C=======================================================================
C
      SUBROUTINE DFFACT(NDMAXI,FCTORI)
      DIMENSION
     *          FCTORI(0:NDMAXI)
C
C=======================================================================
C
C     DFFACT calculates the factorials of n! up to n = NDMAXI
C     The logarithm of n! is stored in a vector (the log allows to
C     avoid too large numbers)
C
C=======================================================================
C
      FCTORI(0)=1.0
C
      IF (NDMAXI.EQ.0) RETURN
C
      DO I=1,NDMAXI
         FCTORI(I)=LOG(REAL(I))+FCTORI(I-1)
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE DFGAMM(NDMAXI,GAMFRA)
      DIMENSION
     *          GAMFRA(0:NDMAXI)
C
C=======================================================================
C
C     DFFACT calculates the gamma function Gamma(n+1/2)
C     The logarithm of Gamma(n+1/2) is stored in a vector (the log 
C     allows to avoid too large numbers)
C
C     Ref. Abramovitz & Stegun, Chap. 6, 6.1.8, 6.1.12
C
C=======================================================================
C
      PIARGU=4.0*ATAN(1.0)
C
      GAMFRA(0)=SQRT(PIARGU)
C
      IF (NDMAXI.EQ.0) RETURN
C
      DO I=1,NDMAXI
         GAMFRA(I)=LOG(I+0.5)+GAMFRA(I-1)
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE DFHERM(ARGHER,NORHER,HERPOL,DHERPO,NDHERM)
      DIMENSION
     *          HERPOL(0:NDHERM),DHERPO(0:NDHERM)
C
C=======================================================================
C
C     DFHERM calculates Hermite polynomials at point X=ARGHER up
C     to N=NORHER-th degree.  It also calculates the derivatives
C     DHERPO of these polynomials
C
C                        DHERPO(I)=2*I*HERPOL(I-1)
C
C=======================================================================
C
      HERPOL(0)=1.0
      DHERPO(0)=0.0
C
      IF (NORHER.EQ.0) RETURN
C
      HERPOL(1)=ARGHER+ARGHER
      DHERPO(1)=2.0000
C
      IF (NORHER.EQ.1) RETURN
C
      DO I=1,NORHER-1
C
         POLYNM=ARGHER*HERPOL(I)-REAL(I)*HERPOL(I-1)
         HERPOL(I+1)=POLYNM+POLYNM
C
         DPOLYN=REAL(I+1)*HERPOL(I)
         DHERPO(I+1)=DPOLYN+DPOLYN
C
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE DLAGUE(XPOINT,XLAGUE,FACTOR,NDFACT,NLAGUE,LORDER)
      PARAMETER
     *         (NDFACT=50)
      PARAMETER
     *         (NDIMEN=NDFACT,NDHERM=NDFACT)
C
      DIMENSION
     *          HERPOL(0:NDHERM),DHERPO(0:NDHERM)
      DIMENSION
     *          FACTOR(0:NDFACT)
      DIMENSION
     *          XLAGUE(-1:NDIMEN)
C
C=======================================================================
C
C     This subroutine calculates the generalized Laguerre polynomials
C     at point XPOINT up to order NLAGUE.
C
C                           k-1/2
C     The output contains  L     (x) with k=0,...,LORDER
C                           n
C
C                            -1/2
C          XLAGUE(-1) -->   L   (x)
C             .              n
C             .
C                            +1/2
C          XLAGUE( 0) -->   L   (x)
C             .              n
C             .
C             .
C                            k+1/2
C          XLAGUE(k)  -->   L     (x)
C                            n
C
C=======================================================================
C
C      We calculate the Hermite Polynomials in HERPOL  up to order
C                                              NORHER = 2*NLAGUE+1
C
      NORHER=2*NLAGUE+1
C
      CALL DFHERM(SQRT(XPOINT),NORHER,HERPOL,DHERPO,NDHERM)
C
C      Initializing the first two Laguerre polynomials
C
      EXPAUX=FACTOR(NLAGUE)*REAL(4**NLAGUE)
C
      XLAGUE(-1)=(-1)**NLAGUE*(HERPOL(NORHER-1))/EXPAUX
C
      XLAGUE( 0)=(-1)**NLAGUE*HERPOL(NORHER)/(2.0*EXPAUX*SQRT(XPOINT))
C
C      Main loop: we calculate L(l+1/2,n)
C
      DO K=0,LORDER-1
         XLAGUE(K+1)=1.0/XPOINT*((XPOINT+REAL(K)+0.5)*XLAGUE(K)-
     *                             REAL(K+0.5+NLAGUE)*XLAGUE(K-1))
      END DO
C
C=======================================================================
C
      RETURN
      END
C=======================================================================
