C FILE NAME = wsu01_main_05.f   ! Keep this symbol here: $ident@string$
C
C=======================================================================
C
C       PROGRAM 'WS_UNIVERSAL' SOLVES THE SCHROEDINGER EQUATION WITH
C       A WOODS-SAXON POTENTIAL
C
C=======================================================================
C
      PROGRAM WS_UNIVERSAL
C=======================================================================
      USE params
C=======================================================================
C
      CHARACTER
     *          FILE_P*256,FILE_N*256,FILEDF*256,SELPAR*20
      CHARACTER
     *          N_ONLY*3,P_ONLY*3,VERSIO*5
      COMPLEX
     *          ALPHAC(0:NDLAMB,-NDLAMB:+NDLAMB)
      DIMENSION
     *          DEFTET(1:NDTETR),DEFOKT(1:NDOKTR),
     *          LAMTET(0:NDTETR),LAMOKT(0:NDOKTR)
      DIMENSION
     *          OMEGAX(0:NDOMEG),OMEGAY(0:NDOMEG),
     *          OMEGAZ(0:NDOMEG),OMEGA3(0:NDOMEG)
C
      COMMON
     *       /NUCLEONIC_NUM/ IZ_ACT,IN_ACT
      COMMON
     *       /MULTIPOLE_LMU/ IFMULT,MAXMUL,NOFLEV,IINDIV,LAMIND,MIUIND
      COMMON
     *       /CRAN3D_GLOBAL/ ROUANG(0:NDOMEG,1:NDDEFO),
     *                       SPIDEF(0:NDOMEG,1:NDDEFO),
     *                       ANGTHE(1:NDDEFO),
     *                       ANGPHI(1:NDDEFO)
      COMMON
     *       /NUCLEONIC_YES/ P_ONLY,N_ONLY
     *       /NUCLEONIC_PAR/ SELPAR
      COMMON
     *       /SIGNATURE_FIX/ ISIGNY,IAUTOM
      COMMON
     *       /ALPHACMUL_DEF/ ALPHAC
     *       /REALALMUL_DEF/ IREALA
      COMMON
     *       /TETRAHEDR_DEF/ DEFTET
     *       /OKTAHEDRL_DEF/ DEFOKT
      COMMON
     *       /TETANDOKT_DEF/ IFTETR,IFOKTR
      COMMON
     *       /DEFORMATN_INP/ FILEDF
     *       /CHOOSEFIL_OUT/ NAMEFI
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
      COMMON
     *       /INITWS_NUMBRS/ PINUMB,PIOVE2
      COMMON
     *       /PARAMS_R0PARM/ R0PARM
C
C=======================================================================
C
C     According to principles  of the modular version  of the code
C     the user controls the input from data file(s) and the output
C     from this MAIN program.  All the technicalities of execution
C     are in principle  hidden from the user. Moreover,  all other
C     packages such as shell-correction, plotting, 3D cranking etc
C     can be connected  by merely calling  the corresponding modul
C     head routine within this main program.
C
C=======================================================================
C
      CALL WS_CPUTIM('WSUNIV',1)
C
C=======================================================================
C     Initialising default values and reading the user input
C=======================================================================
C
      CALL WS_PREDEF(NOFOME,OMEGAX,OMEGAY,OMEGAZ)
C
      IZ_FIX=IZ_ACT
      IN_FIX=IN_ACT
C
      ND_OMG=NDOMEG
C
      MDGAUS=MAX(NDXHRM,NDYHRM,NDZHRM,2*NDOSGS+3)
C
      CALL WS_SHOWOF(NDGAUS,MDGAUS,NDZHRM,NDOSGS,VERSIO)
C
      PINUMB=4.0000*ATAN(1.0)
      PIOVE2=PINUMB/2.0000000
C
C=======================================================================
C
      I_CALL=0
C
   1  CONTINUE
C
      I_CALL=I_CALL+1
C
C      Reading the actual data for one single run
C
      CALL WS_NAMELI(IZ_FIX,IN_FIX,NOFOME,OMEGAX,OMEGAY,OMEGAZ,
     *               OMEGA3,P_ONLY,N_ONLY,FILE_P,FILE_N,SELPAR,
     *                                                  LASTDF)
C
C      Setting the numerical factors for calculating Y_(lm). We have 3
C      sources of usage for Ylm:
c        - the WS basis: LMXBAS
C        - the WS nuclear surface: LMXDEF
C        - the multipole moments: LMXMUL
C      LAMMAX is initialized to the maximum of all these 3 integers
C
      LAMAUX=MAX(LMXBAS,LMXDEF)
      LAMAX1=MAX(LAMAUX,LMXMUL)
      MAXYLM=MAX(LAMAX1,MAXMUL)
      LAMMAX=MAXYLM
C
      CALL WS_LOADIN(MAXYLM)
C
C      Verifying whether the y-signature is conserved
C
      CALL WS_TESIGN(ISIGNY,IAUTOM,ALPHAC,NDLAMB)
C
C      Executing the real runs for the WSUNIV
C
                                                        ISOSPI=0
      CALL WS_WSEXEC(IZ_FIX,IN_FIX,FILE_P,FILE_N,NAMEFI,ISOSPI,
     *               P_ONLY,N_ONLY,OMEGAX,OMEGAY,OMEGAZ,OMEGA3,
     *                             NOFOME,VERSIO,I_CALL,SELPAR)
C
                                                        ISOSPI=1
      CALL WS_WSEXEC(IZ_FIX,IN_FIX,FILE_P,FILE_N,NAMEFI,ISOSPI,
     *               P_ONLY,N_ONLY,OMEGAX,OMEGAY,OMEGAZ,OMEGA3,
     *                             NOFOME,VERSIO,I_CALL,SELPAR)
C
c      CALL WS_PRIROU(ROUANG,SPIDEF,ANGTHE,ANGPHI,I_CALL,NOFOME,
c     *                                               OMEGA3)
      IF (NOFOME.GT.0) CALL WS_PRIALG(NOFOME,OMEGAY)
C
      IF (LASTDF.EQ.0) GO TO 1
C
      IF (LASTDF.EQ.1) GO TO 2
C
C=======================================================================
C     P R I N T I N G   T H E  R U N  -  S U M M A R Y   TABLES
C=======================================================================
C
   2  CONTINUE
C
      CALL WS_ITEPRI
C
      CALL WS_TIMPRI
C
C=======================================================================
C
      IF (ISCREN.EQ.1) THEN
C
          WRITE(0,'(//,''STOP - ALL RIGHT from WSUNIV <<= '',/)')
C
      END IF
C
C=======================================================================
C
      STOP 'ALLRIGHT'
      END
