C FILE NAME = wsu08_math_04.f   ! Keep this symbol here: $ident@string$
C
C=======================================================================
C=======================================================================
C=====                                ==================================
C=====      MATHEMATICAL ROUTINES     ==================================
C=====                                ==================================
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_NEWTON(THANGL,FIANGL,FRIGHT,GRIGHT,EPSF_S,EPSG_S,
     *                                   APROAC,FNSYST,LIMITE,IERNEW)
      EXTERNAL
     *         FNSYST
      COMMON
     *       /INITWS_NUMBRS/ PINUMB,PIOVE2
C
C=======================================================================
C=======================================================================
C
C         This routine solves the system of two nonlinear equations
C         of the form:
C                          FUNCTF(THANGL,FIANGL) = FRIGHT
C                          FUNCTG(THANGL,FIANGL) = GRIGHT
C
C         using the Newton method. This version of the routine will
C         check the accuracy of the solution  only in  terms of the
C         function values
C                          ABS(FUNCTF-FRIGHT) < EPSF_S
C                          ABS(FUNCTG-GRIGHT) < EPSG_S
C
C=======================================================================
C         Used to calculate the distance of a point from the surface
C=======================================================================
C
      ITERAC=0
C
      DO

         ITERAC=ITERAC+1
C
C         Terminating the iteration loop  when exceeding the maximum number
C         of iterations
         IF (ITERAC.GT.LIMITE) THEN
             IERNEW=-1
             EXIT
         END IF
C
         CALL WS_FNSYST(THANGL,FIANGL,DF_DTH,DF_DFI,DG_DTH,DG_DFI,
     *                                              FUNCTF,FUNCTG)
C
         DIFERF=FUNCTF-FRIGHT
         DIFERG=FUNCTG-GRIGHT
C
C         Testing accuracy
         IF (ABS(DIFERF).LT.EPSF_S.AND.ABS(DIFERG).LT.EPSG_S) THEN
             IERNEW=0
             LIMITE=ITERAC
             EXIT
         END IF
C
         DETSYS=DF_DTH*DG_DFI-DF_DFI*DG_DTH
C
C         Testing saddle point vicinity
         IF (ABS(DETSYS).LT.1.0D-20) THEN
             THANGL=1.001*THANGL
             FIANGL=1.001*FIANGL
             CYCLE
         END IF
C
         DELTAT=(DIFERF*DG_DFI-DIFERG*DF_DFI)/DETSYS
         DELTAF=(DIFERG*DF_DTH-DIFERF*DG_DTH)/DETSYS
C
         THANGL=THANGL-DELTAT
         FIANGL=FIANGL-DELTAF
C
C         Testing end of intervals
         IF (THANGL.GT.PINUMB) THANGL=PINUMB-APROAC
         IF (THANGL.LT.0.0000) THANGL=      +APROAC
         IF (FIANGL.LT.0.0000) FIANGL=      +APROAC
         IF (FIANGL.GT.PINUMB) FIANGL=PINUMB-APROAC
C
      END DO
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C=======================================================================
C====                                ===================================
C====    MATHEMATICAL ROUTINE SET    ===================================
C====                                ===================================
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_ORDER1(VECTOR,INDEXV,MAXVEC,NDVECT)
      DIMENSION
     *          VECTOR(1:NDVECT),INDEXV(1:NDVECT)
C
C=======================================================================
C
C     This routine rearranges vector  VECTOR, of dimension MAXVEC
C     so that it is in ascending order  on return.  Vector INDEXV
C     contains, on return, in its k-th element the original index
C     of resulting VECTOR(k).
C
C     On input INDEXV must contain original indices.
C
C     VECTORS IN QUESTION ARE DIMENSIONNED BETWEEN  1  AND NDVECT
C
C=======================================================================
C                                   Method: heapsort of Williams
C=======================================================================
C                                   Based on "Numerical Recipes"
C=======================================================================
C
      IF (MAXVEC.LT.1)
     *
     *    STOP 'ORDER1 called with negative number of vector elements'
C
      IF (MAXVEC.EQ.1) RETURN
C
C=======================================================================
C
      LACTIV=MAXVEC/2+1
      INDACT=MAXVEC
C
      DO WHILE (INDACT.GT.1)
C
         IF (LACTIV.GT.1) THEN
C
             LACTIV=LACTIV-1
C
             VELEMT=VECTOR(LACTIV)
             IELEMT=INDEXV(LACTIV)
C
         ELSE
C
             VELEMT=VECTOR(INDACT)
             IELEMT=INDEXV(INDACT)
C
             VECTOR(INDACT)=VECTOR(1)
             INDEXV(INDACT)=INDEXV(1)
C
             INDACT=INDACT-1
C
             IF (INDACT.EQ.1) THEN
                 VECTOR(1)=VELEMT
                 INDEXV(1)=IELEMT
                 EXIT
             END IF
C
         END IF
C
         I=LACTIV
         J=LACTIV+LACTIV
C
         DO WHILE (J.LE.INDACT)
C
            IF (J.LT.INDACT.AND.VECTOR(J).LT.VECTOR(J+1))
     *          J=J+1
C
            IF (VELEMT.LT.VECTOR(J)) THEN
C
                VECTOR(I)=VECTOR(J)
                INDEXV(I)=INDEXV(J)
C
                I=J
                J=J+J
C
            ELSE
                J=INDACT+1
            END IF
C
         END DO
C
         VECTOR(I)=VELEMT
         INDEXV(I)=IELEMT
C
      END DO
C
C=======================================================================
C
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_ORDER0(VECTOR,INDEXV,MAXVEC,NDVECT)
      DIMENSION
     *          VECTOR(0:NDVECT),INDEXV(0:NDVECT)
      DIMENSION
     *          VECTOR_(1:NDVECT+1),INDEXV_(0:NDVECT+1)
C
C=======================================================================
C
      IF (MAXVEC.LT.1)
     *
     *    STOP 'ORDER0 called with negative number of vector elements'
C
      IF (MAXVEC.EQ.1) RETURN
C
C=======================================================================
C
      VECTOR_(1:NDVECT+1)=VECTOR(0:NDVECT)
C
      CALL WS_ORDER1(VECTOR_,INDEXV_,MAXVEC+1,NDVECT+1)
C
      VECTOR(0:NDVECT)=VECTOR_(1:NDVECT+1)
      INDEXV(0:NDVECT)=INDEXV_(1:NDVECT+1)
C
C=======================================================================
C
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_ORDLOW(VECTOR,KINDEX,LDIMEN,NUMLOW,AUXVEC,NDIMEN,
     *                                                 MINDEX,LINDEX)
      DIMENSION
     *          VECTOR(1:NDIMEN),
     *          KINDEX(1:NDIMEN)
      DIMENSION
     *          AUXVEC(1:NDIMEN),
     *          MINDEX(1:NDIMEN),
     *          LINDEX(1:NDIMEN)
C
C=======================================================================
C     This subroutine finds the first (highest) NUMLOW numbers
C     out of  LDIMEN  values of the vector, whose dimension is
C                                                       NDIMEN
C=======================================================================
C
      IF (LDIMEN.GT.NDIMEN) STOP 'LDIMEN.GT.NDIMEN in ORDLOW'
C
C=======================================================================
C
      DO I=1,LDIMEN
         LINDEX(I)=0
         MINDEX(I)=0
         AUXVEC(I)=VECTOR(I)
      END DO
C
      DO JCOUNT=1,NUMLOW
C
         XINFIN=-1.000E+32
C
         DO I=1,LDIMEN
C
            IF (MINDEX(I).EQ.0) THEN
C
                IF (VECTOR(I).GE.XINFIN) THEN
                    XINFIN=VECTOR(I)
                    INDEXM=I
                END IF
C
            END IF
C
         END DO
C
         MINDEX(INDEXM)=KINDEX(INDEXM)
         LINDEX(JCOUNT)=KINDEX(INDEXM)
C
      END DO
C
      DO JCOUNT=1,NUMLOW
         VECTOR(JCOUNT)=AUXVEC(LINDEX(JCOUNT))
         KINDEX(JCOUNT)=LINDEX(JCOUNT)
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_GAULEG(XLOWER,XUPPER,XLEGEN,WEIGHL,NGAUSL,NDGUSL)
C=======================================================================
      USE params
C=======================================================================
      DIMENSION
     *          XLEGEN(1:NDGUSL),
     *          WEIGHL(1:NDGUSL)
C
C=======================================================================
C
C     Subroutine GAULEG calculates weights and nodes of the Gauss
C     Legendre integration formula
C
C     Integral(XLOWER,XUPPER)[f(x)dx] = Sum (i=1,N)[W(i)*f(X(i))]
C
C     The nodes are stored in vector XLEGEN  while the weights in
C     vector WEIGHL.  The integration limits, usually -1 and + 1,
C     Here can be given as arbitrary real parametrs such that
C
C                            XLOWER < XUPPER.
C
C     must be compiled  i n   t h e    a u t o d o u b l e   mode
C
C=======================================================================
C
      EPSLEG=3.000000D-17
      MGAUSL=(NGAUSL+1)/2
C
      X_SUMM=0.5D0*(XUPPER+XLOWER)
      X_DIFF=0.5D0*(XUPPER-XLOWER)
C
      PINUMB=4.0D0*ATAN(1.0D0)
C
      DO IGAUSL=1,MGAUSL
C
         RUNING=COS(PINUMB*(IGAUSL-0.25D0)/(NGAUSL+0.5D0))
C
         ZGREAT=1.0D+30
         NCOUNT=0
C
         DO
C
            PUNCT1=1
            PUNCT2=0
C
            NCOUNT=NCOUNT+1
C
            DO JCOUNT=1,NGAUSL
C
               PUNCT3=PUNCT2
               PUNCT2=PUNCT1
               PUNCT1=((2*JCOUNT-1)*RUNING*PUNCT2-(JCOUNT-1)*PUNCT3)
     *                   /JCOUNT
C
            END DO
C
            PPUNCT=NGAUSL*(RUNING*PUNCT1-PUNCT2)/(RUNING*RUNING-1)
            ARGUM1=RUNING
            RUNING=ARGUM1-PUNCT1/PPUNCT
            ARGUM2=ABS(RUNING-ARGUM1)
C
            IF (ARGUM2.LE.ZGREAT) THEN
                ZGREAT=ARGUM2
                ARGUMB=RUNING
            END IF
C
            IF (ARGUM2.LE.EPSLEG.OR.NCOUNT.GE.MAXLEG) EXIT

         END DO
C
         LGAUSL=NGAUSL+1-IGAUSL
C
         XLEGEN(IGAUSL)=X_SUMM-X_DIFF*ARGUMB
         XLEGEN(LGAUSL)=X_SUMM+X_DIFF*ARGUMB
C
         WEIGHL(IGAUSL)=2.0D0*X_DIFF/((1-ARGUMB*ARGUMB)*PPUNCT*PPUNCT)
C
         WEIGHL(LGAUSL)=WEIGHL(IGAUSL)
C
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DFHERM(ARGHER,NORHER,HERPOL,DHERPO,NDHERM)
      DIMENSION
     *          HERPOL(0:NDHERM),DHERPO(0:NDHERM)
C
C=======================================================================
C
C     DFHERM calculates Hermite polynomials at point X=ARGHER up
C     to N=NORHER-th degree.  It also calculates the derivatives
C     DHERPO of these polynomials
C
C                        DHERPO(I)=2*I*HERPOL(I-1)
C
C=======================================================================
C
      HERPOL(0)=1.0D0
      DHERPO(0)=0.0D0
C
      IF (NORHER.EQ.0) RETURN
C
      HERPOL(1)=ARGHER+ARGHER
      DHERPO(1)=2.0000
C
      IF (NORHER.EQ.1) RETURN
C
      DO I=1,NORHER-1
C
         POLYNM=ARGHER*HERPOL(I)-REAL(I,KIND=8)*HERPOL(I-1)
         HERPOL(I+1)=POLYNM+POLYNM
C
         DPOLYN=REAL(I+1,KIND=8)*HERPOL(I)
         DHERPO(I+1)=DPOLYN+DPOLYN
C
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_NORFAC(SQFACT,ND2MAX)
      DIMENSION
     *          SQFACT(0:ND2MAX)
C
C=======================================================================
C
      SQFACT(0)=SQRT(SQRT(4.0D0*ATAN(1.0D0)))
C
      DO I=1,ND2MAX
         SQFACT(I)=SQFACT(I-1)*SQRT(REAL(2*I,KIND=8))
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_GAUHER(XNODES,WEIGHT,NGAUSS,NDGAUS,EPSHER)
      DOUBLE PRECISION
     *                 PIMIN4,P1_AUX,P2_AUX,P3_AUX,P_SQRT,ZARGUM,
     *                                                    ZBEFOR
      PARAMETER
     *         (PIMIN4=.7511255444649425D0,MAXITE=10)  !  PI**(-1/4)
      DIMENSION
     *          WEIGHT(1:NDGAUS),XNODES(1:NDGAUS)
C
C=======================================================================
C
C     This subroutine gives Gauss-Hermite nodes and weights placed
C     in 'XNODES' and 'WEIGHT'. The integral has the form:
C
C     INT(-inf,+inf) [exp(-x**x) f(x) dx] = SUM(1,N) [W(I)F(X(I))]
C
C     where:   N <=> NGAUSS denotes the order of the Gauss formula
C              W <=> WEIGHT (the weight vector)
C              X <=> XNODES (the  node  vector)
C
C     The nodes will be stored in descending order.
C
C=======================================================================
C=============================================== T.R.WERNER, AUGUST 1993
C======== FROM "NUMERICAL  RECIPES" 2-ND ED. ===========================
C=======================================================================
C=======================================================================
C
      NODE_2=(NGAUSS+1)/2
C
C=======================================================================
C
      DO I_NODE=1,NODE_2
         XNODES(I_NODE)=0.0D0
      END DO
C
C=======================================================================
C
      DO I_NODE=1,NODE_2
C
         IF (I_NODE.EQ.1) THEN
C
             ZARGUM=SQRT(REAL(2*NGAUSS+1,KIND=8))
     *             -1.85575D0*(2*NGAUSS+1)**(-1.D0/6)
C
         ELSE IF (I_NODE.EQ.2) THEN
C
             ZARGUM=ZARGUM-1.14*NGAUSS**0.426/ZARGUM
C
         ELSE IF (I_NODE.EQ.3) THEN
C
             ZARGUM=1.86*ZARGUM-0.86*XNODES(1)
C
         ELSE IF (I_NODE.EQ.4) THEN
C
             ZARGUM=1.91*ZARGUM-0.91*XNODES(2)
C
         ELSE
C
             ZARGUM=2*ZARGUM-XNODES(I_NODE-2)
C
         END IF
C
         DO ITS=1,MAXITE
C
            P1_AUX=PIMIN4
            P2_AUX=0.00D0
C
            DO J=1,NGAUSS
C
               P3_AUX=P2_AUX
               P2_AUX=P1_AUX
               P1_AUX=ZARGUM*SQRT(2.D0/J)*P2_AUX
     *       - SQRT(REAL(J-1,KIND=8)/REAL(J,KIND=8))*P3_AUX
C
            END DO
C
            P_SQRT=SQRT(2.D0*NGAUSS)*P2_AUX
C
            ZBEFOR=ZARGUM
            ZARGUM=ZBEFOR-P1_AUX/P_SQRT
C
            IF (ABS(ZARGUM-ZBEFOR).LE.EPSHER) GO TO 1
C
         END DO
C
         STOP 'GAUHER'
C
    1    CONTINUE
C
         XNODES(I_NODE         )=+ZARGUM
         XNODES(NGAUSS+1-I_NODE)=-ZARGUM
C
         WEIGHT(I_NODE         )=2.000/(P_SQRT*P_SQRT)
         WEIGHT(NGAUSS+1-I_NODE)=WEIGHT(I_NODE)
C
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DLAGUE(XPOINT,XLAGUE,FACTOR,NLAGUE,LORDER)
C=======================================================================
      USE params
C=======================================================================
C
      DIMENSION
     *          HERPOL(0:NDHERM),DHERPO(0:NDHERM)
      DIMENSION
     *          FACTOR(0:NDFACT)
      DIMENSION
     *          XLAGUE(-1:NDMAIN)
C
C=======================================================================
C
C     This subroutine calculates the generalized Laguerre polynomials
C     at point XPOINT up to order NLAGUE.
C
C                            k-1/2
C     The output conatains  L      (x) with k=0...IALPHA
C                            n
C
C                            -1/2
C          XLAGUE(-1) -->   L   (x)
C             .              n
C             .
C                            +1/2
C          XLAGUE( 0) -->   L   (x)
C             .              n
C             .
C             .
C                            k+1/2
C          XLAGUE(k)  -->   L     (x)
C                            n
C
C=======================================================================
C
C=======================================================================
C     We calculate the Hermite Polynomials in HERPOL  up to order
C                                             NORHER = 2*NLAGUE+1
C=======================================================================
C
      NORHER=2*NLAGUE+1
C
      CALL WS_DFHERM(SQRT(XPOINT),NORHER,HERPOL,DHERPO,NDHERM)
C
C=======================================================================
C     Initializing the first two Laguerre polynomials
C=======================================================================
C
      EXPAUX=FACTOR(NLAGUE)*REAL(4**NLAGUE,KIND=8)
C
      XLAGUE(-1)=(-1)**NLAGUE*(HERPOL(NORHER-1))/EXPAUX
C
      XLAGUE( 0)=(-1)**NLAGUE*HERPOL(NORHER)/(2.0*EXPAUX*SQRT(XPOINT))
C
C=======================================================================
C     Main loop: we calculate L(l+1/2,n)
C=======================================================================
C
      DO K=0,LORDER-1
         XLAGUE(K+1)=1.0/XPOINT*((XPOINT+REAL(K,KIND=8)+0.5D0)
     *                           *XLAGUE(K)-
     *                                  (REAL(K+NLAGUE,KIND=8)+0.5D0)
     *                            *XLAGUE(K-1))
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=====================================================================
C=====================================================================
C
      SUBROUTINE WS_zufall(n,a)
      DIMENSION :: a(n)
C
      CALL RANDOM_NUMBER(a)
C
      END
C
C=====================================================================
C=====================================================================
C
      SUBROUTINE WS_RANDIN(iseed)
C
      CALL RANDOM_SEED(iseed)
C
      RETURN
      END
C
C=======================================================================
c=====================================================================
C
      SUBROUTINE WS_INTER2(X1,X2,X3,Y1,Y2,Y3,A,B,C)
C
C=======================================================================
C         Parabolic interpolation routine finds parameters a,b,c
C         of a parabola Y=A*X**2+B*X+C
C=======================================================================
C
      A=0.0D0
      B=0.0D0
      C=0.0D0
C
      IF (X1.EQ.X2) X2=X2+0.000001D0
      IF (X2.EQ.X3) X3=X3+0.000001D0
C
      F1=(Y3-Y2)*(X2-X1)-(Y2-Y1)*(X3-X2)
      F2=(X3-X2)*(X2-X1)*(X1-X3)
C
      IF (ABS(F2).GT.1.D-14) THEN
C
          A=-F1/F2
          B=(Y2-Y1)/(X2-X1)-A*(X1+X2)
          C=Y1-A*X1**2-B*X1
C
         RETURN
C
      ELSE
C
         WRITE(6,'(''Error in paraboliC interpolation WS_INTER2'')')
         WRITE(6,'(''X1 = '',e12.5,'' X2 = '',E12.5,'' X3 = '',E12.5)')
     *               X1,X2,X3
         WRITE(6,'(''Y1 = '',e12.5,'' Y2 = '',E12.5,'' Y3 = '',E12.5)')
     *               Y1,Y2,y3
C
      END IF
C
      RETURN
      END
C
C=======================================================================
