C FILE NAME = wsu11_crk3_04.f   ! Keep this symbol here: $ident@string$
C
C=======================================================================
C=======================================================================
C=====                                ==================================
C=====   3-D CRANKING ROUTINE SET     ==================================
C=====                                ==================================
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_CRAN3D(IZ_FIX,IN_FIX,NXMAXX,NYMAXX,NZMAXX,LDBASE,
     *                     ICHARG,NOFOME,NOFLEV,OMEGA3,KRANKG,THEANG,
     *                     PHIANG,RYTEXT,EMINST,EMAXST,EMINWF,EMAXWF,
     *                     NILWAV,NILSLB,TOTROU,TOTSPI,ISIGNY,IFMULT,
     *                                   MAXMUL,IINDIV,LAMIND,MIUIND)
C=======================================================================
      USE params
C=======================================================================
C
      CHARACTER
     *          RYTEXT(0:NDREVE)*5
C
      COMPLEX
     *          WAVOCC
      COMPLEX
     *          H_STAT,BIG_PP,BIG_PM
C
      DIMENSION
     *          TOTROU(0:NDOMEG,0:NDISOS),
     *          TOTSPI(0:NDOMEG,0:NDISOS)
      DIMENSION
     *          OMEGAX(0:NDOMEG),OMEGAY(0:NDOMEG),
     *          OMEGAZ(0:NDOMEG),OMEGA3(0:NDOMEG)
      DIMENSION
     *          ESPECT(1:NDBASE)
      DIMENSION
     *          EDOUBL(1:2*NDSTAT),LSTATE(1:2*NDSTAT)
      DIMENSION
     *          HAUXPP(1:NDBASE,1:NDBASE),
     *          HAUXPM(1:NDBASE,1:NDBASE)
      DIMENSION
     *          AMOMUL(0:NDMULT,-NDMULT:NDMULT)
C
C                   Inputs
C
      COMMON
     *       /SPECTR_H_STAT/ H_STAT(1:NDBASE,1:NDBASE)
      COMMON
     *       /PUT_MM_SPNUMS/ NUMBSP(0:NDREVE)
      COMMON
     *       /CRANKG_SIGDIM/ LDSTAT(0:NDREVE)
C
C                   Outputs
C
      COMMON
     *       /CRAN3D_MAT_PP/ BIG_PP(1:NDBASE,1:NDBASE,0:NDREVE)
     *       /CRAN3D_MAT_PM/ BIG_PM(1:NDBASE,1:NDBASE,0:NDREVE)
      COMMON
     *       /CRAN3D_ALLWAV/ WAVOCC(1:NDBASE,1:2*NDSTAT,0:NDREVE)
      COMMON
     *       /CRAN3D_SPLV3D/ SPENER(1:2*NDSTAT,0:NDISOS),
     *                       SPOCCU(1:2*NDSTAT,0:NDISOS),
     *                       SPALIG(1:2*NDSTAT,0:NDISOS),
     *                       SPSPAL(1:2*NDSTAT,0:NDISOS),
     *                       SPPARI(1:2*NDSTAT,0:NDISOS),
     *                       SPSIMP(1:2*NDSTAT,0:NDISOS)
      COMMON
     *       /CONSER_SIMPLY/ ISIMPY
C
C=======================================================================
C
C        This subroutine diagonalizes the cranked mean-field. The
C        The rotational axis OMEGA3 can be tilted relative to the
C        axes of the system. It is defined in this version by the
C        two spherical angles THEANG and PHIANG.
C
c        Three-dimensional cranking requires the calculation of the
C        matrix elements of the 3 components of the total angular
C        momentum Jx, Jy and Jz. Because the time-reversal symmetry
C        is no-longer conserved, and Jx and Jz have a non-diagonal
C        block structure in the t-basis, we must diagonalize a matrix
C        whose size is double than in the standard 1-D-cranking.
C
C=======================================================================
C
      IF (KRANKG.GE.0) STOP 'No 3D cranking from CRAN3D'
C
C=======================================================================
C
      CALL WS_CPUTIM('CRAN3D',1)
C
C=======================================================================
C
                       NFERMI=IN_FIX
      IF (ICHARG.EQ.1) NFERMI=IZ_FIX
C
C      Getting the angles of the rotational frequency vector in radians
C
      PIARGU=4.0*ATAN(1.0)
C
      PHIARG=PHIANG*PIARGU/180.0
      THEARG=THEANG*PIARGU/180.0
C
      DO IOMEGA=1,NOFOME
C
         ISIMPY=0
C
C         Calculating the x, y and z components of the rotational
C         frequency vector
C
         OMEGAC=OMEGA3(IOMEGA)
C
         OMEGAX(IOMEGA)=OMEGAC*SIN(THEARG)*COS(PHIARG)
	 OMEGAY(IOMEGA)=OMEGAC*SIN(THEARG)*SIN(PHIARG)
	 OMEGAZ(IOMEGA)=OMEGAC*COS(THEARG)
C
         OMEG_X=OMEGAX(IOMEGA)
         OMEG_Y=OMEGAY(IOMEGA)
         OMEG_Z=OMEGAZ(IOMEGA)
C
	 IF (ABS(OMEG_X).LE.1.0E-10.AND.ABS(OMEG_Z).LE.1.0E-10)
     *           ISIMPY=1
C
C         Initializing the cranked mean-field matrix
C
         DO IKETHO=1,LDBASE
            DO IBRAHO=1,LDBASE
C
               BIG_PP(IBRAHO,IKETHO,0)=CMPLX(0.0,0.0)
               BIG_PP(IBRAHO,IKETHO,1)=CMPLX(0.0,0.0)
C
               BIG_PM(IBRAHO,IKETHO,0)=CMPLX(0.0,0.0)
               BIG_PM(IBRAHO,IKETHO,1)=CMPLX(0.0,0.0)
C
            END DO
         END DO
C
C         Calculating the angular momentum components matrix elements
C         in the t-basis. For jx and jz, one calculates the <+|-> part
C         (<-|+> = conjg(<+|->)) while for jy, one calculates the <+|+>
C         part (<-|-> = conjg(<+|+>)). The static mean-field was also
C         calculated in the <+|+> block only.
C
C         Calculating matrix elements: Lx and Sx.
C
         IF (ABS(OMEG_X).GT.1.0E-10) THEN
C
             CALL WS_INT_LX(NXMAXX,NYMAXX,NZMAXX,HAUXPM,OMEG_X,LDBASE)
C
             CALL WS_ADDIMA(-1,HAUXPM,BIG_PM(1,1,1),LDBASE)
C
             CALL WS_INT_SX(NXMAXX,NYMAXX,NZMAXX,HAUXPM,OMEG_X,LDBASE)
C
             CALL WS_ADDREA(-1,HAUXPM,BIG_PM(1,1,1),LDBASE)
C
	 END IF
C
C         Calculating matrix elements: Ly and Sy
C
         IF (ABS(OMEG_Y).GT.1.0E-10) THEN
C
             CALL WS_INT_LY(NXMAXX,NYMAXX,NZMAXX,HAUXPM,OMEG_Y,LDBASE)
C
             CALL WS_ADDIMA(-1,HAUXPM,BIG_PP(1,1,0),LDBASE)
             CALL WS_ADDIMA(-1,HAUXPM,BIG_PP(1,1,1),LDBASE)
C
             CALL WS_INT_SY(NXMAXX,NYMAXX,NZMAXX,HAUXPM,OMEG_Y,LDBASE)
C
             CALL WS_ADDREA(-1,HAUXPM,BIG_PP(1,1,0),LDBASE)
             CALL WS_ADDREA(+1,HAUXPM,BIG_PP(1,1,1),LDBASE)
C
	 END IF
C
C         Calculating matrix elements: Lz and Sz
C
         IF (ABS(OMEG_Z).GT.1.0E-10) THEN
C
             CALL WS_INT_LZ(NXMAXX,NYMAXX,NZMAXX,HAUXPM,OMEG_Z,LDBASE)
C
             CALL WS_ADDIMA(-1,HAUXPM,BIG_PM(1,1,1),LDBASE)
C
             CALL WS_INT_SZ(NXMAXX,NYMAXX,NZMAXX,HAUXPM,OMEG_Z,LDBASE)
C
             CALL WS_ADDIMA(-1,HAUXPM,BIG_PM(1,1,1),LDBASE)
C
	 END IF
C
C         Adding the static mean-field part to the two diagonal blocks
C
         LDSTAT(0)=LDBASE
         LDSTAT(1)=LDBASE
C
         NUMBSP(0)=LDBASE
         NUMBSP(1)=LDBASE
C
	 LDOUBL=LDBASE+LDBASE
C
         DO IKETHO=1,LDBASE
            DO IBRAHO=1,LDBASE
               BIG_PP(IBRAHO,IKETHO,0)=
     *         BIG_PP(IBRAHO,IKETHO,0)+H_STAT(IBRAHO,IKETHO)
            END DO
         END DO
C
         DO IKETHO=1,LDBASE
            DO IBRAHO=1,LDBASE
               BIG_PP(IBRAHO,IKETHO,1)=
     *         BIG_PP(IBRAHO,IKETHO,1)+CONJG(H_STAT(IBRAHO,IKETHO))
            END DO
         END DO
C
C         Diagonalizing the matrix of the mean field in the case
C         of broken simplex -> rotation about a tilted axis
C
         IF (ISIMPY.EQ.0) THEN
C
C          Diagonalization for two parities separately (automatic)
C
             IF (ISIGNY.EQ.1) THEN
C
                 CALL WS_DPARI3(ICHARG,LDOUBL)
C
             ELSE
C
                 CALL WS_DIASIZ(ICHARG,LDBASE)
C
             END IF
C
	 ELSE
C
C         Diagonalizing the matrix of the mean field in the case
C         of (accidentally) conserved simplex -> rotation about
C         the Y-axis only
C
C	      Block < t+ | t+ >
             CALL WS_DIAGCM(BIG_PP(1,1,0),ESPECT,LDBASE)
C
             NSTATE=0
C
             DO ISTATE=1,LDBASE
	        NSTATE=NSTATE+1
		LSTATE(NSTATE)=NSTATE
                EDOUBL(ISTATE)=ESPECT(ISTATE)
             END DO
C
C	      Block < t- | t- >
             CALL WS_DIAGCM(BIG_PP(1,1,1),ESPECT,LDBASE)
C
             DO ISTATE=LDBASE+1,LDBASE+LDBASE
	        NSTATE=NSTATE+1
		LSTATE(NSTATE)=NSTATE
                EDOUBL(ISTATE)=ESPECT(ISTATE-LDBASE)
             END DO
C
             CALL WS_ORDER0(EDOUBL,LSTATE,NSTATE,2*NDSTAT)
C
             DO I_BASE=1,LDBASE
                DO IALLST=1,LDOUBL
C
		   I_TRUE=LSTATE(IALLST)
		   I_REAL=I_TRUE-LDBASE
C
		   IF (I_TRUE.GT.LDBASE) THEN
                       WAVOCC(I_BASE,IALLST,0)=0.0
                       WAVOCC(I_BASE,IALLST,1)=BIG_PP(I_BASE,I_REAL,1)
		   ELSE
                       WAVOCC(I_BASE,IALLST,0)=BIG_PP(I_BASE,I_TRUE,0)
                       WAVOCC(I_BASE,IALLST,1)=0.0
		   END IF
C
                END DO
             END DO
C
             DO IALLST=1,LDOUBL
	        SPENER(IALLST,ICHARG)=EDOUBL(IALLST)
             END DO
C
	 END IF
C
         SUMSPE=0.0
	 DO I=1,NFERMI
	    SUMSPE=SUMSPE+SPENER(I,ICHARG)
	 END DO
	 TOTROU(IOMEGA,ICHARG)=SUMSPE
C
C         Cartesian Labels (compulsory)
C
         CALL WS_CART3D(ICHARG,EMINST,EMAXST,OMEGAC,LDBASE)
C
C         Nilsson Labels (optionnal)
C
         CALL WS_NILS3D(ICHARG,EMINST,EMAXST,EMINWF,EMAXWF,OMEGAC,
     *                                       NILWAV,NILSLB,LDBASE)
C
C         X-, Y- and Z-alignments
C
         CALL WS_ANGM3D(NXMAXX,NYMAXX,NZMAXX,ICHARG,IOMEGA,NFERMI,
     *                                             TOTSPI,LDBASE)
C
         CALL WS_PRIDOU(ICHARG,EMINST,EMAXST,OMEGAC,LDOUBL,LDBASE)
C
C=======================================================================
C        Calculating and printing the multipole moments
C=======================================================================
C
C         Electric moments
C
         CALL WS_MOMETS(ICHARG,IZ_FIX,IN_FIX,IFMULT,MAXMUL,NOFLEV,
     *                         KRANKG,ISIGNY,IINDIV,LAMIND,MIUIND)
C
         CALL WS_MOMPRI(MAXMUL,ICHARG,IFMULT,OMEGAC,KRANKG)
C
C         Magnetic moments
C
         CALL WS_MAGMOM(ICHARG,IZ_FIX,IN_FIX,IFMULT,MAXMUL,KRANKG,
     *                                       IINDIV,LAMIND,MIUIND)
C
         CALL WS_MAGPRI(MAXMUL,ICHARG,IFMULT,OMEGAC,KRANKG)
C
C=======================================================================
C
C         Recording results on disk
C
         CALL WS_REVO3D(ICHARG,IZ_FIX,IN_FIX,OMEG_X,OMEG_Y,OMEG_Z,
     *                  THEANG,PHIANG,NOFOME,LDBASE,IFMULT,MAXMUL)
C
      END DO ! end of omega loop
C
C=======================================================================
C
      CALL WS_CPUTIM('CRAN3D',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_PRIDOU(ICHARG,EMINST,EMAXST,OMEGAC,LDOUBL,LDBASE)
C=======================================================================
      USE params
C=======================================================================
C
      CHARACTER
     *          DOUBDB*13,LABEL1*13,LABEL2*13
      CHARACTER
     *          STRNP1*5,STRNP2*5
C
      COMMON
     *       /CRAN3D_SPLV3D/ SPENER(1:2*NDSTAT,0:NDISOS),
     *                       SPOCCU(1:2*NDSTAT,0:NDISOS),
     *                       SPALIG(1:2*NDSTAT,0:NDISOS),
     *                       SPSPAL(1:2*NDSTAT,0:NDISOS),
     *                       SPPARI(1:2*NDSTAT,0:NDISOS),
     *                       SPSIMP(1:2*NDSTAT,0:NDISOS)
      COMMON
     *       /CRAN3D_QUANUM/ ISIGNZ(1:2*NDSTAT,0:NDISOS),
     *                       IDOUBZ(1:2*NDSTAT,0:NDISOS)
C
C=======================================================================
C
C         This subroutine prints a table of the single-particle
C         routhians with their z-signature and doublex labels
C
C=======================================================================
C
      CALL WS_CPUTIM('PRIDOU',1)
C
C=======================================================================
C
C      Preparing the printing of sigle-particle levels to the output
C
      IF (ICHARG.EQ.0) THEN
C
          WRITE(6,'(//,80(''#''),/,''#'',78X,''#'',/,
     *              ''#    NEUTRONS:       Single Particle -'',
     *              ''  Signature/doublex       OMEGA='',  F5.3,
     *                                              ''    #'',/,
     *              ''#'',78X,''#'',/,
     *                              80(''#''),/,''#'',78X,''#'',/,
     *              ''#   No)   Energy    Doublex       Rz       '',
     *              ''No)   Energy    Doublex       Rz    #'',/,
     *              ''#'',78X,''#'')')  OMEGAC
C
      ELSE
C
          WRITE(6,'(//,80(''#''),/,''#'',78X,''#'',/,
     *              ''#    PROTONS:        Single Particle -'',
     *              ''  Signature/doublex       OMEGA='',F5.3,
     *                                              ''    #'',/,
     *              ''#'',78X,''#'',/,
     *                              80(''#''),/,''#'',78X,''#'',/,
     *              ''#   No)   Energy    Doublex       Rz       '',
     *              ''No)   Energy    Doublex       Rz    #'',/,
     *              ''#'',78X,''#'')')  OMEGAC
C
      END IF
C
C      Defining the window range for displaying the single-particle
C      energies
C
      KIKOFF=0
      ISTART=1
C
      DO ISTATE=1,LDOUBL
         IF (SPENER(ISTATE,ICHARG).GE.EMINST.AND.
     *       SPENER(ISTATE,ICHARG).LE.EMAXST) THEN
             KIKOFF=KIKOFF+1
             IF (KIKOFF.EQ.1) ISTART=ISTATE
         END IF
      END DO
C
      KIKOFF=KIKOFF/2-1
C
      IF (ISTART+KIKOFF.GT.2*LDBASE)
     *              STOP 'ISTART+KIKOFF.GT.LDBASE - IN PRIDOU (2)'
C
C=======================================================================
C
C      Printing the ordered single-particle energies with their
C      z-signature and doublex labels
C
      DO L=ISTART,ISTART+KIKOFF
C
         L1=L
         L2=KIKOFF+L+1
C
C         Getting signature corresponding to the first column
C
         IF (ISIGNZ(L1,ICHARG).GT.0.0) THEN
             IPTES1=INT(ISIGNZ(L1,ICHARG)*100+0.49999)
         ELSE
             IPTES1=INT(ISIGNZ(L1,ICHARG)*100-0.49999)
         END IF
C
C         Making special label for signature of the first column
C
         IF (IABS(IPTES1).NE.100) THEN
             WRITE(STRNP1,'(''('',SP,I3,SS,'')'')') IPTES1
         END IF
C
         IF (IPTES1.EQ.+100) THEN
             WRITE(STRNP1,'(''( + )'')')
         END IF
C
         IF (IPTES1.EQ.-100) THEN
             WRITE(STRNP1,'(''( - )'')')
         END IF
C
C         Making the doublex label for the first column
C
         WRITE(DOUBDB(1:13),'(''     XXX     '')')
C
         IF (IDOUBZ(L1,ICHARG).EQ.1)
     *       WRITE(DOUBDB(1:13),'(''  e^(i*pi/4) '')')
C
         IF (IDOUBZ(L1,ICHARG).EQ.3)
     *       WRITE(DOUBDB(1:13),'('' e^(i*3*pi/4)'')')
C
         IF (IDOUBZ(L1,ICHARG).EQ.5)
     *       WRITE(DOUBDB(1:13),'('' e^(i*5*pi/4)'')')
C
         IF (IDOUBZ(L1,ICHARG).EQ.7)
     *       WRITE(DOUBDB(1:13),'('' e^(i*7*pi/4)'')')
C
         LABEL1=DOUBDB
C
C         Getting signature corresponding to the second column
C
         IF (ISIGNZ(L2,ICHARG).GT.0.0) THEN
             IPTES2=INT(ISIGNZ(L2,ICHARG)*100+0.49999)
         ELSE
             IPTES2=INT(ISIGNZ(L2,ICHARG)*100-0.49999)
         END IF
C
C         Making special label for signature of the second column
C
         IF (IABS(IPTES2).NE.100) THEN
             WRITE(STRNP2,'(''('',SP,I3,SS,'')'')') IPTES2
         END IF
C
         IF (IPTES2.EQ.+100) THEN
             WRITE(STRNP2,'(''( + )'')')
         END IF
C
         IF (IPTES2.EQ.-100) THEN
             WRITE(STRNP2,'(''( - )'')')
         END IF
C
C         Making the doublex label for the first column
C
         WRITE(DOUBDB(1:13),'(''     XXX     '')')
C
         IF (IDOUBZ(L2,ICHARG).EQ.1)
     *       WRITE(DOUBDB(1:13),'(''  e^(i*pi/4) '')')
C
         IF (IDOUBZ(L2,ICHARG).EQ.3)
     *       WRITE(DOUBDB(1:13),'('' e^(i*3*pi/4)'')')
C
         IF (IDOUBZ(L2,ICHARG).EQ.5)
     *       WRITE(DOUBDB(1:13),'('' e^(i*5*pi/4)'')')
C
         IF (IDOUBZ(L2,ICHARG).EQ.7)
     *       WRITE(DOUBDB(1:13),'('' e^(i*7*pi/4)'')')
C
         LABEL2=DOUBDB
C
C         Printing the single-particle energies with the labels
C
         WRITE(6,'(''#  '',I3,'')'',F10.5,'' '',A13,
     *                         1X,A5,''     '',
     *                     I3,'')'',F10.5,'' '',A13,
     *                         1X,A5,''   #'')')
     *
     *             L1,SPENER(L1,ICHARG),LABEL1,STRNP1,
     *             L2,SPENER(L2,ICHARG),LABEL2,STRNP2
C
      END DO
C
      WRITE(6,'(''#'',78X,''#'',/,80(''#''))')
C
C=======================================================================
C
      CALL WS_CPUTIM('PRIDOU',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_ANGM3D(NXMAXX,NYMAXX,NZMAXX,ICHARG,IOMEGA,NFERMI,
     *                                                 TOTSPI,LDBASE)
C=======================================================================
      USE params
C=======================================================================
C
      COMPLEX
     *          CMPLPP(1:2*NDSTAT),CMPLPM(1:2*NDSTAT)
C
      DIMENSION
     *          EXPDUM(1:2*NDSTAT)
C=======================================================================
      ALLOCATABLE DOUBPP(:,:),DOUBPM(:,:),HAUXPP(:,:)
C=======================================================================
      DIMENSION
     *          TOTSPI(0:NDOMEG,0:NDISOS)
C
      COMMON
     *       /CRAN3D_SPLV3D/ SPENER(1:2*NDSTAT,0:NDISOS),
     *                       SPOCCU(1:2*NDSTAT,0:NDISOS),
     *                       SPALIG(1:2*NDSTAT,0:NDISOS),
     *                       SPSPAL(1:2*NDSTAT,0:NDISOS),
     *                       SPPARI(1:2*NDSTAT,0:NDISOS),
     *                       SPSIMP(1:2*NDSTAT,0:NDISOS)
      COMMON
     *       /CRAN3D_SPXZ3D/ SPXALI(1:2*NDSTAT,0:NDISOS),
     *                       SPZALI(1:2*NDSTAT,0:NDISOS),
     *                       SPXSPI(1:2*NDSTAT,0:NDISOS),
     *                       SPZSPI(1:2*NDSTAT,0:NDISOS)
      COMMON
     *       /CRAN3D_QUANUM/ ISIGNZ(1:2*NDSTAT,0:NDISOS),
     *                       IDOUBZ(1:2*NDSTAT,0:NDISOS)
      COMMON
     *       /PUT_MM_SPNUMS/ NUMBSP(0:NDREVE)
C
C=======================================================================
C
C        This routine calculates total and intrinsic alignments
C
C=======================================================================
C
      CALL WS_CPUTIM('ANGM3D',1)
C
C=======================================================================
C
      ALLOCATE(DOUBPP(1:NDBASE,1:NDBASE))
      ALLOCATE(DOUBPM(1:NDBASE,1:NDBASE))
      ALLOCATE(HAUXPP(1:NDBASE,1:NDBASE))
C
      DO I=1,2*NDSTAT
         EXPDUM(I)=0.0
	 CMPLPP(I)=CMPLX(0.0,0.0)
	 CMPLPM(I)=CMPLX(0.0,0.0)
      END DO
C
      FACTOR=1.0
C
      IHERMI=+1
      ITIPAR=-1
C
      ICMPLX=0
C
      LSTATE=NUMBSP(0)+NUMBSP(1)
C
C=======================================================================
C         Y-DIRECTION
C=======================================================================
C
      ISIPAR=+1
      IMAREA=-1
C
      CALL WS_INT_LY(NXMAXX,NYMAXX,NZMAXX,HAUXPP,FACTOR,LDBASE)
C
      CALL WS_AVRO3D(SPALIG(1,ICHARG),CMPLPP,HAUXPP,LSTATE,
     *               IMAREA,ISIPAR,ITIPAR,IHERMI,ICMPLX,LDBASE)
C
      IMAREA=+1
C
      CALL WS_INT_SY(NXMAXX,NYMAXX,NZMAXX,HAUXPP,FACTOR,LDBASE)
C
      CALL WS_AVRO3D(SPSPAL(1,ICHARG),CMPLPP,HAUXPP,LSTATE,
     *               IMAREA,ISIPAR,ITIPAR,IHERMI,ICMPLX,LDBASE)
C
C=======================================================================
C         X-DIRECTION
C=======================================================================
C
      ISIPAR=-1
      IMAREA=-1
C
      CALL WS_INT_LX(NXMAXX,NYMAXX,NZMAXX,HAUXPP,FACTOR,LDBASE)
C
      CALL WS_AVRO3D(SPXALI(1,ICHARG),CMPLPP,HAUXPP,LSTATE,
     *               IMAREA,ISIPAR,ITIPAR,IHERMI,ICMPLX,LDBASE)
C
      IMAREA=+1
C
      CALL WS_INT_SX(NXMAXX,NYMAXX,NZMAXX,HAUXPP,FACTOR,LDBASE)
C
      CALL WS_AVRO3D(SPXSPI(1,ICHARG),CMPLPP,HAUXPP,LSTATE,
     *               IMAREA,ISIPAR,ITIPAR,IHERMI,ICMPLX,LDBASE)
C
C=======================================================================
C         Z-DIRECTION
C=======================================================================
C
      ISIPAR=-1
      IMAREA=-1
C
      CALL WS_INT_LZ(NXMAXX,NYMAXX,NZMAXX,HAUXPP,FACTOR,LDBASE)
C
      CALL WS_AVRO3D(SPZALI(1,ICHARG),CMPLPP,HAUXPP,LSTATE,
     *               IMAREA,ISIPAR,ITIPAR,IHERMI,ICMPLX,LDBASE)
C
      IMAREA=-1
C
      CALL WS_INT_SZ(NXMAXX,NYMAXX,NZMAXX,HAUXPP,FACTOR,LDBASE)
C
      CALL WS_AVRO3D(SPZSPI(1,ICHARG),CMPLPP,HAUXPP,LSTATE,
     *               IMAREA,ISIPAR,ITIPAR,IHERMI,ICMPLX,LDBASE)
C
C=======================================================================
C
      DO ISTATE=1,LSTATE
C
         SPALIG(ISTATE,ICHARG)=SPALIG(ISTATE,ICHARG)
     *                        +SPSPAL(ISTATE,ICHARG)
         SPXALI(ISTATE,ICHARG)=SPXALI(ISTATE,ICHARG)
     *                        +SPXSPI(ISTATE,ICHARG)
         SPZALI(ISTATE,ICHARG)=SPZALI(ISTATE,ICHARG)
     *                        +SPZSPI(ISTATE,ICHARG)
C
      END DO
C
C=======================================================================
C
      ALIGJX=0.0
      ALIGJY=0.0
      ALIGJZ=0.0
C
      DO I=1,NFERMI
         ALIGJX=ALIGJX+SPXALI(I,ICHARG)
         ALIGJY=ALIGJY+SPALIG(I,ICHARG)
         ALIGJZ=ALIGJZ+SPZALI(I,ICHARG)
      END DO
C
      ANGMOM=SQRT(ALIGJX**2+ALIGJY**2+ALIGJZ**2)
C
      TOTSPI(IOMEGA,ICHARG)=ANGMOM
C
C=======================================================================
C                 "DOUBLEX" EXPECTATION VALUE
C=======================================================================
C
      CALL WS_INTDBL(NXMAXX,NYMAXX,NZMAXX,DOUBPP,DOUBPM,FACTOR,
     *                                                  LDBASE)
C
      ICMPLX=+1
C
      IMAREA=+1
      IHERMI=+1
      ITIPAR=+1
C
      ISIPAR=+1
C
      CALL WS_AVRO3D(EXPDUM,CMPLPP,DOUBPP,LSTATE,IMAREA,ISIPAR,
     *                             ITIPAR,IHERMI,ICMPLX,LDBASE)
C
      ISIPAR=-1
C
      CALL WS_AVRO3D(EXPDUM,CMPLPM,DOUBPM,LSTATE,IMAREA,ISIPAR,
     *                             ITIPAR,IHERMI,ICMPLX,LDBASE)
C
      DO ISTATE=1,LSTATE
         CMPLPP(ISTATE)=CMPLPP(ISTATE)+CMPLPM(ISTATE)
      END DO
C
C      Assigning the doublex. Numerical accuracy is limited for protons
C      due to the Coulomb potential integrations. As a consequence,
C      the eigenvalues of the doublex can not be tested at less than
c      10^(-3)
C
      SQRT22=0.5*SQRT(2.0)
C
      DO ISTATE=1,LSTATE
C
	 IDOUBZ(ISTATE,ICHARG)=0
C
	 CMPREA=REAL(CMPLPP(ISTATE))
	 CMPIMA=AIMAG(CMPLPP(ISTATE))
C
         IF (ABS(CMPREA-SQRT22).LT.1.E-3.AND.
     *       ABS(CMPIMA-SQRT22).LT.1.E-3) THEN
	         IDOUBZ(ISTATE,ICHARG)=1
	 END IF
C
         IF (ABS(CMPREA+SQRT22).LT.1.E-3.AND.
     *       ABS(CMPIMA-SQRT22).LT.1.E-3) THEN
	         IDOUBZ(ISTATE,ICHARG)=3
	 END IF
C
         IF (ABS(CMPREA+SQRT22).LT.1.E-3.AND.
     *       ABS(CMPIMA+SQRT22).LT.1.E-3) THEN
	         IDOUBZ(ISTATE,ICHARG)=5
	 END IF
C
         IF (ABS(CMPREA-SQRT22).LT.1.E-3.AND.
     *       ABS(CMPIMA+SQRT22).LT.1.E-3) THEN
	         IDOUBZ(ISTATE,ICHARG)=7
	 END IF
C
c	 IF (IDOUBZ(ISTATE,ICHARG).EQ.0) THEN
c	     WRITE(6,'(''ISTATE = '',I4,'' CMPLPP = '',2F14.10)')
c     *                   ISTATE,CMPLPP(ISTATE)
c	 END IF
C
      END DO
C
C=======================================================================
C                 "Z-SIGNATURE" EXPECTATION VALUE
C=======================================================================
C
      CALL WS_INTSGZ(NXMAXX,NYMAXX,NZMAXX,HAUXPP,FACTOR,LDBASE)
C
      ICMPLX=-1
C
      IMAREA=-1
      IHERMI=+1
      ITIPAR=+1
      ISIPAR=-1
C
      CALL WS_AVRO3D(EXPDUM,CMPLPM,HAUXPP,LSTATE,IMAREA,ISIPAR,
     *                             ITIPAR,IHERMI,ICMPLX,LDBASE)
C
      DO ISTATE=1,LSTATE
	 ISIGNZ(ISTATE,ICHARG)=NINT(EXPDUM(ISTATE))
      END DO
C
      DEALLOCATE(DOUBPP,DOUBPM,HAUXPP)
C
C=======================================================================
C
      CALL WS_CPUTIM('ANGM3D',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_AVRO3D(AVRVAL,CPLXAV,OPIMRE,LSTATE,IMAREA,ISIPAR,
     *                                   ITIPAR,IHERMI,ICMPLX,LDBASE)
C=======================================================================
      USE params
C=======================================================================
C
      COMPLEX
     *           CPLXAV(1:2*NDSTAT)
C
      DIMENSION
     *           AVRVAL(1:2*NDSTAT)
      DIMENSION
     *           OPIMRE(1:NDBASE,1:NDBASE)
C
      COMPLEX
     *           WAVOCC,RESULT
C
      COMMON
     *       /CRAN3D_ALLWAV/ WAVOCC(1:NDBASE,1:2*NDSTAT,0:NDREVE)
C
C=======================================================================
C
C     This subroutine calculates single-particle average values of
c     an operator which is:
C        1. Real or imaginary in the simplex basis (IMAREA=+1/-1)
C        2. Diagonal or off-diagonal in simplex (ISIPAR=+1/-1)
C        3. Time-even or time-odd (ITIPAR=+1/-1)
C        4. Hermitian or anti-hermitian (IHERMI=+1/-1)
C
C     Input array OPIMRE should contain
C        1. Real or imaginary parts of matrix elements (IMAREA=+1/-1)
C        2. In simplex plus-plus or plus-minus block (ISIPAR=+1/-1)
C
C     Resulting AVRVAL vector then contains
C        1. Real or imaginary parts of average values (IHERMI=+1/-1)
C
C=======================================================================
C
      CALL WS_CPUTIM('AVRO3D',1)
C
C=======================================================================
C
      IF (ISIPAR.EQ.1) THEN
C
          DO ISTATE=1,LSTATE
C
             RESULT=CMPLX(0.0,0.0)
C
             DO IBRAHO=1,LDBASE
                DO IKETHO=1,LDBASE
                   RESULT=RESULT
     *            +OPIMRE(IBRAHO,IKETHO)*(CONJG(WAVOCC(IBRAHO,ISTATE,0))
     *                                         *WAVOCC(IKETHO,ISTATE,0)
     *                     +IMAREA*ITIPAR*CONJG(WAVOCC(IBRAHO,ISTATE,1))
     *                                         *WAVOCC(IKETHO,ISTATE,1))
                END DO
             END DO
C
	     IF (ICMPLX.EQ.1) THEN
C
                 CPLXAV(ISTATE)=RESULT
C
	     ELSE
C
                 IF (IMAREA.EQ.-1) RESULT=RESULT*CMPLX(0.0,1.0)
C
                 IF (IHERMI.EQ.1) THEN
                     AVRVAL(ISTATE)=REAL(RESULT)
                 ELSE
                     AVRVAL(ISTATE)=AIMAG(RESULT)
                 END IF
C
	     END IF
C
          END DO
C
      ELSE
C
          DO ISTATE=1,LSTATE
C
             RESULT=CMPLX(0.0,0.0)
C
             DO IBRAHO=1,LDBASE
                DO IKETHO=1,LDBASE
                   RESULT=RESULT+OPIMRE(IBRAHO,IKETHO)
     *                           *CONJG(WAVOCC(IBRAHO,ISTATE,1))
     *                                 *WAVOCC(IKETHO,ISTATE,0)
                END DO
             END DO
C
	     IF (ICMPLX.EQ.1) THEN
C
                 CPLXAV(ISTATE)=RESULT
C
                 CPLXAV(ISTATE)=-2.0*IMAREA*ITIPAR*CPLXAV(ISTATE)
C
	     ELSE
C
                 IF (IMAREA.EQ.-1) RESULT=RESULT*CMPLX(0.0,1.0)
C
                 IF (IHERMI.EQ.1) THEN
                     AVRVAL(ISTATE)=REAL(RESULT)
                 ELSE
                     AVRVAL(ISTATE)=AIMAG(RESULT)
                 END IF
C
                 AVRVAL(ISTATE)=-2*IMAREA*ITIPAR*AVRVAL(ISTATE)
C
	     END IF
C
          END DO
C
      END IF
C
C=======================================================================
C
      CALL WS_CPUTIM('AVRO3D',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_STORRO(TOTROU,TOTSPI,ROUANG,SPIDEF,NOFOME,IDEFCN,
     *                                   THEANG,PHIANG,ANGTHE,ANGPHI)
C=======================================================================
      USE params
C=======================================================================
C
      DIMENSION
     *          TOTROU(0:NDOMEG,0:NDISOS),
     *          TOTSPI(0:NDOMEG,0:NDISOS)
      DIMENSION
     *          ROUANG(0:NDOMEG,1:NDDEFO),
     *          SPIDEF(0:NDOMEG,1:NDDEFO)
      DIMENSION
     *          ANGTHE(1:NDDEFO),ANGPHI(1:NDDEFO)
C
C=======================================================================
C
      ANGTHE(IDEFCN)=THEANG
      ANGPHI(IDEFCN)=PHIANG
C
      DO IOMEGA=1,NOFOME
         ROUANG(IOMEGA,IDEFCN)=TOTROU(IOMEGA,0)+TOTROU(IOMEGA,1)
         SPIDEF(IOMEGA,IDEFCN)=TOTSPI(IOMEGA,0)+TOTSPI(IOMEGA,1)
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_PRIROU(ROUANG,SPIDEF,ANGTHE,ANGPHI,NOFDEF,NOFOME,
     *                                                        OMEGA3)
C=======================================================================
      USE params
C=======================================================================
      DIMENSION
     *          ROUANG(0:NDOMEG,1:NDDEFO),
     *          SPIDEF(0:NDOMEG,1:NDDEFO)
      DIMENSION
     *          ANGTHE(1:NDDEFO),ANGPHI(1:NDDEFO)
      DIMENSION
     *          OMEGA3(0:NDOMEG)
C
C=======================================================================
C
      WRITE(6,'(//,60(''*''),/,''*'',58X,''*'',/,
     *        ''*      Theta     Phi     Omega     Rtotal'',
     *        ''      Spin        *'',/,''*'',58X,''*'')')
C
      DO IDEFCN=1,NOFDEF
C
         WRITE(6,'(''*   '',2F8.1,F10.2,F12.2,F11.2,6X,''*'')')
     *
     *            ANGTHE(IDEFCN),ANGPHI(IDEFCN),
     *               OMEGA3(1),ROUANG(1,IDEFCN),
     *                         SPIDEF(1,IDEFCN)
C
         DO IOMEGA=2,NOFOME
C
            WRITE(6,'(''*   '',16X,F10.2,F12.2,F11.2,6X,''*'')')
     *
     *                    OMEGA3(IOMEGA),
     *                    ROUANG(IOMEGA,IDEFCN),
     *                    SPIDEF(IOMEGA,IDEFCN)
C
         END DO
C
      END DO
C
      WRITE(6,'(''*'',58X,''*'',/,60(''*''))')
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_INTDBL(NXMAXX,NYMAXX,NZMAXX,DOUBPP,DOUBPM,FACTOR,
     *                                                        LDBASE)
C=======================================================================
      USE params
C=======================================================================
C
      DIMENSION
     *           DOUBPP(1:NDBASE,1:NDBASE),DOUBPM(1:NDBASE,1:NDBASE)
C
      COMMON
     *       /TPHASE_PHASES/ TPHAPP(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAPM(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAMP(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAMM(0:NDYMAX,0:NDYMAX,0:NDKART)
      COMMON
     *       /SETBAS_INDICE/ INDICE(0:NDXMAX,0:NDYMAX,0:NDZMAX)
     *
     *       /SETBAS_BASIS1/ LAXOFY(0:NDYMAX),LAXOFZ(0:NDZMAX),
     *                       LAYOFZ(0:NDZMAX),LAYOFX(0:NDXMAX),
     *                       LAZOFX(0:NDXMAX),LAZOFY(0:NDYMAX)
     *
     *       /SETBAS_BASIS2/ LAXOYZ(0:NDYMAX,0:NDZMAX),
     *                       LAYOZX(0:NDZMAX,0:NDXMAX),
     *                       LAZOXY(0:NDXMAX,0:NDYMAX)
     *
     *       /SETBAS_NXNYNZ/ NXVECT(1:NDBASE),
     *                       NYVECT(1:NDBASE),
     *                       NZVECT(1:NDBASE)
C
C=======================================================================
C
C     This subroutine calculates the matrix elements of the doublex
C     operator on the modified harmnic oscillator basis.
C
C=======================================================================
C
      DO IKETHO=1,LDBASE
         DO IBRAHO=1,LDBASE
            DOUBPP(IBRAHO,IKETHO)=0.0
            DOUBPM(IBRAHO,IKETHO)=0.0
        END DO
      END DO
C
      SQRT22=0.5*SQRT(2.0)
C
      DO IKETHO=1,LDBASE
C
         NX=NXVECT(IKETHO)
         NY=NYVECT(IKETHO)
         NZ=NZVECT(IKETHO)
C
         DO IBRAHO=1,LDBASE
C
            MX=NXVECT(IBRAHO)
            MY=NYVECT(IBRAHO)
            MZ=NZVECT(IBRAHO)
C
            IF (MZ.EQ.NZ.AND.MX.EQ.NY.AND.MY.EQ.NX) THEN
C
                DOUBPP(IBRAHO,IKETHO)=(-1)**(NX+NZ)*SQRT22
     *
     *               *(TPHAPP(MY,NY,0)+TPHAPP(MY,NY,3))
C
                DOUBPM(IBRAHO,IKETHO)=(-1)**(NX+NZ)*SQRT22
     *
     *               *(TPHAPM(MY,NY,0)+TPHAPM(MY,NY,3))
C
            END IF
C
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_INTSGZ(NXMAXX,NYMAXX,NZMAXX,ELMSGZ,FACTOR,LDBASE)
C=======================================================================
      USE params
C=======================================================================
C
      DIMENSION
     *           ELMSGZ(1:NDBASE,1:NDBASE)
C
      COMMON
     *       /TPHASE_PHASES/ TPHAPP(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAPM(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAMP(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAMM(0:NDYMAX,0:NDYMAX,0:NDKART)
      COMMON
     *       /SETBAS_INDICE/ INDICE(0:NDXMAX,0:NDYMAX,0:NDZMAX)
     *
     *       /SETBAS_BASIS1/ LAXOFY(0:NDYMAX),LAXOFZ(0:NDZMAX),
     *                       LAYOFZ(0:NDZMAX),LAYOFX(0:NDXMAX),
     *                       LAZOFX(0:NDXMAX),LAZOFY(0:NDYMAX)
     *
     *       /SETBAS_BASIS2/ LAXOYZ(0:NDYMAX,0:NDZMAX),
     *                       LAYOZX(0:NDZMAX,0:NDXMAX),
     *                       LAZOXY(0:NDXMAX,0:NDYMAX)
C
C=======================================================================
C
C     This subroutine calculates the matrix elements of the doublex
C     operator on the modified harmnic oscillator basis.
C
C=======================================================================
C
      DO IKETHO=1,LDBASE
         DO IBRAHO=1,LDBASE
            ELMSGZ(IBRAHO,IKETHO)=0.0
        END DO
      END DO
C
      DO NZ=0,NZMAXX
         MZ=NZ
         DO NY=0,LAYOFZ(NZ)
	    MY=NY
            DO NX=0,LAXOYZ(NY,NZ)
	       MX=NX
C
               IBRAHO=INDICE(MX,MY,MZ)
               IKETHO=INDICE(NX,NY,NZ)
C
               ELMSGZ(IBRAHO,IKETHO)=(-1)**(NX+NY)*TPHAPM(MY,NY,3)
C
            END DO
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
