C FILE NAME = wsu12_dens_04.f   ! Keep this symbol here: $ident@string$
C
C=======================================================================
C=======================================================================
C=====                                ==================================
C=====      NUCLEONIC DENSITY SET     ==================================
C=====                                ==================================
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DENSSP(NXMAXX,NYMAXX,NZMAXX,NXHERM,NYHERM,NZHERM,
     *                     ISOSPI,IZ_FIX,IN_FIX,IFDENS,NOBODY,R_MAXI)
C=======================================================================
      USE params
C=======================================================================
C
      COMPLEX
     *          EWAVEF,WAVEUP,WAVEDW,PHASUP,PHASDW
      COMPLEX
     *          CCOEFF,DCOEFF,DTILDA
      COMPLEX
     *          UNIT_I
C
      DIMENSION
     *          EXPXSI(1:NDXDEN),EXPETA(1:NDYDEN),EXPZET(1:NDZDEN)
C=======================================================================
      ALLOCATABLE CCOEFF(:,:,:)
      ALLOCATABLE DCOEFF(:,:,:)
      ALLOCATABLE DTILDA(:,:,:)
      ALLOCATABLE WAVEUP(:,:,:)
      ALLOCATABLE WAVEDW(:,:,:)
C=======================================================================
      DIMENSION
     *          V2OCUP(1:NDBASE)
      DIMENSION
     *          PHASUP(0:2*NDOSGS),
     *          PHASDW(0:2*NDOSGS)
      DIMENSION
     *          HERMIT(0:NDGAUS,1:NDDENS,1:NDKART)
      DIMENSION
     *          HERAUX(0:NDDENS),
     *          DHERAX(0:NDDENS),
     *          SQFACT(0:NDDENS)
C
      COMMON
     *       /DENSSP_DENFIN/ DENSIT(1:NDXDEN,1:NDYDEN,1:NDZDEN),
     *                       DENNOD(1:NDXHRM,1:NDYHRM,1:NDZHRM)
     *
     *       /DENDEN_DIMDEN/ MXHERM,MYHERM,MZHERM
     *
     *       /DENSSP_KSIDSS/ RXIDSS(1:NDDENS),
     *                       RETDSS(1:NDDENS),
     *                       RDZDSS(1:NDDENS)
      COMMON
     *       /SETBAS_INDICE/ INDICE(0:NDXMAX,0:NDYMAX,0:NDZMAX)
     *
     *       /SETBAS_BASIS1/ LAXOFY(0:NDYMAX),LAXOFZ(0:NDZMAX),
     *                       LAYOFZ(0:NDZMAX),LAYOFX(0:NDXMAX),
     *                       LAZOFX(0:NDXMAX),LAZOFY(0:NDYMAX)
     *
     *       /SETBAS_BASIS2/ LAXOYZ(0:NDYMAX,0:NDZMAX),
     *                       LAYOZX(0:NDZMAX,0:NDXMAX),
     *                       LAZOXY(0:NDXMAX,0:NDYMAX)
      COMMON
     *       /SPECTR_SPINOR/ EWAVEF(1:NDBASE,1:NDBASE)
      COMMON
     *       /SETBAS_DIMENS/ LDBASE
C
      COMMON
     *       /DENSSP_FERMIS/ XYZPNS(1:NDDENS,1:NDKART)
      COMMON
     *       /SETBAS_SCALNG/ HOMSCA(1:NDKART)
C
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
      DATA
     *     EPSOCU  / 0.0001 /
C
C=======================================================================
C
C        This subroutine calculates the space representation of the
C        nucleonic density from the Woods-Saxon  deformed potential
C        a t  t h e  u s e r  d e f i n e d   m e s h   p o i n t s
C
C        To make enemies life more difficult numbers of the user de-
C        fined points are limited by parameters called N{x,y,z}HERM
C        although these  have nothing in common with  Gauss-Hermite
C        integration formulae.
C
C=======================================================================
C
      IF (IFDENS.EQ.0) RETURN
C
C=======================================================================
C
      CALL WS_CPUTIM('DENSSP',1)
C
C=======================================================================
C
C        The Hermite polynomials, denoted  "HERACT", are multiplied
C        by the dimensioned normalization constant:
C
C                     (MASS*HOMEG{}/H_BAR**2)**(1/4)
C
C        This corresponds to introducing the one - dimensional wave
C        functions in units of fm**(-1/2). Corresponding  densities
C        in three dimensions are then in units of fm**(-3)
C
C        N{}MAXX - denote the maximum quantum numbers for the H.OSC
C        N{}HERM - denote the maximum numbers of Gauss intg. points
C
C=======================================================================
C
      MXHERM=NXHERM
      MYHERM=NYHERM
      MZHERM=NZHERM
C
C        Attention: for this variant of the density calculations we
C                   will need to introduce the delimiters 'M{}HERM'
C                   independent of the N{}HERM, the latter reserved
C                   for the calculations at the Gauss nodes
C
      IF (MXHERM.GT.NDXDEN) STOP 'MXHERM.GT.NDXDEN in DENSSP <<= '
      IF (MYHERM.GT.NDYDEN) STOP 'MYHERM.GT.NDYDEN in DENSSP <<= '
      IF (MZHERM.GT.NDZDEN) STOP 'MZHERM.GT.NDZDEN in DENSSP <<= '
C
      UNIT_I=CMPLX(0.0,1.0)
C
      DO IPHASE=0,2*NDOSGS
         PHASUP(IPHASE)=UNIT_I**( IPHASE )
         PHASDW(IPHASE)=UNIT_I**(1-IPHASE)
      END DO
C
      IF (ISOSPI.EQ.0) THEN
          N_STAT=IN_FIX/2
          N_PART=IN_FIX
      END IF
C
      IF (ISOSPI.EQ.1) THEN
          N_STAT=IZ_FIX/2
          N_PART=IZ_FIX
      END IF
C
      LIMITS=N_PART
C
      IF (LIMITS.GT.NDBASE) STOP 'LIMITS.GT.NDBASE in DENSSP <<= '
C
C      Calculations with sharp occupation distribution
C
      DO ISTATE=1,LDBASE
                               V2OCUP(ISTATE)=1.0
         IF (ISTATE.GT.N_STAT) V2OCUP(ISTATE)=0.0
      END DO
C
      IF (MOD(N_PART,2).NE.0)  V2OCUP(N_PART/2+1)=0.5
C
      DO IZ=1,NZHERM
         DO IY=1,NYHERM
            DO IX=1,NXHERM
               DENSIT(IX,IY,IZ)=0.0
            END DO
         END DO
      END DO
C
C=======================================================================
C             Defining an auxiliary mesh for plotting the density
C=======================================================================
C
      CALL WS_NORFAC(SQFACT,ND2MAX)
C
      IF (NOBODY.EQ.2) SCBODY=SQRT(2.0)
      IF (NOBODY.EQ.1) SCBODY=(1.00000)
C
      DO II=0,ND2MAX
         HERAUX(II)=0.0
         DHERAX(II)=0.0
      END DO
C
C      Below XYZPNS represents the Cartesian points in fermis. For
C      calculating the density, they will need to be multiplied by
C      HOMSCA. Note: in the exponentials we have the dimensionless
C      variables chi (equal XYZPNS*HOMSCA)
C
      KARTEZ=1
C
      DO IX=1,NXHERM
C
         X0ARGU=((REAL(IX)-1.0)/(REAL(NXHERM)-1.0))
     *                       *(R_MAXI/SCBODY)
C
         CALL WS_DFHERM(X0ARGU*HOMSCA(KARTEZ),NXMAXX,HERAUX,DHERAX,
     *                                                      ND2MAX)
         DO NX=0,NXMAXX
            HERMIT(NX,IX,KARTEZ)=((SQRT(HOMSCA(KARTEZ)))/
     *                             SQFACT(NX))*HERAUX(NX)
         END DO
C
         XYZPNS(IX,KARTEZ)=X0ARGU
C
      END DO
C
      KARTEZ=2
C
      DO IY=1,NYHERM
C
         Y0ARGU=((REAL(IY)-1.0)/(REAL(NYHERM)-1.0))*(R_MAXI/SCBODY)
C
         CALL WS_DFHERM(Y0ARGU*HOMSCA(KARTEZ),NYMAXX,HERAUX,DHERAX,
     *                                                      ND2MAX)
         DO NY=0,NYMAXX
            HERMIT(NY,IY,KARTEZ)=((SQRT(HOMSCA(KARTEZ)))/
     *                             SQFACT(NY))*HERAUX(NY)
         END DO
C
         XYZPNS(IY,KARTEZ)=Y0ARGU
C
      END DO
C
      KARTEZ=3
C
      DO IZ=1,NZHERM
C
         Z0ARGU=((REAL(IZ)-1.0)/(REAL(NZHERM)-1.0))*(R_MAXI/SCBODY)
C
         CALL WS_DFHERM(Z0ARGU*HOMSCA(KARTEZ),NZMAXX,HERAUX,DHERAX,
     *                                                      ND2MAX)
         DO NZ=0,NZMAXX
            HERMIT(NZ,IZ,KARTEZ)=((SQRT(HOMSCA(KARTEZ)))/
     *                             SQFACT(NZ))*HERAUX(NZ)
         END DO
C
         XYZPNS(IZ,KARTEZ)=Z0ARGU
C
      END DO
C
C=======================================================================
C
      ALLOCATE(CCOEFF(0:NDXMAX,0:NDYMAX,1:NDGAUS))
      ALLOCATE(DCOEFF(0:NDXMAX,1:NDGAUS,1:NDGAUS))
      ALLOCATE(DTILDA(0:NDXMAX,1:NDGAUS,1:NDGAUS))
C
      ALLOCATE(WAVEUP(1:NDXDEN,1:NDYDEN,1:NDZDEN))
      ALLOCATE(WAVEDW(1:NDXDEN,1:NDYDEN,1:NDZDEN))
C
      DO ISTATE=1,LIMITS
C
         IF (V2OCUP(ISTATE).GE.EPSOCU) THEN
C
C             Defining the auxiliary products of the wave functions
C
C             NX, NY, NZ correspond to loops over quantum numbers
C             IX, IY, IZ correspond to loops over the coordinates
C                                                     x, y and z
C
             DO IZ=1,NZHERM
                DO NY=0,LAYOFX(NX)
                   DO NX=0,NXMAXX
                      CCOEFF(NX,NY,IZ)=CMPLX(0.0,0.0)
                   END DO
                END DO
             END DO
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO NX=0,NXMAXX
                      DCOEFF(NX,IY,IZ)=CMPLX(0.0,0.0)
                      DTILDA(NX,IY,IZ)=CMPLX(0.0,0.0)
                   END DO
                END DO
             END DO
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO IX=1,NXHERM
                      WAVEUP(IX,IY,IZ)=CMPLX(0.0,0.0)
                      WAVEDW(IX,IY,IZ)=CMPLX(0.0,0.0)
                   END DO
                END DO
             END DO
C
C             Here we work in the {|t+>,|t->} basis, where the |t+> and
C             |t-> vectors can be expressed in terms of vectors
C
C                            |b> = i^ny |nx,ny,nz;sz>
C
C             and we have:
C
C                     |t+> = 1/sqrt(2) [ i^ny|up> -i^(1-ny)|down> ]
C
C             where |up>=|nx,ny,nz;+sz> and |down>=|nx,ny,nz;-sz>.
C
C             The total wave function within the |t+> block is then:
C
C                   |Psi+> = \sum_{nx,ny,nz,sz} C(nx,ny,nz,sz) |t+>
C
C             In the following, EWAVEF contains C(nx,ny,nz,sz) coefficients;
C
C             The total (static) density is a sum of spin-up and spin-down
C             components
C
C             Below we sum up over z; this operation does not depen on the
C             UP and DOWN components
C
             KARTEZ=3
C
             DO NX=0,NXMAXX
                DO NY=0,LAYOFX(NX)
C
                   DO IZ=1,NZHERM
C
                      DO NZ=0,LAZOXY(NX,NY)
C
                         IBRAHO=INDICE(NX,NY,NZ)
C
                         CCOEFF(NX,NY,IZ)=CCOEFF(NX,NY,IZ)
     *                                   +EWAVEF(IBRAHO,ISTATE)
     *                                   *HERMIT(NZ,IZ, KARTEZ)
                      END DO
C
                   END DO
C
                END DO
             END DO
C
C             The summation for the y-components is performed separately
C             for the two phase coefficients
C
             KARTEZ=2
C
             DO NX=0,NXMAXX
C
                DO IY=1,NYHERM
                   DO IZ=1,NZHERM
C
                      DO NY=0,LAYOFX(NX)
C
                         DCOEFF(NX,IY,IZ)=DCOEFF(NX,IY,IZ)
     *                                   +CCOEFF(NX,NY,IZ)
     *                                   *HERMIT(NY,IY,KARTEZ)
     *                                   *PHASUP(NY)
C
                      END DO
C
                   END DO
                END DO
C
             END DO
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO NX=0,NXMAXX
C
                      DO NY=0,LAYOFX(NX)
                         DTILDA(NX,IY,IZ)=DTILDA(NX,IY,IZ)
     *                                   -CCOEFF(NX,NY,IZ)
     *                                   *HERMIT(NY,IY,KARTEZ)
     *                                   *PHASDW(NY)
                      END DO
C
                   END DO
                END DO
             END DO
C
C             Finally, we sum over x,  but we now have two different
C             functions corresponding to the two parts that make the
C             t-vector.
C
             KARTEZ=1
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO IX=1,NXHERM
C
                      DO NX=0,NXMAXX
                         WAVEUP(IX,IY,IZ)=WAVEUP(IX,IY,IZ)
     *                                   +DCOEFF(NX,IY,IZ)
     *                                   *HERMIT(NX,IX,KARTEZ)
                      END DO
C
                   END DO
                END DO
             END DO
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO IX=1,NXHERM
C
                      DO NX=0,NXMAXX
                         WAVEDW(IX,IY,IZ)=WAVEDW(IX,IY,IZ)
     *                                   +DTILDA(NX,IY,IZ)
     *                                   *HERMIT(NX,IX,KARTEZ)
                      END DO
C
                   END DO
                END DO
             END DO
C
C             Having computed the individual wave functions, we deduce
C             the density. The factor of 2 that appears comes from the
C             Kramers degeneracy: we performed all the calculations in
C             the |t+> block of the total Hamiltonian.
C
             SQRT_2=SQRT(2.0)
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO IX=1,NXHERM
C
C                      For the calculation of the norm, we have
C
C                              PSI_total = PSI_up + PSI_down
C
C                      and as PSI_up and PSI_down have opposite spins,
C                      they are orthogonal and consequently the sums
C                      reduce to the following one:
C
                      WAVEUP(IX,IY,IZ)=WAVEUP(IX,IY,IZ)/SQRT_2
                      WAVEDW(IX,IY,IZ)=WAVEDW(IX,IY,IZ)/SQRT_2
C
                      DENAUX          =CABS(WAVEUP(IX,IY,IZ))**2
     *                                +CABS(WAVEDW(IX,IY,IZ))**2
C
                      DENSIT(IX,IY,IZ)=DENSIT(IX,IY,IZ)
     *                                +2.0*DENAUX*V2OCUP(ISTATE)
C
                   END DO
                END DO
             END DO
C
         END IF
C
      END DO
C
      DEALLOCATE(CCOEFF,DCOEFF,DTILDA)
      DEALLOCATE(WAVEUP,WAVEDW)
C
C=======================================================================
C        The coefficients below should be used to calculate
C        the exponential term in the density
C=======================================================================
C
      DO IX=1,NXHERM
         RXIDSS(IX)=XYZPNS(IX,1)*HOMSCA(1)
      END DO
C
      DO IY=1,NYHERM
         RETDSS(IY)=XYZPNS(IY,2)*HOMSCA(2)
      END DO
C
      DO IZ=1,NZHERM
         RDZDSS(IZ)=XYZPNS(IZ,3)*HOMSCA(3)
      END DO
C
      DO IXHERM=1,NXHERM
         EXPXSI(IXHERM)=EXP(-(RXIDSS(IXHERM)**2))
      END DO
C
      DO IYHERM=1,NYHERM
         EXPETA(IYHERM)=EXP(-(RETDSS(IYHERM)**2))
      END DO
C
      DO IZHERM=1,NZHERM
         EXPZET(IZHERM)=EXP(-(RDZDSS(IZHERM)**2))
      END DO
C
      DO IZ=1,NZHERM
         DO IY=1,NYHERM
            DO IX=1,NXHERM
C
               DENSIT(IX,IY,IZ)=DENSIT(IX,IY,IZ)*EXPXSI(IX)
     *                                          *EXPETA(IY)
     *                                          *EXPZET(IZ)
C
            END DO
         END DO
      END DO
C
C=======================================================================
C
      CALL WS_CPUTIM('DENSSP',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DENINT(NXMAXX,NYMAXX,NZMAXX,NXHERM,NYHERM,NZHERM,
     *                     ISOSPI,IZ_FIX,IN_FIX,D_NORM,RMSRAD,RADIUS,
     *                                                        IFDENI)
C=======================================================================
      USE params
C=======================================================================
C
      CHARACTER
     *          NUCLEO(0:NDISOS)*8
      CHARACTER
     *          INFORM(0:NDINFO)*10
C
      COMPLEX
     *          EWAVEF,WAVEUP,WAVEDW,PHASUP,PHASDW
      COMPLEX
     *          CCOEFF,DCOEFF,DTILDA
      COMPLEX
     *          UNIT_I
C
      DIMENSION
     *          EXPXSI(1:NDXHRM),EXPETA(1:NDYHRM),EXPZET(1:NDZHRM)
      DIMENSION
     *          RXIDEN(1:NDXHRM),RETDEN(1:NDYHRM),RDZDEN(1:NDZHRM)
C=======================================================================
      ALLOCATABLE CCOEFF(:,:,:)
      ALLOCATABLE DCOEFF(:,:,:)
      ALLOCATABLE DTILDA(:,:,:)
      ALLOCATABLE WAVEUP(:,:,:)
      ALLOCATABLE WAVEDW(:,:,:)
C=======================================================================
      DIMENSION
     *          V2OCUP(1:NDBASE)
      DIMENSION
     *          PHASUP(0:2*NDOSGS),
     *          PHASDW(0:2*NDOSGS)
C
      COMMON
     *       /DENSSP_DENFIN/ DENSIT(1:NDXDEN,1:NDYDEN,1:NDZDEN),
     *                       DENNOD(1:NDXHRM,1:NDYHRM,1:NDZHRM)
     *
     *       /DENDEN_DIMDEN/ MXHERM,MYHERM,MZHERM
     *
     *       /DENINT_RADINT/ RMSINT,R2XINT,R2YINT,R2ZINT
     *
     *       /DENINT_RADRAD/ RMSTRU,RMSPHE
      COMMON
     *       /SETBAS_INDICE/ INDICE(0:NDXMAX,0:NDYMAX,0:NDZMAX)
     *
     *       /SETBAS_BASIS1/ LAXOFY(0:NDYMAX),LAXOFZ(0:NDZMAX),
     *                       LAYOFZ(0:NDZMAX),LAYOFX(0:NDXMAX),
     *                       LAZOFX(0:NDXMAX),LAZOFY(0:NDYMAX)
     *
     *       /SETBAS_BASIS2/ LAXOYZ(0:NDYMAX,0:NDZMAX),
     *                       LAYOZX(0:NDZMAX,0:NDXMAX),
     *                       LAZOXY(0:NDXMAX,0:NDYMAX)
      COMMON
     *       /SPECTR_SPINOR/ EWAVEF(1:NDBASE,1:NDBASE)
      COMMON
     *       /SETBAS_DIMENS/ LDBASE
C
      COMMON
     *       /WSEXEC_HERSTO/ HERMTS(0:ND2MAX,1:NDGAUS,1:NDKART)
      COMMON
     *       /WSEXEC_INTSTO/ HERWGT(1:NDGAUS,1:NDKART),
     *                       HERPNT(1:NDGAUS,1:NDKART)
      COMMON
     *       /SETBAS_SCALNG/ HOMSCA(1:NDKART)
C
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
      DATA
     *     INFORM(0) / 'NO PAIRING' /,
     *     INFORM(1) / '  P.N.P.  ' /,
     *     INFORM(2) / '  B.C.S.  ' /
      DATA
     *     NUCLEO(0) /  'NEUTRONS'  /
      DATA
     *     NUCLEO(1) /  'PROTONS '  /
      DATA
     *     EPSOCU    /    0.0001    /
C
C=======================================================================
C
C        This subroutine calculates the space representation of the
C        nucleonic density from the Woods-Saxon  deformed potential
C
C        The difference compared to the subroutine  DENSSP  is that
C        this routine calculates the density at the Gauss nodes; in
C        this form the calculated density can be used to obtain the
C        RMS radius and/or other Trace(rho*Operator) related exprns
C
C=======================================================================
C
      IF (IFDENI.EQ.0) RETURN
C
C=======================================================================
C
      CALL WS_CPUTIM('DENINT',1)
C
C=======================================================================
C
C        The Hermite polynomials, denoted  "HERACT", are multiplied
C        by the dimensioned normalization constant:
C
C                     (MASS*HOMEG{}/H_BAR**2)**(1/4)
C
C        This corresponds to introducing the one - dimensional wave
C        functions in units of FM**(-1/2). Corresponding  densities
C        in three dimensions are then in units of FM**(-3)
C
C        N{}MAXX - Denote the maximum quantum numbers for the h.osc
C        N{}HERM - Denote the maximum numbers of Gauss intg. points
C
C=======================================================================
C
      MXHERM=NXHERM
      MYHERM=NYHERM
      MZHERM=NZHERM
C
      UNIT_I=CMPLX(0.0,1.0)
C
      DO IPHASE=0,2*NDOSGS
         PHASUP(IPHASE)=UNIT_I**( IPHASE )
         PHASDW(IPHASE)=UNIT_I**(1-IPHASE)
      END DO
C
      IF (ISOSPI.EQ.0) THEN
          N_STAT=IN_FIX/2
          N_PART=IN_FIX
      END IF
C
      IF (ISOSPI.EQ.1) THEN
          N_STAT=IZ_FIX/2
          N_PART=IZ_FIX
      END IF
C
      LIMITS=N_PART
C
      IF (LIMITS.GT.NDBASE) STOP 'LIMITS.GT.NDBASE in DENINT <<= '
C
C      Calculations with sharp occupation distribution
C
      DO ISTATE=1,LDBASE
                               V2OCUP(ISTATE)=1.0
         IF (ISTATE.GT.N_STAT) V2OCUP(ISTATE)=0.0
      END DO
C
      IF (MOD(N_PART,2).NE.0)  V2OCUP(N_PART/2+1)=0.5
C
C      Initialization
C
      DO IZ=1,NZHERM
         DO IY=1,NYHERM
            DO IX=1,NXHERM
               DENNOD(IX,IY,IZ)=0.0
            END DO
         END DO
      END DO
C
C=======================================================================
C
      ALLOCATE(CCOEFF(0:NDXMAX,0:NDYMAX,1:NDGAUS))
      ALLOCATE(DCOEFF(0:NDXMAX,1:NDGAUS,1:NDGAUS))
      ALLOCATE(DTILDA(0:NDXMAX,1:NDGAUS,1:NDGAUS))
C
      ALLOCATE(WAVEUP(1:NDXHRM,1:NDYHRM,1:NDZHRM))
      ALLOCATE(WAVEDW(1:NDXHRM,1:NDYHRM,1:NDZHRM))
C
      DO ISTATE=1,LIMITS
C
         IF (V2OCUP(ISTATE).GE.EPSOCU) THEN
C
C             Defining the auxiliary products of the wave functions
C
C             NX, NY, NZ correspond to loops over quantum numbers
C             IX, IY, IZ correspond to loops over the coordinates
C                                                     x, y and z
C
             DO IZ=1,NZHERM
                DO NY=0,LAYOFX(NX)
                   DO NX=0,NXMAXX
                      CCOEFF(NX,NY,IZ)=CMPLX(0.0,0.0)
                   END DO
                END DO
             END DO
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO NX=0,NXMAXX
                      DCOEFF(NX,IY,IZ)=CMPLX(0.0,0.0)
                      DTILDA(NX,IY,IZ)=CMPLX(0.0,0.0)
                   END DO
                END DO
             END DO
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO IX=1,NXHERM
                      WAVEUP(IX,IY,IZ)=CMPLX(0.0,0.0)
                      WAVEDW(IX,IY,IZ)=CMPLX(0.0,0.0)
                   END DO
                END DO
             END DO
C
C             Here we work within the {|t+>,|t->} basis that can be
C             expressed by means of the
C
C                                 |b> = i^ny |nx,ny,nz;sz>
C             vectors as
C
C                     |t+> = 1/sqrt(2) [ i^ny |up> -i^(1-ny) |down> ]
C
C             where
C                                 |up>  =|nx,ny,nz;+sz>
C             and
C                                 |down>=|nx,ny,nz;-sz>
C
C             The total wave function can be written within the |t+>
C             block as
C
C                      Psi+ = \sum_{nx,ny,nz,sz} C(nx,ny,nz,sz) |t+>
C
C             In the following the expansion coefficients C are contained
C             in the matrix  EWAVEF, while  HERMIT  represents a spatial
C             part of the |up> and |down> wave functions.
C
C             It follows that the density calculated within one block [e.g.
C             (<+| |+>)] is only a part of the total density; in a static,
C             no rotation case the occupations within the  <+| |+>  and
C             <-| |-> blocks are symmetric and the factor 2 will appear at
C             the end...
C
C             Here, we sum over z, that is identical for  both  UP  and
C             DOWN parts of the wave function
C
             KARTEZ=3
C
             DO NX=0,NXMAXX
                DO NY=0,LAYOFX(NX)
C
                   DO IZ=1,NZHERM
C
                      DO NZ=0,LAZOXY(NX,NY)
C
                         IBRAHO=INDICE(NX,NY,NZ)
C
                         CCOEFF(NX,NY,IZ)=CCOEFF(NX,NY,IZ)
     *                                   +EWAVEF(IBRAHO,ISTATE)
     *                                   *HERMTS(NZ,IZ, KARTEZ)
                      END DO
C
                   END DO
C
                END DO
             END DO
C
C             Now, we sum over y; here we take into account the phases
C             i^ny and i^(1-ny)
C
             KARTEZ=2
C
             DO NX=0,NXMAXX
C
                DO IY=1,NYHERM
                   DO IZ=1,NZHERM
C
                      DO NY=0,LAYOFX(NX)
C
                         DCOEFF(NX,IY,IZ)=DCOEFF(NX,IY,IZ)
     *                                   +CCOEFF(NX,NY,IZ)
     *                                   *HERMTS(NY,IY,KARTEZ)
     *                                   *PHASUP(NY)
C
                      END DO
C
                   END DO
                END DO
C
             END DO
C
             DO IY=1,NYHERM
                DO IZ=1,NZHERM
                   DO NX=0,NXMAXX
C
                      DO NY=0,LAYOFX(NX)
                         DTILDA(NX,IY,IZ)=DTILDA(NX,IY,IZ)
     *                                   -CCOEFF(NX,NY,IZ)
     *                                   *HERMTS(NY,IY,KARTEZ)
     *                                   *PHASDW(NY)
                      END DO
C
                   END DO
                END DO
             END DO
C
C             At last, we perform the summation over x, but we now have two
C             different functions corresponding to the two parts that make
C             the t-vector:
C
             KARTEZ=1
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO IX=1,NXHERM
C
                      DO NX=0,NXMAXX
C
                         WAVEUP(IX,IY,IZ)=WAVEUP(IX,IY,IZ)
     *                                   +DCOEFF(NX,IY,IZ)
     *                                   *HERMTS(NX,IX,KARTEZ)
                      END DO
C
                   END DO
                END DO
             END DO
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO IX=1,NXHERM
C
                      DO NX=0,NXMAXX
C
                         WAVEDW(IX,IY,IZ)=WAVEDW(IX,IY,IZ)
     *                                   +DTILDA(NX,IY,IZ)
     *                                   *HERMTS(NX,IX,KARTEZ)
                      END DO
C
                   END DO
                END DO
             END DO
C
C             Now we have computed the individual wave functions, and
C             we proceed to calculate the density. The factor 2 appears
c             from the Kramers degeneracy equivalent to multiplying the
C             result for the |t+> block by 2
C
             SQRT_2=SQRT(2.0)
C
             DO IZ=1,NZHERM
                DO IY=1,NYHERM
                   DO IX=1,NXHERM
C
C                      For the calculation of the norm, we have:
C                           PSI_total = PSI_up + PSI_down
C                      and as PSI_up  and PSI_down have opposite spins,
C                      they are orthogonal; that's why the sum reduces
C                      to the following one:
C
                      WAVEUP(IX,IY,IZ)=WAVEUP(IX,IY,IZ)/SQRT_2
                      WAVEDW(IX,IY,IZ)=WAVEDW(IX,IY,IZ)/SQRT_2
C
                      DENAUX          =CABS(WAVEUP(IX,IY,IZ))**2
     *                                +CABS(WAVEDW(IX,IY,IZ))**2
C
                      DENNOD(IX,IY,IZ)=DENNOD(IX,IY,IZ)
     *                                +2.0*DENAUX*V2OCUP(ISTATE)
C
                   END DO
                END DO
             END DO
C
         END IF
C
      END DO
C
      DEALLOCATE(CCOEFF,DCOEFF,DTILDA)
      DEALLOCATE(WAVEUP,WAVEDW)
C
C=======================================================================
C         Integrating the density over the volume
C=======================================================================
C
      DENSUM=0.0
C
      RKSIXS=0.0
      RKSIYS=0.0
      RKSIZS=0.0
C
      DO IZ=1,NZHERM
         DO IY=1,NYHERM
            DO IX=1,NXHERM
C
C               Attention: HERWGT are divided by the HOMSCA-coeffs.
C
               W_HERM=HERWGT(IX,1)*HERWGT(IY,2)*HERWGT(IZ,3)
C
               RKSIX2=HERPNT(IX,1)**2
               RKSIY2=HERPNT(IY,2)**2
               RKSIZ2=HERPNT(IZ,3)**2
C
               DENSUM=DENSUM+DENNOD(IX,IY,IZ)*W_HERM
C
               RKSIXS=RKSIXS+DENNOD(IX,IY,IZ)*W_HERM*RKSIX2
               RKSIYS=RKSIYS+DENNOD(IX,IY,IZ)*W_HERM*RKSIY2
               RKSIZS=RKSIZS+DENNOD(IX,IY,IZ)*W_HERM*RKSIZ2
C
            END DO
         END DO
      END DO
C
C      The coefficients below should be used to calculate
C      the exponential term in the density
C
      DO IX=1,NXHERM
         RXIDEN(IX)=HERPNT(IX,1)
      END DO
C
      DO IY=1,NYHERM
         RETDEN(IY)=HERPNT(IY,2)
      END DO
C
      DO IZ=1,NZHERM
         RDZDEN(IZ)=HERPNT(IZ,3)
      END DO
C
      DO IXHERM=1,NXHERM
         EXPXSI(IXHERM)=EXP(-(RXIDEN(IXHERM)**2))
      END DO
C
      DO IYHERM=1,NYHERM
         EXPETA(IYHERM)=EXP(-(RETDEN(IYHERM)**2))
      END DO
C
      DO IZHERM=1,NZHERM
         EXPZET(IZHERM)=EXP(-(RDZDEN(IZHERM)**2))
      END DO
C
      DO IZ=1,NZHERM
         DO IY=1,NYHERM
            DO IX=1,NXHERM
C
               DENNOD(IX,IY,IZ)=DENNOD(IX,IY,IZ)*EXPXSI(IX)
     *                                          *EXPETA(IY)
     *                                          *EXPZET(IZ)
            END DO
         END DO
      END DO
C
      D_NORM=DENSUM
C
      R2XINT=RKSIXS/HOMSCA(1)**2/N_PART
      R2YINT=RKSIYS/HOMSCA(2)**2/N_PART
      R2ZINT=RKSIZS/HOMSCA(3)**2/N_PART
C
      R2_SUM=RKSIXS/HOMSCA(1)**2
     *      +RKSIYS/HOMSCA(2)**2
     *      +RKSIZS/HOMSCA(3)**2
C
      RMSRAD=SQRT(R2_SUM/N_PART)
C
      RMSINT=RMSRAD
      RMSTRU=RMSRAD
C
C=======================================================================
C
      IF (IWRITE.GE.1)
     *    WRITE(NOUTPT,'(/,80(''#''),/,''#'',78X,''#'',/,
     *               ''#   Spatial Density Test for '',A8,
     *               '':   Z ='',I3,''   Integrated.rho ='',F10.6,
     *               ''   #'',/,
     *               ''#                                        N ='',
     *               I3,''          RMS.RAD.='',F10.6,''   #'',/,
     *               ''#   Pairing Variant: '',A10,8X,
     *               ''Ratio:  SQRT(5/3)*RMS/GEO.='',F10.6,
     *               ''   #'',/,''#'',78X,''#'',/,80(''#''))')
     *
     *               NUCLEO(ISOSPI),IZ_FIX,D_NORM,IN_FIX,RMSRAD,
     *               INFORM(0),(SQRT(5./3.)*RMSRAD)/RADIUS
C
C=======================================================================
C
      CALL WS_CPUTIM('DENINT',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_RMSCUT(NXMAXX,NYMAXX,NZMAXX,NXHERM,NYHERM,NZHERM,
     *                     ISOSPI,IZ_FIX,IN_FIX,RMSRAD,RADIUS,IF_XYZ,
     *                                   BASEX2,BASEY2,BASEZ2,EPSHER)
C=======================================================================
      USE params
C=======================================================================
C
      CHARACTER
     *          NUCLEO(0:NDISOS)*8,INFORM(0:NDINFO)*10
C
      COMPLEX
     *          EWAVEF,WAVAUX,PHASUP,PHASDW
      COMPLEX
     *          FUNXUP,FUNXDW,FUNYUP,FUNYDW,FUNZUP,FUNZDW
      COMPLEX
     *          XFACTO,XFACUP,XFACDW,
     *          YFACTO,YFACUP,YFACDW,
     *          ZFACTO,ZFACUP,ZFACDW
      COMPLEX
     *          UNIT_I
C
      DIMENSION
     *          XHERMS(1:NDXDEN),YHERMS(1:NDYDEN),ZHERMS(1:NDZDEN),
     *          WXHRMS(1:NDXDEN),WYHRMS(1:NDYDEN),WZHRMS(1:NDZDEN)
      DIMENSION
     *          DNPURX(1:NDXDEN,1:NDBASE),
     *          DNPURY(1:NDYDEN,1:NDBASE),
     *          DNPURZ(1:NDZDEN,1:NDBASE)
      DIMENSION
     *          FUNXUP(1:NDXDEN),FUNXDW(1:NDXDEN),
     *          FUNYUP(1:NDYDEN),FUNYDW(1:NDYDEN),
     *          FUNZUP(1:NDZDEN),FUNZDW(1:NDZDEN)
      DIMENSION
     *          V2OCUP(1:NDBASE)
      DIMENSION
     *          HERMIT(0:NDGAUS,1:NDGAUS,1:NDKART)
      DIMENSION
     *          PHASUP(0:2*NDOSGS),
     *          PHASDW(0:2*NDOSGS)
      DIMENSION
     *          RKSIXS(1:NDBASE),
     *          RKSIYS(1:NDBASE),
     *          RKSIZS(1:NDBASE)
      DIMENSION
     *          HERAUX(0:NDGAUS),
     *          DHERAX(0:NDGAUS)
      DIMENSION
     *          SQFACT(0:NDGAUS)
C
      COMMON
     *       /SETBAS_NXNYNZ/ NXVECT(1:NDBASE),
     *                       NYVECT(1:NDBASE),
     *                       NZVECT(1:NDBASE)
      COMMON
     *       /RMSCUT_DENXYZ/ DENSIX(1:NDXDEN),
     *                       DENSIY(1:NDYDEN),
     *                       DENSIZ(1:NDZDEN)
      COMMON
     *       /SPECTR_SPINOR/ EWAVEF(1:NDBASE,1:NDBASE)
C
C             Attention here: For spherical nuclei R2XCUT,R2YCUT&R2ZCUT
C                             are all equal and equal to the rms radii.
C
C                                RMLIKE=sqrt{[R2XCUT+R2YCUT+R2ZCUT]/3}
C
C                             gives r.m.s. radius only at zero deformtn.
      COMMON
     *       /RMSCUT_RADCUT/ RMLIKE,R2XCUT,R2YCUT,R2ZCUT
     *
     *       /DENINT_RADRAD/ RMSTRU,RMSPHE
      COMMON
     *       /VERIFS_SYMETS/ ISPHER,ICYLIN,IPCONS
      COMMON
     *       /SETBAS_DIMENS/ LDBASE
C
      COMMON
     *       /SETBAS_SCALNG/ HOMSCA(1:NDKART)
C
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
      DATA
     *     INFORM(0) / 'NO PAIRING' /,
     *     INFORM(1) / '  P.N.P.  ' /,
     *     INFORM(2) / '  B.C.S.  ' /
      DATA
     *     NUCLEO(0) /  'NEUTRONS'  /
      DATA
     *     NUCLEO(1) /  'PROTONS '  /
      DATA
     *     EPSWAV    /   1.0E-10    /
      DATA
     *     EPSSPH    /   1.0E-10    /
      DATA
     *     EPSOCU    /   0.00010    /
C
C=======================================================================
C
C        This subroutine calculates the space representation of the
C        nucleonic density from the Woods-Saxon  deformed potential.
C        Here we calculate the density at the Gauss nodes projected
C        onto the Ox, Oy and Oz axes.
C
C        In addition, we calculate the contributions to the density
C        from each orbital separately.
C
C        Pairing taken (optionally) into account using the approach
C        employed already in DENSSP
C
C        Attention: This routine  recognizes  whether the spherical
C                   nuclei have single-j shell orbitals  fully occu-
C                   pied (or not)  by calculating and comparing the
C                   mean values of x^2, y^2 and z^2 along the axes.
C                   Here we perform the one-dimensional integration
C                   and thus  this routine  is much faster than the
C                   DENINT subroutine.
C
C=======================================================================
C
      IF (IF_XYZ.EQ.0) RETURN
C
C=======================================================================
C
      CALL WS_CPUTIM('RMSCUT',1)
C
C=======================================================================
C
C        The Hermite polynomials, denoted  "HERACT", are multiplied
C        by the dimensioned normalization constant:
C
C                     (MASS*HOMEG{}/H_BAR**2)**(1/4)
C
C        This corresponds to introducing the one - dimensional wave
C        functions in units of FM**(-1/2). Corresponding  densities
C        in three dimensions are then in units of FM**(-3)
C
C        N{}MAXX - Denote the maximum quantum numbers for the h.osc
C        N{}HERM - Denote the maximum numbers of Gauss intg. points
C
C=======================================================================
C
      UNIT_I=CMPLX(0.0,1.0)
C
      DO IPHASE=0,2*NDOSGS
         PHASUP(IPHASE)=UNIT_I**( IPHASE )
         PHASDW(IPHASE)=UNIT_I**(1-IPHASE)
      END DO
C
C=======================================================================
C
      IF (ISOSPI.EQ.0) THEN
          N_STAT=IN_FIX/2
          N_PART=IN_FIX
      END IF
C
      IF (ISOSPI.EQ.1) THEN
          N_STAT=IZ_FIX/2
          N_PART=IZ_FIX
      END IF
C
C=======================================================================
C         Calculations with sharp occupation distribution
C=======================================================================
C
      DO ISTATE=1,LDBASE
                               V2OCUP(ISTATE)=1.0
         IF (ISTATE.GT.N_STAT) V2OCUP(ISTATE)=0.0
      END DO
C
      IF (MOD(N_PART,2).NE.0)  V2OCUP(N_PART/2+1)=0.5
C
C=======================================================================
C
                       LIMITS=IZ_FIX
      IF (ISOSPI.EQ.0) LIMITS=IN_FIX
C
      IF (LIMITS.GT.NDBASE) STOP 'LIMITS.GT.NDBASE in RMSCUT <<= '
C
C=======================================================================
C=======================================================================
C
C         Below  we calculate the zeros of the Hermite polynomials
C         (dimensionless) and the weight coefficients of the Gauss
C         Hermite integration formulas. Below we are interested in
C         the cuts of the wave-functions in the x-y,  y-z and  z-x
C         planes, what corresponds to the projections of the type:
C
C                 psi(x,0,0);  psi(0,y,0)  and  psi(0,0,z)
C
C         Consequently,  the numbers of nodes must be odd below in
C         order to have those zero-projections.
C
C         Recall, that in the present version of the code the no's
C         of the Gauss integration points must be   e v e n  since
C         the reflexions  in the principal planes  have been coded
C         according to such a scheme
C
C=======================================================================
C
      SQRT_2=SQRT(2.0)
C
C         Below, we must have the projections  passing by the axes
C         and this implies the simultaneous presence of the points
C         of the type {x,0,0}, {0,y,0} etc.  Since the rest of the
C         program  must use EVEN numbers of the integration points
C         here we change into the ODD numbers
C
      NXHRMS=NXHERM+1
      NYHRMS=NYHERM+1
      NZHRMS=NZHERM+1
C
      IF (NXHRMS.GT.NDXDEN.OR.NYHRMS.GT.NDYDEN.OR.NZHRMS.GT.NDZDEN)
     *                                                       THEN
C
          STOP 'Increase the number of the Gauss points in RMSCUT <<= '
C
      END IF
C
      CALL WS_GAUHER(XHERMS,WXHRMS,NXHRMS,NDXDEN,EPSHER)
      CALL WS_GAUHER(YHERMS,WYHRMS,NYHRMS,NDYDEN,EPSHER)
      CALL WS_GAUHER(ZHERMS,WZHRMS,NZHRMS,NDZDEN,EPSHER)
C
C=======================================================================
C
      DO IX=1,NXHRMS
         WXHRMS(IX)=WXHRMS(IX)/HOMSCA(1)
      END DO
C
      DO IY=1,NYHRMS
         WYHRMS(IY)=WYHRMS(IY)/HOMSCA(2)
      END DO
C
      DO IZ=1,NZHRMS
         WZHRMS(IZ)=WZHRMS(IZ)/HOMSCA(3)
      END DO
C
C=======================================================================
C
      CALL WS_NORFAC(SQFACT,ND2MAX)
C
C=======================================================================
C
      KARTEZ=1
C
      DO IX=1,NXHRMS
C
         X0ARGU=XHERMS(IX)
C
         CALL WS_DFHERM(X0ARGU,NXMAXX,HERAUX,DHERAX,ND2MAX)
C
         DO NX=0,NXMAXX
            HERMIT(NX,IX,KARTEZ)=((SQRT(HOMSCA(KARTEZ)))/
     *                             SQFACT(NX))*HERAUX(NX)
C
         END DO
C
      END DO
C
C=======================================================================
C
      KARTEZ=2
C
C=======================================================================
C
      DO IY=1,NYHRMS
C
         Y0ARGU=YHERMS(IY)
C
         CALL WS_DFHERM(Y0ARGU,NYMAXX,HERAUX,DHERAX,ND2MAX)
C
         DO NY=0,NYMAXX
            HERMIT(NY,IY,KARTEZ)=((SQRT(HOMSCA(KARTEZ)))/
     *                             SQFACT(NY))*HERAUX(NY)
         END DO
C
      END DO
C
C=======================================================================
C
      KARTEZ=3
C
C=======================================================================
C
      DO IZ=1,NZHRMS
C
         Z0ARGU=ZHERMS(IZ)
C
         CALL WS_DFHERM(Z0ARGU,NZMAXX,HERAUX,DHERAX,ND2MAX)
C
         DO NZ=0,NZMAXX
            HERMIT(NZ,IZ,KARTEZ)=((SQRT(HOMSCA(KARTEZ)))/
     *                             SQFACT(NZ))*HERAUX(NZ)
         END DO
C
      END DO
C
C=======================================================================
C=======================================================================
C
      PINUMB=4.0*ATAN(1.0)
C
      DO ISTATE=1,LIMITS
C
         RKSIXS(ISTATE)=0.0
         RKSIYS(ISTATE)=0.0
         RKSIZS(ISTATE)=0.0
C
         DO IX=1,NXHRMS
            FUNXUP(IX)=CMPLX(0.0,0.0)
            FUNXDW(IX)=CMPLX(0.0,0.0)
         END DO
C
         DO IY=1,NYHRMS
            FUNYUP(IY)=CMPLX(0.0,0.0)
            FUNYDW(IY)=CMPLX(0.0,0.0)
         END DO
C
         DO IZ=1,NZHRMS
            FUNZUP(IZ)=CMPLX(0.0,0.0)
            FUNZDW(IZ)=CMPLX(0.0,0.0)
         END DO
C
         IF (V2OCUP(ISTATE).LT.EPSOCU) GO TO 1
C
C=======================================================================
C        Defining the auxiliary products of the wave functions
C=======================================================================
C
C        NX, NY, NZ correspond to loops over quantum numbers
C        IX, IY, IZ correspond to loops over coordinates x, y and z
C
C=======================================================================
C
         DO IBRAHO=1,LDBASE
C
            WAVAUX=+EWAVEF(IBRAHO,ISTATE)
C
            IF (ABS(WAVAUX).LT.EPSWAV) GO TO 2
C
            NX=NXVECT(IBRAHO)
            NY=NYVECT(IBRAHO)
            NZ=NZVECT(IBRAHO)
C
C                             The z-projection
C                             ================
            IX=NXHRMS/2+1
            IY=NYHRMS/2+1
C
            ZFACTO=WAVAUX*HERMIT(NX,IX,1)*HERMIT(NY,IY,2)
            ZFACUP=ZFACTO*PHASUP(NY)
            ZFACDW=ZFACTO*PHASDW(NY)
C
            DO IZ=1,NZHRMS
C
               FUNZUP(IZ)=FUNZUP(IZ)+ZFACUP*HERMIT(NZ,IZ,3)
               FUNZDW(IZ)=FUNZDW(IZ)+ZFACDW*HERMIT(NZ,IZ,3)
C
            END DO
C
C                             The y-projection
C                             ================
            IX=NXHRMS/2+1
            IZ=NZHRMS/2+1
C
            YFACTO=WAVAUX*HERMIT(NX,IX,1)*HERMIT(NZ,IZ,3)
            YFACUP=YFACTO*PHASUP(NY)
            YFACDW=YFACTO*PHASDW(NY)
C
            DO IY=1,NYHRMS
C
               FUNYUP(IY)=FUNYUP(IY)+YFACUP*HERMIT(NY,IY,2)
               FUNYDW(IY)=FUNYDW(IY)+YFACDW*HERMIT(NY,IY,2)
C
            END DO
C
C                             The x-projection
C                             ================
            IY=NYHRMS/2+1
            IZ=NZHRMS/2+1
C
            XFACTO=WAVAUX*HERMIT(NY,IY,2)*HERMIT(NZ,IZ,3)
            XFACUP=XFACTO*PHASUP(NY)
            XFACDW=XFACTO*PHASDW(NY)
C
            DO IX=1,NXHRMS
C
               FUNXUP(IX)=FUNXUP(IX)+XFACUP*HERMIT(NX,IX,1)
               FUNXDW(IX)=FUNXDW(IX)+XFACDW*HERMIT(NX,IX,1)
C
            END DO
C
   2        CONTINUE
C
         END DO
C
C=======================================================================
C
C        For the calculation of the norm, we have:
C
C                      PSI_total = PSI_up + PSI_down
C
C        and since Psi_up and Psi_down have opposite spins,  they are
C        orthogonal; this is why the sum reduces to the following one
C
C=======================================================================
C
         DO IX=1,NXHRMS
C
            DNPURX(IX,ISTATE)=CABS(FUNXUP(IX)/SQRT_2)**2
     *                       +CABS(FUNXDW(IX)/SQRT_2)**2
         END DO
C
         DO IY=1,NYHRMS
C
            DNPURY(IY,ISTATE)=CABS(FUNYUP(IY)/SQRT_2)**2
     *                       +CABS(FUNYDW(IY)/SQRT_2)**2
         END DO
C
         DO IZ=1,NZHRMS
C
            DNPURZ(IZ,ISTATE)=CABS(FUNZUP(IZ)/SQRT_2)**2
     *                       +CABS(FUNZDW(IZ)/SQRT_2)**2
         END DO
C
C=======================================================================
C
C        We have computed the individual wave functions, and we will
C        integrate the r^2 operator by integrating:   Int x^4 rho dx.
C        This makes sense only in the case of the spherical symmetry
C        and provides a gain in c.p.u. for such a case
C
C=======================================================================
C        Attention:   Gauss weights are divided by the HOMSCA-coeffs.
C=======================================================================
C
         DO IX=1,NXHRMS
            RKSIX4=XHERMS(IX)**4
            W_HERM=WXHRMS(IX)
            RKSIXS(ISTATE)=RKSIXS(ISTATE)
     *                    +DNPURX(IX,ISTATE)*W_HERM*RKSIX4
         END DO
C
         DO IY=1,NYHRMS
            RKSIY4=YHERMS(IY)**4
            W_HERM=WYHRMS(IY)
            RKSIYS(ISTATE)=RKSIYS(ISTATE)
     *                    +DNPURY(IY,ISTATE)*W_HERM*RKSIY4
         END DO
C
         DO IZ=1,NZHRMS
            RKSIZ4=ZHERMS(IZ)**4
            W_HERM=WZHRMS(IZ)
            RKSIZS(ISTATE)=RKSIZS(ISTATE)
     *                    +DNPURZ(IZ,ISTATE)*W_HERM*RKSIZ4
         END DO
C
   1     CONTINUE
C
C=======================================================================
C
      END DO
C
C=======================================================================
C     Summing up over all the occupied states
C=======================================================================
C
      R2XSUM=0.0
      R2YSUM=0.0
      R2ZSUM=0.0
C
      DO IX=1,NXHRMS
         DENSIX(IX)=0.0
      END DO
      DO IY=1,NYHRMS
         DENSIY(IY)=0.0
      END DO
      DO IZ=1,NZHRMS
         DENSIZ(IZ)=0.0
      END DO
C
C=======================================================================
C
      DO ISTATE=1,LIMITS
C
         RKSIXS(ISTATE)=RKSIXS(ISTATE)/HOMSCA(1)**4
         RKSIYS(ISTATE)=RKSIYS(ISTATE)/HOMSCA(2)**4
         RKSIZS(ISTATE)=RKSIZS(ISTATE)/HOMSCA(3)**4
C
         IF (V2OCUP(ISTATE).GT.EPSOCU) THEN
C
             R2XSUM=R2XSUM+RKSIXS(ISTATE)
             R2YSUM=R2YSUM+RKSIYS(ISTATE)
             R2ZSUM=R2ZSUM+RKSIZS(ISTATE)
C
             DO IX=1,NXHRMS
                DENSIX(IX)=DENSIX(IX)+DNPURX(IX,ISTATE)*2.
             END DO
C
             DO IY=1,NYHRMS
                DENSIY(IY)=DENSIY(IY)+DNPURY(IY,ISTATE)*2.
             END DO
C
             DO IZ=1,NZHRMS
                DENSIZ(IZ)=DENSIZ(IZ)+DNPURZ(IZ,ISTATE)*2.
             END DO
C
         END IF
C
      END DO
C
C     Below we have the factor of 4 from the volume element, then
C     the factor of 1/2, from integrating the function from minus
C     infinity to plus infinity rather than from zero to infinity
C     and an additional factor of two coming from the Kramers de-
C     generacy
C
      R2XSUM=SQRT(4.0*PINUMB*R2XSUM/N_PART)
      R2YSUM=SQRT(4.0*PINUMB*R2YSUM/N_PART)
      R2ZSUM=SQRT(4.0*PINUMB*R2ZSUM/N_PART)
C
      R2XCUT=R2XSUM**2
      R2YCUT=R2YSUM**2
      R2ZCUT=R2ZSUM**2
C
      ISPCON=1
C
      IF (ABS(R2XSUM-R2YSUM).GT.EPSSPH.OR.
     *    ABS(R2XSUM-R2ZSUM).GT.EPSSPH.OR.
     *    ABS(R2YSUM-R2ZSUM).GT.EPSSPH) THEN
C
C         Checking for full (or not) spherical configurations
C
          ISPCON=0
C
          IF (ISPHER.EQ.1) THEN
C
              RMLIKE=SQRT((R2XCUT+R2YCUT+R2ZCUT)/3.0)
C
              IF (IWRITE.GE.1)
     *        WRITE(NOUTPT,'(/,80(''#''),/,''#'',78X,''#'',/,
     *                  ''#   You have a spherical shape but '',
     *                  ''(?) non-closed j-shells?    Z ='',I3,
     *                  '' N ='',I3,''   #'',/,
     *                  ''#   <x^2> ='',F8.5,''  <y^2> ='',F9.5,
     *                  ''  <z^2> ='',F9.5,''   Average= '',F9.6,
     *                  ''   #'',/,
     *                  ''#   Pairing Variant: '',A10,
     *                  ''                   Ratio: RMS/GEO.='',F10.6,
     *                  ''   #'',/,''#'',78X,''#'',/,80(''#''))')
     *
     *                  IZ_FIX,IN_FIX,R2XSUM,R2YSUM,R2ZSUM,RMLIKE,
     *                  INFORM(0),SQRT(5./3.0)*RMLIKE/RADIUS
C
          END IF
C
      END IF
C
C=======================================================================
C
      IF (ISPHER.NE.1) THEN
C
          RMLIKE=SQRT((R2XCUT+R2YCUT+R2ZCUT)/3.0)
C
          IF (IWRITE.GE.1)
     *    WRITE(NOUTPT,'(/,80(''#''),/,''#'',78X,''#'',/,
     *                ''#   You calculate density sections for '',
     *                ''a deformed case:        Z ='',I3,'' N ='',I3,
     *                ''   #'',/,
     *                ''#   Section integrals:   x^2 ='',F8.5,
     *                ''      y^2 ='',F8.5,''      z^2 ='',F8.5,
     *                ''   #'',/,
     *                ''#   Ratios:          x^2/z^2 ='',F8.5,
     *                ''  y^2/z^2 ='',F8.5,''  y^2/x^2 ='',F8.5,
     *                ''   #'',/,
     *                ''#                    bx2/bz2 ='',F8.5,
     *                ''  by2/bz2 ='',F8.5,''  by2/bx2 ='',F8.5,
     *                ''   #'',/,
     *                  ''#   Pairing Variant: '',A10,
     *               ''                     Ratio: RML/GEO.='',F8.5,
     *               ''   #'',/,''#'',78X,''#'',/,80(''#''))')
     *
     *               IZ_FIX,IN_FIX,
     *               R2XSUM,R2YSUM,R2ZSUM,
     *               R2XSUM/R2ZSUM,R2YSUM/R2ZSUM,R2YSUM/R2XSUM,
     *               BASEX2/BASEZ2,BASEY2/BASEZ2,BASEY2/BASEX2,
     *               INFORM(0),             RMLIKE/RADIUS
C
      END IF
C
C=======================================================================
C
      IF (ISPCON.EQ.1.AND.ISPHER.EQ.1) THEN
C
          RMSRAD=(R2XSUM+R2YSUM+R2ZSUM)/3
C
C=======================================================================
C
          IF (IWRITE.GE.1)
     *    WRITE(NOUTPT,'(/,80(''#''),/,''#'',78X,''#'',/,
     *               ''#   Spatial Density Test for '',A8,
     *               '' (Ox,Oy,Oz- projections): Z ='',I3,'' N ='',I3,
     *               ''   #'',/,
     *               ''#                                           '',
     *               ''             RMS.RAD.='',F10.6,''   #'',/,
     *               ''#   Pairing Variant: '',A10,
     *               ''                   Ratio: RMS/GEO.='',F10.6,
     *               ''   #'',/,''#'',78X,''#'',/,80(''#''))')
     *
     *               NUCLEO(ISOSPI),IZ_FIX,IN_FIX,RMSRAD,
     *               INFORM(0),SQRT(5.0/3.0)*RMSRAD/RADIUS
C
          RMSPHE=RMSRAD
C
      END IF
C
C=======================================================================
C
      CALL WS_CPUTIM('RMSCUT',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DELTJK(NXMAXX,NYMAXX,NZMAXX,NXHERM,NYHERM,NZHERM,
     *                     LIMLOW,LIMUPP,ISOSPI,IZ_FIX,IN_FIX,IFDELT,
     *                                                 EPSHER,NOSCIL)
C=======================================================================
      USE params
C=======================================================================
C
C=======================================================================
      ALLOCATABLE DENBRA(:,:,:),DENKET(:,:,:)
C=======================================================================
      DIMENSION
     *          HERMT2(0:ND2MAX,1:NDGAUS,1:NDKART),
     *          HERWG2(1:NDGAUS,1:NDKART),
     *          HERPN2(1:NDGAUS,1:NDKART),
     *
     *          IDELST(1:NDLEVS)
      DIMENSION
     *          DUMMY1(0:ND2MAX,1:NDGAUS,1:NDKART),
     *          DUMMY2(0:ND2MAX,1:NDGAUS,1:NDKART)
C
      COMMON
     *       /DELTJK_DELTME/ BRAKET(1:NDELEM,1:NDELEM)
     *
     *       /DELTJK_DELTDA/ IDELIL,IDELFI,IDELLA
      COMMON
     *       /WSEXEC_HERSTO/ HERMTS(0:ND2MAX,1:NDGAUS,1:NDKART)
      COMMON
     *       /WSEXEC_INTSTO/ HERWGT(1:NDGAUS,1:NDKART),
     *                       HERPNT(1:NDGAUS,1:NDKART)
C
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
C=======================================================================
C
C        This subroutine calculates the matrix elements of the usual
C        delta-interaction within the single-particle wave functions
C        of the deformed Woods-Saxon potential:
C
C                  Int {Psi_j * Psi_j*} {Psi_k * Psi_k*}
C
C=======================================================================
C
      IF (IFDELT.NE.1) RETURN
C
C=======================================================================
C
      CALL WS_CPUTIM('DELTJK',1)
C
C=======================================================================
C
C        The Hermite polynomials, denoted  "HERMT2", are multiplied
C        by the dimensioned normalization constant:
C
C                     (MASS*HOMEG{}/H_BAR**2)**(1/4)
C
C        This corresponds to introducing the one - dimensional wave
C        functions in units of fm**(-1/2). Corresponding  densities
C        in three dimensions are then in units of fm**(-3)
C
C        N{}MAXX - Denote the maximum quantum numbers for the h.osc
C        N{}HERM - Denote the maximum numbers of Gauss intg. points
C
      ALLOCATE(DENBRA(1:NXHERM,1:NYHERM,1:NZHERM))
      ALLOCATE(DENKET(1:NXHERM,1:NYHERM,1:NZHERM))
C
      IF (ISOSPI.EQ.0) THEN
          LFERMI=IN_FIX/2
      END IF
C
      IF (ISOSPI.EQ.1) THEN
          LFERMI=IZ_FIX/2
      END IF
C
C      Calculating  wave functions in nodes  which are appropriate for
C      performing integration of matrix elements of two-body operators
C
      NOBODY=2
      IDERIV=0
C
      NOSACT=NOSCIL
C
      KARTEZ=1
      NGAUSS=NXHERM
C
      CALL WS_DEFINT(NOSACT,EPSHER,KARTEZ,NGAUSS,NOBODY,IDERIV,
     *                      HERMT2,DUMMY1,DUMMY2,HERWG2,HERPN2)
C
      KARTEZ=2
      NGAUSS=NYHERM
C
      CALL WS_DEFINT(NOSACT,EPSHER,KARTEZ,NGAUSS,NOBODY,IDERIV,
     *                      HERMT2,DUMMY1,DUMMY2,HERWG2,HERPN2)
C
      KARTEZ=3
      NGAUSS=NZHERM
C
      CALL WS_DEFINT(NOSACT,EPSHER,KARTEZ,NGAUSS,NOBODY,IDERIV,
     *                      HERMT2,DUMMY1,DUMMY2,HERWG2,HERPN2)
C
C      Finding the spatial representation for levels: MIN < I < MAX
C
      IDELIL=0
      IDELFI=MAX(1,LFERMI-LIMLOW)
      IDELLA=LFERMI+LIMUPP
C
      DO ISTATE=IDELFI,IDELLA
C
         IDELIL=IDELIL+1
C
         IDELST(IDELIL)=ISTATE
C
         IF (IDELIL.GT.NDELEM) STOP 'IDELIL.GT.NDELEM in DELTJK'
C
      END DO
C
C=======================================================================
C        Integrating  the  Psi_j**2  *  Psi_k**2  over  the volume
C=======================================================================
C
      ELEMAX=0
C
      DO INDBRA=1,IDELIL
C
         CALL WS_DENSTA(NXMAXX,NYMAXX,NZMAXX,NXHERM,NYHERM,NZHERM,
     *                                EPSHER,NOSCIL,INDBRA,DENBRA)
C
         DO INDKET=INDBRA,IDELIL
C
            CALL WS_DENSTA(NXMAXX,NYMAXX,NZMAXX,NXHERM,NYHERM,NZHERM,
     *                                   EPSHER,NOSCIL,INDKET,DENKET)
C
            ELEMEN=0
C
            DO IX=1,NXHERM
               DO IY=1,NYHERM
                  DO IZ=1,NZHERM
C
C                     Attention: HERWG2 are divided by the homsca-coeffs.
C
                     W_HERM=HERWG2(IX,1)*HERWG2(IY,2)*HERWG2(IZ,3)
C
                     ELEMEN=ELEMEN+DENBRA(IX,IY,IZ)
     *                            *DENKET(IX,IY,IZ)*W_HERM
C
                  END DO
               END DO
            END DO
C
            BRAKET(INDBRA,INDKET)=ELEMEN
C
            ELEMAX=MAX(ELEMAX,ELEMEN)
C
         END DO
      END DO
C
C      Defining the lower-left corner of the delta-matrix
C
      DO INDBRA=1,IDELIL
         DO INDKET=1,INDBRA
            BRAKET(INDBRA,INDKET)=BRAKET(INDKET,INDBRA)
         END DO
      END DO
C
C=======================================================================
C                            PRINTING
C=======================================================================
C
      DO I=1,IDELIL
         IF (IDELST(I).EQ.LFERMI) GO TO 1
      END DO
C
   1  CONTINUE
C
      ISHIFT=I
C
      IF (IWRITE.GE.1) THEN
C
          WRITE(NOUTPT,'(/,80(''#''),/,''#'',78X,''#'',/,
     *                 ''#     MATRIX ELEMENTS OF THE TWO-BODY '',
     *                 ''DELTA-OPERATOR AROUND THE FERMI LEVEL    #'',/,
     *                 ''#     ALL MATRIX ELEMENTS DIVIDED BY NOR'',
     *                 ''MALISATION FACTOR   = '',E13.6,4X,''#'',/,
     *                 ''#'',78X,''#'',/,80(''#''),/,''#'',78X,''#'',/,
     *                 ''#     KET'',8I4,'' ||'',I3,7I4,''    #'',/,
     *                 ''#  BRA                                   '',
     *                 ''                                      #'')')
     *
     *                 ELEMAX,(ISHIFT-8+I,I=1,16)
C
          DO I=ISHIFT-8+1,ISHIFT+8
C
             WRITE(NOUTPT,'(''#  '',I3,3X,8F4.2,2X,8F4.2,''    #'')')
     *                    I,(BRAKET(I,J)/ELEMAX,J=ISHIFT-8+1,ISHIFT+8)
C
             IF (I.EQ.LFERMI) WRITE(NOUTPT,'(''#'',78X,''#'')')
C
          END DO
C
      END IF
C
      DEALLOCATE(DENBRA,DENKET)
C
C=======================================================================
C
      CALL WS_CPUTIM('DELTJK',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DENSTA(NXMAXX,NYMAXX,NZMAXX,NXHERM,NYHERM,NZHERM,
     *                                   EPSHER,NOSCIL,ISTATE,DENDEL)
C=======================================================================
      USE params
C=======================================================================
C
      COMPLEX
     *          EWAVEF,WAVEUP,WAVEDW,PHASUP,PHASDW,
     *
     *          CCOEFF,DCOEFF,DTILDA,
     *
     *          UNIT_I
C
      DIMENSION
     *          DENDEL(1:NDXHRM,1:NDYHRM,1:NDZHRM)
      DIMENSION
     *          CCOEFF(0:NDXMAX,0:NDYMAX,1:NDGAUS),
     *          DCOEFF(0:NDXMAX,1:NDGAUS,1:NDGAUS),
     *          DTILDA(0:NDXMAX,1:NDGAUS,1:NDGAUS),
     *
     *          PHASUP(0:2*NDOSGS),PHASDW(0:2*NDOSGS),
     *
     *          HERMT2(0:ND2MAX,1:NDGAUS,1:NDKART),
     *          HERWG2(1:NDGAUS,1:NDKART),
     *          HERPN2(1:NDGAUS,1:NDKART)
      DIMENSION
     *          DUMMY1(0:ND2MAX,1:NDGAUS,1:NDKART),
     *          DUMMY2(0:ND2MAX,1:NDGAUS,1:NDKART)
C
      COMMON
     *       /SETBAS_INDICE/ INDICE(0:NDXMAX,0:NDYMAX,0:NDZMAX)
     *
     *       /SETBAS_BASIS1/ LAXOFY(0:NDYMAX),LAXOFZ(0:NDZMAX),
     *                       LAYOFZ(0:NDZMAX),LAYOFX(0:NDXMAX),
     *                       LAZOFX(0:NDXMAX),LAZOFY(0:NDYMAX)
     *
     *       /SETBAS_BASIS2/ LAXOYZ(0:NDYMAX,0:NDZMAX),
     *                       LAYOZX(0:NDZMAX,0:NDXMAX),
     *                       LAZOXY(0:NDXMAX,0:NDYMAX)
     *
     *       /SPECTR_SPINOR/ EWAVEF(1:NDBASE,1:NDBASE)
      COMMON
     *       /WSEXEC_HERSTO/ HERMTS(0:ND2MAX,1:NDGAUS,1:NDKART)
      COMMON
     *       /WSEXEC_INTSTO/ HERWGT(1:NDGAUS,1:NDKART),
     *                       HERPNT(1:NDGAUS,1:NDKART)
C
C=======================================================================
C
C        This subroutine calculates the single-particle density
C
C                    |Psi_j(x,y,z)|^2 = Psi_j * Psi_j
c
C        of the state j in the coordinate space. Hermite polynomials
C        are used to prepare the further integration of the density
C        over the coordinates.
C
C        There are two ways to use this subroutine:
C         1) call it N times (N number os s.p. states) and store the
C            results for each state in a matrix of size
C                 NDXHRM x NDYHRM x NDZHRM x NDBASE
C         2) call it N(N+1) times and reusing the same matrix each time
C
C        Solution 2) is implemented in this version, to lower the
C        amount of memory required.
C
C=======================================================================
C
      CALL WS_CPUTIM('DENSTA',1)
C
C=======================================================================
C
C        The Hermite polynomials, denoted  "HERMT2", are multiplied
C        by the dimensioned normalization constant:
C
C                     (MASS*HOMEG{}/H_BAR**2)**(1/4)
C
C        This corresponds to introducing the one - dimensional wave
C        functions in units of fm**(-1/2). Corresponding  densities
C        in three dimensions are then in units of fm**(-3)
C
C        N{}MAXX - Denote the maximum quantum numbers for the h.osc
C        N{}HERM - Denote the maximum numbers of Gauss intg. points
C
      UNIT_I=CMPLX(0.0,1.0)
C
      DO IPHASE=0,2*NDOSGS
         PHASUP(IPHASE)=UNIT_I**( IPHASE )
         PHASDW(IPHASE)=UNIT_I**(1-IPHASE)
      END DO
C
C      Calculating  wave functions in nodes  which are appropriate for
C      performing integration of matrix elements of two-body operators
C
      NOBODY=2
      IDERIV=0
C
      NOSACT=NOSCIL
C
      KARTEZ=1
      NGAUSS=NXHERM
C
      CALL WS_DEFINT(NOSACT,EPSHER,KARTEZ,NGAUSS,NOBODY,IDERIV,
     *                      HERMT2,DUMMY1,DUMMY2,HERWG2,HERPN2)
C
      KARTEZ=2
      NGAUSS=NYHERM
C
      CALL WS_DEFINT(NOSACT,EPSHER,KARTEZ,NGAUSS,NOBODY,IDERIV,
     *                      HERMT2,DUMMY1,DUMMY2,HERWG2,HERPN2)
C
      KARTEZ=3
      NGAUSS=NZHERM
C
      CALL WS_DEFINT(NOSACT,EPSHER,KARTEZ,NGAUSS,NOBODY,IDERIV,
     *                      HERMT2,DUMMY1,DUMMY2,HERWG2,HERPN2)
C
C=======================================================================
C       FINDING THE SPATIAL REPRESENTATION FOR LEVELS: MIN < I < MAX
C=======================================================================
C
C         Defining the auxiliary products of the wave functions
C
C         The Z-direction
C
      KARTEZ=3
C
      DO NX=0,NXMAXX
         DO NY=0,LAYOFX(NX)
            DO IZ=1,NZHERM
C
               CCOEFF(NX,NY,IZ)=CMPLX(0.0,0.0)
C
               DO NZ=0,LAZOXY(NX,NY)
C
                  IBRAHO=INDICE(NX,NY,NZ)
C
                  CCOEFF(NX,NY,IZ)=CCOEFF(NX,NY,IZ)
     *                            +EWAVEF(IBRAHO,ISTATE)
     *                            *HERMT2(NZ,IZ, KARTEZ)
               END DO
C
            END DO
          END DO
      END DO
C
C      The Y-direction
C
C      Phase "up"
C
      KARTEZ=2
C
      DO NX=0,NXMAXX
         DO IY=1,NYHERM
            DO IZ=1,NZHERM
C
               DCOEFF(NX,IY,IZ)=CMPLX(0.0,0.0)
C
               DO NY=0,LAYOFX(NX)
C
                  DCOEFF(NX,IY,IZ)=DCOEFF(NX,IY,IZ)
     *                            +CCOEFF(NX,NY,IZ)
     *                            *HERMT2(NY,IY,KARTEZ)
     *                            *PHASUP(NY)
C
               END DO
C
            END DO
         END DO
      END DO
C
C      Phase "down"
C
      DO NX=0,NXMAXX
         DO IY=1,NYHERM
            DO IZ=1,NZHERM
C
               DTILDA(NX,IY,IZ)=CMPLX(0.0,0.0)
C
               DO NY=0,LAYOFX(NX)
C
                  DTILDA(NX,IY,IZ)=DTILDA(NX,IY,IZ)
     *                            -CCOEFF(NX,NY,IZ)
     *                            *HERMT2(NY,IY,KARTEZ)
     *                            *PHASDW(NY)
C
               END DO
C
            END DO
         END DO
      END DO
C
C      The X-direction
C
      KARTEZ=1
C
      DO IX=1,NXHERM
         DO IY=1,NYHERM
            DO IZ=1,NZHERM
C
               WAVEUP=CMPLX(0.0,0.0)
               WAVEDW=CMPLX(0.0,0.0)
C
               DO NX=0,NXMAXX
C
                  WAVEUP=WAVEUP+DCOEFF(NX,IY,IZ)
     *                         *HERMT2(NX,IX,KARTEZ)
C
                  WAVEDW=WAVEDW+DTILDA(NX,IY,IZ)
     *                         *HERMT2(NX,IX,KARTEZ)
               END DO
C
               DENDEL(IX,IY,IZ)=0.5*(CABS(WAVEUP)**2+CABS(WAVEDW)**2)
C
            END DO
         END DO
      END DO
C
C=======================================================================
C
      CALL WS_CPUTIM('DENSTA',0)
C
C=======================================================================
C
      RETURN
      END
C
