C FILE NAME = wsu13_geom_04.f   ! Keep this symbol here: $ident@string$
C
C=======================================================================
C=======================================================================
C=====                                ==================================
C=====     GEOMETRICAL FUNCTIONS      ==================================
C=====                                ==================================
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_QGFOLD
C=======================================================================
      USE params
C=======================================================================
      COMMON
     *       /GAUSSINTG_ANG/ NTHPNT,NFIPNT
      COMMON
     *       /QGFOLD_BTHBFI/ NN_THE,NN_PHI
      COMMON
     *       /INITWS_NUMBRS/ PINUMB,PIOVE2
C
      COMMON
     *       /DRSURF_THEPHI/ R_OFTP(1:ND_THE,1:ND_PHI),
     *                       DRDTHE(1:ND_THE,1:ND_PHI),
     *                       DRDPHI(1:ND_THE,1:ND_PHI)
      COMMON
     *       /QGFOLD_SINCOS/ SIN_PH(1:NDPHTH),COS_PH(1:NDPHTH),
     *                       SIN_TH(1:NDPHTH),COS_TH(1:NDPHTH)
      COMMON
     *       /Q2GAUS_POINTS/ XPOINT(1:NDPHTH),YPOINT(1:NDPHTH),
     *                       XWEIGH(1:NDPHTH),YWEIGH(1:NDPHTH)
C
C=======================================================================
C      This subroutine defines the points on the nuclear surface
C      for a given set of the THETA and PHI angles  according to
C      the Gauss-Legendre quadrature formulae. They will be used
C      to integrate the Coulomb potential
C=======================================================================
C
      XLOWER=0.0000
      XUPPER=PINUMB
C
      YLOWER=0.0000
      YUPPER=PINUMB+PINUMB
C
      NXPOIN=NTHPNT
      NYPOIN=NFIPNT
C
      NN_THE=NTHPNT/2
      NN_PHI=NFIPNT/2
C
C=======================================================================
C
      CALL WS_GAULEG(XLOWER,XUPPER,XPOINT,XWEIGH,NXPOIN,NDPHTH)
C
      CALL WS_GAULEG(YLOWER,YUPPER,YPOINT,YWEIGH,NYPOIN,NDPHTH)
C
C=======================================================================
C
      DO IFIANG=1,2*NN_PHI
C
         FANGLE=YPOINT(IFIANG)
C
         SIN_PH(IFIANG)=SIN(FANGLE)
         COS_PH(IFIANG)=COS(FANGLE)
C
      END DO
C
C=======================================================================
C
      DO ITHANG=1,2*NN_THE
C
         TANGLE=XPOINT(ITHANG)
C
         SIN_TH(ITHANG)=SIN(TANGLE)
         COS_TH(ITHANG)=COS(TANGLE)
C
      END DO
C
C=======================================================================
C
      DO ITHANG=1,2*NN_THE
C
         TANGLE=XPOINT(ITHANG)
C
         DO IFIANG=1,2*NN_PHI
C
            FANGLE=YPOINT(IFIANG)
C
C            Calculating the surface-integration element (R-R')*DS
C
            NODIMS=0
            LEVELY=1
C
            CALL WS_DRSURF(TANGLE,FANGLE,R_OFTP(ITHANG,IFIANG),
     *                                   DRDTHE(ITHANG,IFIANG),
     *                                   DRDPHI(ITHANG,IFIANG),
     *                                          NODIMS,LEVELY)
C
         END DO
C
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_GEOMFC(IZNUMB,INNUMB,FCHOM0,R0PARM,HOMEGA,
     *                                   MAXLAM,ISOTRO,CVOLFC,
     *                                   CMSXFC,CMSYFC,CMSZFC,
     *                                   XMOMFC,YMOMFC,ZMOMFC)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *          ALPHAC
C
      COMMON
     *       /HFBASE_BASE_2/ BASEX2,BASEY2,BASEZ2
      COMMON
     *       /ALPHACMUL_DEF/ ALPHAC(0:NDLAMB,-NDLAMB:NDLAMB)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
      COMMON
     *       /WSGEOM_GAUS2L/ XPOINT(1:NDPHTH),XWEIGH(1:NDPHTH),
     *                       YPOINT(1:NDPHTH),YWEIGH(1:NDPHTH)
C
      DATA
     *        HBOM00 / 41.0 /
C
C=======================================================================
C
C        THIS SUBROUTINE WS_CALCULATES THE GEOMETRICAL FACTORS RELATED
C        TO THE  DEFINITION OF THE NUCLEAR SURFACE  IN TERMS OF THE
C        SPHERICAL HARMONICS AND DERIVES THE CORRESPONDING HARMONIC
C        OSCILLATOR FREQUENCIES
C
C        A) HBAROX,HBAROY,HBAROZ =>> DERIVED OSCILLATOR FREQUENCIES
C
C        B) CMSXFC,CMSYFC,CMSZFC =>> COORDINATES OF  CENTER OF MASS
C
C        C) XMOMFC,YMOMFC,ZMOMFC =>> CLASSICAL  MOMENTS  OF INERTIA
C
C=======================================================================
C
C        M E A N I N G  O F  S O M E  I N P U T  P A R A M E T E R S
C
C
C        A) THE NUCLEUS:
C
C        INNUMB, IZNUMB => THE N- AND THE Z-NUMBERS  OF  THE NUCLEUS
C                          FOR WHICH THE SURFACE IS DEFINED.
C
C        B) ITS DEFORMATION:
C
C        ALPHAR,        => A REAL  ARRAY  SPECIFYING THE NUCLEAR DE-
C                          FORMATIONS IN TERMS OF THE SPHERICAL HAR-
C                          MONICS. THE INTEGER PARAMETER "IREALA" IS
C                          EXPLICITLY SET TO  1  TO SPECIFY THAT THE
C                          DEFORMATIONS ARE REAL.
C
C        LMXBAS   => => => THE MAXIMUM MULTIPOLARITY  OF THE DEFORM.
C                          IN QUESTION (THE HIGHEST ORDER OF Y_LM'S)
C
C
C        C) THE RADIUS  P A R A M E T E R:
C
C        R0PARM   => => => THE RADIUS IS EQUAL TO R0PARM*A**(1/3) FM
C
C
C        D) PRECISION OF THE INTEGRATIONS IN GAUSS-LEGENDRE FORMULAS
C
C        N1POIN, N2POIN => THE NUMBER OF GAUSS INTEGRATION POINTS IN
C                          ONE (E.G.THETA) AND THE SECOND (E.G. PHI)
C                          DIRECTIONS
C
C=======================================================================
C
      ZERO=0.0D0
C
      A_MASS=IZNUMB+INNUMB
C
      HOMEGA=HBOM00*FCHOM0/A_MASS**(1./3.)
C
      IREALA=1
C
C      Initialization of the Gauss-Legendre integration mesh
C
      PINUMB=4.0*ATAN(1.0)
C
      NXPOIN=32
      NYPOIN=32
C
      XLOWER=0.0
      YLOWER=0.0
      XUPPER=PINUMB
      YUPPER=2.0000*PINUMB
C
      CALL WS_GAULEG(XLOWER,XUPPER,XPOINT,XWEIGH,NXPOIN,NDPHTH)
C
      CALL WS_GAULEG(YLOWER,YUPPER,YPOINT,YWEIGH,NYPOIN,NDPHTH)
C
C=======================================================================
C      Computing the geometrical characteristics of the nuclear
C      surface corresponding to the WS basis. At this point, ALPHAC
C      should contain the deformations of the basis, not of the actual
C      physical surface
C=======================================================================
C
C      Note: LMXBAS below correspond to the maximum multipolarity
C            of the surface for the basis.
C
      IF (LMXBAS.GT.0) ISOTRO=0
      IF (LMXBAS.EQ.0) ISOTRO=1
C
      IFBASE=1
C
      CALL WS_GEOMFR(CMSXFC,CMSYFC,CMSZFC,XMOMFC,YMOMFC,ZMOMFC,
     *               CVOLFC,NXPOIN,NYPOIN,IZNUMB,INNUMB,R0PARM,
     *                                                  IFBASE)
C
C      Note: the call below print the deformation of the basis
C
      CALL WS_DEFPRI(LMXBAS,IZNUMB,INNUMB)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE WS_GEOMFR(CMSXFC,CMSYFC,CMSZFC,XMOMFC,YMOMFC,ZMOMFC,
     *                     CVOLFC,NXPOIN,NYPOIN,IZ_FIX,IN_FIX,R0PARM,
     *                                                        IFBASE)
      EXTERNAL
     *        WS_CVOLFN,WS_XMASFN,WS_YMASFN,WS_ZMASFN,
     *                  WS_BASXFN,WS_BASYFN,WS_BASZFN,
     *                  WS_XMOMFN,WS_YMOMFN,WS_ZMOMFN
      COMMON
     *       /HFBASE_BASE_2/ BASEX2,BASEY2,BASEZ2
      COMMON
     *       /NCONST_CONSTS/ HBAR_C,XMASSP,XMASSN,HCOVE2,E2OVHC
      COMMON
     *       /SETBAS_CENMAS/ XCMPOS,YCMPOS,ZCMPOS
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
C=======================================================================
C
C         This subroutine calculates the geometrical auxiliary factors
C
C         1. CVOLFC - Volume conservation factor
C
C         2. CMSXFC - Position of the centre of mass  (x - projection)
C         3. CMSYFC - Position of the centre of mass  (y - projection)
C         4. CMSZFC - Position of the centre of mass  (z - projection)
C
C         5. XMOMFC - Moment of inertia  with respect  to  x - a x i s
C         6. YMOMFC - Moment of inertia  with respect  to  y - a x i s
C         7. ZMOMFC - Moment of inertia  with respect  to  z - a x i s
C
C         8. BASEX2 - The basis s t r e t c h i n g factor x - a x i s
C         9. BASEY2 - The basis s t r e t c h i n g factor y - a x i s
C        10. BASEZ2 - The basis s t r e t c h i n g factor z - a x i s
C
C=======================================================================
C
      PINUMB=4.0*ATAN(1.0)
C
      VOLUM0=4.0/3.0000000*PINUMB*R0PARM**3*REAL(IZ_FIX+IN_FIX)
C
      XMASS0=REAL(IZ_FIX)*XMASSP+REAL(IN_FIX)*XMASSN
C
      DENS_0=XMASS0/VOLUM0/HBAR_C**2
C
      RADAUX=R0PARM*REAL(IZ_FIX+IN_FIX)**(1.0/3.0)
C
C=======================================================================
C
      XLOWER=0.0000
      YLOWER=0.0000
C
      XUPPER=PINUMB
      YUPPER=2.0000*PINUMB
C
C=======================================================================
C         C o n s t a n t      v o l u m e      c o n d i t i o n
C=======================================================================
C
      CALL WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                        WS_CVOLFN,RESULT)
C
      F=RESULT/4.000/PINUMB
C
      CVOLFC=1.00000/F**(1.0/3.0)
C
C=======================================================================
C         C e n t r e   o f   m a s s   ( c o o r d i n a t e   x)
C=======================================================================
C
      CALL WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                        WS_XMASFN,RESCMX)
C
      CMSXFC=0.2500*(CVOLFC*RADAUX)**4/VOLUM0*RESCMX
      XCMPOS=CMSXFC/(RADAUX*CVOLFC)
C
C=======================================================================
C         C e n t r e   o f   m a s s   ( c o o r d i n a t e   y)
C=======================================================================
C
      CALL WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                        WS_YMASFN,RESCMY)
C
      CMSYFC=0.2500*(CVOLFC*RADAUX)**4/VOLUM0*RESCMY
      YCMPOS=CMSYFC/(RADAUX*CVOLFC)
C
C=======================================================================
C         C e n t r e   o f   m a s s   ( c o o r d i n a t e   z)
C=======================================================================
C
      CALL WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                        WS_ZMASFN,RESCMZ)
C
      CMSZFC=0.2500*(CVOLFC*RADAUX)**4/VOLUM0*RESCMZ
      ZCMPOS=CMSZFC/(RADAUX*CVOLFC)
C
C=======================================================================
C         B a s i s    s t r e t c h i n g    f a c t o r s
C=======================================================================
C
      IF (IFBASE.EQ.1) THEN
C
C          Basis:  x - s t r e t c h i n g   c o e f f i c i e n t
C
          CALL WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                            WS_BASXFN,RESBX2)
C
          BASEX2=(CVOLFC*RADAUX)**5/VOLUM0/5.0000*RESBX2
C
C          Basis:  y - s t r e t c h i n g   c o e f f i c i e n t
C
          CALL WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                            WS_BASYFN,RESBY2)
C
          BASEY2=(CVOLFC*RADAUX)**5/VOLUM0/5.0000*RESBY2
C
C          Basis:  z - s t r e t c h i n g   c o e f f i c i e n t
C
          CALL WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                            WS_BASZFN,RESBZ2)
C
          BASEZ2=(CVOLFC*RADAUX)**5/VOLUM0/5.0000*RESBZ2
C
      END IF
C
C=======================================================================
C         T h e   m o m e n t   o f   i n e r t i a  (component x)
C=======================================================================
C
      CALL WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                        WS_XMOMFN,RESXMT)
C
      XMOMFC=DENS_0*(CVOLFC*RADAUX)**5/5.0000*RESXMT
C
C=======================================================================
C         T h e   m o m e n t   o f   i n e r t i a  (component y)
C=======================================================================
C
      CALL WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                        WS_YMOMFN,RESYMT)
C
      YMOMFC=DENS_0*(CVOLFC*RADAUX)**5/5.0000*RESYMT
C
C=======================================================================
C         T h e   m o m e n t   o f   i n e r t i a  (component z)
C=======================================================================
C
      CALL WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                        WS_ZMOMFN,RESZMT)
C
      ZMOMFC=DENS_0*(CVOLFC*RADAUX)**5/5.0000*RESZMT
C
C=======================================================================
C=======================================================================
C         P R I N T I N G     T H E     R E S U L T     T A B L E
C=======================================================================
C=======================================================================
C
      IF (IWRITE.GE.3) THEN
C
          WRITE(NOUTPT,'(//,80(''#''),/,''#'',78X,''#'',/,
     *       ''#  A u x i l i a r y   G e o m e t r i c a l'',
     *       ''  F a c t o r s  (S u m m a r y)   #'',/,
     *       ''#'',78X,''#'',/,
     *       ''#  C.Volum     C_ms-x     C_ms-y     C_ms-z '',
     *       ''     Mom-X      Mom-Y      Mom-Z   #'',/,
     *       ''#  '',F7.5,''    '',F7.5,''    '',F7.5,
     *                    ''    '',F7.5,''     '',F6.2,
     *                    ''     '',F6.2,''     '',F6.2,''   #'',/,
     *       ''#'',78X,''#'',/,
     *       ''#  Radaux      B_x**2     B_y**2     B_z**2 '',
     *       ''     R0Par      Volum0    Dens_0   #'',/,
     *       ''#  '',F6.4,''     '',F7.4,''    '',F7.4,
     *                     ''    '',F7.4,''      '',F5.3,
     *                     ''      '',F6.1,  ''    '',F6.2,''   #'',/,
     *                                ''#'',78X,''#'',/,80(''#''),/)')
     *
     *                                    CVOLFC,CMSXFC,CMSYFC,CMSZFC,
     *                                    XMOMFC,YMOMFC,ZMOMFC,RADAUX,
     *                                    BASEX2,BASEY2,BASEZ2,
     *                                    R0PARM,VOLUM0,DENS_0*HBAR_C**2
C
      END IF
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_Q2GAUS(NXPOIN,NYPOIN,XLOWER,XUPPER,YLOWER,YUPPER,
     *                                                 FUNCTN,RESULT)
C=======================================================================
      USE params
C=======================================================================
      EXTERNAL
     *         FUNCTN
      COMMON
     *       /WSGEOM_GAUS2L/ XPOINT(1:NDPHTH),XWEIGH(1:NDPHTH),
     *                       YPOINT(1:NDPHTH),YWEIGH(1:NDPHTH)
C
C=======================================================================
C
C         This subroutine performs a two-dimensional integration
C         of the function FUNCTN(X,Y) between the limits
C
C                   XLOWER < XUPPER and YLOWER < YUPPER
C
C         using the double Gauss-Legendre procedure.  The orders
C         of the Gauss-Legendre formulas are:  NXPOIN and NYPOIN
C
C=======================================================================
C
      SINT_X=0.0
C
      DO INDEXX=1,NXPOIN
C
         XARGUM=XPOINT(INDEXX)
C
         SINT_Y=0.0
C
         DO INDEXY=1,NYPOIN
C
            YARGUM=YPOINT(INDEXY)
C
            SINT_Y=SINT_Y+YWEIGH(INDEXY)*FUNCTN(XARGUM,YARGUM)
C
         END DO
C
         SINT_X=SINT_X+SINT_Y*XWEIGH(INDEXX)
C
      END DO
C
      RESULT=SINT_X
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_FORMFN(THANGL,PHANGL,MAXLAM,RESULT)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *        ALPHAC(0:NDLAMB,-NDLAMB:+NDLAMB)
      COMPLEX
     *        F
      COMPLEX
     *        YLMHAR(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        DYLMDT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        DYLMDF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDTT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDTF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDFF(0:NDLAMB,-NDLAMB:+NDLAMB)
C
      COMMON
     *       /LIMITY_LAMMIU/ MIUACT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *                       LAMACT(0:NDLAMB)
      COMMON
     *       /FORMFN_YLMHAR/ YLMHAR,DYLMDT,DYLMDF,
     *                       YLMDTT,YLMDTF,YLMDFF
C
      COMMON
     *       /ALPHACMUL_DEF/ ALPHAC
C
C=======================================================================
C      This subroutine calculates the form-function =(1+SUM ALFA*Y)
C      at point (theta,phi) = (THANGL,PHANGL). The result is returned
C      in RESULT. Summation is extended up to MAXLAM
C=======================================================================
C
      LEVELY=0
C
      CALL WS_ZERHAR(YLMHAR,DYLMDT,DYLMDF,YLMDTT,YLMDTF,YLMDFF,
     *                                                  MAXLAM)
C
C=======================================================================
C
      DO LAMBDA=0,MAXLAM
C
         IF (LAMACT(LAMBDA).LT.0) GO TO 1
C
         DO MIUIND=0,LAMBDA
C
            IF (MIUACT(LAMBDA,MIUIND).LT.-99) GO TO 2
C
                CALL WS_YL_MIU(YLMHAR,DYLMDT,DYLMDF,YLMDTT,YLMDTF,
     *                         YLMDFF,LAMBDA,MIUIND,THANGL,PHANGL)
   2        CONTINUE
C
         END DO
C
   1     CONTINUE
C
      END DO
C
C=======================================================================
C
      F=CMPLX(0.0,0.0)
C
C=======================================================================
C         S u m m i n g   u p  t h e   (ALPHAC*) times YLM  T E R M S
C=======================================================================
C
      DO LAMBDA=0,MAXLAM
         DO MIUIND=-LAMBDA,+LAMBDA
C
            F=F+CONJG(ALPHAC(LAMBDA,MIUIND))*YLMHAR(LAMBDA,MIUIND)
C
         END DO
      END DO
C
      RESULT=1.0+REAL(F)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_CVOLFN(THETA,PHI)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
C=======================================================================
C
      CALL WS_FORMFN(THETA,PHI,LAMMAX,R)
C
      WS_CVOLFN=R**3*SIN(THETA)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_ZMASFN(THETA,PHI)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
C=======================================================================
C
      CALL WS_FORMFN(THETA,PHI,LAMMAX,R)
C
      WS_ZMASFN=R**4*SIN(THETA)*COS(THETA)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_YMASFN(THETA,PHI)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
C=======================================================================
C
      CALL WS_FORMFN(THETA,PHI,LAMMAX,R)
C
      WS_YMASFN=R**4*SIN(THETA)**2*SIN(PHI)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_XMASFN(THETA,PHI)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
C=======================================================================
C
      CALL WS_FORMFN(THETA,PHI,LAMMAX,R)
C
      WS_XMASFN=R**4*SIN(THETA)**2*COS(PHI)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_BASZFN(THETA,PHI)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C=======================================================================
C
      CALL WS_FORMFN(THETA,PHI,LAMMAX,R)
C
      WS_BASZFN=R**5*COS(THETA)**2*SIN(THETA)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_BASYFN(THETA,PHI)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
C=======================================================================
C
      CALL WS_FORMFN(THETA,PHI,LAMMAX,R)
C
      WS_BASYFN=R**5*SIN(THETA)**3*SIN(PHI)**2
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_BASXFN(THETA,PHI)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
C=======================================================================
C
      CALL WS_FORMFN(THETA,PHI,LAMMAX,R)
C
      WS_BASXFN=R**5*SIN(THETA)**3*COS(PHI)**2
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_ZMOMFN(THETA,PHI)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
C=======================================================================
C
      CALL WS_FORMFN(THETA,PHI,LAMMAX,R)
C
      WS_ZMOMFN=R**5*SIN(THETA)**3
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_YMOMFN(THETA,PHI)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
C=======================================================================
C
      CALL WS_FORMFN(THETA,PHI,LAMMAX,R)
C
      WS_YMOMFN=R**5*(SIN(THETA)**2*COS(PHI)**2
     *               +COS(THETA)**2)*SIN(THETA)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_XMOMFN(THETA,PHI)
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
C=======================================================================
C
      CALL WS_FORMFN(THETA,PHI,LAMMAX,R)
C
      WS_XMOMFN=R**5*(SIN(THETA)**2*SIN(PHI)**2
     *               +COS(THETA)**2)*SIN(THETA)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_QLMGEO(MAXLAM,IZNUMB,INNUMB,QLMGEO)
C=======================================================================
      USE params
C=======================================================================
      CHARACTER
     *          NUNITS*3,NUCAVR*7
C
      COMPLEX
     *          ALPHAC(0:NDLAMB,-NDLAMB:+NDLAMB)
C
      DIMENSION
     *          QLMGEO(0:NDLAMB,0:NDLAMB)
C
      COMMON
     *       /NUCLEONIC_UNI/ NUNITS,NUCAVR
      COMMON
     *       /WSGEOM_GAUS2L/ XPOINT(1:NDPHTH),XWEIGH(1:NDPHTH),
     *                       YPOINT(1:NDPHTH),YWEIGH(1:NDPHTH)
      COMMON
     *       /PARAMS_RADVOL/ RADIUS,CVOLFC,RCVOLM
     *       /PARAMS_R0PARM/ R0PARM
      COMMON
     *       /ALPHACMUL_DEF/ ALPHAC
C
C=======================================================================
C     Subroutine defining the standard sets  of the potential
C     parameters; several standard choices are pre-programmed
C     here see comments inside
C=======================================================================
C
C      Defining the natural constants common for the run
C
      CALL WS_NCONST(NUNITS,NUCAVR)
C
C      Initialization of the Gauss-Legendre integration mesh
C
      PINUMB=4.0*ATAN(1.0)
C
      NXPOIN=64
      NYPOIN=64
C
      XLOWER=0.0
      YLOWER=0.0
      XUPPER=PINUMB
      YUPPER=2.0000*PINUMB
C
      CALL WS_GAULEG(XLOWER,XUPPER,XPOINT,XWEIGH,NXPOIN,NDPHTH)
C
      CALL WS_GAULEG(YLOWER,YUPPER,YPOINT,YWEIGH,NYPOIN,NDPHTH)
C
C      Definition of the units for the multipole moments
C
      CALL WS_DEFUNI(MAXLAM)
C
      A_MASS=REAL(IZNUMB+INNUMB)
C
      RADIUS=R0PARM*A_MASS**(1./3.)
C
C      Determining the spherical harmonics needed for execution
C
      CALL WS_LIMITY(ALPHAC)
C
      NLEGDX=NXPOIN
      NLEGDY=NXPOIN
C
      IFBASE=0
C
      CALL WS_GEOMFR(CMSXFC,CMSYFC,CMSZFC,XMOMFC,YMOMFC,ZMOMFC,
     *               CVOLFC,NLEGDX,NLEGDY,IZNUMB,INNUMB,R0PARM,
     *                                                  IFBASE)
C
      ISOSPI=2
C
      CALL WS_UNIQLM(NLEGDX,NLEGDY,MAXLAM,IZNUMB,INNUMB,ISOSPI,
     *                                    RADIUS,CVOLFC,QLMGEO)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_UNIQLM(NLEGDX,NLEGDY,MAXLAM,IZ_FIX,IN_FIX,ISOSPI,
     *                                          RADIUS,CVOLFC,QLMUNI)
C=======================================================================
      USE params
C=======================================================================
      DIMENSION
     *          QLMOPR(0:NDLAMB,0:NDLAMB),
     *          QLMUNI(0:NDLAMB,0:NDLAMB)
      COMMON
     *       /WSGEOM_GAUS2L/ XPOINT(1:NDPHTH),WEIGHX(1:NDPHTH),
     *                       YPOINT(1:NDPHTH),WEIGHY(1:NDPHTH)
      COMMON
     *       /DEFUNI_OURUNI/ QUNITS(0:NDMULT,0:NDMULT)
C
C=======================================================================
C
C      This subroutine performs the integration of the multipole
C      moment operator (calculated in DEFQLM) over theta and phi
C      according to the Legendre integration method
C
C=======================================================================
C
      IF (MAXLAM.GT.NDLAMB) STOP 'MAXLAM.GT.NDLAMB in WS_UNIQLM'
C
C=======================================================================
C
      DO LAMBDA=0,MAXLAM
         DO MIUACT=0,LAMBDA
            QLMUNI(LAMBDA,MIUACT)=0.0
         END DO
      END DO
C
C      Performing the integration
C
      DO IPOINX=1,NLEGDX
C
         XARGUM=XPOINT(IPOINX)
C
         DO IPOINY=1,NLEGDY
C
            YARGUM=YPOINT(IPOINY)
C
            CALL WS_DEFQLM(XARGUM,YARGUM,MAXLAM,QLMOPR)
C
            DO LAMBDA=0,MAXLAM
               DO MIUACT=0,LAMBDA

                  QLMUNI(LAMBDA,MIUACT)=QLMUNI(LAMBDA,MIUACT)
     *                                 +QLMOPR(LAMBDA,MIUACT)
     *                                 *WEIGHY(IPOINY)
     *                                 *WEIGHX(IPOINX)
C
               END DO
            END DO
C
         END DO
C
      END DO
C
C=======================================================================
C      The tricky part: what normalization factor should be used ?
C      We use here the normalization assuming the matter is uniformly
C      distributed inside the nucleus: constant density rho_0
C=======================================================================
C
      PINUMB=4.0*ATAN(1.0)
      VOLUM0=4.0*PINUMB*RADIUS**3/3.0
C
                       N_PART=IN_FIX
      IF (ISOSPI.EQ.1) N_PART=IZ_FIX
      IF (ISOSPI.EQ.2) N_PART=IZ_FIX+IN_FIX
C
C      Density: different for neutron and proton
C
      DENS_0=REAL(N_PART)/VOLUM0
C
      DO LAMBDA=0,MAXLAM
C
C         Normalization "Q_NORM" gives the volume in [Fm**3]
C
         Q_NORM=(RADIUS*CVOLFC)**(LAMBDA+3)/REAL(LAMBDA+3)
C
         DO MIUACT=0,LAMBDA
C
C            Multiplying by "DENS_0" we normalize the density to
C            the total number of particles,  protons or neutrons
C            Additionally, we multiply the result by QUNITS, which
C            accounts for some special units.
C
C     ===>>> THIS SHOULD NOT BE DONE !!!!
C
C            Reason: we do not want the multipole moments to be compared
C                    their experimental values (like e.g. quadrupole), but
C                    we are after the electromagnetic transition: the mul-
C                    tipole moment that enter the latter are 'naked ones'
C
            QLMUNI(LAMBDA,MIUACT)=DENS_0*Q_NORM*QLMUNI(LAMBDA,MIUACT)
     *                                         *QUNITS(LAMBDA,MIUACT)
C
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DEFQLM(ANGTHE,ANGPHI,MAXLAM,QLMOPR)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *          YLMHAR(0:NDLAMB,-NDLAMB:+NDLAMB),
     *          DYLMDT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *          DYLMDF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *          YLMDTT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *          YLMDTF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *          YLMDFF(0:NDLAMB,-NDLAMB:+NDLAMB)
      DIMENSION
     *          QLMOPR(0:NDLAMB,0:NDLAMB)
      COMMON
     *         /FORMFN_YLMHAR/ YLMHAR,DYLMDT,DYLMDF,
     *                         YLMDTT,YLMDTF,YLMDFF
C
C=======================================================================
C
C      This subroutine calculates the multipole operators at point
C      R = RTHPHI. The multipole operators are defined as:
C
C       Q_lm(theta,phi) = R^(l + 3) * sin(theta) * Y_lm(theta,phi)
C
C      Integration over theta (ANGTHE) and phi (ANGPHI) should give
C      the actual expectation value of the multipole operator.
C
C      N.B. The spherical harmonics YLMFCT, calculated at point
C           theta = ANGTHE and phi = ANGPHI respect the Varshalovitch
C           convention.
C
C=======================================================================
C
      RTHPHI=0.0
C
C      Defining the spherical harmonics
C
      CALL WS_FORMFN(ANGTHE,ANGPHI,MAXLAM,RTHPHI)
C
C      Calculating the multipole operator for each (l,m)
C
      SINTHE=SIN(ANGTHE)
C
      DO LAMBDA=0,MAXLAM
C
         DO MIUACT=0,LAMBDA
            QLMOPR(LAMBDA,MIUACT)=RTHPHI**(LAMBDA+3)
     *                           *SINTHE*YLMHAR(LAMBDA,MIUACT)
         END DO
C
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_LOADIN(MAXLAM)
C=======================================================================
      USE params
C=======================================================================
C
      COMMON
     *       /LOADIN_COEFAB/ ALMS00(0:NDLAMB,0:NDLAMB,0:NDLAMB),
     *                       ALMSS0(0:NDLAMB,0:NDLAMB,0:NDLAMB),
     *                       ALMSSS(0:NDLAMB,0:NDLAMB,0:NDLAMB),
     *
     *                       BLMS00(0:NDLAMB,0:NDLAMB,0:NDLAMB),
     *                       BLMSS0(0:NDLAMB,0:NDLAMB,0:NDLAMB),
     *                       BLMSSS(0:NDLAMB,0:NDLAMB,0:NDLAMB)
     *
     *       /LOADIN_FACTRS/ FACTOR(0:NDFACT),FACT_2(0:NDFACT+1)
     *
     *       /LOADIN_NORMNG/ XNORM(0:NDLAMB,-NDLAMB:+NDLAMB),
     *                       PHASE(0:NDFACT)
C
C=======================================================================
C        C a l c u l a t i n g   t h e   f a c t o r i a l
C=======================================================================
C
      FACTOR(0)=1.0D0
      FACTOR(1)=1.0D0
C
      DO I=2,NDFACT
         FACTOR(I)=REAL(I,KIND=8)*FACTOR(I-1)
      END DO
C
C=======================================================================
C        Calculating the normalization factors for the Ylm functions
C=======================================================================
C
      PI_4=16.0D0*ATAN(1.0D0)
C
      DO LAMBDA=0,MAXLAM
         DO MIU=0,LAMBDA
C
            XNORM(LAMBDA,MIU)=(2.0000*LAMBDA+1.000000)
     *                       / FACTOR(LAMBDA-IABS(MIU))
     *                       / FACTOR(LAMBDA+IABS(MIU))/PI_4
C
            XNORM(LAMBDA,MIU)=SQRT(XNORM(LAMBDA,MIU))
C
         END DO
      END DO
C
C=======================================================================
C        Calculating the coefficients for the summations in spherical
C        harmonics
C=======================================================================
C        C a l c u l a t i n g     d o u b l e    f a c t o r i a l s
C=======================================================================
C
      FACT_2(0)=1.0D0
      FACT_2(1)=1.0D0
C
      DO I=2,NDFACT,2
         FACT_2(I)=REAL(I,KIND=8)*FACT_2(I-2)
      END DO
C
      DO I=3,NDFACT,2
         FACT_2(I)=REAL(I,KIND=8)*FACT_2(I-2)
      END DO
C
C=======================================================================
C        C a l c u l a t i n g    t h e    p h a s e    f a c t o r s
C=======================================================================
C
      PHASE(0)=1.00
      DO I=1,NDFACT
         PHASE(I)=-PHASE(I-1)
      END DO
C
C=======================================================================
C        Calculating the coefficients for the YLM summations (L-M)-even
C=======================================================================
C
      DO LAMBDA=0,MAXLAM
         DO MIU=LAMBDA,0,-2
C
            ZNORMA=XNORM(LAMBDA,MIU)
C
            DO IS=MIU,LAMBDA,2
               ALMS00(LAMBDA,MIU,IS)=ZNORMA*PHASE((MIU+IS)/2)
     *                              *FACT_2(LAMBDA+MIU)
     *                              /FACT_2(LAMBDA+ IS)
     *                              *FACT_2(LAMBDA-MIU)
     *                              /FACT_2(LAMBDA- IS)
     *                              *FACTOR(LAMBDA+IS)
     *                              /FACT_2(IS  +  MIU)
     *                              /FACT_2(IS  -  MIU)
            END DO
C
            DO IS=MIU,LAMBDA,2
               ALMSS0(LAMBDA,MIU,IS)=ALMS00(LAMBDA,MIU,IS)*IS
               ALMSSS(LAMBDA,MIU,IS)=ALMS00(LAMBDA,MIU,IS)*IS*(IS-1)
            END DO
C
         END DO
      END DO
C
C=======================================================================
C        Calculating the coefficients for the Ylm summations (L-M)-odd
C=======================================================================
C
      DO LAMBDA=0,MAXLAM
         DO MIU=LAMBDA-1,0,-2
C
            ZNORMA=XNORM(LAMBDA,MIU)
C
            DO IS=MIU,LAMBDA-1,2
               BLMS00(LAMBDA,MIU,IS)=ZNORMA*PHASE((MIU+IS)/2)
     *                              *FACT_2(LAMBDA+MIU-1)
     *                              /FACT_2(LAMBDA+ IS-1)
     *                              *FACT_2(LAMBDA-MIU-1)
     *                              /FACT_2(LAMBDA- IS-1)
     *                              *FACTOR(LAMBDA+IS)
     *                              /FACT_2(IS  +  MIU)
     *                              /FACT_2(IS  -  MIU)
            END DO
C
            DO IS=MIU,LAMBDA-1,2
               BLMSS0(LAMBDA,MIU,IS)=BLMS00(LAMBDA,MIU,IS)*IS
               BLMSSS(LAMBDA,MIU,IS)=BLMS00(LAMBDA,MIU,IS)*IS*(IS-1)
            END DO
C
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_YL_MIU(YLM,YLMT,YLMF,YLMTT,YLMTF,YLMFF,
     *                                LAMBDA,MIU,THETA,PHI)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *          YLM  (0:NDLAMB,-NDLAMB:+NDLAMB),
     *          YLMT (0:NDLAMB,-NDLAMB:+NDLAMB),
     *          YLMF (0:NDLAMB,-NDLAMB:+NDLAMB),
     *          YLMTT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *          YLMTF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *          YLMFF(0:NDLAMB,-NDLAMB:+NDLAMB)
      COMPLEX
     *          YLMCCC,YLMTCC,YLMFCC,YLMTTC,YLMTFC,YLMFFC,
     *                                      EXPPHI,UNIT_I
      DIMENSION
     *          SINUS (-2:NDLAMB)
C
      COMMON
     *       /LOADIN_COEFAB/ ALMS00(0:NDLAMB,0:NDLAMB,0:NDLAMB),
     *                       ALMSS0(0:NDLAMB,0:NDLAMB,0:NDLAMB),
     *                       ALMSSS(0:NDLAMB,0:NDLAMB,0:NDLAMB),
     *
     *                       BLMS00(0:NDLAMB,0:NDLAMB,0:NDLAMB),
     *                       BLMSS0(0:NDLAMB,0:NDLAMB,0:NDLAMB),
     *                       BLMSSS(0:NDLAMB,0:NDLAMB,0:NDLAMB)
     *
     *       /LOADIN_NORMNG/ XNORM(0:NDLAMB,-NDLAMB:+NDLAMB),
     *                       PHASE(0:NDFACT)
C
C=======================================================================
C
C      Calculation of the spherical harmonic Y_lm(theta,phi) at point
C      (theta,phi) = (THETA,PHI) for multipolarity (l,m) = (LAMBDA,MIU)
C
C=======================================================================
C
      UNIT_I=CMPLX(0.0,1.0)
C
      YLMCCC=CMPLX(0.0,0.0)
      YLMTCC=CMPLX(0.0,0.0)
      YLMFCC=CMPLX(0.0,0.0)
      YLMTTC=CMPLX(0.0,0.0)
      YLMTFC=CMPLX(0.0,0.0)
      YLMFFC=CMPLX(0.0,0.0)
C
C=======================================================================
C     Calculating the spherical harmonics for (LAMBDA-MIU)-even
C=======================================================================
C
      SINTHT=SIN(THETA)
      COSTHT=COS(THETA)
C
      COST_2=COSTHT*COSTHT
C
      ANGLE=MIU*PHI
C
      EXPPHI=CMPLX(COS(ANGLE),SIN(ANGLE))
C
      SINUS(-2)=0.0
      SINUS(-1)=0.0
      SINUS( 0)=1.0
C
      SINUS(+1)=SINTHT
      DO I=2,LAMBDA
         SINUS(I)=SINUS(I-1)*SINTHT
      END DO
C
      IF (MOD((LAMBDA-MIU),2).EQ.0) THEN
C
          SUM=0.0
C
          DO IS=MIU,LAMBDA,2
             SUM=SUM+ALMS00(LAMBDA,MIU,IS)*SINUS(IS)
          END DO
C
          YLMCCC=SUM*EXPPHI
C
          YLMFCC=YLMCCC*UNIT_I*MIU
C
          SUM=0.0
          DO IS=MIU,LAMBDA,2
             SUM=SUM+ALMSS0(LAMBDA,MIU,IS)*SINUS(IS-1)
          END DO
C
          YLMTCC=SUM*COSTHT*EXPPHI
C
          YLMFFC=-MIU*MIU*YLMCCC
          YLMTFC=+MIU*UNIT_I*YLMTCC
C
          SUM1=0.0
          SUM2=0.0
C
          DO IS=MIU,LAMBDA,2
             SUM1=SUM1+ALMSS0(LAMBDA,MIU,IS)*SINUS( IS )
             SUM2=SUM2+ALMSSS(LAMBDA,MIU,IS)*SINUS(IS-2)
          END DO
C
          YLMTTC=EXPPHI*(COST_2*SUM2-SUM1)
C
      ELSE
C
C=======================================================================
C         Calculating the spherical harmonics for (lambda-miu)-odd
C=======================================================================
C
          SUM=0.0
C
          DO IS=MIU,LAMBDA-1,2
             SUM=SUM+BLMS00(LAMBDA,MIU,IS)*SINUS(IS)
          END DO
C
          SUM_TH=SUM*COSTHT
C
          YLMCCC=SUM_TH*EXPPHI
C
          YLMFCC=YLMCCC*UNIT_I*MIU
C
          SUM1=0.0
          DO IS=MIU,LAMBDA-1,2
             SUM1=SUM1+BLMSS0(LAMBDA,MIU,IS)*SINUS(IS-1)
          END DO
C
          YLMTCC=EXPPHI*(COST_2*SUM1-SINTHT*SUM)
C
          YLMFFC=-MIU*MIU*YLMCCC
          YLMTFC=+MIU*UNIT_I*YLMTCC
C
          SUM2=0.0
C
          DO IS=MIU,LAMBDA-1,2
             SUM2=SUM2+BLMSSS(LAMBDA,MIU,IS)*SINUS(IS-2)
          END DO
C
          YLMTTC=EXPPHI*(COST_2*COSTHT*SUM2-SUM_TH
     *              -3.0*SINTHT*COSTHT*SUM1)
C
      END IF
C
      YLM(LAMBDA,+MIU)=YLMCCC
      YLM(LAMBDA,-MIU)=PHASE(MIU)*CONJG(YLMCCC)
C
      YLMT(LAMBDA,+MIU)=YLMTCC
      YLMF(LAMBDA,+MIU)=YLMFCC
C
      YLMT(LAMBDA,-MIU)=PHASE(MIU)*CONJG(YLMTCC)
      YLMF(LAMBDA,-MIU)=PHASE(MIU)*CONJG(YLMFCC)
C
      YLMTT(LAMBDA,+MIU)=YLMTTC
      YLMTF(LAMBDA,+MIU)=YLMTFC
      YLMFF(LAMBDA,+MIU)=YLMFFC
C
      YLMTT(LAMBDA,-MIU)=PHASE(MIU)*CONJG(YLMTTC)
      YLMTF(LAMBDA,-MIU)=PHASE(MIU)*CONJG(YLMTFC)
      YLMFF(LAMBDA,-MIU)=PHASE(MIU)*CONJG(YLMFFC)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_ZERHAR(YLMHAR,DYLMDT,DYLMDF,YLMDTT,YLMDTF,YLMDFF,
     *                                                        MAXLAM)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *        YLMHAR(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        DYLMDT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        DYLMDF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDTT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDTF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDFF(0:NDLAMB,-NDLAMB:+NDLAMB)
      COMPLEX
     *        C_ZERO
C
C=======================================================================
C        Zeroing the auxiliary matrices for the spherical harmonic
C        calculation routine.
C=======================================================================
C
      C_ZERO=CMPLX(0.0,0.0)
C
      DO LAMBDA=0,MAXLAM
         DO MIUIND=-LAMBDA,+LAMBDA
            YLMHAR(LAMBDA,MIUIND)=C_ZERO
            DYLMDT(LAMBDA,MIUIND)=C_ZERO
            DYLMDF(LAMBDA,MIUIND)=C_ZERO
            YLMDTT(LAMBDA,MIUIND)=C_ZERO
            YLMDTF(LAMBDA,MIUIND)=C_ZERO
            YLMDFF(LAMBDA,MIUIND)=C_ZERO
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE WS_LIMITY(ALPHAC)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *        ALPHAC(0:NDLAMB,-NDLAMB:+NDLAMB)
      COMMON
     *       /LIMITY_LAMMIU/ MIUACT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *                       LAMACT(0:NDLAMB)
      COMMON
     *       /MULTIPOLE_LMU/ IFMULT,MAXMUL,NOFLEV,IINDIV,LAMIND,MIUIND
C
C=======================================================================
C         This subroutine determines the  LAMBDA  and  MIU values
C         for which the spherical harmonics and their derivatives
C         are needed in the program
C=======================================================================
C
      DO LAMBDA=0,NDLAMB
         LAMACT(LAMBDA)=-100
         DO MIU=-LAMBDA,+LAMBDA
            MIUACT(LAMBDA,MIU)=-100
         END DO
      END DO
C
      DO LAMBDA=0,NDLAMB
         DO MIU=-LAMBDA,+LAMBDA
            IF (ABS(ALPHAC(LAMBDA,MIU)).GT.1.E-7) THEN
                LAMACT(LAMBDA)=LAMBDA
                MIUACT(LAMBDA,MIU)=MIU
            END IF
         END DO
      END DO
CC
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C
C      SUBROUTINES WS_ROTWIG, WS_BIGWIG and WS_SMALLW PERFORM THE
C      EULER ROTATION OF THE WAVE-FUNCTIONS
C
C=======================================================================
C
C
      SUBROUTINE WS_ROTWIG(IFWIGN,MAXLAM,ALFAWG,BETAWG,GAMAWG,ALPHAC,
     *                                                        NDLAMP)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *          ALPHAC(0:NDLAMP,-NDLAMP:+NDLAMP),
     *          ALPH1C(0:NDLAMP,-NDLAMP:+NDLAMP)
      COMPLEX
     *          BIGDJM(-NDLAMP:+NDLAMP,-NDLAMP:+NDLAMP)
C
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
C=======================================================================
C         Turning the nucleus in space using the Wigner functions
C=======================================================================
C
      IF (NDLAMB.NE.NDLAMP) STOP 'NDLAMB.NE.NDLAMP in ROTWIG'
C
      IF (MAXLAM.GT.NDLAMP) STOP 'MAXLAM.GT.NDLAMB in ROTWIG'
C
C=======================================================================
C
      IF (IFWIGN.NE.1) RETURN
C
C=======================================================================
C
      PINUMB=4.0*ATAN(1.000000)
C
      ALFAWG=PINUMB*ALFAWG/180.
      BETAWG=PINUMB*BETAWG/180.
      GAMAWG=PINUMB*GAMAWG/180.
C
      DO LAMBDA=0,MAXLAM
C
         CALL WS_BIGWIG(LAMBDA,ALFAWG,BETAWG,GAMAWG,BIGDJM,NDLAMP)
C
         DO MIUACT=-LAMBDA,LAMBDA
            ALPH1C(LAMBDA,MIUACT)=CMPLX(0.0,0.0)
         END DO
C
         DO MIUACT=-LAMBDA,LAMBDA
            DO MIURUN=-LAMBDA,LAMBDA
C
               ALPH1C(LAMBDA,MIUACT)=ALPH1C(LAMBDA,MIUACT)
     *                              +CONJG (BIGDJM(MIUACT,MIURUN))
     *                              *ALPHAC(LAMBDA,MIURUN)
            END DO
         END DO
C
         DO MIUACT=-LAMBDA,LAMBDA
            ALPHAC(LAMBDA,MIUACT)=ALPH1C(LAMBDA,MIUACT)
         END DO
C
      END DO
C
      ALFAWG=ALFAWG*180./PINUMB
      BETAWG=BETAWG*180./PINUMB
      GAMAWG=GAMAWG*180./PINUMB
C
C=======================================================================
C
      WRITE(NOUTPT,'(/,80(''#''),/,''#'',78X,''#'',/,
     *                 ''#  You have requested to rotate the defor'',
     *                 ''mation space using the Euler angles:  #'',/,
     *                 ''#              o                       '',
     *                 ''    o                                o  #'',/,
     *                 ''#  ALFAWG ='',I4,
     *                 ''                BETAWG ='',I4,
     *                 ''                     GAMAWG ='',I4,''   #'',/,
     *                 ''#'',78X,''#'',/,80(''#''))')
     *
     *                 NINT(ALFAWG),NINT(BETAWG),NINT(GAMAWG)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_BIGWIG(MAXLAM,ALPHWG,BETAWG,GAMAWG,BIGDJM,NDLAMP)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *          BIGDJM,EMALPH,EMGAMA
      DIMENSION
     *          DWIGNE(-NDLAMP:+NDLAMP,-NDLAMP-1:+NDLAMP+1)
      DIMENSION
     *          BIGDJM(-NDLAMP:+NDLAMP,-NDLAMP:+NDLAMP)
C
C=======================================================================
C     This subroutine calculates the  'big D'  Wigner functions
C     using the 'small d' Wigner functions calculated in SMALLW
C=======================================================================
C
      IF (NDLAMB.NE.NDLAMP) STOP 'NDLAMB.NE.NDLAMP in BIGWIG'
C
C=======================================================================
C
      CALL WS_SMALLW(MAXLAM,BETAWG,DWIGNE,NDLAMP)
C
C=======================================================================
C
      DO INDBRA=-MAXLAM,+MAXLAM
         DO INDKET=-MAXLAM,+MAXLAM
C
            EMALPH = CMPLX (COS(INDBRA*ALPHWG), -SIN(INDBRA*ALPHWG))
            EMGAMA = CMPLX (COS(INDKET*GAMAWG), -SIN(INDKET*GAMAWG))
C
            BIGDJM(INDBRA,INDKET)=EMALPH
     *                           *CMPLX(DWIGNE(INDBRA,INDKET),0.0)
     *                           *EMGAMA
C
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_SMALLW(J_SPIN,BETAWG,DWIGNE,NDLAMP)
C=======================================================================
      USE params
C=======================================================================
      DIMENSION
     *          FACTOR(0:NDFACT),SQRAUX(0:NDFACT**2)
      DIMENSION
     *          DWIGNE(-NDLAMP:+NDLAMP,-NDLAMP-1:+NDLAMP+1)
      DIMENSION
     *          WIGJJM(-NDLAMP:+NDLAMP)
C
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
C=======================================================================
C
C     This subroutine calculates the "small d" Wigner functions of
C     angle BETAWG given in radians. Variable  J_SPIN  is equal to
C     the upper index J of the Wigner function. We limit oursleves
C     to the integer J-values.
C
C     The phases correspond to the Varshalovich convention.
C
C     Here we calculate only the functions d(J,J,M) and obtain all
C     the other functions by the fast recurrence relation as given
C     by Varshalovich et al., p.93
C
C=======================================================================
C
      IF (NDLAMB.NE.NDLAMP) STOP 'NDLAMB.NE.NDLAMP in SMALLW'
C
      IF (J_SPIN.GT.NDLAMB) THEN
C
          WRITE(NOUTPT,'(/,''J_SPIN='',I3,'' EXCEEDS NDLAMB='',I2)')
     *                                        J_SPIN,NDLAMB
C
          STOP 'J_SPIN exceeds NDLAMB in SMALLW'
C
      END IF
C
C=======================================================================
C
      FACTOR(0)=1.0D0
      FACTOR(1)=1.0D0
C
      DO I=2,2*J_SPIN
         FACTOR(I)=REAL(I,KIND=8)*FACTOR(I-1)
      END DO
C
C=======================================================================
C
      SQRAUX(0)=0.0D0
C
      DO I=1,NDFACT**2
         SQRAUX(I)=SQRT(REAL(I,KIND=8))
      END DO
C
C=======================================================================
C     Here calculating the functions d^J_J_M, for (-J).ge.M.le.(+J)
C=======================================================================
C
      COSBO2=+COS(0.5*BETAWG)
      SINBO2=-SIN(0.5*BETAWG)
C
      DO MWIGNE=-J_SPIN,+J_SPIN
C
         COSSIN=COSBO2**(J_SPIN+MWIGNE)
     *         *SINBO2**(J_SPIN-MWIGNE)
C
         X_NORM=SQRT(FACTOR(J_SPIN+J_SPIN)
     *             /(FACTOR(J_SPIN+MWIGNE)
     *              *FACTOR(J_SPIN-MWIGNE)))
C
         WIGJJM(MWIGNE)=COSSIN*X_NORM
C
      END DO
C
C=======================================================================
C     Here using the recurrence relations to calculate all other M's
C     Preparations first...
C=======================================================================
C
      COSBET=COS(BETAWG)
      SINBET=SIN(BETAWG)
C
      COSBEM=0.5*(1.0-COSBET)
      COSBEP=0.5*(1.0+COSBET)
C
      DO M_RIGH=-J_SPIN,+J_SPIN
C
         DWIGNE(J_SPIN,M_RIGH)=WIGJJM(M_RIGH)
C
      END DO
C
C=======================================================================
C     ... and then the actual recurrence relations
C=======================================================================
C
      DO M_LEFT=J_SPIN-1,-J_SPIN,-1
C
         CDENOM=+SQRAUX((J_SPIN+M_LEFT+1)*(J_SPIN-M_LEFT))
C
         DO M_RIGH=J_SPIN,-J_SPIN,-1
C
            COEFM1=-SQRAUX((J_SPIN+M_RIGH)*(J_SPIN-M_RIGH+1))
            COEFP1=+SQRAUX((J_SPIN-M_RIGH)*(J_SPIN+M_RIGH+1))
C
            COEF00=+REAL(M_RIGH)
C
            DWIGNE(M_LEFT,M_RIGH)=
     *
     *            (COEFM1*COSBEM*DWIGNE(M_LEFT+1,M_RIGH-1)
     *            +COEF00*SINBET*DWIGNE(M_LEFT+1, M_RIGH )
     *            +COEFP1*COSBEP*DWIGNE(M_LEFT+1,M_RIGH+1))/CDENOM
C
         END DO
C
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C
C      SUBROUTINES WS_FNSYST, WS_RONSUR AND WS_DSURF ARE NEEDED BY THE
C      MODULE wsu21_macr_01.f
C
C=======================================================================
C
      SUBROUTINE WS_FNSYST(THANGL,FIANGL,DERFDT,DERFDF,DERGDT,DERGDF,
     *                                                 FUNCTF,FUNCTG)
      COMMON
     *       /DISTAN_SPACEP/ X_KART,Y_KART,Z_KART
      COMMON
     *       /PARAMS_RADVOL/ RADIUS,CVOLFC,RCVOLM
C
C=======================================================================
C         This subroutine defines FUNCTF, FUNCTG and auxiliary partial
C         derivatives DERRDT and DERRDF for the distance function
C=======================================================================
C
      COSTHE=COS(THANGL)
      SINTHE=SIN(THANGL)
      COS_FI=COS(FIANGL)
      SIN_FI=SIN(FIANGL)
C
C=======================================================================
C
      NODIMS=0
      LEVELY=2
C
      CALL WS_RONSUR(THANGL,FIANGL,REQUAT,DERRDT,DERRDF,D2RDT2,
     *                             D2RDTF,D2RDF2,NODIMS,LEVELY)
C
C=======================================================================
C
      REQUAT=REQUAT/RCVOLM
C
      DERRDT=DERRDT/RCVOLM
      DERRDF=DERRDF/RCVOLM
C
      D2RDT2=D2RDT2/RCVOLM
      D2RDTF=D2RDTF/RCVOLM
      D2RDF2=D2RDF2/RCVOLM
C
C=======================================================================
C         Calculating XNORML = R scalar product N (see page D.10 /notes)
C=======================================================================
C
      PROJN1=+X_KART*COS_FI+Y_KART*SIN_FI
      PROJN2=-X_KART*SIN_FI+Y_KART*COS_FI
C
      XNORML=+SINTHE*PROJN1+COSTHE*Z_KART
C
C=======================================================================
C         Calculating the first derivatives of the auxiliary 'XNORML'
C=======================================================================
C
      DNORDT=COSTHE*PROJN1-Z_KART*SINTHE
      DNORDF=SINTHE*PROJN2
C
      D2NDT2=-XNORML
      D2NDTF=+COSTHE*PROJN2
      D2NDF2=-SINTHE*PROJN1
C
C=======================================================================
C         Defining the equations to be solved: DDIST2/DT=0, DDIST2/DF=0
C=======================================================================
C
      FUNCTF=DERRDT*(REQUAT-XNORML)-REQUAT*DNORDT
      FUNCTG=DERRDF*(REQUAT-XNORML)-REQUAT*DNORDF
C
      FUNCTF=FUNCTF+FUNCTF
      FUNCTG=FUNCTG+FUNCTG
C
C=======================================================================
C         For the Newton's method - the corresponding derivative calcs.
C=======================================================================
C
      DERFDT=D2RDT2*(REQUAT-XNORML)
     *      -REQUAT* D2NDT2+DERRDT*(DERRDT-DNORDT-DNORDT)
      DERFDF=D2RDTF*(REQUAT-XNORML)
     *      -REQUAT* D2NDTF+DERRDT*(DERRDF-DNORDF)-DERRDF*DNORDT
C
      DERFDT=DERFDT+DERFDT
      DERFDF=DERFDF+DERFDF
C
      DERGDT=DERFDF
C
      DERGDF=D2RDF2*(REQUAT-XNORML)
     *      -REQUAT* D2NDF2+DERRDF*(DERRDF-2.0*DNORDF)
      DERGDF=DERGDF+ DERGDF
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_RONSUR(THANGL,PHANGL,RSURFC,DRDTHE,DRDPHI,
     *                                   D2RDTT,D2RDTF,D2RDFF,
     *                                          NODIMS,LEVELY)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *        ALPHAC(0:NDLAMB,-NDLAMB:+NDLAMB)
      COMPLEX
     *        D2FDTT,D2FDTF,D2FDFF
      COMPLEX
     *        Z_SUMM,SUMMDT,SUMMDF
      COMPLEX
     *        YLMHAR(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        DYLMDT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        DYLMDF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDTT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDTF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDFF(0:NDLAMB,-NDLAMB:+NDLAMB)
C
      COMMON
     *       /LIMITY_LAMMIU/ MIUACT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *                       LAMACT(0:NDLAMB)
      COMMON
     *       /FORMFN_YLMHAR/ YLMHAR,DYLMDT,DYLMDF,
     *                       YLMDTT,YLMDTF,YLMDFF
      COMMON
     *       /PARAMS_RADVOL/ RADIUS,CVOLFC,RCVOLM
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
      COMMON
     *       /ALPHACMUL_DEF/ ALPHAC
C
C=======================================================================
C
C         This subroutine calculates the form-function (DIMENSION fm)
C                         F = (1+SUM ALFA*Y) * RCVOLM
C         its partial derivatives DRDTHE and DRDPHI;  also the second
C         partial derivatives:  D2RDTT, D2RDTF, & D2RDFF;  the factor
C                            RCVOLM=RADIUS*CVOLFC
C         above  'CVOLFC' is  the constant-volume-function. Therefore
C         the result of this function has the dimension equal to 'fm'
C
C=======================================================================
C
      CALL WS_ZERHAR(YLMHAR,DYLMDT,DYLMDF,YLMDTT,YLMDTF,YLMDFF,
     *                                                  LAMMAX)
C
C=======================================================================
C
      DO LAMBDA=0,LAMMAX
C
         IF (LAMACT(LAMBDA).LT.0) GO TO 1
C
         DO MIUIND=0,LAMBDA
C
            IF (MIUACT(LAMBDA,MIUIND).LT.-99) GO TO 2
                CALL WS_YL_MIU(YLMHAR,DYLMDT,DYLMDF,YLMDTT,YLMDTF,
     *                         YLMDFF,LAMBDA,MIUIND,THANGL,PHANGL)
   2        CONTINUE
         END DO
C
   1     CONTINUE
C
      END DO
C
C=======================================================================
C
      D2FDTT=CMPLX(0.0,0.0)
      D2FDTF=CMPLX(0.0,0.0)
      D2FDFF=CMPLX(0.0,0.0)
C
C=======================================================================
C         S u m m i n g   u p  t h e  (ALPHAC*) times d2_YLM  T E R M S
C=======================================================================
C
      DO LAMBDA=0,LAMMAX
         DO MIUIND=-LAMBDA,+LAMBDA
C
            D2FDFF=D2FDFF+CONJG(ALPHAC(LAMBDA,MIUIND))
     *                         *YLMDFF(LAMBDA,MIUIND)
            D2FDTF=D2FDTF+CONJG(ALPHAC(LAMBDA,MIUIND))
     *                         *YLMDTF(LAMBDA,MIUIND)
            D2FDTT=D2FDTT+CONJG(ALPHAC(LAMBDA,MIUIND))
     *                         *YLMDTT(LAMBDA,MIUIND)
C
         END DO
      END DO
C
C=======================================================================
C
      IF (NODIMS.EQ.1) THEN
C
          D2RDFF=REAL(D2FDFF)
          D2RDTF=REAL(D2FDTF)
          D2RDTT=REAL(D2FDTT)
C
      ELSE
C
          D2RDFF=REAL(D2FDFF)*RCVOLM
          D2RDTF=REAL(D2FDTF)*RCVOLM
          D2RDTT=REAL(D2FDTT)*RCVOLM
C
      END IF
C
C=======================================================================
C         In addition the  f i r s t  derivatives are also calculated
C=======================================================================
C
      SUMMDT=CMPLX(0.0,0.0)
      SUMMDF=CMPLX(0.0,0.0)
      Z_SUMM=CMPLX(0.0,0.0)
C
C=======================================================================
C         S u m m i n g    u p    t h e    ALPHAC * Y_lm    t e r m s
C=======================================================================
C
      DO LAMBDA=0,LAMMAX
         DO MIUIND=-LAMBDA,+LAMBDA
C
            Z_SUMM=Z_SUMM+CONJG(ALPHAC(LAMBDA,MIUIND))
     *                         *YLMHAR(LAMBDA,MIUIND)
            SUMMDT=SUMMDT+CONJG(ALPHAC(LAMBDA,MIUIND))
     *                         *DYLMDT(LAMBDA,MIUIND)
            SUMMDF=SUMMDF+CONJG(ALPHAC(LAMBDA,MIUIND))
     *                         *DYLMDF(LAMBDA,MIUIND)
C
         END DO
      END DO
C
C=======================================================================
C
      IF (NODIMS.EQ.1) THEN
C
          DRDTHE=REAL(SUMMDT)
          DRDPHI=REAL(SUMMDF)
C
          R_OFTP=(1.00+REAL(Z_SUMM))
C
      ELSE
C
          DRDTHE=REAL(SUMMDT)*RCVOLM
          DRDPHI=REAL(SUMMDF)*RCVOLM
C
          R_OFTP=(1.00+REAL(Z_SUMM))*RCVOLM
C
      END IF
C
      RSURFC=R_OFTP
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DRSURF(THANGL,PHANGL,R_OFTP,DRDTHE,DRDPHI,NODIMS,
     *                                                        LEVELY)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *        Z_SUMM,SUMMDT,SUMMDF
      COMPLEX
     *        ALPHAC(0:NDLAMB,-NDLAMB:+NDLAMB)
      COMPLEX
     *        YLMHAR(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        DYLMDT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        DYLMDF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDTT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDTF(0:NDLAMB,-NDLAMB:+NDLAMB),
     *        YLMDFF(0:NDLAMB,-NDLAMB:+NDLAMB)
C
      COMMON
     *       /LIMITY_LAMMIU/ MIUACT(0:NDLAMB,-NDLAMB:+NDLAMB),
     *                       LAMACT(0:NDLAMB)
      COMMON
     *       /FORMFN_YLMHAR/ YLMHAR,DYLMDT,DYLMDF,
     *                       YLMDTT,YLMDTF,YLMDFF
      COMMON
     *       /PARAMS_RADVOL/ RADIUS,CVOLFC,RCVOLM
      COMMON
     *       /READDF_LAMMAX/ LAMMAX,LMXBAS,LMXDEF,LMXMUL
C
      COMMON
     *       /ALPHACMUL_DEF/ ALPHAC
C
C=======================================================================
C
C         This subroutine calculates the form-function (dimension fm)
C
C                         F = (1+SUM ALFA*Y) * RCVOLM
C
C         and its partial derivatives DRDTHE and DRDPHI.  The factor
C
C                             RCVOLM=RADIUS*CVOLFC,
C
C         where CVOLFC is the constant-volume function. Consequently
C         the result of this function has the dimension equal to 'fm'
C
C=======================================================================
C
      CALL WS_ZERHAR(YLMHAR,DYLMDT,DYLMDF,YLMDTT,YLMDTF,YLMDFF,
     *                                                  LAMMAX)
C
C=======================================================================
C
      DO LAMBDA=0,LAMMAX
C
         IF (LAMACT(LAMBDA).LT.0) GO TO 1
C
         DO MIUIND=0,LAMBDA
C
            IF (MIUACT(LAMBDA,MIUIND).LT.-99) GO TO 2
C
            CALL WS_YL_MIU(YLMHAR,DYLMDT,DYLMDF,YLMDTT,YLMDTF,YLMDFF,
     *                                   LAMBDA,MIUIND,THANGL,PHANGL)
   2        CONTINUE
C
         END DO
C
   1     CONTINUE
C
      END DO
C
      SUMMDT=CMPLX(0.0,0.0)
      SUMMDF=CMPLX(0.0,0.0)
      Z_SUMM=CMPLX(0.0,0.0)
C
C=======================================================================
C         S u m m i n g   u p  t h e  (ALPHAC*) times YLM  T E R M S
C=======================================================================
C
      DO LAMBDA=0,LAMMAX
         DO MIUIND=-LAMBDA,+LAMBDA
C
            Z_SUMM=Z_SUMM+CONJG(ALPHAC(LAMBDA,MIUIND))
     *                         *YLMHAR(LAMBDA,MIUIND)
            SUMMDT=SUMMDT+CONJG(ALPHAC(LAMBDA,MIUIND))
     *                         *DYLMDT(LAMBDA,MIUIND)
            SUMMDF=SUMMDF+CONJG(ALPHAC(LAMBDA,MIUIND))
     *                         *DYLMDF(LAMBDA,MIUIND)
C
         END DO
      END DO
C
      DRDTHE=0.0
      DRDPHI=0.0
      R_OFTP=0.0
C
      IF (NODIMS.EQ.0) THEN
C
          DRDTHE=REAL(SUMMDT)*RCVOLM
          DRDPHI=REAL(SUMMDF)*RCVOLM
C
          R_OFTP=(1.00+REAL(Z_SUMM))*RCVOLM
C
      ELSE
C
          DRDTHE=REAL(SUMMDT)
          DRDPHI=REAL(SUMMDF)
C
          R_OFTP=(1.00+REAL(Z_SUMM))
C
      END IF
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================

