C FILE NAME = wsu16_matr_04.f   ! Keep this symbol here: $ident@string$
C
C=======================================================================
C=======================================================================
C====                                 ==================================
C====    MATRIX OPERATION ROUTINES    ==================================
C====                                 ==================================
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_CCSPRS(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL,FORM_A,
     *                     BMATRX,NDBROW,NDBCOL,LDBROW,LDBCOL,FORM_B,
     *                     CMATRX,NDCROW,NDCCOL,LDCROW,LDCCOL,EPSMUL)
C
      CHARACTER
     *          FORM_A*1,FORM_B*1
      COMPLEX
     *          ZERO_C,T
      COMPLEX
     *          AMATRX(1:NDAROW,1:NDACOL),
     *          BMATRX(1:NDBROW,1:NDBCOL),
     *          CMATRX(1:NDCROW,1:NDCCOL)
C
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
      DATA
     *     ZERO_C / (0,0) /
C
C=======================================================================
C
C     MESSAGE FOR THE USERS: This general purpose routine has been
C     tested on  the complex 500*500 matrices with two variants of
C     sub-dimensioning:
C
C     a. Square Matrices (full)
C
C     b. Matrices every second index value corresponding  to  zero
C
C     There were the following timing properties discovered on the
C     RS6000 in Strasbourg (times in seconds):
C
C                            CCSPRS (KAPF)  Option O3   F06ZAF(NAG)
C
C    1000*1000(full  matrix)      86.59        92.12       32.68
C                                  9.00 (if's)  1.30
C    1000*1000(500*500 used)      11.31        11.88        4.28
C
C    1000*1000(every secnd0)      22.62        24.28       44.52
C
C     500*500 (full  matrix)      10.5         11.80        4.18
C
C     500*500 (250*250 used)       1.50                     0.50
C
C     500*500 (every secnd0)       3.02                     6.10
C
C     250*250 (250*250 used)       1.50                     0.56
C
C=======================================================================
C
C     CONCLUSION:    CCSPRS IS STRONGLY ADVISABLE FOR BIG MATRICES
C     ==========     WITH MANY ZEROS. FOR SMALLER MATRICES YOU ARE
C                    BETTER OFF WITH THE NAG.  IN OTHER WORDS:
C                    FULL MATRICES 3 TIMES FASTER WITH  N A G
C                    HALF-MATRCES 2 TIMES FASTER WITH  CCSPRS
C
C     Because of this, the subroutine originally called  CCMULT is
C     here called CCFULL  (multiplication  of two complex matrices
C     of the SPARSE (SPRS) structure
C
C=======================================================================
C
C     This subroutine multiplies two complex matrices, AMATRX  and
C     BMATRX,  (or their transforms  -  see below) and stores  the
C     result in  matrix CMATRX
C
C     Input parameters:
C     ================
C
C     AMATRX          - the first matrix to be multiplied
C
C     NDAROW,NDACOL   - its physical dimensions identical to those
C                       declared in the caller program
C
C     LDAROW,LDACOL   - its actual dimensions
C
C     FORM_A          - its form to be used when multiplying
C                       (see below)
C
C     BMATRX          - second matrix to be multiplied
C     NDBROW,NDBCOL   - its physical dimensions identical to those
C                       declared in the caller program
C     LDBROW,LDBCOL   - its actual dimensions
C     FORM_B          - its form to be used (see below)
C
C     CMATRX          - result of the multiplication
C     NDCROW,NDCCOL   - its physical dimensions identical to those
C                       declared in the caller program
C     LDCROW,LDCCOL   - its actual dimensions
C
C     For rectangular matrices the dimensions of the type LD{} are
C     verified by the program depending on the values of  'FORM_A'
C     and 'FORM_B'; experience demonstrates, that the users have a
C     problem here.  Unfortunately an automatic test verifies only
C     about a half of the possible erroneous calls
C
C     NOTE:  Not all matrix dimension are actually needed and used;
C     the only purpose  of their presence  in the argument list is
C     to provide the information helpful for the consistency check,
C     just to be sure that the user knows what he/she is doing ....
C
C     One-character variables FORM_A and FORM_B indicate what form
C     of a given matrix you want to multiply.  Their meaning is as
C     follows:
C
C            'N'  -  just the matrix (as itis);
C            'T'  -  its transpose;
C            'Z'  -  its complex conjugation;
C            'C'  -  its hermitian conjugation ('H' also works);
C
C     Therefore e.g. FORM_A='C' and form_b='Z'  implies the action
C                                 +   *
C                            C = A x B
C
C=======================================================================
C
C     Remarks:  Below the values of LDIMRC, LDIMMM, and LDIMCC will
C     be determined. They are defined as follows:
C
C     .............................................................
C
C                            A C T U A L  DIMENSIONS
C
C       FORM_A FORM_B        AMATRX           BMATRX
C      ..................................................
C
C       'N/Z'  'N/Z'    LDIMRC x LDIMMM  LDIMMM x LDIMCC
C
C       'N/Z'  'T/C'    LDIMRC x LDIMMM  LDIMCC x LDIMMM
C
C       'T/C'  'N/Z'    LDIMMM x LDIMRC  LDIMMM x LDIMCC
C
C       'T/C'  'T/C'    LDIMMM x LDIMRC  LDIMCC x LDIMMM
C
C
C=======================================================================
C                                                 T.R. Werner, Feb 1997
C=======================================================================
C                         Present form by Dudek and Schunck, March 2000
C=======================================================================
C
      IF (LDAROW.GT.NDAROW.OR.LDACOL.GT.NDACOL) THEN
C
          WRITE(NOUTPT,'(/,''LDAROW='',I4,'' AND NDAROW='',I4,/,
     *                     ''LDACOL='',I4,'' AND NDACOL='',I4,/,
     *                     ''STOP from CCSPRS'')')
     *
     *                       LDAROW,NDAROW,LDACOL,NDACOL
C
          STOP 'Dimensions in CCSPRS were incorrect <<= '
C
      END IF
C
      IF (LDBROW.GT.NDBROW.OR.LDBCOL.GT.NDBCOL) THEN
C
          WRITE(NOUTPT,'(/,''LDBROW='',I4,'' AND NDBROW='',I4,/,
     *                     ''LDBCOL='',I4,'' AND NDBCOL='',I4,/,
     *                     ''STOP from CCSPRS'')')
     *
     *                       LDBROW,NDBROW,LDBCOL,NDBCOL
C
          STOP 'Dimensions in CCSPRS were incorrect <<= '
C
      END IF
C
      IF (LDCROW.GT.NDCROW.OR.LDCCOL.GT.NDCCOL) THEN
C
          WRITE(NOUTPT,'(/,''LDCROW='',I4,'' AND NDCROW='',I4,/,
     *                     ''LDCCOL='',I4,'' AND NDCCOL='',I4,/,
     *                     ''STOP from CCSPRS'')')
     *
     *                       LDCROW,NDCROW,LDCCOL,NDCCOL
C
          STOP 'Dimensions in CCSPRS were incorrect <<= '
C
      END IF
C
C=======================================================================
C     Setting a uniform notation for the 'FORMAT' of the operations
C=======================================================================
C
      IF (FORM_A.EQ.'n') FORM_A='N'
      IF (FORM_A.EQ.'t') FORM_A='T'
      IF (FORM_A.EQ.'z') FORM_A='Z'
      IF (FORM_A.EQ.'c') FORM_A='C'
      IF (FORM_A.EQ.'h') FORM_A='C'
      IF (FORM_A.EQ.'H') FORM_A='C'
C
      IF (FORM_B.EQ.'n') FORM_B='N'
      IF (FORM_B.EQ.'t') FORM_B='T'
      IF (FORM_B.EQ.'z') FORM_B='Z'
      IF (FORM_B.EQ.'c') FORM_B='C'
      IF (FORM_B.EQ.'h') FORM_B='C'
      IF (FORM_B.EQ.'H') FORM_B='C'
C
C=======================================================================
C
      IF (FORM_A.EQ.'N'.OR.FORM_A.EQ.'Z') THEN
          LDIMRC=LDAROW
          LDIMM1=LDACOL
      ELSE
          LDIMRC=LDACOL
          LDIMM1=LDAROW
      END IF
C
C=======================================================================
C
      IF (FORM_B.EQ.'N'.OR.FORM_B.EQ.'Z') THEN
          LDIMCC=LDBCOL
          LDIMM2=LDBROW
      ELSE
          LDIMCC=LDBROW
          LDIMM2=LDBCOL
      END IF
C
C=======================================================================
C
      IF (LDIMM1.NE.LDIMM2) THEN
C
          WRITE(NOUTPT,'(/,''The middle indice in CCSPRS'',
     *                     '' must be equal'',/,
     *                     ''LDIMM1='',I4,
     *                     '' should agree with LDIMM2='',I4,/)')
     *                                   LDIMM1,LDIMM2
C
          STOP 'Wrong middle index in CCSPRS <<= '
      ELSE
          LDIMMM=LDIMM1
      END IF
C
C=======================================================================
C
      IF (LDIMRC.NE.LDCROW.OR.LDIMCC.NE.LDCCOL) THEN
C
          WRITE(NOUTPT,'(/,''The indices LDIMRC='',I4,
     *                     '' and LDCROW='',I4,
     *                     '' must be equal in CCSPRS'')') LDIMRC,LDCROW
C
          WRITE(NOUTPT,'(/,''The indices LDIMCC='',I4,
     *                     '' and LDCCOL='',I4,
     *                     '' must be equal in CCSPRS'')') LDIMCC,LDCCOL
C
          STOP 'Mismatch in CCSPRS <<= '
C
      END IF
C
C=======================================================================
C
      IF (FORM_A.EQ.'N'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C
C          CMATRX = AMATRX x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(M,L)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M) * T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'N'.AND.FORM_B.EQ.'Z') THEN
C
C=======================================================================
C                         *
C          CMATRX = AMATRX x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=CONJG(BMATRX(M,L))
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M)*T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'N'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                                  T
C          CMATRX = AMATRX x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(L,M)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M)*T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'N'.AND.FORM_B.EQ.'C') THEN
C
C=======================================================================
C                                  +
C          CMATRX = AMATRX x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=CONJG(BMATRX(L,M))
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M)*T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'Z'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C                         *
C          CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(M,L)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + CONJG(AMATRX(K,M))*T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'Z'.AND.FORM_B.EQ.'Z') THEN
C
C=======================================================================
C                         *         *
C          CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=CONJG(BMATRX(M,L))
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + CONJG(AMATRX(K,M))*T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'Z'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                         *         T
C          CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(L,M)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + CONJG(AMATRX(K,M))*T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'Z'.AND.FORM_B.EQ.'C') THEN
C
C=======================================================================
C                         *         +
C          CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=CONJG(BMATRX(L,M))
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + CONJG(AMATRX(K,M))*T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'T'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C                             T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                        T                  T
C          Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
           CALL WS_TRANSP(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(M,L)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M)*T
                     END DO
                 END IF
              END DO
           END DO
C
           CALL WS_TRANSP(AMATRX,NDAROW,NDACOL,LDACOL,LDAROW)
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'T'.AND.FORM_B.EQ.'Z') THEN
C
C=======================================================================
C                             T         *
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                        +                 T
C          Here, we first multiply BMATRX x AMATRX = CMATRX
C
C=======================================================================
C
           CALL WS_TRANSP(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=CONJG(BMATRX(M,L))
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M)*T
                     END DO
                 END IF
              END DO
           END DO
C
           CALL WS_TRANSP(AMATRX,NDAROW,NDACOL,LDACOL,LDAROW)
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'T'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                             T         T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                                           T
C          Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
         DO L=1,LDIMCC
            DO K=1,LDIMRC
               CMATRX(K,L)=ZERO_C
            END DO
            DO M=1,LDIMMM
               T=AMATRX(M,L)
               IF (T.NE.ZERO_C) THEN
                  DO K=1,LDIMRC
                     CMATRX(K,L) = CMATRX(K,L) + BMATRX(K,M)*T
                  END DO
               END IF
            END DO
         END DO
C
         CALL WS_TRANSP(CMATRX,NDCROW,NDCCOL,LDIMRC,LDIMCC)
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'T'.AND.FORM_B.EQ.'C') THEN
C
C=======================================================================
C                             T         +
C              CMATRX = AMATRX  X BMATRX
C
C=======================================================================
C                                        *                  T
C          Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
         DO L=1,LDIMCC
            DO K=1,LDIMRC
               CMATRX(K,L)=ZERO_C
            END DO
            DO M=1,LDIMMM
               T=AMATRX(M,L)
               IF (ABS(T).GT.EPSMUL) THEN
                  DO K=1,LDIMRC
                     CMATRX(K,L) = CMATRX(K,L) + CONJG(BMATRX(K,M))*T
                  END DO
               END IF
            END DO
         END DO
C
         CALL WS_TRANSP(CMATRX,NDCROW,NDCCOL,LDIMRC,LDIMCC)
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'C'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C                             +
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                        T        *         T
C          Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
           CALL WS_TRANCJ(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(M,L)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M)*T
                     END DO
                 END IF
              END DO
           END DO
C
           CALL WS_TRANCJ(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'C'.AND.FORM_B.EQ.'Z') THEN
C
C=======================================================================
C                             +         *
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                        T        *         T
C          Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
           CALL WS_TRANCJ(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=CONJG(BMATRX(M,L))
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M)*T
                     END DO
                 END IF
              END DO
           END DO
C
           CALL WS_TRANCJ(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'C'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                             +         T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                                 *         T
C          Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
         DO L=1,LDIMCC
            DO K=1,LDIMRC
               CMATRX(K,L)=ZERO_C
            END DO
            DO M=1,LDIMMM
               T=CONJG(AMATRX(M,L))
               IF (ABS(T).GT.EPSMUL) THEN
                  DO K=1,LDIMRC
                     CMATRX(K,L) = CMATRX(K,L) + BMATRX(K,M)*T
                  END DO
               END IF
            END DO
         END DO
C
         CALL WS_TRANSP(CMATRX,NDCROW,NDCCOL,LDIMRC,LDIMCC)
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'C'.AND.FORM_B.EQ.'C') THEN
C
C=======================================================================
C                             +         +
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                                          +
C         Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             DO K=1,LDIMRC
                CMATRX(K,L)=ZERO_C
             END DO
             DO M=1,LDIMMM
                T=AMATRX(M,L)
                IF (ABS(T).GT.EPSMUL) THEN
                    DO K=1,LDIMRC
                       CMATRX(K,L) = CMATRX(K,L) + BMATRX(K,M)*T
                    END DO
                END IF
             END DO
          END DO
C
          CALL WS_TRANCJ(CMATRX,NDCROW,NDCCOL,LDIMRC,LDIMCC)
C
C=======================================================================
C
      ELSE
C
          STOP 'Ill-defined FORM in CCSPRS <<= '
C
      END IF
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_CRSPRS(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL,FORM_A,
     *                     BMATRX,NDBROW,NDBCOL,LDBROW,LDBCOL,FORM_B,
     *                     CMATRX,NDCROW,NDCCOL,LDCROW,LDCCOL,EPSMUL)
C
      CHARACTER
     *          FORM_A*1,FORM_B*1
      COMPLEX
     *          ZERO_C,T
      COMPLEX
     *          AMATRX(1:NDAROW,1:NDACOL),
     *          CMATRX(1:NDCROW,1:NDCCOL)
      DIMENSION
     *          BMATRX(1:NDBROW,1:NDBCOL)
C
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
      DATA
     *     ZERO_C / (0,0) /
C
C=======================================================================
C
C     MESSAGE FOR THE USERS: This general purpose routine has been
C     tested on  the complex 500*500 matrices with two variants of
C     sub-dimensioning:
C
C     a. Square Matrices (full)
C
C     b. Matrices every second index value corresponding  to  zero
C
C     There were the following timing properties discovered on the
C     RS6000 in Strasbourg (times in seconds):
C
C                            CCSPRS (KAPF)  Option O3   F06ZAF(NAG)
C
C     1000*1000(full  matrix)      86.59        92.12        32.68
C                                   9.00 (if's)  1.30
C     1000*1000(500*500 used)      11.31        11.88         4.28
C
C     1000*1000(every secnd0)      22.62        24.28        44.52
C
C      500*500 (full  matrix)      10.5         11.80         4.18
C
C      500*500 (250*250 used)       1.50                      0.50
C
C      500*500 (every secnd0)       3.02                      6.10
C
C      250*250 (250*250 used)       1.50                      0.56
C
C=======================================================================
C
C     CONCLUSION:    CRSPRS IS STRONGLY ADVISABLE FOR BIG MATRICES
C     ==========     WITH MANY ZEROS. FOR SMALLER MATRICES YOU ARE
C                    BETTER OFF WITH THE NAG.  IN OTHER WORDS:
C                    FULL MATRICES 3 TIMES FASTER WITH  N A G
C                    HALF-MATRCES 2 TIMES FASTER WITH  CRSPRS
C
C     Because of this,  the subroutine originally called CCMULT is
C     here called CCFULL   (multiplication of two complex matrices
C     of the SPARSE (SPRS) structure
C
C=======================================================================
C
C     This subroutine multiplies two complex matrices, AMATRX  and
C     BMATRX,  (or their transforms  -  see below) and stores  the
C     result in  matrix CMATRX
C
C     Input parameters:
C     ================
C
C     AMATRX          - the first matrix to be multiplied
C
C     NDAROW,NDACOL   - its physical dimensions identical to those
C                       declared in the caller program
C
C     LDAROW,LDACOL   - its actual dimensions
C
C     FORM_A          - its form to be used when multiplying
C                       (see below)
C
C     BMATRX          - second matrix to be multiplied
C     NDBROW,NDBCOL   - its physical dimensions identical to those
C                       declared in the caller program
C     LDBROW,LDBCOL   - its actual dimensions
C     FORM_B          - its form to be used (see below)
C
C     CMATRX          - result of the multiplication
C     NDCROW,NDCCOL   - its physical dimensions identical to those
C                       declared in the caller program
C     LDCROW,LDCCOL   - its actual dimensions
C
C     For rectangular matrices the dimensions of the type LD{} are
C     verified by the program depending on the values of  'FORM_A'
C     and 'FORM_B'; experience demonstrates, that the users have a
C     problem here.  Unfortunately an automatic test verifies only
C     about a half of the possible erroneous calls
C
C     NOTE:  Not all matrix dimension are actually needed and used;
C     the only purpose  of their presence  in the argument list is
C     to provide the information helpful for the consistency check,
C     just to be sure that the user knows what he/she is doing ....
C
C     One-character variables FORM_A and FORM_B indicate what form
C     of a given matrix you want to multiply.  Their meaning is as
C     follows:
C
C            'N'  -  just the matrix (as itis);
C            'T'  -  its transpose;
C            'Z'  -  its complex conjugation;
C            'C'  -  its hermitian conjugation ('H' also works);
C
C     Therefore e.g. FORM_A='C' and form_b='Z'  implies the action
C                                 +   *
C                            C = A x B
C
C=======================================================================
C
C     Remarks: Below the values of LDIMRC, LDIMMM, and LDIMCC will
C     be determined. They are defined as follows:
C
C     ............................................................
C
C                            A C T U A L  DIMENSIONS
C
C       FORM_A FORM_B        AMATRX           BMATRX
C     ...................................................
C
C       'N/Z'  'N/Z'    LDIMRC x LDIMMM  LDIMMM x LDIMCC
C
C       'N/Z'  'T/C'    LDIMRC x LDIMMM  LDIMCC x LDIMMM
C
C       'T/C'  'N/Z'    LDIMMM x LDIMRC  LDIMMM x LDIMCC
C
C       'T/C'  'T/C'    LDIMMM x LDIMRC  LDIMCC x LDIMMM
C
C
C=======================================================================
C                                                 T.R. Werner, Feb 1997
C=======================================================================
C                         Present form by Dudek and Schunck, March 2000
C=======================================================================
C
      IF (LDAROW.GT.NDAROW.OR.LDACOL.GT.NDACOL) THEN
C
          WRITE(NOUTPT,'(/,''LDAROW='',I4,'' AND NDAROW='',I4,/,
     *                     ''LDACOL='',I4,'' AND NDACOL='',I4,/,
     *                     ''STOP from CCSPRS'')')
     *
     *                       LDAROW,NDAROW,LDACOL,NDACOL
C
          STOP 'Dimensions in CRSPRS were incorrect <<= '
C
      END IF
C
      IF (LDBROW.GT.NDBROW.OR.LDBCOL.GT.NDBCOL) THEN
C
          WRITE(NOUTPT,'(/,''LDBROW='',I4,'' AND NDBROW='',I4,/,
     *                     ''LDBCOL='',I4,'' AND NDBCOL='',I4,/,
     *                     ''STOP from CCSPRS'')')
     *
     *                       LDBROW,NDBROW,LDBCOL,NDBCOL
C
          STOP 'Dimensions in CRSPRS were incorrect <<= '
C
      END IF
C
      IF (LDCROW.GT.NDCROW.OR.LDCCOL.GT.NDCCOL) THEN
C
          WRITE(NOUTPT,'(/,''LDCROW='',I4,'' AND NDCROW='',I4,/,
     *                     ''LDCCOL='',I4,'' AND NDCCOL='',I4,/,
     *                     ''STOP from CCSPRS'')')
     *
     *                       LDCROW,NDCROW,LDCCOL,NDCCOL
C
          STOP 'Dimensions in CRSPRS were incorrect <<= '
C
      END IF
C
C=======================================================================
C     Setting a uniform notation for the 'FORMAT' of the operations
C=======================================================================
C
      IF (FORM_A.EQ.'n') FORM_A='N'
      IF (FORM_A.EQ.'t') FORM_A='T'
      IF (FORM_A.EQ.'z') FORM_A='Z'
      IF (FORM_A.EQ.'c') FORM_A='C'
      IF (FORM_A.EQ.'h') FORM_A='C'
      IF (FORM_A.EQ.'H') FORM_A='C'
C
      IF (FORM_B.EQ.'n') FORM_B='N'
      IF (FORM_B.EQ.'t') FORM_B='T'
      IF (FORM_B.EQ.'z') FORM_B='N'
      IF (FORM_B.EQ.'c') FORM_B='T'
      IF (FORM_B.EQ.'h') FORM_B='T'
      IF (FORM_B.EQ.'H') FORM_B='T'
C
C=======================================================================
C
      IF (FORM_A.EQ.'N'.OR.FORM_A.EQ.'Z') THEN
          LDIMRC=LDAROW
          LDIMM1=LDACOL
      ELSE
          LDIMRC=LDACOL
          LDIMM1=LDAROW
      END IF
C
C=======================================================================
C
      IF (FORM_B.EQ.'N') THEN
          LDIMCC=LDBCOL
          LDIMM2=LDBROW
      ELSE
          LDIMCC=LDBROW
          LDIMM2=LDBCOL
      END IF
C
C=======================================================================
C
      IF (LDIMM1.NE.LDIMM2) THEN
C
          WRITE(NOUTPT,'(/,''The middle indice in CRSPRS'',
     *                     '' must be equal'',/,
     *                     ''LDIMM1='',I4,
     *                     '' should agree with LDIMM2='',I4,/)')
     *                       LDIMM1,LDIMM2
C
          STOP 'Wrong middle index in CRSPRS <<= '
      ELSE
          LDIMMM=LDIMM1
      END IF
C
C=======================================================================
C
      IF (LDIMRC.NE.LDCROW.OR.LDIMCC.NE.LDCCOL) THEN
C
          WRITE(NOUTPT,'(/,''The indices LDIMRC='',I4,
     *                     '' and LDCROW='',I4,
     *                     '' must be equal in CRSPRS'')')
     *                            LDIMRC,LDCROW
C
          WRITE(NOUTPT,'(/,''The indices LDIMCC='',I4,
     *                     '' and LDCCOL='',I4,
     *                     '' must be equal in CRSPRS'')')
     *                            LDIMCC,LDCCOL
C
          STOP 'Mismatch in CRSPRS <<= '
C
      END IF
C
C=======================================================================
C
      IF (FORM_A.EQ.'N'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C
C              CMATRX = AMATRX x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(M,L)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M) * T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'N'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                                      T
C              CMATRX = AMATRX x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(L,M)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M)*T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'Z'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C                             *
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(M,L)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + CONJG(AMATRX(K,M))*T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'Z'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                             *         T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(L,M)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + CONJG(AMATRX(K,M))*T
                     END DO
                 END IF
              END DO
           END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'T'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C                             T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                        T                  T
C          Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
           CALL WS_TRANSP(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(M,L)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M)*T
                     END DO
                 END IF
              END DO
           END DO
C
           CALL WS_TRANSP(AMATRX,NDAROW,NDACOL,LDACOL,LDAROW)
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'T'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                             T         T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                                           T
C          Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
         DO L=1,LDIMCC
            DO K=1,LDIMRC
               CMATRX(K,L)=ZERO_C
            END DO
            DO M=1,LDIMMM
               T=AMATRX(M,L)
               IF (T.NE.ZERO_C) THEN
                  DO K=1,LDIMRC
                     CMATRX(K,L) = CMATRX(K,L) + BMATRX(K,M)*T
                  END DO
               END IF
            END DO
         END DO
C
         CALL WS_TRANSP(CMATRX,NDCROW,NDCCOL,LDIMRC,LDIMCC)
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'C'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C                             +
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                        T        *         T
C          Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
           CALL WS_TRANCJ(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
           DO L=1,LDIMCC
              DO K=1,LDIMRC
                 CMATRX(K,L)=ZERO_C
              END DO
              DO M=1,LDIMMM
                 T=BMATRX(M,L)
                 IF (ABS(T).GT.EPSMUL) THEN
                     DO K=1,LDIMRC
                        CMATRX(K,L) = CMATRX(K,L) + AMATRX(K,M)*T
                     END DO
                 END IF
              END DO
           END DO
C
           CALL WS_TRANCJ(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'C'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                             +         T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C                                                 *         T
C          Here, we first multiply BMATRX x AMATRX  = CMATRX
C
C=======================================================================
C
         DO L=1,LDIMCC
            DO K=1,LDIMRC
               CMATRX(K,L)=ZERO_C
            END DO
            DO M=1,LDIMMM
               T=CONJG(AMATRX(M,L))
               IF (ABS(T).GT.EPSMUL) THEN
                  DO K=1,LDIMRC
                     CMATRX(K,L) = CMATRX(K,L) + BMATRX(K,M)*T
                  END DO
               END IF
            END DO
         END DO
C
         CALL WS_TRANSP(CMATRX,NDCROW,NDCCOL,LDIMRC,LDIMCC)
C
C=======================================================================
C
      ELSE
C
          STOP 'Ill-defined FORM in CCSPRS <<= '
C
      END IF
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DIAMUL(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL,FORM_A,
     *                     BMATRX,NDBROW,NDBCOL,LDBROW,LDBCOL,FORM_B,
     *                                   C_VECT,NDDIMC,LDDIMC,EPSMUL)
C
      CHARACTER
     *          FORM_A*1,FORM_B*1
      COMPLEX
     *          ZERO_C,T
      COMPLEX
     *          AMATRX(1:NDAROW,1:NDACOL),
     *          BMATRX(1:NDBROW,1:NDBCOL)
      COMPLEX
     *          C_VECT(1:NDDIMC)
C
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
      COMMON
     *       /DIAMUL_DDWORK/ IDDCNT
C
      DATA
     *     ZERO_C / (0,0) /
C
C=======================================================================
C
C     MESSAGE FOR THE USERS: This general purpose routine has been
C     tested on  the complex 300*300 matrices with two variants of
C     sub-dimensioning:
C
C     a. Square Matrices (full)
C
C     b. Matrices every second index value corresponding  to  zero
C
C     There were the following timing properties discovered on the
C     RS6000 in Strasbourg (times in seconds):
C
C                                 DIAMUL      DIAMUL (no if)
C
C     300*300 (full,    x100)       246.        238
C
C     300*300 (every 2, x100)        65.3
C
C     100*100 (full,    x100)        10.4
C
C     100*100 (every 2, x100)         2.8
C
C=======================================================================
C
C     CONCLUSION: DIAMUL IS STRONGLY ADVISABLE FOR BIG MATRICES
C     ==========  WITH MANY ZEROS
C
C=======================================================================
C
C     This subroutine calculates  diagonal of the product of matries
C     AMATRX and BMATRX, or their transforms - see below, and stores
C     the result in vector C_VECT
C
C     Input parameters:
C     ================
C
C     AMATRX          - the first matrix to be multiplied
C
C     NDAROW,NDACOL   - its physical dimensions identical to those
C                       declared in the caller program
C
C     LDAROW,LDACOL   - its actual dimensions
C
C     FORM_A          - its form to be used when multiplying
C                       (see below)
C
C     BMATRX          - second matrix to be multiplied
C     NDBROW,NDBCOL   - its physical dimensions identical to those
C                       declared in the caller program
C     LDBROW,LDBCOL   - its actual dimensions
C     FORM_B          - its form to be used (see below)
C
C     C_VECT          - result for the diagonal elements
C     NDDIMC          - its physical dimensions identical to those
C                       declared in the caller program
C     LDDIMC          - its actual dimensions
C
C     For rectangular matrices the dimensions of the type LD{} are
C     verified by the program depending on the values of  'FORM_A'
C     and 'FORM_B'; experience demonstrates, that the users have a
C     problem here.  Unfortunately an automatic test verifies only
C     about a half of the possible erroneous calls
C
C     NOTE:  Not all matrix dimension are actually needed and used;
C     the only purpose  of their presence  in the argument list is
C     to provide the information helpful for the consistency check,
C     just to be sure that the user knows what he/she is doing ....
C
C     One-character variables FORM_A and FORM_B indicate what form
C     of a given matrix you want to multiply.  Their meaning is as
C     follows:
C
C            'N'  -  just the matrix (as itis);
C            'T'  -  its transpose;
C            'Z'  -  its complex conjugation;
C            'C'  -  its hermitian conjugation ('H' also works);
C
C     Therefore e.g. FORM_A='C' and form_b='Z'  implies the action
C                                 +   *
C                            C = A x B
C
C=======================================================================
C
C     Remarks:  Below the values of LDIMRC, LDIMMM, and LDIMCC will
C     be determined. They are defined as follows:
C
C     .............................................................
C
C                            A C T U A L  DIMENSIONS
C
C       FORM_A FORM_B        AMATRX           BMATRX
C      ..................................................
C
C       'N/Z'  'N/Z'    LDIMRC x LDIMMM  LDIMMM x LDIMCC
C
C       'N/Z'  'T/C'    LDIMRC x LDIMMM  LDIMCC x LDIMMM
C
C       'T/C'  'N/Z'    LDIMMM x LDIMRC  LDIMMM x LDIMCC
C
C       'T/C'  'T/C'    LDIMMM x LDIMRC  LDIMCC x LDIMMM
C
C
C=======================================================================
C                                                 T.R. Werner, Feb 1997
C=======================================================================
C                         Present form by Dudek and Schunck, March 2000
C=======================================================================
C
      IF (LDAROW.GT.NDAROW.OR.LDACOL.GT.NDACOL) THEN
C
          WRITE(NOUTPT,'(/,''LDAROW='',I4,'' AND NDAROW='',I4,/,
     *                     ''LDACOL='',I4,'' AND NDACOL='',I4,/,
     *                     ''STOP from DIAMUL'')')
     *
     *                       LDAROW,NDAROW,LDACOL,NDACOL
C
          STOP 'Dimensions in DIAMUL were incorrect <<= '
C
      END IF
C
      IF (LDBROW.GT.NDBROW.OR.LDBCOL.GT.NDBCOL) THEN
C
          WRITE(NOUTPT,'(/,''LDBROW='',I4,'' AND NDBROW='',I4,/,
     *                     ''LDBCOL='',I4,'' AND NDBCOL='',I4,/,
     *                     ''STOP from DIAMUL'')')
     *
     *                       LDBROW,NDBROW,LDBCOL,NDBCOL
C
          STOP 'Dimensions in DIAMUL were incorrect <<= '
C
      END IF
C
      IF (LDDIMC.GT.NDDIMC) THEN
C
          WRITE(NOUTPT,'(/,''LDDIMC='',I4,'' AND NDDIMC='',I4,/,
     *                     ''STOP from DIAMUL'')')
     *
     *                       LDDIMC,NDDIMC
C
          STOP 'Dimensions in DIAMUL were incorrect <<= '
C
      END IF
C
C=======================================================================
C         Setting a uniform notation for the 'FORMAT' of the operations
C=======================================================================
C
      IF (FORM_A.EQ.'n') FORM_A='N'
      IF (FORM_A.EQ.'t') FORM_A='T'
      IF (FORM_A.EQ.'z') FORM_A='Z'
      IF (FORM_A.EQ.'c') FORM_A='C'
      IF (FORM_A.EQ.'h') FORM_A='C'
      IF (FORM_A.EQ.'H') FORM_A='C'
C
      IF (FORM_B.EQ.'n') FORM_B='N'
      IF (FORM_B.EQ.'t') FORM_B='T'
      IF (FORM_B.EQ.'z') FORM_B='Z'
      IF (FORM_B.EQ.'c') FORM_B='C'
      IF (FORM_B.EQ.'h') FORM_B='C'
      IF (FORM_B.EQ.'H') FORM_B='C'
C
C=======================================================================
C
      IF (FORM_A.EQ.'N'.OR.FORM_A.EQ.'Z') THEN
          LDIMRC=LDAROW
          LDIMM1=LDACOL
      ELSE
          LDIMRC=LDACOL
          LDIMM1=LDAROW
      END IF
C
C=======================================================================
C
      IF (FORM_B.EQ.'N'.OR.FORM_B.EQ.'Z') THEN
          LDIMCC=LDBCOL
          LDIMM2=LDBROW
      ELSE
          LDIMCC=LDBROW
          LDIMM2=LDBCOL
      END IF
C
C=======================================================================
C
      IF (LDIMM1.NE.LDIMM2) THEN
C
          WRITE(NOUTPT,'(/,''The middle indice in DIAMUL'',
     *                     '' must be equal'',/,
     *                     ''LDIMM1='',I4,
     *                     '' should agree with LDIMM2='',I4,/)')
     *                                   LDIMM1,LDIMM2
C
          STOP 'Wrong middle index in DIAMUL <<= '
      ELSE
          LDIMMM=LDIMM1
      END IF
C
C=======================================================================
C
      IF (LDIMRC.NE.LDDIMC) THEN
C
          WRITE(NOUTPT,'(''Verify: LDIMRC='',I4,
     *                   '' not equal LDDIMC='',I4)')
     *                         LDIMRC,LDDIMC
C
          STOP 'Mismatch in DIAMUL <<= '
C
      END IF
C
C=======================================================================
C
      IF (IDDCNT.LE.1) THEN
C
          IF (IWRITE.GT.2)
     *
     *    WRITE(NOUTPT,'(/,80(''|''),/,''||'',76X,''||'',/,
     *                ''|| To gain on c.p.u. your matrix '',
     *                ''multiplications are cut on the level '',E7.1,
     *                '' ||'',/,
     *                ''||                                      '',
     *                ''                               ====== ||'',/,
     *                ''|| DID YOU VERIFY THAT THIS IS ALLOWED'',
     *                '' IN YOUR PROBLEM HERE ???        DIAMUL ||'',/,
     *                ''||'',76X,''||'',/,80(''|''))')   EPSMUL
C
          IDDCNT=IDDCNT+1
C
      END IF
C
C=======================================================================
C
      IF (FORM_A.EQ.'N'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C
C         CMATRX = AMATRX x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             C_VECT(L)=ZERO_C
             DO M=1,LDIMMM
                T=BMATRX(M,L)
                IF (ABS(T).GT.EPSMUL) THEN
                    C_VECT(L) = C_VECT(L) + AMATRX(L,M)*T
                END IF
             END DO
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'N'.AND.FORM_B.EQ.'Z') THEN
C
C=======================================================================
C                             *
C              CMATRX = AMATRX x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             C_VECT(L)=ZERO_C
             DO M=1,LDIMMM
                T=CONJG(BMATRX(M,L))
                IF (ABS(T).GT.EPSMUL) THEN
                    C_VECT(L) = C_VECT(L) + AMATRX(L,M)*T
                END IF
             END DO
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'N'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                                      T
C              CMATRX = AMATRX x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             C_VECT(L)=ZERO_C
             DO M=1,LDIMMM
                T=BMATRX(L,M)
                IF (ABS(T).GT.EPSMUL) THEN
                    C_VECT(L) = C_VECT(L) + AMATRX(L,M)*T
                END IF
             END DO
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'N'.AND.FORM_B.EQ.'C') THEN
C
C=======================================================================
C                                      +
C              CMATRX = AMATRX x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             C_VECT(L)=ZERO_C
             DO M=1,LDIMMM
                T=CONJG(BMATRX(L,M))
                IF (ABS(T).GT.EPSMUL) THEN
                    C_VECT(L) = C_VECT(L) + AMATRX(L,M)*T
                END IF
             END DO
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'Z'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C                             *
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             C_VECT(L)=ZERO_C
             DO M=1,LDIMMM
                T=BMATRX(M,L)
                IF (ABS(T).GT.EPSMUL) THEN
                    C_VECT(L) = C_VECT(L) + CONJG(AMATRX(L,M))*T
                END IF
             END DO
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'Z'.AND.FORM_B.EQ.'Z') THEN
C
C=======================================================================
C                             *         *
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             C_VECT(L)=ZERO_C
             DO M=1,LDIMMM
                T=CONJG(BMATRX(M,L))
                IF (ABS(T).GT.EPSMUL) THEN
                    C_VECT(L) = C_VECT(L) + CONJG(AMATRX(L,M))*T
                END IF
             END DO
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'Z'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                             *         T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             C_VECT(L)=ZERO_C
             DO M=1,LDIMMM
                T=BMATRX(L,M)
                IF (ABS(T).GT.EPSMUL) THEN
                    C_VECT(L) = C_VECT(L) + CONJG(AMATRX(L,M))*T
                END IF
             END DO
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'Z'.AND.FORM_B.EQ.'C') THEN
C
C=======================================================================
C                             *         +
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             C_VECT(L)=ZERO_C
             DO M=1,LDIMMM
                T=CONJG(BMATRX(L,M))
                IF (ABS(T).GT.EPSMUL) THEN
                    C_VECT(L) = C_VECT(L) + CONJG(AMATRX(L,M))*T
                END IF
             END DO
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'T'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C                             T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             T=ZERO_C
             DO M=1,LDIMMM
                T = T + AMATRX(M,L)*BMATRX(M,L)
             END DO
             C_VECT(L)=T
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'T'.AND.FORM_B.EQ.'Z') THEN
C
C=======================================================================
C                             T         *
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             T=ZERO_C
             DO M=1,LDIMMM
                T = T + AMATRX(M,L)*CONJG(BMATRX(M,L))
             END DO
             C_VECT(L)=T
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'T'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                             T         T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             T=ZERO_C
             DO M=1,LDIMMM
                T = T + AMATRX(M,L)*BMATRX(L,M)
             END DO
             C_VECT(L)=T
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'T'.AND.FORM_B.EQ.'C') THEN
C
C=======================================================================
C                             T         +
C              CMATRX = AMATRX  X BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             T=ZERO_C
             DO M=1,LDIMMM
                T = T + AMATRX(M,L)*CONJG(BMATRX(L,M))
             END DO
             C_VECT(L)=T
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'C'.AND.FORM_B.EQ.'N') THEN
C
C=======================================================================
C                             +
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             T=ZERO_C
             DO M=1,LDIMMM
                T = T + CONJG(AMATRX(M,L))*BMATRX(M,L)
             END DO
             C_VECT(L)=T
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'C'.AND.FORM_B.EQ.'Z') THEN
C
C=======================================================================
C                             +         *
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             T=ZERO_C
             DO M=1,LDIMMM
                T = T + CONJG(AMATRX(M,L))*CONJG(BMATRX(M,L))
             END DO
             C_VECT(L)=T
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'C'.AND.FORM_B.EQ.'T') THEN
C
C=======================================================================
C                             +         T
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             T=ZERO_C
             DO M=1,LDIMMM
                T = T + CONJG(AMATRX(M,L))*BMATRX(L,M)
             END DO
             C_VECT(L)=T
          END DO
C
C=======================================================================
C
      ELSE IF (FORM_A.EQ.'C'.AND.FORM_B.EQ.'C') THEN
C
C=======================================================================
C                             +         +
C              CMATRX = AMATRX  x BMATRX
C
C=======================================================================
C
          DO L=1,LDIMCC
             T=ZERO_C
             DO M=1,LDIMMM
                T = T + CONJG(AMATRX(M,L))*CONJG(BMATRX(L,M))
             END DO
             C_VECT(L)=T
          END DO
C
C=======================================================================
C
      ELSE
C
          STOP 'Ill-defined FORM in DDFULL <<= '
C
      END IF
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_CCFULL(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL,FORM_A,
     *                     BMATRX,NDBROW,NDBCOL,LDBROW,LDBCOL,FORM_B,
     *                     CMATRX,NDCROW,NDCCOL,LDCROW,LDCCOL,EPSMUL)
C
      CHARACTER
     *          FORM_A*1,FORM_B*1
      CHARACTER
     *          FORM0A*1,FORM0B*1
      COMPLEX
     *          AMATRX(1:NDAROW,1:NDACOL),
     *          BMATRX(1:NDBROW,1:NDBCOL),
     *          CMATRX(1:NDCROW,1:NDCCOL)
C
C=======================================================================
C     FOR REASONS EXPLAINED BELOW THIS SUBROUTINE WS_IS ADVISED ON RS2
C     FOR THE MULTIPLICATION OF FULL MATRICES; IF ONLY ~70% OR LESS
C     OF THE MATRIX ELEMENTS ARE ZERO, THEN CCSPRS IS ADISABLE
C=======================================================================
C
C     MESSAGE FOR THE USERS:  This general purpose routine has been
C     tested on the complex 1000*1000 and 500*500 matrices with two
C     variants of sub-dimensioning:
C
C     a. Square Matrices (full)
C
C     b. Matrices every second index value corresponding  to  zero
C
C     There were the following timing properties discovered on the
C     RS6000 in Strasbourg (times in seconds):
C                                                         CCFULL=
C                            CCSPRS (KAPF)  Option O3   F06ZAF(NAG)
C                                                       ESSL of IBM
C    1000*1000(full  matrix)      86.59        92.12       32.68
C                                  9.00 (if's)  1.30
C    1000*1000(500*500 used)      11.31        11.88        4.28
C
C    1000*1000(every secnd0)      22.62        24.28       44.52
C
C     500*500 (full  matrix)      10.5         11.80        4.18
C
C     500*500 (250*250 used)       1.50                     0.50
C
C     500*500 (every secnd0)       3.02                     6.10
C
C     250*250 (250*250 used)       1.50                     0.56
C
C=======================================================================
C
C     CONCLUSION: CCSPRS IS STRONGLY ADVISABLE FOR BIG MATRICES
C     ==========  WITH MANY ZEROS. FOR SMALLER MATRICES YOU ARE
C                 BETTER OFF WITH THE ESSL/NAG. IN OTHER WORDS:
C                 FULL MATRICES 3 TIMES FASTER WITH ESSL/ N A G
C                 HALF-MATRCES  2 TIMES FASTER WITH CCSPRS
C
C     Because of this, the subroutine is called here CCFULL; it
C     is just  a more convenient form  of a call to the routine
C     F06ZAF from NAG
C
C=======================================================================
C
C     This subroutine multiplies two complex matrices, AMATRX  and
C     BMATRX,  (or their transforms  -  see below) and stores  the
C     result in  matrix CMATRX
C
C     Input parameters:
C     ================
C
C     AMATRX          - the first matrix to be multiplied
C
C     NDAROW,NDACOL   - its physical dimensions identical to those
C                       declared in the caller program
C
C     LDAROW,LDACOL   - its actual dimensions
C
C     FORM_A          - its form to be used when multiplying
C                       (see below)
C
C     BMATRX          - second matrix to be multiplied
C     NDBROW,NDBCOL   - its physical dimensions identical to those
C                       declared in the caller program
C     LDBROW,LDBCOL   - its actual dimensions
C     FORM_B          - its form to be used (see below)
C
C     CMATRX          - result of the multiplication
C     NDCROW,NDCCOL   - its physical dimensions identical to those
C                       declared in the caller program
C     LDCROW,LDCCOL   - its actual dimensions
C
C     For rectangular matrices the dimensions of the type LD{} are
C     verified by the program depending on the values of  'FORM_A'
C     and 'FORM_B'; experience demonstrates, that the users have a
C     problem here.  Unfortunately an automatic test verifies only
C     about a half of the possible erroneous calls
C
C     NOTE:  Not all matrix dimension are actually needed and used;
C     the only purpose  of their presence  in the argument list is
C     to provide the information helpful for the consistency check,
C     just to be sure that the user knows what he/she is doing ....
C
C     One-character variables FORM_A and FORM_B indicate what form
C     of a given matrix you want to multiply.  Their meaning is as
C     follows:
C
C            'N'  -  just the matrix (as itis);
C            'T'  -  its transpose;
C            'C'  -  its hermitian conjugation ('H' also works);
C
C     Therefore e.g. FORM_A='C' and form_b='N'  implies the action
C                                 +
C                            C = A x B
C
C=======================================================================
C
C     Remarks:  Below the values of LDIMRC, LDIMMM, and LDIMCC will
C     be determined. They are defined as follows:
C
C     .............................................................
C
C                            A C T U A L  DIMENSIONS
C
C       FORM_A FORM_B        AMATRX           BMATRX
C     .............................................................
C
C       'T/C'  'T/C'    LDIMMM x LDIMRC  LDIMCC x LDIMMM
C
C=======================================================================
C                                                 T.R. Werner, Feb 1997
C=======================================================================
C
      IF (LDAROW.GT.NDAROW.OR.LDACOL.GT.NDACOL) THEN
C
          WRITE(6,'(/,''LDAROW='',I4,'' AND NDAROW='',I4,/,
     *                ''LDACOL='',I4,'' AND NDACOL='',I4,/,
     *                ''STOP from CCFULL'')')
     *
     *                  LDAROW,NDAROW,LDACOL,NDACOL
C
          STOP 'Dimensions in CCFULL were incorrect <<= '
C
      END IF
C
      IF (LDBROW.GT.NDBROW.OR.LDBCOL.GT.NDBCOL) THEN
C
          WRITE(6,'(/,''LDBROW='',I4,'' AND NDBROW='',I4,/,
     *                ''LDBCOL='',I4,'' AND NDBCOL='',I4,/,
     *                ''STOP from CCFULL'')')
     *
     *                  LDBROW,NDBROW,LDBCOL,NDBCOL
C
          STOP 'Dimensions in CCFULL were incorrect <<= '
C
      END IF
C
      IF (LDCROW.GT.NDCROW.OR.LDCCOL.GT.NDCCOL) THEN
C
          WRITE(6,'(/,''LDCROW='',I4,'' AND NDCROW='',I4,/,
     *                ''LDCCOL='',I4,'' AND NDCCOL='',I4,/,
     *                ''STOP from CCFULL'')')
     *
     *                  LDCROW,NDCROW,LDCCOL,NDCCOL
C
          STOP 'Dimensions in CCFULL were incorrect <<= '
C
      END IF
C
C=======================================================================
C     Setting a uniform notation for the 'FORMAT' of the operations
C=======================================================================
C
      IF (FORM_A.EQ.'n') FORM_A='N'
      IF (FORM_A.EQ.'t') FORM_A='T'
      IF (FORM_A.EQ.'z') FORM_A='Z'
      IF (FORM_A.EQ.'c') FORM_A='C'
      IF (FORM_A.EQ.'h') FORM_A='C'
      IF (FORM_A.EQ.'H') FORM_A='C'
C
      IF (FORM_B.EQ.'n') FORM_B='N'
      IF (FORM_B.EQ.'t') FORM_B='T'
      IF (FORM_B.EQ.'z') FORM_B='Z'
      IF (FORM_B.EQ.'c') FORM_B='C'
      IF (FORM_B.EQ.'h') FORM_B='C'
      IF (FORM_B.EQ.'H') FORM_B='C'
C
C=======================================================================
C
      IF (FORM_A.EQ.'N'.OR.FORM_A.EQ.'Z') THEN
          LDIMRC=LDAROW
          LDIMM1=LDACOL
      ELSE
          LDIMRC=LDACOL
          LDIMM1=LDAROW
      END IF
C
C=======================================================================
C
      IF (FORM_B.EQ.'N'.OR.FORM_B.EQ.'Z') THEN
          LDIMCC=LDBCOL
          LDIMM2=LDBROW
      ELSE
          LDIMCC=LDBROW
          LDIMM2=LDBCOL
      END IF
C
C=======================================================================
C
      IF (LDIMM1.NE.LDIMM2) THEN
C
          WRITE(6,'(/,''The middle indice in CCFULL must be equal'',/,
     *                ''LDIMM1='',I4,'' should agree with LDIMM2='',
     *                I4,/)') LDIMM1,LDIMM2
C
          STOP 'Wrong middle index in CCFULL <<= '
      ELSE
          LDIMMM=LDIMM1
      END IF
C
C=======================================================================
C
      IF (LDIMRC.NE.LDCROW.OR.LDIMCC.NE.LDCCOL)
     *    STOP 'Mismatch in CCFULL <<= '
C
      FORM0A=FORM_A
      FORM0B=FORM_B
C
C=======================================================================
C
      IF (FORM_A.EQ.'Z') THEN
C
          CALL WS_CONJUG(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
          FORM0A='N'
C
      END IF
C
C=======================================================================
C
      IF (FORM_B.EQ.'Z') THEN
C
          CALL WS_CONJUG(BMATRX,NDBROW,NDBCOL,LDBROW,LDBCOL)
C
          FORM0B='N'
C
      END IF
C
C=======================================================================
C
C         NAG version starts here - 4 active lines below to uncomment
C
C=======================================================================
C
C=======================================================================
C=========================    PORTABILITY    ===========================
C=======================================================================
C=========================    block no. 3    ===========================
C=======================================================================
C============== This routine is not called at all. Comment =============
C==============   out calling F06ZAF to keep the program   =============
C==============               portable !!!                 =============
C=======================================================================
C
CCC      CALL WS_F06ZAF(FORM0A,FORM0B,LDIMRC,LDIMCC,LDIMMM,
CCC     *            UNIT_C,AMATRX,NDAROW,
CCC     *                   BMATRX,NDBROW,ZERO_C,CMATRX,NDCROW)
C
C
C=======================================================================
C================   end of portability block no 3   ====================
C=======================================================================
C
      IF (FORM_A.EQ.'Z') THEN
C
          CALL WS_CONJUG(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
C
      END IF
C
C=======================================================================
C
      IF (FORM_B.EQ.'Z') THEN
C
          CALL WS_CONJUG(BMATRX,NDBROW,NDBCOL,LDBROW,LDBCOL)
C
      END IF
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_TRANSP(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
      COMPLEX
     *        AMATRX(1:NDAROW,1:NDACOL)
      COMPLEX
     *        ROWMAT,COLMAT
C
C=======================================================================
C     This subroutine transforma a matrix AMATRX into its transpose;
C     the original matrix is overwritten
C=======================================================================
C
      DO J=1,LDACOL
         DO I=J+1,LDAROW
C
            ROWMAT=AMATRX(I,J)
            COLMAT=AMATRX(J,I)
C
            AMATRX(I,J)=COLMAT
            AMATRX(J,I)=ROWMAT
C
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_TRANCJ(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
      COMPLEX
     *        AMATRX(1:NDAROW,1:NDACOL)
      COMPLEX
     *        R,C
C
C=======================================================================
C     This subroutine transforma a matrix AMATRX into its hermitian
C     conjugate; the original matrix is overwritten
C=======================================================================
C
      DO J=1,LDACOL
         DO I=J,LDAROW
C
            R=AMATRX(I,J)
            C=AMATRX(J,I)
C
            AMATRX(I,J)=CONJG(C)
            AMATRX(J,I)=CONJG(R)
C
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_CONJUG(AMATRX,NDAROW,NDACOL,LDAROW,LDACOL)
      COMPLEX
     *        AMATRX(1:NDAROW,1:NDACOL)
C
C=======================================================================
C     This subroutine transforma a matrix AMATRX into its complex
C     conjugate; the original matrix is overwritten
C=======================================================================
C
      DO J=1,LDACOL
         DO I=1,LDAROW
C
            AMATRX(I,J)=CONJG(AMATRX(I,J))
C
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_ADDIMA(IPLUMI,H_IMAG,H_CMPL,LDBASE)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *          H_CMPL(1:NDBASE,1:NDBASE)
      DIMENSION
     *          H_IMAG(1:NDBASE,1:NDBASE)
C
      DATA
     *          R_ZERO / 0.000 /
C
C=======================================================================
C        This subroutine adds the imaginary part of the Hamiltonian
C        matrix  (H_IMAG)  to the complex one (H_CMPL) with a phase
C                                                            IPLUMI
C=======================================================================
C
      DO IBRAHO=1,LDBASE
         DO IKETHO=1,LDBASE
C
            H_CMPL(IBRAHO,IKETHO)=H_CMPL(IBRAHO,IKETHO)
     *                           +IPLUMI
     *                           *CMPLX(R_ZERO,H_IMAG(IBRAHO,IKETHO))
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_ADDREA(IPLUMI,H_REAL,H_CMPL,LDBASE)
C=======================================================================
      USE params
C=======================================================================
      COMPLEX
     *          H_CMPL(1:NDBASE,1:NDBASE)
      DIMENSION
     *          H_REAL(1:NDBASE,1:NDBASE)
      DATA
     *          R_ZERO / 0.000 /
C
C=======================================================================
C     This subroutine adds a real part of a hamiltonian matrix
C     (H_REAL) to a complex one,  introducing the phase IPLUMI
C=======================================================================
C
      DO IBRAHO=1,LDBASE
         DO IKETHO=1,LDBASE
C
            H_CMPL(IBRAHO,IKETHO)=H_CMPL(IBRAHO,IKETHO)
     *                           +IPLUMI
     *                           *CMPLX (H_REAL(IBRAHO,IKETHO),R_ZERO)
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
