C FILE NAME = wsu24_bcsp_01.f   ! Keep this symbol here: $ident@string$
C
C=======================================================================
C=======================================================================
C=====                                ==================================
C=====  BCS PAIRING PACKAGE           ==================================
C=====                                ==================================
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_BCSPAI(IZ_FIX,IN_FIX,ISOSPI,NLPAIR,NOFBLK,N_BLOK,
     *                     FACTOG,G_PAIR,XLAMBD,PDELTA,PCOREL,PCOREC,
     *                     IFPROJ,IFSUMG,PROJ_E,ESPECT,BCSBAR,PAIREN,
     *                                   PROJCR,SUMSNG,SUMGPA,V2OCCU)
C=======================================================================
      USE params
C=======================================================================
      EXTERNAL
     *          WS_EQSBCS
C
      DIMENSION
     *          ESPECT(1:NDLEVS),V2OCCU(1:NDLEVS)
      DIMENSION
     *          NBLOCK(1:NDLEVS),N_BLOK(1:NDLEVS)
C
      COMMON
     *       /BCSBCS_RES_WS/ ESINGP(1:NDLEVS)
      COMMON
     *       /BCSBCS_CONTRL/ NOKEPT,NACTIV,ISEGMT,ICNTRL
      COMMON
     *       /BCSBCS_PROAUX/ GACTIV,DELTAC,XIMAX0,XIMAXD
      COMMON
     *       /BCSBCS_STARTS/ STARTD,STARTL
      COMMON
     *       /BCSBCS_RHSRHS/ TWOVRG,PARTIC
      COMMON
     *       /BCSBCS_BLCKLV/ NBLOCK
      COMMON
     *       /BCSBCS_LIMPAI/ NOLEVT
C
C=======================================================================
C
C          EPS_XY = Precision in the arguments in the SYSPAI alg.
C          EPS_FG = Precision in the functions in the SYSPAI alg.
C
C          DELMIN = Minimum value for the pairing DELTA2 (program
C                   does not like DELTA**2=0)
C
C=======================================================================
C
      DATA
     *     EPSNWT / 1.0E-10 /
      DATA
     *     DELMIN / .625E-1/
      DATA
     *     MXITER /  100 /
C
C=======================================================================
C
C       This package calculates pairing energies, v**2, etc, within
C       the traditional simple algorithm
C
C       The package consists of the following subroutines:
C
C       "WS_EQSBCS", "FUNMIN", "SYSPAI"
C
C=======================================================================
C
C                S O M E    I N P U T   P A R A M E T E R S :
C
C=======================================================================
C
C       NLPAIR - The actual number of the single - particle levels
C
C       N_PART - Total number of particles, including those placed
C                on the blocked levels
C
C       NOFBLK - Number of blocked levels. The blocking means that
C                these levels  are simply eliminated  from pairing
C                calculations.  In the case of blocking  activated,
C                only the pairing energy of the zero seniority, of
C                the corresponding  even sub-system  is calculated.
C                The total energy, and the single particle  energy
C                contain blocked levels  (related  single particle
C                contribution)
C
C       N_BLOK - A vector of dimension  NDBLOK  containing the in-
C                dices of blocked levels (not used if NOFBLK=0)
C
C       ESINGP - A vector of dimension  NDLEVS,  containing single
C                particle energies  (e.g. Woods-Saxon s.p. levels)
C
C       G_PAIR - The monopole pairing strength constant "G"
C
C=======================================================================
C
      IF (NLPAIR.GT.NDLEVS) STOP 'NLPAIR.GT.NDLEVS in BCSPAI'
      IF (NLPAIR.LT.     1) STOP 'NLPAIR.LT.     1 in BCSPAI'
C
      DO I=1,NDBASE
         ESINGP(I)=ESPECT(I)
      END DO
C
C=======================================================================
C         Calculating the pairing "G" out of the average DELTA
C=======================================================================
C
                       N_PART=IN_FIX
      IF (ISOSPI.EQ.1) N_PART=IZ_FIX
C
      GACTIV=G_PAIR*FACTOG
      EPABAR=BCSBAR
C
      A_MASS=REAL(IZ_FIX+IN_FIX)
C
      NOLEVT=2*(N_PART/2)
C
C=======================================================================
C          Defining "NBLOCK" vector out of N_BLOK(...)
C=======================================================================
C
      DO LEVELS=1,NDLEVS
         NBLOCK(LEVELS)=0
      END DO
C
C=======================================================================
C          If blocking should be taken into account ...
C=======================================================================
C
      IF (NOFBLK.GE.1) THEN
C
          DO LEVELS=1,NOFBLK     ! The number of all-together blocked
C                                                              levels
             KBLCKD=N_BLOK(LEVELS)
C
             IF (KBLCKD.NE.0) THEN
C
                IF (KBLCKD.LT.1.OR.KBLCKD.GT.NOLEVT) THEN
C
                    WRITE(6,'(/,''KBLCKD='',I3,'' < 1'','' .OR. '',
     *                               ''KBLCKD='',I3,'' > NOLEVT='',I3)')
     *                                 KBLCKD,KBLCKD,NOLEVT
C
                    STOP 'KBLCKD.LT.1.OR.KBLCKD.GT.NOLEVT'
C
                END IF
C
                NBLOCK(KBLCKD)=1
C
            END IF
C
         END DO
C
      END IF
C
C=======================================================================
C         Calculating the number of particles on the active levels
C=======================================================================
C
      NOKEPT=N_PART-NOFBLK
      PARTIC=NOKEPT
C
C=======================================================================
C         Solving the usual BCS-equations
C=======================================================================
C
      TWOVRG=2.00/GACTIV
C
      BCSLAM=0.5*(ESINGP(N_PART/2)+ESINGP(N_PART/2+1))
C
      XSTART=BCSLAM
      YSTART=12.000/SQRT(A_MASS)
C
      STARTL=XSTART
      STARTD=YSTART
C
      CALL WS_NEWTN2(XSTART,YSTART,XSOLUT,YSOLUT,WS_EQSBCS,MXITER,
     *                                       IERROR,EPSNWT,ITDONE)
C
      BCSLAM=XSOLUT
      BCSDEL=YSOLUT
      DELTA2=BCSDEL**2
C
      IF (IERROR.EQ.1) THEN
          BCSLAM=0.5*(ESINGP(N_PART/2)+ESINGP(N_PART/2+1))
          BCSDEL=0.1
      END IF
C
C=======================================================================
C        Calculating the occupation factors - squared
C=======================================================================
C
      DO LEVELS=1,NOLEVT
         EMINLA=ESINGP(LEVELS)-BCSLAM
	 IF (NBLOCK(LEVELS).EQ.0) THEN
             V2OCCU(LEVELS)=0.5*(1.0-EMINLA/(SQRT(EMINLA**2+DELTA2)))
	 ELSE
	     V2OCCU(LEVELS)=0.5
	 END IF
      END DO
C
C=======================================================================
C        Calculating the particle number projection energy correction
C=======================================================================
C
      PROJ_E=WS_PROJEN(NOLEVT,V2OCCU,G_PAIR,NBLOCK,NDLEVS)
C
C      Compatibility with projection block
      PROJCR=PROJ_E
C
C=======================================================================
C        WS_CORREL calculates the pairing correlation energy i.e.
C                         (pairing - no_pairing)
C=======================================================================
C
      SINGLE=0.0
      SUMPAR=0.0
C
      PDELKW=BCSDEL**2
      PDELTA=BCSDEL
C
      CORPAI=WS_CORREL(N_PART,ESINGP,V2OCCU,PDELKW,G_PAIR,SINGLE,
     *                                      IFSUMG,SUMPAR,NDBASE)
C
C      Compatibility with projection block
      PAIREN=SUMPAR
      SUMSNG=SINGLE
      SUMGPA=-REAL(N_PART/2)*G_PAIR
C
C=======================================================================
C        Estimated projection energy  can be added optionally
C=======================================================================
C
      IF (IFPROJ.EQ.0) THEN
          PCOREL=PAIREN-SUMSNG
      ELSE
          PCOREL=(PAIREN+PROJCR)-(SUMSNG+SUMGPA)
      END IF
C
C=======================================================================
C        Pairing CORRECTION energy is equal to the pairing CORRELATION
C        energy - minus the average pairing correlation energy  EPABAR
C=======================================================================
C
      PCOREC=PCOREL-EPABAR
C
c      CALL WS_BCSOUT(IZ_FIX,IN_FIX,ISOSPI,G_PAIR,FACTOG,DELTAP,
c     *                      PCOREL,PCOREC,NLPAIR,ESINGP,V2OCCU)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_PROJEN(NOLEVT,V2OCCU,G_PAIR,NBLOCK,NDLEVS)
      DIMENSION
     *          NBLOCK(1:NDLEVS),V2OCCU(1:NDLEVS)
C
      DATA
     *     EPSPRJ / 0.0001 /
C
C=======================================================================
C        This function calculates the particle number projection
C        energy correction (simple PNP after variation estimate)
C=======================================================================
C
      UV3SUM=0.0
      VU3SUM=0.0
C
      U4V4SM=0.0
      U2V2SM=0.0
C
      DO LEVELS=1,NOLEVT
C
         VOCCU2=V2OCCU(LEVELS)
C
         IF (LEVELS.EQ.NBLOCK(LEVELS)) VOCCU2=1.
C
         UOCCU2=1.00-VOCCU2
         UVFACT=SQRT(VOCCU2*UOCCU2)
         UV3SUM=UV3SUM+VOCCU2*UVFACT
         VU3SUM=VU3SUM+UOCCU2*UVFACT
         U4V4SM=U4V4SM+UVFACT**4
         U2V2SM=U2V2SM+UVFACT*UVFACT
C
      END DO
C
C=======================================================================
C
      IF (U2V2SM.LT.EPSPRJ) THEN
          WS_PROJEN=0.0
          RETURN
      END IF
C
C=======================================================================
C
      WS_PROJEN=-G_PAIR*(UV3SUM*VU3SUM-U4V4SM)/U2V2SM
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_CORREL(N_PART,ESINGP,V2OCCU,PDELKW,G_PAIR,SINGLE,
     *                                     IFSUMG,SUMPAR,NDBASE)
      DIMENSION
     *          ESINGP(1:NDBASE),V2OCCU(1:NDBASE)
C
C=======================================================================
C         This routine calculates the expectation value of the usual
C         m o n o p o l e     p a i r i n g    h a m i l t o n i a n
C         WS_CORREL denotes "pairing correlation" and  not "correction"
C=======================================================================
C
      NOLEVT=2*(N_PART/2)
C
      SINGLE=0.0
      SUMPAR=0.0
C
C      Without the term -G_PAIR*V2OCCU(K)**2, the pairing energy
C      calculated in CORREl comes down to the same expression as
C      within the HFB formalism.
C
      DO K=1,NOLEVT
C         SUMPAR=SUMPAR+2.0*ESINGP(K)*V2OCCU(K)-G_PAIR*V2OCCU(K)**2
         SUMPAR=SUMPAR+2.0*ESINGP(K)*V2OCCU(K)
      END DO
C
C      This term must be used only when the 'correlation energy' in the
C      pairing correction is seeked. It corresponds to the G\sum_i v_i^4
C      term in the BCS energy.
C
      IF (IFSUMG.EQ.1) THEN
          SUMPAR=SUMPAR+(N_PART/2)*G_PAIR
      END IF
C
      DO K=1,N_PART/2
         SINGLE=SINGLE+2.0*ESINGP(K)
      END DO
C
C      Below: WS_CORREL denotes "CORRELATION" => energy with pairing
C             minus the energy without pairing correlations
C
      SUMPAR=SUMPAR-PDELKW/G_PAIR
C
C      SUMPAR=\sum_i 2*v_i^2*e_i - Delta^2/G_pair (+ N*G_pair/2)
C
C      The term in brakets must be conserved only if correlation
C      energy is wanted. The 'true' pairing energy does not have
C      this term
C
      WS_CORREL=SUMPAR-SINGLE
C
C      The above definition corresponds to Bolsterli et al. PR C5.
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_EQSBCS(BCSLAM,BCSDEL,AG_BCS,AF_BCS,DGOVDL,DGOVDD,
     *                                                 DFOVDL,DFOVDD)
C=======================================================================
      USE params
C=======================================================================
      DIMENSION
     *          NBLOCK(1:NDLEVS)
C
      COMMON
     *       /BCSBCS_RES_WS/ ESINGP(1:NDLEVS)
      COMMON
     *       /BCSBCS_STARTS/ STARTD,STARTL
      COMMON
     *       /BCSBCS_RHSRHS/ TWOVRG,PARTIC
      COMMON
     *       /BCSBCS_BLCKLV/ NBLOCK
      COMMON
     *       /BCSBCS_LIMPAI/ NOLEVT
C
      DATA
     *     DELMIN / .1E-9 /,
     *     DELMAX / .5E+2 /
C
C=======================================================================
C
C        This subroutine calculates the rhs of BCS-equations and
C        their derivatives.  It is called  in subroutine  SYSPAI
C
C        Left hand sides are:  The particle number  and 2/G_PAIR
C
C=======================================================================
C
C        BCSLMB - Denotes the pairing BCS - lambda
C
C=======================================================================
C
      AF_BCS=0
      AG_BCS=0
C
      DFOVDL=0
      DFOVDD=0
      DGOVDL=0
      DGOVDD=0
C
      IF (BCSDEL.LT.0.0) BCSDEL=-BCSDEL
C
      IF (BCSDEL.GT.DELMAX) THEN
          BCSDEL=DELMIN
          BCSLAM=STARTL
      END IF
C
      DELTA2=BCSDEL*BCSDEL
C
      DO LEVELS=1,NOLEVT
C
         IF (NBLOCK(LEVELS).EQ.1) GO TO 1
C
         ENEDIF=ESINGP(LEVELS)-BCSLAM
         QUASI2=ENEDIF*ENEDIF +DELTA2
         QUASIP=SQRT(QUASI2)
         ONEO32=1.0/(QUASIP*QUASI2)
C
         AF_BCS=AF_BCS+1.0000/QUASIP
         AG_BCS=AG_BCS-ENEDIF/QUASIP+1.00
C
         DFOVDL=DFOVDL+ENEDIF*ONEO32
         DFOVDD=DFOVDD+ONEO32

         DGOVDL=DGOVDL+ONEO32
         DGOVDD=DGOVDD+ENEDIF*ONEO32
C
   1     CONTINUE
C
      END DO
C
      AF_BCS=AF_BCS-TWOVRG
      AG_BCS=AG_BCS-PARTIC
C
      DFOVDD=-DFOVDD*BCSDEL
C
      DGOVDL=+DGOVDL*DELTA2
      DGOVDD=+DGOVDD*BCSDEL
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_NEWTN2(XSTART,YSTART,XSOLUT,YSOLUT,WS_SYSDEF,
     *                               NOFITE,IERROR,EPSNWT,ITDONE)
      EXTERNAL
     *         WS_SYSDEF
C
      COMMON
     *       /BCSBCS_STARTS/ STARTD,STARTL
C
      DATA
     *        EPSDET / 1.0E-30 /
     *        EPSDEF / 1.0E-30 /
C
C=======================================================================
C
C         This routine solves the system of two linear quations of the
C         form:
C                      f(x,y) = 0    and    g(x,y) = 0
C
C         using the Newton's method. The accuracy can be controlled in
C         terms of the function values; the user defines EPSNWT.
C
C         The system of equations with the right-hand sides equal zero
C         is defined in terms of the user provided subroutine 'WS_SYSDEF';
C         the latter will provide at each iteration point the function
C         values as well as both partial derivatives.
C
C         NOFITE - The maximum number of iterations as defined by user
C
C         ITDONE - The number of iteration actually performed. This is
C                  equal to the number of the calls to 'WS_SYSDEF'.
C
C         Error-Index Coding:
C
C         IERROR = 0 ->> requested precision is satisfied hurra!
C
C         IERROR = 2 ->> the determinant is zero and some coeffs.
C                        of the system of two equations are zero;
C                        equations are most likely lin.dependent
C         IERROR = 4 ->> the determinant is 0,  and coeffitients
C                        of the system such, that solutions most
C                        likely do not exist
C         IERROR = 6 ->> the determinant is 0,  and coeffitients
C                        such, that the system  in question  may
C                        have infinitely many solutions
C         IERROR = 8 ->> within  given number of iterations, the
C                        required precision cannot be guaranteed
C
C         If IERROR =-1, congratulations! The WS_SYSDEF routine has been
C                        written in such a way, that iterations could
C                        not start what is next to impossible.
C
C=======================================================================
C
      IERROR=-1
C
      X_NEXT=XSTART
      Y_NEXT=YSTART
C
      CALL WS_SYSDEF(X_NEXT,Y_NEXT,FFUNCT,GFUNCT,DFOVDX,DFOVDY,
     *                                           DGOVDX,DGOVDY)
C
      ITEACT=1
C
C=======================================================================
C     Starting the iterations
C=======================================================================
C
   1  CONTINUE
C
      ITEACT=ITEACT+1
C
      A11MAT=DFOVDX
      A12MAT=DFOVDY
      A21MAT=DGOVDX
      A22MAT=DGOVDY
C
      B1COEF=-FFUNCT
      B2COEF=-GFUNCT
C
      DETBAS=A11MAT*A22MAT-A12MAT*A21MAT
      DT1SOL=B1COEF*A22MAT-B2COEF*A12MAT
      DT2SOL=A11MAT*B2COEF-A21MAT*B1COEF
C
C     Analysing properties of the determinant; if DETBAS.NE.0, there
C                                              will be nothing to do
C                                              at this place
      IF (ABS(DETBAS).LT.EPSDET) THEN
C
C         The determinant is unfortunately equal to zero (or almost)
C
          IF (ABS(A11MAT*A22MAT).GT.EPSDET) THEN
C
C             Some coefficients of the system must be non-zero; then:
C
              RATIOA=A11MAT/A12MAT
              RATIOB=A21MAT/A22MAT
C
              IF (ABS(RATIOA-RATIOB).LT.10.*EPSDEF) THEN
C
C                 The columns of the determinant are linearly depndt.
C                 this is equivalent to the linear dependence of the
C                 lines as well;  the related condition not verified
C
                  IF (ABS(A12MAT*B2COEF-A22MAT*B1COEF).GT.10*EPSDEF)
     *                                                        THEN
C                     Contradiction: actual values of the 'b-coeffs'
C                                    are  in conflict  with those of
                      IERROR=4     ! the 'a-coeffs', in this case...
                  ELSE
C                     ... while here - they are not in conflict, but
C                     the equations get independent from one another
                      IERROR=6
                  END IF
              END IF
          ELSE
              IERROR=2
          END IF
      END IF
C
      SOLUTX=DT1SOL/DETBAS
      SOLUTY=DT2SOL/DETBAS
C
      X_NEXT=X_NEXT+SOLUTX
      Y_NEXT=Y_NEXT+SOLUTY
C
      CALL WS_SYSDEF(X_NEXT,Y_NEXT,FFUNCT,GFUNCT,DFOVDX,DFOVDY,
     *                                           DGOVDX,DGOVDY)
C
C     Checking the precision & verifying the iteration count
C
      IF (ABS(FFUNCT)+ABS(GFUNCT).LT.EPSNWT) THEN
          XSOLUT=X_NEXT
          YSOLUT=Y_NEXT
          ITDONE=ITEACT
          IERROR=0
          RETURN
      END IF
C
      IF (ITEACT.LT.NOFITE) THEN
          GO TO 1
      ELSE
          XSOLUT=STARTL
          YSOLUT=0
          IERROR=8
          RETURN
      END IF
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_BCSOUT(IZ_FIX,IN_FIX,ISOSPI,G_PAIR,FACTOG,DELTAP,
     *                            PCOREL,PCOREC,NLPAIR,ESINGP,V2OCCU)
C=======================================================================
      USE params
C=======================================================================
C
      CHARACTER
     *          NUCLEO(0:1)*9,COMENT(0:2)*10
C
      DIMENSION
     *          ESINGP(1:NDLEVS),V2OCCU(1:NDLEVS)
C
      DATA
     *     NUCLEO(0) / 'NEUTRONS:' /,
     *     NUCLEO(1) / 'PROTONS: ' /
      DATA
     *     COMENT(0) / 'G-fitted: ' /,
     *     COMENT(1) / 'G-average:' /,
     *     COMENT(2) / 'G-userdef:' /
C
C=======================================================================
C
      ICOMEN=0
C
      WRITE(6,'(/,80(''#''),/,''#'',78X,''#'',/,
     *            ''#                     Standard BCS Pairing'',
     *            '' Calculations                        #'',/,
     *            ''#'',78X,''#'',/,
     *            ''#     '',A9,''   Z ='',I3,''   N ='',I3,4X,A10,
     *            '' G_PAIR='',F5.3,''  GFACTO='',F5.3,''     #'',/,
     *            ''#'',T63,''GACTIV='',F5.3,''     #'')')
     *
     *            NUCLEO(ISOSPI),IZ_FIX,IN_FIX,COMENT(ICOMEN),
     *                                         G_PAIR/FACTOG,
     *                                                FACTOG,
     *                                                G_PAIR
C
      WRITE(6,'(''#'',78X,''#'',/,
     *          ''#                                   '',
     *          ''DELTA='',F4.2,''  E_PAIR='',F6.2,'' ECOREC='',F6.2,
     *          ''    #'')')
     *
     *                                   DELTAP,PCOREL,PCOREC
C
      LEVMIN=1
      LEVMAX=NLPAIR
C
      DO I=1,NLPAIR
C
         IF (V2OCCU(I).LT.1.E-5) THEN
	     LEVMAX=I
	     GO TO 1
	 END IF
C
      END DO
C
  1   CONTINUE
C
      KIKOFF=LEVMAX-LEVMIN+1
      ISTART=1
C
      KIKOFF=KIKOFF/2-1
C
      IF (ISTART+KIKOFF.GT.LEVMAX)
     *    STOP 'ISTART+KIKOFF.GT.LDBASE - IN PAIRLN <<= '
C
      WRITE(6,'(''#'',78X,''#'',/,
     *          ''#       No)     ESPAIR      V2occu        '',
     *            ''   No)     ESPAIR      V2occu        #'',/,
     *          ''#'',78X,''#'')')
C
      DO L=ISTART,ISTART+KIKOFF
C
         L1=L
         L2=KIKOFF+L+1
C
         WRITE(6,'(''#   '',I6,'') '',2F11.5,10X,
     *                   I3,'') '',2F11.5,5X,''   #'')')
     *             L1,ESINGP(L1),V2OCCU(L1),
     *             L2,ESINGP(L2),V2OCCU(L2)
      END DO
C
      WRITE(6,'(''#'',78X,''#'',/,80(''#''))')
C
C
C=======================================================================
C
      RETURN
      END
C=======================================================================
