C FILE NAME = wsu25_lipk_01.f   ! Keep this symbol here: $ident@string$
C
C=======================================================================
C=======================================================================
C=====                                ==================================
C=====  LIPKIN-NOGAMI PAIRING         ==================================
C=====                                ==================================
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_PAIRLN(IZ_FIX,IN_FIX,NUMLEV,ENERSP,ICODEL,DEL_LN,
     *                     PCOREC,PCOREL,IFSUMG,XLAMBD,XLAMB2,ISOSPI,
     *                                                 V2CORR,IPRINT)
C=======================================================================
      USE params
C=======================================================================
C
      CHARACTER
     *          NUCLEO(0:1)*9,COMENT(0:2)*10
C
      DIMENSION
     *          ENERSP(1:NDLEVS),VV_LIN(1:NDLEVS),V2CORR(1:NDLEVS)
      DIMENSION
     *          NBLOCK(1:NDLEVS),N_BLCK(1:NDBLOK)
C
      COMMON
     *       /EXTRAS_PSTRNG/ GPAIRN,GPAIRP
      COMMON
     *       /PAIRIN_STRENG/ GFACTN,GFACTP
      COMMON
     *       /EXTRAS_LIPKCO/ RLNOGN,RLNOGP
      COMMON
     *       /PAIRIN_WINDOW/ LEVPUP,LEVPDW,LEVNUP,LEVNDW
      COMMON
     *       /LIPKIN_LNAUXR/ SPECEN(1:NDLEVS),VV_COM(1:NDLEVS),
     *                                        G_PAIR,DELTAC
      COMMON
     *       /LIPKIN_LNAUXI/ NUMPAR,INWIND
C
      DATA
     *     NUCLEO(0) / 'NEUTRONS:' /,
     *     NUCLEO(1) / 'PROTONS: ' /
      DATA
     *     COMENT(0) / 'G-fitted: ' /,
     *     COMENT(1) / 'G-average:' /,
     *     COMENT(2) / 'G-userdef:' /
      DATA
     *                    SQ2REC,ITER,  EPS
     *    / 7.071067811865475D-1,  50,1.D-9/
C
C=======================================================================
C
C        This package calculates the pairing energies, v**2, etc,
C        within the lipkin-nogami model
C
C        The package consists of:  PAIRLN,WS_ETOTLN,WS_EQNSLN,SYSTEC.
C
C        Input
C        -----
C
C              NUMLEV - Number of levels (two states per level)
C
C              LEVMIN - Number of the first active level. Levels
C                       below are assumed doubly occupied
C
C              LEVMAX - Number of the last active level. Levels
C                       above are assumed empty
C
C              N_PART - Total number of particles (including those
C                       on blocked levels and those below LEVMIN -
C                       can be odd)
C
C              NOFBLO - Number of blocked levels
C
C              N_BLCK - Vector of dimension NOFBLO containing indices
C                       of blocked levels (not used if NOFBLO=0)
C
C              ENERSP - Vector of dimension NUMLEV containing single
C                       particle energies
C
C              G_PAIR - Pairing constant G
C
C              ICODEL - ICODEL=0 ==> Variable delta
C                       ICODEL=1 ==> Constant delta (=DEL_LN)
C
C              DEL_LN - Initial value of delta
C
C       Output
C       ------
C
C              VV_LIN - Vector of dimension NUMLEV containing
C                       v coefficients (n o t   s q u a r e d).
C                        + for levels below LEVMIN v=1
C                        + for blocked levels      v=sqrt(1/2)
C                        + for levels above LEVMAX v=0
C
C              V2CORR - Vector of dimension NUMLEV Containing
c                       squares of corrected v coefficients:
C                          V(i)=<a^+_i a_i>_LN
C                        + for levels below LEVMIN v=1
c                        + for blocked levels      v=1/2
c                        + for levels above LEVMAX v=0
c
C              DEL_LN  - Delta for Lipkin-Nogami
C
C              ECORHF - Correction to HF energy:
C                         ECORHF=-D**2/G-G*SUM_V(I)**4
C                                -4*XLAMB2*SUM_(U(I)*V(I))**2+G*NPARTIC/2
C
C              EPAIRR - Pairing energy
C                         EPAIRR= ECORHF+2*SUM_E(I)*V(I)**2
C                                -2*SUM(OCCUPIED)_E(I)
C
C              XLAMBD - LAMBDA
C
C              XL21   - LAMBDA_2
C
C         NOTE
C         ----
C
C            Energies are calculated for even subsystem only;
C            if there are blocked levels, they are simply excluded
c            from calculations: no quasiparticle energies are added !!
C
C                   See NUCL. PHYS. A201 (1973) 357.
C
C=======================================================================
C========== BASED ON ROUTINE BY G.A.LEANDER AND W.NAZAREWICZ, MARCH 1985
C================================ MODIFIED BY T.R.WERNER, SEPTEMBER 1994
C=======================================================================
C
      IF (ISOSPI.EQ.0) THEN
C
          G_PAIR=GPAIRN*GFACTN
	  N_PART=IN_FIX
	  RLNBAR=RLNOGN
	  FACTOG=GFACTN
C
C          If N is odd, then one level is blocked
C
          IF (MOD(IN_FIX,2).EQ.1) THEN
C
              LFERMI=IN_FIX/2
C
	      NOFBLO=1
	      N_BLCK(1)=LFERMI+1
C
          ELSE
C
	      NOFBLO=0
C
          END IF
C
      END IF
C
      IF (ISOSPI.EQ.1) THEN
C
          G_PAIR=GPAIRP*GFACTP
	  N_PART=IZ_FIX
	  RLNBAR=RLNOGP
	  FACTOG=GFACTP
C
C          If N is odd, then one level is blocked
C
          IF (MOD(IZ_FIX,2).EQ.1) THEN
C
              LFERMI=IZ_FIX/2
C
	      NOFBLO=1
	      N_BLCK(1)=LFERMI+1
C
          ELSE
C
	      NOFBLO=0
C
          END IF
C
      END IF
C
      LEVMIN=1
      LEVMAX=N_PART
C
C=======================================================================
C
      IF (NUMLEV.GT.NDLEVS)            STOP 'PAIRLN01'
      IF (NUMLEV.LT.LEVMAX)            STOP 'PAIRLN02'
      IF (1     .GT.LEVMIN)            STOP 'PAIRLN03'
      IF (LEVMIN.GE.LEVMAX-3)          STOP 'PAIRLN04'
      IF (ICODEL.NE.0.AND.ICODEL.NE.1) STOP 'PAIRLN05'
C
C=======================================================================
C
      DO I=LEVMIN,LEVMAX
         NBLOCK(I)=0
      END DO
C
C=======================================================================
C         Blocking user-predefined states
C=======================================================================
C
      IF (NOFBLO.GE.1) THEN
C
	 DO I=1,NOFBLO
	    K=N_BLCK(I)
	    IF (K.LT.LEVMIN.OR.K.GT.LEVMAX)
     *          STOP 'Blocked states out of the energy window'
	    NBLOCK(K)=1
         END DO
C
      END IF
C
C=======================================================================
C        Transforming the running index: levels within the window
C        are counted with indices 1,2, ...  and placed in  SPECEN
C=======================================================================
C
      INWIND=0
      DO I=LEVMIN,LEVMAX
         IF (NBLOCK(I).EQ.0) THEN
             INWIND=INWIND+1
             SPECEN(INWIND)=ENERSP(I)
         END IF
      END DO
C
      IF (NOFBLO.NE.(LEVMAX-LEVMIN+1-INWIND)) STOP 'PAIRLN07'
C
C=======================================================================
C        Number of particles on active levels within energy window
C=======================================================================
C
      NUMPAR=N_PART-NOFBLO-2*(LEVMIN-1)
      PARTIC=NUMPAR
C
      IF (MOD(NUMPAR,2).NE.0) STOP 'PAIRLN08'
C
C=======================================================================
C        Calculating the sum of the single-particle levels within
C                                                   energy window
C=======================================================================
C
      FERMLV=0.5*(SPECEN(NUMPAR/2)+SPECEN(NUMPAR/2+1))
      E_SNGP=0.0
      IUNDER=0
C
      DO I=1,INWIND
	 E=SPECEN(I)
         IF (E.LT.FERMLV) THEN
	     IUNDER=IUNDER+1
	     E_SNGP=E_SNGP+E
	 END IF
      END DO
C
      E_SNGP=2*E_SNGP
C
      IF (2*IUNDER.NE.NUMPAR) STOP 'PAIRLN09'
C
C=======================================================================
C         Lipkin with everybody knows what - as such
C=======================================================================
C
C     ==>> INITIAL VALUES
C
C          XLAMB2=G_PAIR/3.5       DLACZEGO?????????
C
C          DEL_LN=12/SQRT(2.*NUMB_N)
C
C=======================================================================
C
      DEL_LN=MAX(DEL_LN,0.004)
      XLAMBD=FERMLV
      XLAMB2=G_PAIR/3.5
C
C=======================================================================
C         The case of the constant pairing-delta
C=======================================================================
C
      IF (ICODEL.EQ.1) THEN
C
          DELTAC=DEL_LN
          E_CALK=WS_ETOTLN(XLAMBD,XLAMB2,DDELTA,SUMESP)
C
C=======================================================================
C
C          DDELTA=DELTAC - G* \sum_{k} v_{k} u_{k}
C
C        ==>> DONE
C
C=======================================================================
C
         GO TO 4
C
      END IF
C
C=======================================================================
C     Iterating the selfconsistent LN solutions for Delta, Lambda
C                                                     and Lambda2
C=======================================================================
C
      DELTAC=MAX(DEL_LN,0.1)
      STEPDL=DELTAC/(ITER-1)
      DELTAC=DELTAC+STEPDL
C
      IFIRST=1
      ITEACT=0
C
   1  CONTINUE
C
      ITEACT=ITEACT+1
      DELTAC=DELTAC-STEPDL
C
      E_CALK=WS_ETOTLN(XLAMBD,XLAMB2,DDELTA,SUMESP)
C
C=======================================================================
C
C=======================================================================
C
      IF (ABS(DDELTA).LE.EPS) GO TO 4
C
         IF (ITEACT.EQ.1) THEN
             DDELT1=DDELTA
             D1=DELTAC
             GO TO 1
         END IF
C
         IF (DDELTA*DDELT1.LT.0.D0) GO TO 2
C
         IF (ITEACT.EQ.2.AND.ABS(DDELTA).GT.ABS(DDELT1)) THEN
             DELTAC=DELTAC+STEPDL
             STEPDL=-STEPDL
             GO TO 1
         END IF
C
         IF (ITEACT.EQ.ITER) THEN
             IF (STEPDL.LT.0.D0.AND.IFIRST.EQ.1) THEN
                 IFIRST=0
                 ITEACT=2
                 STEPDL=2*STEPDL
                 GO TO 1
             ELSE
                 STOP 'PAIRLN10'
             END IF
         END IF
C
         DDELT1=DDELTA
         D1=DELTAC
C
      GO TO 1
C
C=======================================================================
C
C     ==>> Solving the pairing gap equation
C
C=======================================================================
C
   2  CONTINUE
C
      D=DELTAC
      ITEACT=0
C
   3  CONTINUE
C
      DELTAC=(D1*DDELTA-DDELT1*D)/(DDELTA-DDELT1)
C
      E_CALK=WS_ETOTLN(XLAMBD,XLAMB2,DG0,SUMESP)
C
C=======================================================================
C
C        ==>> DONE
C
C=======================================================================
C
         IF (ABS(DG0).LT.EPS) GO TO 4
C
         ITEACT=ITEACT+1
         IF (ITEACT.GT.ITER) STOP 'PAIRLN11'
C
         IF (DDELTA*DG0.LE.0.) THEN
             DDELT1=DDELTA
             D1=D
         END IF
C
         D=DELTAC
         DDELTA=DG0
      GO TO 3
C
    4 DEL_LN =DELTAC
C
      EPAIRR=E_CALK-E_SNGP+0.5*G_PAIR*PARTIC
      ECORHF=E_CALK-SUMESP+0.5*G_PAIR*PARTIC
C
      PCOREL=E_CALK-E_SNGP
C
      IF (IFSUMG.EQ.1) THEN
          PCOREL=PCOREL+0.5*PARTIC*G_PAIR
      END IF
C
      ESMOLN=RLNBAR
      PCOREC=PCOREL-ESMOLN
C
C=======================================================================
C
C                DEFINING V'S
C
C=======================================================================
C
      S2=0.
      S4=0.
      SC=0.
      SX=0.
C
      DO K=1,INWIND
         V2COEF=VV_COM(K)
C
C       B=(2*V_{k}2-1.)*2*V_{k}*SQRT(1.-V_{k}2)
C        =2*(V_{k}2-u_{k}2)*V_{k}*u_{k}
C      s2=4 \sum_{k} v_{k}^2 u_{k}^2
C      s4=\sum_{k} (2*V_{k}*SQRT(1.-V_{k}2))**4
C      SC=\sum_{k} 4*V_{k}^2*u_{k}^2(V_{k}2-u_{k}2)
C      SX=\sum_{k} (2*(V_{k}2-u_{k}2)*V_{k}*u_{k})**2
C
C
C
         V=SQRT(V2COEF)
         VV_LIN(K)=V
         S=2*V*SQRT(1.-V2COEF)
         B=(2*V2COEF-1.)*S
         S2=S2+S**2
         S4=S4+S**4
         SC=SC+S*B
         SX=SX+B**2
      END DO
C
C      OD=\sum_{k} v_{k}^2 u_{k}^2/
c        (\sum_{k} (2*(V_{k}2-u_{k}2)*V_{k}*u_{k})**2*4 *
C         \sum_{k} v_{k}^2 u_{k}^2-
C        (\sum_{k} 4*V_{k}^2*u_{k}^2(V_{k}2-u_{k}2))**2+
C         \sum_{k} v_{k}^2 u_{k}^2*(
C      16(\sum_{k} v_{k}^2 u_{k}^2)^2-
C         \sum_{k} (2*V_{k}*SQRT(1.-V_{k}2))**4))
C
C
C
      OD=0.25*S2/(SX*S2-SC*SC+0.25*S2*(S2*S2-S4))
C
      DO K=1,INWIND
         V2COEF=VV_COM(K)
         S=2*SQRT(V2COEF*(1.-V2COEF))
         C=2*V2COEF-1.
         V2CORR(K)=V2COEF+OD*S*S*(C*S2-SC)
      END DO
C
C      Storing the values of the V (not the V2) in VV_LIN
C
      DO I=1,INWIND
         VV_COM(I)=VV_LIN(I)
      END DO
C
      IF (LEVMIN.GT.1) THEN
          DO I=1,LEVMIN-1
             VV_LIN(I)=1.0
          END DO
      END IF
C
      IF (LEVMAX.LT.NUMLEV) THEN
          DO I=LEVMAX+1,NUMLEV
             VV_LIN(I)=0.0
          END DO
      END IF
C
      K=0
      DO I=LEVMIN,LEVMAX
         IF (NBLOCK(I).EQ.0) THEN
             K=K+1
             VV_LIN(I)=VV_COM(K)
         ELSE
             VV_LIN(I)=SQ2REC
         END IF
      END DO
C
      IF (K.NE.INWIND) STOP 'PAIRLN12'
C
C      Defining here the V coefficients that will be used (eventually)
C      in the rest of the code. We actually use the v2 (squared)
C
      DO I=1,INWIND
          VV_COM(I)=V2CORR(I)
      END DO
C
      IF (LEVMIN.GT.1) THEN
          DO I=1,LEVMIN-1
             V2CORR(I)=1.0
          END DO
      END IF
C
      IF (LEVMAX.LT.NUMLEV) THEN
          DO I=LEVMAX+1,NUMLEV
             V2CORR(I)=0.0
          END DO
      END IF
C
      K=0
      DO I=LEVMIN,LEVMAX
         IF (NBLOCK(I).EQ.0) THEN
             K=K+1
             V2CORR(I)=VV_COM(K)
         ELSE
             V2CORR(I)=0.5
         END IF
      END DO
C
      IF (K.NE.INWIND) STOP 'PAIRLN13'
C
C=======================================================================
C
      ICOMEN=0
C
      IF (IPRINT.EQ.1) THEN
C
          WRITE(6,'(/,80(''#''),/,''#'',78X,''#'',/,
     *            ''#      Lipkin-Nogami Projection and the '',
     *            ''Standard BCS Pairing Calculations      #'',/,
     *            ''#'',78X,''#'',/,
     *            ''#     '',A9,''   Z ='',I3,''   N ='',I3,4X,A10,
     *            '' G_PAIR='',F5.3,''  GFACTO='',F5.3,''     #'',/,
     *            ''#'',T63,''GACTIV='',F5.3,''     #'')')
     *
     *            NUCLEO(ISOSPI),IZ_FIX,IN_FIX,COMENT(ICOMEN),
     *                                         G_PAIR/FACTOG,
     *                                                FACTOG,
     *                                                G_PAIR
C
         WRITE(6,'(''#'',78X,''#'',/,
     *          ''#                                   '',
     *          ''DELTA='',F4.2,''  E_PAIR='',F6.2,'' ECOREC='',F6.2,
     *          ''    #'')')
     *
     *                                   DEL_LN,PCOREL,PCOREC
C
         KIKOFF=LEVMAX-LEVMIN+1
         ISTART=1
C
         KIKOFF=KIKOFF/2-1
C
         IF (ISTART+KIKOFF.GT.LEVMAX)
     *       STOP 'ISTART+KIKOFF.GT.LDBASE - IN PAIRLN <<= '
C
         WRITE(6,'(''#'',78X,''#'',/,
     *          ''#       No)     ESPAIR      V2lipk        '',
     *            ''   No)     ESPAIR      V2lipk        #'',/,
     *          ''#'',78X,''#'')')
C
         DO L=ISTART,ISTART+KIKOFF
C
            L1=L
            L2=KIKOFF+L+1
C
            WRITE(6,'(''#   '',I6,'') '',2F11.5,10X,
     *                   I3,'') '',2F11.5,5X,''   #'')')
     *             L1,SPECEN(L1),V2CORR(L1),
     *             L2,SPECEN(L2),V2CORR(L2)
         END DO
C
         WRITE(6,'(''#'',78X,''#'',/,80(''#''))')
C
      END IF
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WS_ETOTLN(XLAMBD,XLAMB2,DIFDEL,SUMESP)
C=======================================================================
      USE params
C=======================================================================
      EXTERNAL
     *          WS_EQNSLN
      COMMON
     *       /LIPKIN_LNAUXR/ SPECEN(1:NDLEVS),VV_COM(1:NDLEVS),
     *                                        G_PAIR,DELTAC
      COMMON
     *       /LIPKIN_LNAUXI/ NUMPAR,INWIND
      DATA
     *       EPSXY, EPSFG, ITER, ZERO
     *     / 1.D-9, 1.D-9,  200, 0.D0 /
C
C=======================================================================
C
C        Calculation of the Lipkin-Nogami energy term
C
C               ETOTLN=2 *       \sum_{k} v^2_k e_k-
C                     -G *       \sum_{k} v^4_k-
C                     -4 * XLAMB2\sum_{k} (u_k v_k)^2-
C                     -G *      [\sum_{k}  u_k v_k]^2
C        INPUT:
C
C        XLAMBD - Initial value for fermi energy (LAMBDA)
C        XLAMB2 - Initial value for LAMBDA2 in the LN method
C
C        OUTPUT:
C
C        XLAMBD - Selfconsistent value of LAMBDA
C        XLAMB2 - Selfconsistent value of LAMBDA2
C        DIFDEL - Pairing gap equation FUNCTION
C
C                 DIFDEL = DELTA - G_PAIR*SUM(U*V)
C
C=======================================================================
C
      PARTIC=NUMPAR
C
C=======================================================================
C          Here two nonlinear equations viz.
C=======================================================================
C
      CALL WS_SYSTEC(XLAMBD,XLAMB2,PARTIC,ZERO,WS_EQNSLN,EPSXY,EPSFG,
     *                                         ITER,IOK,DEVF,DEVG)
C
C=======================================================================
C
      IF (IOK.EQ.0) STOP 'WS_ETOTLN00'
C
C     SUMESP=2\sum_{k} v^2_k e_k
C     SUM_V4= \sum_{k} v^4_k
C     SUMUV2= \sum_{k} (u_k v_k)^2
C
      SUMESP=0
      SUM_V4=0
      SUMUV2=0
      SUMUV1=0
C
      DO K=1,INWIND
C
         V2COEF=VV_COM(K)
C
         U2COEF=ABS(1.00000-V2COEF)
         UVCOEF=SQRT(U2COEF*V2COEF)
         SUMESP=SUMESP+2*SPECEN(K)*V2COEF
         SUM_V4=SUM_V4+V2COEF*V2COEF
         SUMUV2=SUMUV2+UVCOEF*UVCOEF
         SUMUV1=SUMUV1+UVCOEF
C
      END DO
C
C=======================================================================
C                  DIFDEL= DELTAC-G * \sum_{k} u_k v_k
C=======================================================================
C
      DIFDEL=DELTAC-G_PAIR*SUMUV1
      WS_ETOTLN=SUMESP-G_PAIR*SUM_V4-4*XLAMB2*SUMUV2
     *                -G_PAIR*SUMUV1*SUMUV1
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_EQNSLN(XLAMBD,XLAMB2,F1FUNC,F2FUNC,IFSTOR)
C=======================================================================
      USE params
C=======================================================================
      COMMON
     *       /LIPKIN_LNAUXR/ SPECEN(1:NDLEVS),VV_COM(1:NDLEVS),
     *                                        G_PAIR,DELTAC
      COMMON
     *       /LIPKIN_LNAUXI/ NUMPAR,INWIND
C
      DATA
     *      EPS,  EPSF,  ITER
     *    /1.E-9, 1.E-9,  100  /
C
C=======================================================================
C
C       Calculation of occupation probabilities VV2 and functions
C
C       F1FUNC(XLAMBD,XLAMB2) = \sum (2V**2)
C       and
C       F2FUNC(XLAMBD,XLAMB2)
C
C=======================================================================
C
C Input
C       XLAMBD - Fermi energy (LAMBDA)
C       XLAMB2 - LAMBDA2 in the Lipkin-Nogami method
C       IFSTOR - IF 1 the new V2COEF's are stored in a COMMON
C Output
C
C       F1FUNC - \sum (2V**2);  F1FUNC = PARTIC implies AV(N)=N)
C
C       F2FUNC - XLAMB2-G_PAIR/4*(SUM(U**3*V)SUM(V**3*U)-SUM(U**4*V**4))
C                              /((SUM(U**2*V**2))**2   - SUM(U**4*V**4))
C
C                               F2FUNC = 0      implies AV(H*N**2) = 0
C
C=======================================================================
C
      ALPHA=4*XLAMB2-G_PAIR
      D2=DELTAC**2
C
      SVCOE2=0
      SUVCO4=0
      SU3VCO=0
      SUV3CO=0
      SUVCO2=0
C
      DO 2 K=1,INWIND
C
C=======================================================================
C
C       ==>> INITIAL GUESS FOR EP
C
C=======================================================================
C
         E=SPECEN(K)-XLAMBD
         EQ=SQRT(E*E+D2)
         V2COEF=0.5*(1-E/EQ)
C
         IF (D2.LT.1.D-5) GO TO 1
C
         F=V2COEF*ALPHA
         DERF=-0.5*ALPHA*D2/EQ**3-1
C
C=======================================================================
C
C        ==>> ITERATION LOOP
C
C=======================================================================
C
         IF (ABS(F).GE.EPS) THEN
            X=E
            DO I=1,ITER
               DX=F/DERF
               X=X-DX
               EQ=SQRT(D2+X*X)
               V2COEF=0.5*(1-X/EQ)
               F=E+ALPHA*V2COEF-X
               DERF=-ALPHA*0.5*D2/EQ**3-1
               IF(ABS(DX).LT.EPS.AND.ABS(F).LE.EPSF) GO TO 1
            END DO
            IF (D2.LT.0.02D0.AND.E.GE.-ALPHA.AND.E.LE.0.0D0) THEN
               V2COEF=-E/ALPHA
               GO TO 1
            ELSE
               PRINT *,' ITERATIONS IN WS_EQNSLN...'
               STOP 'WS_EQNSLN00'
            END IF
         END IF
C
    1    CONTINUE
C
         U2COEF=1.00-V2COEF
         UVCOEF=SQRT(U2COEF*V2COEF)
         SVCOE2=SVCOE2+V2COEF
         SUVCO4=SUVCO4+U2COEF*U2COEF*V2COEF*V2COEF
         SUVCO2=SUVCO2+U2COEF*V2COEF
         SU3VCO=SU3VCO+UVCOEF*U2COEF
         SUV3CO=SUV3CO+UVCOEF*V2COEF
C
         IF (IFSTOR.EQ.1) VV_COM(K)=V2COEF
C
    2 CONTINUE
C
      F1FUNC=2*SVCOE2
      F2FUNC=XLAMB2-G_PAIR/4*(SU3VCO*SUV3CO-SUVCO4)
     *                      /(SUVCO2*SUVCO2-SUVCO4)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_SYSTEC(XARGUM,YARGUM,FSOUGH,GSOUGH,WS_FNPROJ,
     *                     EPS_XY,EPS_FG,NOFITE,IERROR,DIFFEF,DIFFEG)
      PARAMETER
     *          (TP=1.001,TM=0.999,DETMIN=1.0E-16)
      DIMENSION
     *           EX(0:7),EY(0:7)
      EXTERNAL
     *           WS_FNPROJ
      DATA
     *           EX /  TP,TP,0.00,TM,  TM,TM,0.00,TP/
     *           EY /0.00,TM,  TM,TM,0.00,TP,  TP,TP/
     *    BERX,BERY /0.010,0.010/
C
C=======================================================================
C
C                    DEDICATED VERSION OF SYSTEM
C
C     THE SUBROUTINE WS_SOLVES (USING NEWTON'S METHOD) SYSTEM OF TWO
C        NONLINEAR EQUATIONS COMPUTING DERIVATIVES NUMERICALLY
C
C=======================================================================
C
      DEVMIN=1.0E+37
C
      DO 3 I=3,NOFITE
C
         DELTAX=BERX
         DELTAY=BERY
C
         IERROR=I
C
         CALL WS_FNPROJ(XARGUM,YARGUM,FUNCF0,FUNCG0,0)
C
         DLFUNF=FUNCF0-FSOUGH
         DLFUNG=FUNCG0-GSOUGH
C
         DEVIAF=ABS(DLFUNF)
         DEVIAG=ABS(DLFUNG)
C
         DEVFPG=DEVIAF+DEVIAG
C
C=======================================================================
C        ==>> CHECK IF THE BEST SO FAR
C=======================================================================
C
         IF (DEVFPG.LT.DEVMIN) THEN
             DIFFEF=DEVIAF
             DIFFEG=DEVIAG
             DEVMIN=DEVFPG
             X_BEST=XARGUM
             Y_BEST=YARGUM
         END IF
C
C=======================================================================
C
C        ==>> CHECK IF DONE
C
C=======================================================================
C
         IF (DEVFPG.LE.EPS_FG) GO TO 4
C
C=======================================================================
C
C        ==>> DERIVATIVES WITH RESPECT TO Y
C
C=======================================================================
C
    1    CALL WS_FNPROJ(XARGUM,YARGUM+DELTAY,FUNCFY,FUNCGY,0)
C
         DELTFY=FUNCFY-FUNCF0
         IF (ABS(DELTFY)/(ABS(FUNCF0)+ABS(FUNCFY)+DETMIN).LE.1.0E-10)
     *                                                          THEN
             DELTAY=2*DELTAY
             GO TO 1
         END IF
         DELTGY=FUNCGY-FUNCG0
         IF (ABS(DELTGY)/(ABS(FUNCG0)+ABS(FUNCGY)+DETMIN).LE.1.0E-10)
     *                                                          THEN
             DELTAY=2*DELTAY
             GO TO 1
         END IF
C
         DFOVDY=DELTFY/DELTAY
         DGOVDY=DELTGY/DELTAY
C
C=======================================================================
C       ==>> DERIVATIVES WITH RESPECT TO X
C=======================================================================
C
    2    CALL WS_FNPROJ(XARGUM+DELTAX,YARGUM,FUNCFX,FUNCGX,0)
C
         DELTFX=FUNCFX-FUNCF0
         IF (ABS(DELTFX)/(ABS(FUNCF0)+ABS(FUNCFX)+DETMIN).LE.1.E-10)
     *                                                         THEN
            DELTAX=2*DELTAX
            GO TO 2
         END IF
C
         DELTGX=FUNCGX-FUNCG0
         IF (ABS(DELTGX)/(ABS(FUNCG0)+ABS(FUNCGX)+DETMIN).LE.1.E-10)
     *                                                         THEN
            DELTAX=2*DELTAX
            GO TO 2
         END IF
C
         DFOVDX=DELTFX/DELTAX
         DGOVDX=DELTGX/DELTAX
C
         DETERM=DFOVDX*DGOVDY-DFOVDY*DGOVDX
C
C=======================================================================
C        ==>> CHECK FOR PLATEAU
C=======================================================================
C
         IF (ABS(DETERM).LE.DETMIN) THEN
             II=MOD(I-1,8)
             XARGUM=EX(II)*XARGUM
             YARGUM=EY(II)*YARGUM
             GO TO 3
         END IF
C
C=======================================================================
C        ==>> INCREMENTS IN ARGUMENTS
C=======================================================================
C
         DX_NEW=(DLFUNF*DGOVDY-DLFUNG*DFOVDY)/DETERM
         DY_NEW=(DLFUNG*DFOVDX-DLFUNF*DGOVDX)/DETERM
C
C=======================================================================
C
C        ==>> CHECK IF INCREMENTS SMALL ENOUGH
C
C=======================================================================
C
         IF (ABS(DX_NEW)+ABS(DY_NEW).LE.EPS_XY) GO TO 4
C
C=======================================================================
C
C        ==>> CORRECT X AND Y AND PROCEED
C
C=======================================================================
C
         XARGUM=XARGUM-DX_NEW
         YARGUM=YARGUM-DY_NEW
C
    3 END DO
C
C=======================================================================
C     ==>> NO CONVERGENCE
C=======================================================================
C
      IERROR=0
C
C=======================================================================
C     ==>> EXIT
C=======================================================================
C
   4  CONTINUE
C
      XARGUM=X_BEST
      YARGUM=Y_BEST
C
C=======================================================================
C
      CALL WS_FNPROJ(XARGUM,YARGUM,FUNCF0,FUNCG0,1)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
