C FILE NAME = wsu14_moms_07.f   ! Keep this symbol here: $ident@string$
C
C=======================================================================
C=======================================================================
C=====                                ==================================
C=====     MULTIPOLE MOMENTS          ==================================
C=====                                ==================================
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_MOMETS(ICHARG,IZ_FIX,IN_FIX,IFMULT,MAXMUL,NOFLEV,
     *                            KRANKG,ISIGNY,IINDIV,LAMIND,MIUIND)
C=======================================================================
      USE params
C=======================================================================
C
      COMPLEX
     *          RESULT,RESODD,SINGPA
      COMPLEX
     *          EWAVEF,BIG_PP,BIG_PM,WAVOCC
C
C=======================================================================
      ALLOCATABLE ELEMUL(:,:)
C=======================================================================
      DIMENSION
     *          QUADRU(1:NDBASE)
C
      COMMON
     *       /PAIRIN_SWITCH/ IFPAIR,IFPROJ,IF_HFB,IF_LIP
      COMMON
     *       /TPHASE_PHASES/ TPHAPP(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAPM(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAMP(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAMM(0:NDYMAX,0:NDYMAX,0:NDKART)
      COMMON
     *       /SPECTR_SPINOR/ EWAVEF(1:NDBASE,1:NDBASE)
      COMMON
     *       /WSEXEC_V2OCCU/ V2OCCU(1:NDBASE)
      COMMON
     *       /CRAN3D_MAT_PP/ BIG_PP(1:NDBASE,1:NDBASE,0:NDREVE)
     *       /CRAN3D_MAT_PM/ BIG_PM(1:NDBASE,1:NDBASE,0:NDREVE)
      COMMON
     *       /CRAN3D_ALLWAV/ WAVOCC(1:NDBASE,1:2*NDBASE,0:NDREVE)
      COMMON
     *       /MOMETS_MULMOM/ QMOMUL(0:NDMULT,-NDMULT:NDMULT,0:NDISOS)
      COMMON
     *       /SETBAS_DIMENS/ LDBASE
      COMMON
     *       /CONSER_SIMPLY/ ISIMPY
      DATA
     *        EPSMUL /.1E-16 /
C
C=======================================================================
C
C       This subroutine calculates and returns in "QMOMUL" the average
C       values of the electromagnetic multipole moments
C
C                     R**lambda * Y(lambda,miu)
C
C       For the parity/signature case (ISIGNY=1) only the even moments
C       are calculated. After the calculations, the total multipole
C       moments as well as the individual multipole moments are stored.
C       The current subroutine can not deal with odd nuclei in the
C       cranking case.
C
C=======================================================================
C
      IF (IFMULT.NE.1) RETURN
C
C=======================================================================
C
      CALL WS_CPUTIM('MOMETS',1)
C
C=======================================================================
C
      DO LAMBDA=0,NDMULT
         DO MIU=-LAMBDA,LAMBDA
            QMOMUL(LAMBDA,MIU,ICHARG)=0.0
         END DO
      END DO
C
C      LDMAXI:
C
      LDMAXI=180
C
C      By default, calculations are done with no cranking, when
C      each level is occupied by TWO particles.
C
C      In the cranking case, each level is occupied by ONE particle
C      only.
C
      IF (ICHARG.EQ.0) THEN
          NFERMI=IN_FIX/2
	  IMODUL=MOD(IN_FIX,2)
      ELSE
          NFERMI=IZ_FIX/2
	  IMODUL=MOD(IZ_FIX,2)
      END IF
C
      IF (KRANKG.LT.0) THEN
C
          IMODUL=0
C
                           NFERMI=IZ_FIX
          IF (ICHARG.EQ.0) NFERMI=IN_FIX
C
      END IF
C
C      In case of pairing, levels beyond the Fermi level should also
C      be included. For pairing applications, we take a pairing window
C      equal to twice the number of particles.
C
      IF (IFPAIR.EQ.1) THEN
          LDSUMM=2*NFERMI
      ELSE
          LDSUMM=NFERMI
      END IF
C
C      In case of no-cranking, we compute first the density matrix from
C      the WS eigenstates
C
      IF (KRANKG.EQ.0) THEN
C
          CALL WS_DENMAC()
C
      END IF
C
C=======================================================================
C
C      Here the real parts of moments are calculated and stored
C      in the "QMOMUL" matrix for non-negative values of "miu".
C
C      With the current phase convention, multipole moments have a
C      non-zero real part only if the simplex is conserved.
C
C=======================================================================
C
      ALLOCATE(ELEMUL(1:NDBASE,1:NDBASE))
C
      IMAREA=1
C
      DO LAMBDA=0,MAXMUL
         DO MIU=0,LAMBDA
C
            IF (MOD(LAMBDA,2).EQ.0.OR.ISIGNY.NE.1) THEN
C
                CALL WS_INTMUL(TPHAPP(0,0,0),ELEMUL,LAMBDA,MIU,IMAREA)
C
                RESULT=CMPLX(0.0,0.0)
C
C                Case of no-cranking at all
C
                IF (KRANKG.EQ.0) THEN
C
                    DO IBRA=1,LDBASE
                       DO IKET=1,LDBASE
C
                          RESULT=RESULT + ELEMUL(IBRA,IKET)
     *                                  *(BIG_PP(IKET,IBRA,0)
     *                                  + BIG_PP(IKET,IBRA,1))
C
                       END DO
                    END DO
C
C                    For specific purposes, one calculates the individual
C                    quadrupole moments beyond the pairing window (or
C                    the Fermi level if no pairing)
C
		    IF (IINDIV.EQ.1.AND.LAMBDA.EQ.LAMIND
     *                                .AND.MIU.EQ.MIUIND) THEN
C
                        DO ISTATE=LDSUMM+1,LDMAXI
C
                           SINGPA=CMPLX(0.0,0.0)
C
                           DO J=1,LDBASE
                              DO K=1,LDBASE
C
                                 SINGPA=SINGPA+CONJG(BIG_PP(J,ISTATE,0))
     *
     *                                              *ELEMUL(J,K)
     *
     *                                              *BIG_PP(K,ISTATE,0)
C
                              END DO
                           END DO
C
		           QUADRU(ISTATE)=REAL(SINGPA)*V2OCCU(ISTATE)
C
		         END DO
C
		    END IF
C
C                    Case of odd nuclei: we include the last particle
C                    occupying the level NFERMI+1. This is only neces-
C                    sary for KRANKG=0 (no rotation) where levels are
C                    doubly-degenerate, and in the case of no pairing.
C
                    ODDPAR=0.0
C
		    IF (IMODUL.EQ.1.AND.IFPAIR.EQ.0) THEN
C
                        ISTATE=NFERMI+1
C
                        RESODD=CMPLX(0.0,0.0)
C
                        DO J=1,LDBASE
                           DO K=1,LDBASE
C
                              RESODD=RESODD+CONJG(BIG_PP(J,ISTATE,0))
     *
     *                                           *ELEMUL(J,K)
     *
     *                                           *BIG_PP(K,ISTATE,0)
C
                           END DO
                        END DO
C
                        ODDPAR=REAL(RESODD)*V2OCCU(ISTATE)
C
		    END IF
C
C                    Final result for the quadrupole moment. The factor
C                    2 comes from the Kramers degeneracy.
C
                    QMOMUL(LAMBDA,MIU,ICHARG)=REAL(RESULT)+ODDPAR
C
	        END IF
C
C	         Case of 1D-cranking (y-simplex is automatically conserved)
C
	        IF (KRANKG.GT.0) THEN
C
                    DO ISTATE=1,LDSUMM
C
		       SINGPA=CMPLX(0.0,0.0)
C
                       DO J=1,LDBASE
                          DO K=1,LDBASE
C
                             SINGPA=SINGPA+ELEMUL(J,K)
     *
     *          *CONJG(BIG_PP(J,ISTATE,0))*BIG_PP(K,ISTATE,0)
     *                                    *V2OCCU(ISTATE)
     *
     *                                    +ELEMUL(J,K)
     *
     *          *CONJG(BIG_PP(J,ISTATE,1))*BIG_PP(K,ISTATE,1)
     *                                    *V2OCCU(ISTATE)
C
                          END DO
                       END DO
C
C                       For specific purposes, one stores the individual
C                       quadrupole moments
C
                       IF (IINDIV.EQ.1.AND.LAMBDA.EQ.LAMIND
     *                                   .AND.MIU.EQ.MIUIND) THEN
		           QUADRU(ISTATE)=REAL(SINGPA)
                       END IF
C
		       RESULT=RESULT+SINGPA
C
		    END DO
C
                    QMOMUL(LAMBDA,MIU,ICHARG)=REAL(RESULT)
C
	        END IF
C
C                Case of a 3D-cranking with accidentally conserved simplex
C
	        IF ((KRANKG.LT.0.AND.ISIMPY.EQ.1) .OR.
     *              (KRANKG.LT.0.AND.LAMBDA.EQ.0.AND.MIU.EQ.0)) THEN
c
                    DO ISTATE=1,LDSUMM
C
		       SINGPA=CMPLX(0.0,0.0)
C
                       DO J=1,LDBASE
                          DO K=1,LDBASE
C
                             SINGPA=SINGPA+ELEMUL(J,K)
     *
     *          *CONJG(WAVOCC(J,ISTATE,0))*WAVOCC(K,ISTATE,0)
     *                                    *V2OCCU(ISTATE)
     *
     *                                    +ELEMUL(J,K)
     *
     *          *CONJG(WAVOCC(J,ISTATE,1))*WAVOCC(K,ISTATE,1)
     *                                    *V2OCCU(ISTATE)
C
                          END DO
                       END DO
C
		       RESULT=RESULT+SINGPA
C
C                       For specific purposes, one stores the individual
C                       quadrupole moments
C
                       IF (IINDIV.EQ.1.AND.LAMBDA.EQ.LAMIND
     *                                   .AND.MIU.EQ.MIUIND) THEN
		           QUADRU(ISTATE)=REAL(SINGPA)
                       END IF
C
		    END DO
C
                    QMOMUL(LAMBDA,MIU,ICHARG)=REAL(RESULT)
C
	       END IF
C
            ELSE
C
                QMOMUL(LAMBDA,MIU,ICHARG)=0.0
C
            END IF
C
         END DO
      END DO
C
C=======================================================================
C
C      Here the imaginary parts of moments are calculated and stored
C      in the "QMOMUL" matrix for negative values of "miu".
C
C      With the current phase convention, multipole moments have a
C      non-zero imaginary part only if the simplex is NOT conserved.
C      In the current version of the code, it can happen ONLY in the
C      case of 3D cranking.
C
C=======================================================================
C
      IF (KRANKG.LT.0.AND.MAXMUL.GT.0) THEN
C
          IMAREA=-1
C
          DO LAMBDA=1,MAXMUL
             DO MIU=-LAMBDA,-1
C
                IF (MOD(LAMBDA,2).EQ.1.OR.ISIGNY.NE.1) THEN
C
                    CALL WS_INTMUL(TPHAPM(0,0,0),ELEMUL,LAMBDA,MIU,
     *                                                      IMAREA)
C
                    RESULT=CMPLX(0.0,0.0)
C
                    DO ISTATE=1,LDSUMM
C
		       SINGPA=CMPLX(0.0,0.0)
C
                       DO J=1,LDBASE
                          DO K=1,LDBASE
C
                             SINGPA=SINGPA+CONJG(WAVOCC(J,ISTATE,0))
     *
     *                                          *ELEMUL(J,K)
     *
     *                                          *WAVOCC(K,ISTATE,1)
     *
     *                                          *V2OCCU(ISTATE)
C
                          END DO
                       END DO
C
		       RESULT=RESULT+SINGPA
C
C                       For specific purposes, one stores the individual
C                       quadrupole moments
C
                       IF (IINDIV.EQ.1.AND.LAMBDA.EQ.LAMIND
     *                                   .AND.MIU.EQ.MIUIND) THEN
		           QUADRU(ISTATE)=-2.0*REAL(SINGPA)
                       END IF
C
		    END DO
C
                    QMOMUL(LAMBDA,MIU,ICHARG)=-2.0*REAL(RESULT)
C
                ELSE
C
                    QMOMUL(LAMBDA,MIU,ICHARG)=0.0
C
                END IF
C
             END DO
          END DO
C
      END IF
C
      DEALLOCATE(ELEMUL)
C
C=======================================================================
C
C      If requested, printing of the individual quadrupole moments
C
C=======================================================================
C
      IF (IINDIV.EQ.1) THEN
C
                           TIMREV=2.0
          IF (KRANKG.NE.0) TIMREV=1.0
C
          WRITE(6,'(/,80(''#''),/,''#'',78x,''#'',/,
     *              ''#                  Single-particle Multipole'',
     *              '' Moment [barn]                     #'',/,
     *              ''#'',78x,''#'',/,80(''#'')/,''#'',78x,''#'',/,
     *              ''#    No)     q'',I1,I1,''   Q(tot)'',
     *              ''    No)     q'',I1,I1,''   Q(tot)'',
     *              ''    No)     q'',I1,I1,''   Q(tot)      #'',/,
     *              ''#'',78x,''#'')')
     *                LAMIND,MIUIND,LAMIND,MIUIND,LAMIND,MIUIND
C
          KIKOFF=LDMAXI
          ISTART=1
C
          KIKOFF=KIKOFF/3-1
C
          IF (ISTART+KIKOFF.GT.LDMAXI)
     *        STOP 'ISTART+KIKOFF.GT.LDBASE - IN PAIRLN <<= '
C
          SUMFIR=0.0
          DO L=ISTART,ISTART+KIKOFF
	     SUMFIR=SUMFIR+QUADRU(L)
	  END DO
C
          SUMSEC=SUMFIR
          DO L=ISTART+KIKOFF+1,ISTART+2*KIKOFF
	     SUMSEC=SUMSEC+QUADRU(L)
	  END DO
C
          S_COL1=0.0
          S_COL2=SUMFIR
          S_COL3=SUMSEC
C
          DO L=ISTART,ISTART+KIKOFF
C
             L1=L
             L2=KIKOFF+L+1
             L3=KIKOFF+KIKOFF+L+1
C
	     S_COL1=S_COL1+QUADRU(L1)*TIMREV
	     S_COL2=S_COL2+QUADRU(L2)*TIMREV
	     S_COL3=S_COL3+QUADRU(L3)*TIMREV
C
             WRITE(6,'(''#'',3(3X,I3,'')'',F9.5,F8.3),6X,''#'')')
     *             L1,QUADRU(L1)*TIMREV,S_COL1,
     *             L2,QUADRU(L2)*TIMREV,S_COL2,
     *             L3,QUADRU(L3)*TIMREV,S_COL3
          END DO
C
          WRITE(6,'(''#'',78X,''#'',/,80(''#''))')
C
      END IF
C
C=======================================================================
C
      CALL WS_CPUTIM('MOMETS',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_INTMUL(C_PHAS,ELEMUL,LAMBDA,MIU,IMAREA)
C=======================================================================
      USE params
C=======================================================================
C
      DIMENSION
     *           ELEMUL(1:NDBASE,1:NDBASE)
      DIMENSION
     *           C_PHAS(0:NDYMAX,0:NDYMAX)
C
      COMMON
     *       /DEVMUL_COEMUL/ COMULT(0:NDMULT,0:NDOSGS,0:NDOSGS,1:NDKART)
      COMMON
     *       /SETBAS_NXNYNZ/ NXVECT(1:NDBASE),
     *                       NYVECT(1:NDBASE),
     *                       NZVECT(1:NDBASE)
      COMMON
     *       /SETBAS_DIMENS/ LDBASE
      COMMON
     *       /INTMUL_COEYLM/ COEYLM(0:NDMULT,0:NDMULT)
      COMMON
     *       /INTMUL_IMAREA/ NREYLM(0:NDMULT,0:NDMULT),
     *                       KREYLM(0:NDMULT,0:NDMULT,NDTERM,0:NDKART),
     *
     *                       NIMYLM(0:NDMULT,1:NDMULT),
     *                       KIMYLM(0:NDMULT,1:NDMULT,NDTERM,0:NDKART)
      COMMON
     *       /DEFUNI_OURUNI/ QUNITS(0:NDMULT,0:NDMULT)
C
C=======================================================================
C
C        This subroutine calculates matrix elements of multipole
C        operators within the harmonic oscillator basis.  Factor
C                         IMAREA = 1 (-1)
C        requests calculation of the real (imaginary) part
C
C=======================================================================
C
      CALL WS_CPUTIM('INTMUL',1)
C
C=======================================================================
C
      IF (MIU.GE.0) THEN
          JPHASE=IMAREA
      ELSE
          JPHASE=(-1)**MIU
      END IF
C
      DO IKETHO=1,LDBASE
         DO IBRAHO=1,LDBASE
            ELEMUL(IBRAHO,IKETHO)=0.0
         END DO
      END DO
C
C=======================================================================
C         Calculating the real part
C=======================================================================
C
      IF (IMAREA.EQ.1) THEN
C
          DO NUTERM=1,NREYLM(LAMBDA,MIU)
C
             DO IKETHO=1,LDBASE
C
	        NX_KET=NXVECT(IKETHO)
	        NY_KET=NYVECT(IKETHO)
	        NZ_KET=NZVECT(IKETHO)
C
                DO IBRAHO=1,LDBASE
C
	           NX_BRA=NXVECT(IBRAHO)
	           NY_BRA=NYVECT(IBRAHO)
	           NZ_BRA=NZVECT(IBRAHO)
C
                   test=
     *
     *                    KREYLM(LAMBDA,MIU,NUTERM,0)
     *
     *          *  COMULT(KREYLM(LAMBDA,MIU,NUTERM,1),
     *                           NX_BRA,NX_KET,1)
     *
     *          *  COMULT(KREYLM(LAMBDA,MIU,NUTERM,2),
     *                           NY_BRA,NY_KET,2)
     *
     *          *  COMULT(KREYLM(LAMBDA,MIU,NUTERM,3),
     *                           NZ_BRA,NZ_KET,3)
     *
     *          *  JPHASE
C
                   ELEMUL(IBRAHO,IKETHO)=ELEMUL(IBRAHO,IKETHO)
     *                                  +test
C
                END DO
             END DO
          END DO
C
      ELSE
C
C=======================================================================
C         Calculating the imaginary part
C=======================================================================
C
          IF (MIU.EQ.0) STOP 'NO IMAGINARY PART FOR MIU=0 in INTMUL'
C
          DO NUTERM=1,NIMYLM(LAMBDA,MIU)
C
             DO IKETHO=1,LDBASE
C
	        NX_KET=NXVECT(IKETHO)
	        NY_KET=NYVECT(IKETHO)
	        NZ_KET=NZVECT(IKETHO)
C
                DO IBRAHO=1,LDBASE
C
	           NX_BRA=NXVECT(IBRAHO)
	           NY_BRA=NYVECT(IBRAHO)
	           NZ_BRA=NZVECT(IBRAHO)
C
                   ELEMUL(IBRAHO,IKETHO)=ELEMUL(IBRAHO,IKETHO)
     *
     *                  + KIMYLM(LAMBDA,MIU,NUTERM,0)
     *
     *          *  COMULT(KIMYLM(LAMBDA,MIU,NUTERM,1),
     *                           NX_BRA,NX_KET,1)
     *
     *          *  COMULT(KIMYLM(LAMBDA,MIU,NUTERM,2),
     *                           NY_BRA,NY_KET,2)
     *
     *          *  COMULT(KIMYLM(LAMBDA,MIU,NUTERM,3),
     *                           NZ_BRA,NZ_KET,3)
     *
     *           * JPHASE
C
                END DO
             END DO
          END DO
C
      END IF
C
C=======================================================================
C         Normalization factor and phase. COEYLM contains the overall
C         numerical factor in fronto the expansion of the solid harmo-
C         nics in cartesian coordinates. C_PHAS takes into account
C         the particular structure of the t-basis.
C=======================================================================
C
      DO IKETHO=1,LDBASE
         NY_KET=NYVECT(IKETHO)
         DO IBRAHO=1,LDBASE
            NY_BRA=NYVECT(IBRAHO)
C
            ELEMUL(IBRAHO,IKETHO) = ELEMUL(IBRAHO,IKETHO)
     *
     *                        * COEYLM(LAMBDA,MIU)
     *
     *                        * C_PHAS(NY_BRA,NY_KET)
C
         END DO
      END DO
C
C      Follows multiplication by the units defined once for all in
C      the subroutine DEFUNI
C
      NIU=ABS(MIU)
C
      DO IKETHO=1,LDBASE
         DO IBRAHO=1,LDBASE
            ELEMUL(IBRAHO,IKETHO)=ELEMUL(IBRAHO,IKETHO)*
     *                               QUNITS(LAMBDA,NIU)
         END DO
      END DO
C
C=======================================================================
C
      CALL WS_CPUTIM('INTMUL',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_MOMPRI(NMOMUL,ISOSPI,IFMULT,OMEGAC,KRANKG)
C=======================================================================
      USE params
C=======================================================================
C
      CHARACTER
     *          PRIMOM*15
      CHARACTER
     *          PRIMIU*2
C
      DIMENSION
     *          PRIMOM(-NDMULT:NDMULT)
C
      COMMON
     *       /MOMETS_MULMOM/ QMOMUL(0:NDMULT,-NDMULT:NDMULT,0:NDISOS)
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
C=======================================================================
C
C       This subroutine prints the average values  of  electromagnetic
c       multipole moments: R**lambda*CONJG(Y(lambda,miu))
C
C=======================================================================
C
      IF (ISOSPI.EQ.1) THEN
C
          IF (IWRITE.GE.1)
     *    WRITE(NOUTPT,'(/,80(''#''),/,       ''#'',78X,''#'',/,
     *          ''#  Multipole Moments in units of (10 Fermi)'',
     *          ''**Lambda   [OMEGA='',F4.2,'']   PROTONS   #'',/,
     *          ''#'',78X,''#'')')    OMEGAC
C
      END IF
C
      IF (ISOSPI.EQ.0) THEN
C
          IF (IWRITE.GE.1)
     *    WRITE(NOUTPT,'(/,80(''#''),/,       ''#'',78X,''#'',/,
     *          ''#  Multipole Moments in units of (10 Fermi)'',
     *          ''**Lambda   [OMEGA='',F4.2,'']  NEUTRONS   #'',/,
     *          ''#'',78X,''#'')')    OMEGAC
C
      END IF
C
C=======================================================================
C
      IF (KRANKG.LT.0) THEN
C
          WRITE(6,'(1H#,2X,''Real part  for a non-negative projection'',
     *                                                        36X,1H#,/,
     *              1H#,2X,''Imaginary part for a negative projection'',
     *                                                        36X,1H#,/,
     *                                      1H#,78X,1H#,/,
     *                                          80(1H#))')
C
      ELSE
C
          WRITE(6,'(80(1H#))')
C
      ENDIF
C
C=======================================================================
C
      DO MIU=-NDMULT,NDMULT
         PRIMOM(MIU)='  .............'
      END DO
C
      DO LAMBDA=0,MIN(NMOMUL,NDMULT)
C
         DO MIU=-LAMBDA,LAMBDA
C
            QMOMEN = QMOMUL(LAMBDA,MIU,ISOSPI)
C
            IF (KRANKG.GE.0) THEN
                WRITE(PRIMIU,'(I1,1X)') ABS(MIU)
            ELSE
                WRITE(PRIMIU,'(I2)') MIU
                IF (PRIMIU(1:2).EQ.' 0') PRIMIU(1:2)='0 '
                IF (PRIMIU(1:1).EQ.' ' ) PRIMIU(1:1)='+'
            ENDIF
C
            IF (ABS(QMOMEN).LT.1.0E-10) THEN
C
                WRITE(PRIMOM(MIU),'(''  Q'',I1,A2,''='',4X,''ZERO'')')
     *                                                   LAMBDA,PRIMIU
C
            ELSE
C
               IF (ABS(QMOMEN).LT.1.0E-3.OR.QMOMEN.GT.1000.OR.
     *                                      QMOMEN.LT.-100) THEN
C
                   WRITE(PRIMOM(MIU),'(''  Q'',I1,A2,''='',1PE8.1)')
     *                                          LAMBDA,PRIMIU,QMOMEN
C
               ELSE
C
                   WRITE(PRIMOM(MIU),'(''  Q'',I1,A2,''='',F8.4)')
     *                                        LAMBDA,PRIMIU,QMOMEN
C
               END IF
C
            END IF
C
         END DO
C
C=======================================================================
C
         WRITE(6,'(1H#,78X,1H#,/,1H#,5A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=0,4)
         IF (KRANKG.LT.0)
     *   WRITE(6,'(1H#,          15X,4A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-1,-4,-1)
C
C=======================================================================
C
         IF (LAMBDA.EQ.5) THEN
         WRITE(6,'(1H#,78X,1H#,/,1H#,60X, A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=5,5)
         IF (KRANKG.LT.0)
     *   WRITE(6,'(1H#,              60X, A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-5,-5)
         END IF
C
C=======================================================================
C
         IF (LAMBDA.EQ.6) THEN
         WRITE(6,'(1H#,78X,1H#,/,1H#,45X,2A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=5,6)
         IF (KRANKG.LT.0)
     *   WRITE(6,'(1H#,              45X,2A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-5,-6,-1)
         END IF
C
C=======================================================================
C
         IF (LAMBDA.EQ.7) THEN
         WRITE(6,'(1H#,78X,1H#,/,1H#,30X,3A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=5,7)
         IF (KRANKG.LT.0)
     *   WRITE(6,'(1H#,              30X,3A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-5,-7,-1)
         END IF
C
C=======================================================================
C
         IF (LAMBDA.EQ.8) THEN
         WRITE(6,'(1H#,78X,1H#,/,1H#,15X,4A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=5,8)
         IF (KRANKG.LT.0)
     *   WRITE(6,'(1H#,              15X,4A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-5,-8,-1)
         END IF
C
C=======================================================================
C
         IF (LAMBDA.EQ.9) THEN
         WRITE(6,'(1H#,78X,1H#,/,1H#,    5A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=5,9)
         IF (KRANKG.LT.0)
     *   WRITE(6,'(1H#,                  5A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-5,-9,-1)
         END IF
C
      END DO
C
      WRITE(6,'(1H#,78X,1H#,/,80(1H#),/)')
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_MAGPRI(NMOMAG,ISOSPI,IFMULT,OMEGAC,KRANKG)
C=======================================================================
      USE params
C=======================================================================
C
      DIMENSION
     *          DIPKAR(1:NDKART)
C
      CHARACTER
     *          PRIMOM(-NDMULT:NDMULT)*15
      CHARACTER
     *          PRIMIU*2
C
      COMMON
     *       /DEFUNI_MAGUNI/ AUNITS(0:NDMULT,0:NDMULT)
      COMMON
     *       /MAGMOM_AMOMUL/ AMOMUL(0:NDMULT,-NDMULT:NDMULT,0:NDISOS)
      COMMON
     *       /READWRITE_PAR/ NOUTPT,IWRITE,ISCREN
C
C=======================================================================
C
C       This subroutine prints the average values of magnetic multipole
C       moments
C
C=======================================================================
C
      IF (NMOMAG.EQ.0) RETURN
C
      KRAOLD=KRANKG
      KRANKG=-5
C
C=======================================================================
C
      IF (ISOSPI.EQ.1) THEN
C
          IF (IWRITE.GE.1)
     *    WRITE(NOUTPT,'(/,80(''#''),/,''#'',78X,''#'',/,
     *          ''#  Magnetic Moments in units of mu_N = hbar*c/M'',
     *          ''       [OMEGA='',F4.2,'']   PROTONS   #'',/,
     *          ''#'',78X,''#'')')    OMEGAC
C
      END IF
C
      IF (ISOSPI.EQ.0) THEN
C
          IF (IWRITE.GE.1)
     *    WRITE(NOUTPT,'(/,80(''#''),/,''#'',78X,''#'',/,
     *          ''#  Magnetic Moments in units of mu_N = hbar*c/M'',
     *          ''       [OMEGA='',F4.2,'']   NEUTRONS  #'',/,
     *          ''#'',78X,''#'')')    OMEGAC
C
      END IF
C
C=======================================================================
C
      IF (KRANKG.LE.0) THEN
C
          WRITE(6,'(1H#,2X,''Real part  for a non-negative projection'',
     *                                                        36X,1H#,/,
     *              1H#,2X,''Imaginary part for a negative projection'',
     *                                                        36X,1H#,/,
     *                                      1H#,78X,1H#,/,
     *                                          80(1H#))')
C
      ELSE
C
          WRITE(6,'(80(''#''))')
C
      ENDIF
C
C=======================================================================
C
      DO MIU=-NDMULT,NDMULT
         PRIMOM(MIU)='  .............'
      END DO
C
      DO LAMBDA=1,MIN(NMOMAG,NDMULT)
C
         DO MIU=-LAMBDA,LAMBDA
C
            AMOMEN = AMOMUL(LAMBDA,MIU,ISOSPI)
C
            IF (KRANKG.GT.0) THEN
                WRITE(PRIMIU,'(I1,1X)') MIU
            ELSE
                WRITE(PRIMIU,'(I2)') MIU
                IF (PRIMIU(1:2).EQ.' 0') PRIMIU(1:2)='0 '
                IF (PRIMIU(1:1).EQ.' ' ) PRIMIU(1:1)='+'
            ENDIF
C
            IF (ABS(AMOMEN).LT.1.0E-10) THEN
C
                WRITE(PRIMOM(MIU),'(''  M'',I1,A2,''='',4X,''ZERO'')')
     *                                                   LAMBDA,PRIMIU
            ELSE
C
               IF (ABS(AMOMEN).LT.1.0E-3) THEN
C
                   WRITE(PRIMOM(MIU),'(''  M'',I1,A2,''='',1PE8.1)')
     *                                          LAMBDA,PRIMIU,AMOMEN
               ELSE
C
                   WRITE(PRIMOM(MIU),'(''  M'',I1,A2,''='',F8.4)')
     *                                        LAMBDA,PRIMIU,AMOMEN
               END IF
C
            END IF
C
         END DO
C
C=======================================================================
C
         WRITE(6,'(1H#,78X,1H#,/,1H#,5A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=0,4)
         IF (KRANKG.LE.0)
     *   WRITE(6,'(1H#,          15X,4A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-1,-4,-1)
C
C=======================================================================
C
         IF (LAMBDA.EQ.5) THEN
         WRITE(6,'(1H#,78X,1H#,/,1H#,60X, A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=5,5)
         IF (KRANKG.LE.0)
     *   WRITE(6,'(1H#,              60X, A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-5,-5)
         END IF
C
C=======================================================================
C
         IF (LAMBDA.EQ.6) THEN
         WRITE(6,'(1H#,78X,1H#,/,1H#,45X,2A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=5,6)
         IF (KRANKG.LE.0)
     *   WRITE(6,'(1H#,              45X,2A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-5,-6,-1)
         END IF
C
C=======================================================================
C
         IF (LAMBDA.EQ.7) THEN
         WRITE(6,'(1H#,78X,1H#,/,1H#,30X,3A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=5,7)
         IF (KRANKG.LE.0)
     *   WRITE(6,'(1H#,              30X,3A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-5,-7,-1)
         END IF
C
C=======================================================================
C
         IF (LAMBDA.EQ.8) THEN
         WRITE(6,'(1H#,78X,1H#,/,1H#,15X,4A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=5,8)
         IF (KRANKG.LE.0)
     *   WRITE(6,'(1H#,              15X,4A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-5,-8,-1)
         END IF
C
C=======================================================================
C
         IF (LAMBDA.EQ.9) THEN
         WRITE(6,'(1H#,78X,1H#,/,1H#,    5A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=5,9)
         IF (KRANKG.LE.0)
     *   WRITE(6,'(1H#,                  5A15,3X,1H#)')
     *                                        (PRIMOM(MIU),MIU=-5,-9,-1)
         END IF
C
      END DO
C
      WRITE(6,'(1H#,78X,1H#)')
C
C=======================================================================
C
      PI=4.0*ATAN(1.00)
C
      DIPKAR(1)=-SQRT(8*PI/3)*AMOMUL(1, 1,ISOSPI)/AUNITS(1,1)
      DIPKAR(2)= SQRT(8*PI/3)*AMOMUL(1,-1,ISOSPI)/AUNITS(1,1)
      DIPKAR(3)= SQRT(4*PI/3)*AMOMUL(1, 0,ISOSPI)/AUNITS(1,0)
C
      WRITE(6,'(79(1H#),/,                          1H#,77X,1H#,/,
     *                 1H#,2X,''Cartesian components of mag'',
     *                        ''netic dipole moment'',
     *                                      29X,1H#,/,
     *                                              1H#,77X,1H#,/,
     *                 1H#,2X,''M1_X ='',F9.4,2X,
     *                     2X,''M1_Y ='',F9.4,2X,
     *                     2X,''M1_Z ='',F9.4,3X,
     *                     2X,''M1 =  '',F9.4,2X,      1H#,/,
     *                                             1H#,77X,1H#,/,
     *                                               79(1H#),/)')
     *
     *
     *          (DIPKAR(KARTEZ),KARTEZ=1,NDKART),
     *           SQRT(DIPKAR(1)**2+DIPKAR(2)**2+DIPKAR(3)**2)
C
C=======================================================================
C
      KRANKG=KRAOLD
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DEFUNI(NMUMAX)
C=======================================================================
      USE params
C=======================================================================
      COMMON
     *       /DEFUNI_OURUNI/ QUNITS(0:NDMULT,0:NDMULT)
      COMMON
     *       /DEFUNI_MAGUNI/ AUNITS(0:NDMULT,0:NDMULT)
C
C=======================================================================
C
C       This subroutine defines the units of the multipole moments;
C       they  d i f f e r  from the electromagnetic  units used by
C       T. D. Jackson, ch.4, eq.(4.3) in the following way:
C
C       The spherical harmonics used by the program, have the form
C       identical with those used by Jackson and also Varshalovich.
C
C       The multipole moments of Jackson are defined as the average
C       values of the above spherical harmonics.  Our multipole mo-
C       ments have additional multiplicative factors stored in the
C       the matrix QUNITS.
C
C=======================================================================
C
      SQR4PI=SQRT(16.0*ATAN(1.0))
C
      DO LAMBDA=0,NMUMAX
         DO MIU=0,LAMBDA
            QUNITS(LAMBDA,MIU)=1.0
            AUNITS(LAMBDA,MIU)=1.0
         END DO
      END DO
C
C=======================================================================
C                   E X P L I C I T      D E F I N I T I O S
C=======================================================================
C
      QUNITS(0,0)=+SQR4PI
C
C=======================================================================
C        With the above, our Q(0,0) - moment gives the particle number
C=======================================================================
C
      QUNITS(1,0)=+SQR4PI/SQRT(3.0)
      QUNITS(1,1)=-SQR4PI/SQRT(3.0)*SQRT(2.0)
C
C=======================================================================
C        With the above, our Q(1,0)- and Q(1,1) - moments give the
C        expectation values of "Z" and "X", respectively.
C=======================================================================
C
      QUNITS(2,0)=+SQR4PI/SQRT(5.0)*2.0
      QUNITS(2,1)=-SQR4PI/SQRT(15.)*SQRT(2.0)
      QUNITS(2,2)=+SQR4PI/SQRT(5.0)*2.0*SQRT(2.0)
C
C=======================================================================
C
C        With the above, our Q(2,0)-moment gives the expectation value
C        of "( 2*Z**2 - X**2 - Y**2 )". In addition, our Q(2,2)-moment
C        contains  Bohr's SQRT(2), what implies,  that using this unit
C        is equivalent to treating  Q(2,+2) & Q(2,-2)), combined. Con-
C        sequently, Bohr's "BETA = SQRT(Q(2,0)**2 + Q(2,2)**2)", while
C        Bohr's "GAMMA = ATAN (Q(2,2)/Q(2,0))".  In order to introduce
C        the above into the transition program we have to define
C
C                  Q_{2+2} = Q_{2,+2; Bohr} / sqrt(2)
C        and
C
C                  Q_{2-2} = Q_{2,-2; Bohr} / sqrt(2)
C
C        where Q_{2,\mu; Bohr} are those obtained with our units.
C
C#######################################################################
C
C        THE INSTRUCTION BELOW WAS USED INCORRECTLY IN SOME PREVIOUS
C        VERSIONS OF THE CODE. WHEN LEFT ACTIVE, THE BOHR'S SYMMETRY
C        CONDITION 'GAMMA -> GAMMA + 60' DOES NOT APPLY ANYMORE.
C
C#######################################################################
C
CCC      Never more  QUNITS(2,2)=QUNITS(2,2)*SQRT(2.0) Never more    CCC
C
C#######################################################################
C
C=======================================================================
C        All higher multipoles are left in peace, so far.
C=======================================================================
C
C=======================================================================
C        Moreover, to make enemies life difficult, the moments are
C        calculated in units of ( 10 * Fermi ) ** Lambda.
C=======================================================================
C
      DO LAMBDA=0,NMUMAX
         DO MIU=0,LAMBDA
            QUNITS(LAMBDA,MIU)=QUNITS(LAMBDA,MIU)/(10.0)**LAMBDA
         END DO
      END DO
C
C=======================================================================
C     So far, no ingenious units have been invented for magnetic
c     moments apart from the nuclear magneton, but wait a while...
C=======================================================================
C
      DO LAMBDA=0,NMUMAX
         DO MIU=0,LAMBDA
            AUNITS(LAMBDA,MIU)=1.0
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_DEFYLM
C=======================================================================
      USE params
C=======================================================================
C
      DIMENSION
     *          KOELOC(0:NDMULT,0:NDMULT,1:NDKART),
     *
     *          NRELOC(0:NDMULT,0:NDMULT),
     *          KRELOC(0:NDMULT,0:NDMULT,1:NDTERM,0:NDKART),
     *
     *          NIMLOC(0:NDMULT,1:NDMULT),
     *          KIMLOC(0:NDMULT,1:NDMULT,1:NDTERM,0:NDKART)
C
      COMMON
     *       /INTMUL_COEYLM/ COEYLM(0:NDMULT,0:NDMULT)
      COMMON
     *       /INTMUL_IMAREA/ NREYLM(0:NDMULT,0:NDMULT),
     *                       KREYLM(0:NDMULT,0:NDMULT,NDTERM,0:NDKART),
     *
     *                       NIMYLM(0:NDMULT,1:NDMULT),
     *                       KIMYLM(0:NDMULT,1:NDMULT,NDTERM,0:NDKART)
C
      DATA
     *        MULMAX / 9 /
C
C=======================================================================
C
C         This subroutine defines the terms appearing in the
C
C                   s o l i d    h a r m o n i c s
C
C         (spherical harmonics  *  R**lambda). The solid harmo-
C         are written in the cartesian representation, using
C
C                  Eq. 5.1.7 (19) of Varshalovich
C
C         The expansion contains real and imaginary terms. For
C         optimization purposes, they are stored in different
C         arrays. The number of terms of each kind (real or imagi-
C         nary) is stored in "NREYLM" and "NIMYLM", respectively.
C
C         Information on expansion coefficients is stored in the
C         two arrays KRELOC and KIMLOC (which are kept in memory
C         as KREYLM and KIMYLM). The structure of these arrays is
C
C                   KRELOC ( lambda, mu, N, k)
C
C         where:
C            - (lambda, mu) refer to the solid harmonic expanded
C            - N is the number of (real or imaginary) terms in the
C              expansion
C            - k contains the information on the coefficients
C                  k=0: coefficient in front of the current term
C                       in the expansion
C                  k=1: power of x for this particular term
C                  k=2: power of y for this particular term
C                  k=3: power of z for this particular term
C
C         KOELOC: Integer matrix storing the structure of the
C                 overall common factors  "COEYLM" according
C                 to the following convention:
C
C                 COEYLM = INT1 * SQRT(INT2/INT3) / SQRT(4PI)
C
C                 KOELOC(,,1) = INT1
C                 KOELOC(,,2) = INT2
C                 KOELOC(,,3) = INT3
C
C=======================================================================
C
C         This subroutine defines multipoles up to the value
C         of "MULMAX" defined above. This value must not be
C         smaller than any of the following input parameters:
C
C                      NMUCON < or =  MULMAX
C                      NMUCOU < or =  MULMAX
C                      NMUPRI < or =  MULMAX
C
C         As a safety measure, the code requires that the
C         dimension  declaration "NDMULT" is necessarily
C         equal to "MULMAX".
C
C         At the same time other dimension declarations have
C         to be changed in the following way:
C
C         For  MULMAX = NDMULT   ==>>     4  (5, 6, 7, 8, 9)
C         one  needs    NDPOLS   ==>>    15 (21, 36, 45, 55)
C
C         and "NDTERM"  has to take the following values:
C                       MULMAX = NDMULT<=5  ==>>  NDTERM= 6;
C                       MULMAX = NDMULT<=7  ==>>  NDTERM=10;
C                       MULMAX = NDMULT<=9  ==>>  NDTERM=15;
C                       MULMAX = NDMULT =10 ==>>  NDTERM=21;
C
C=======================================================================
C         Here start definitions for the given lambda and miu values:
C=======================================================================
C                        LAMBDA=0  MIU=0  YLM=C
C                               C=1/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  0,  0,      K), K= 1, 3) /  1,  1,  1/
C
C=======================================================================
C          Y00 REAL PART ONLY
C=======================================================================
C
      DATA (KRELOC(  0,  0,  1,  K), K= 0, 3) /  1,  0,  0,  0/
C
      DATA  NRELOC(  0,  0) /1/
C
C=======================================================================
C          LAMBDA=1  MIU=0  YLM=C * Z
C          C=SQRT(3)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  1,  0,      K), K= 1, 3) /  1,  3,  1/
C
C=======================================================================
C          Y10 REAL PART ONLY
C=======================================================================
C
      DATA (KRELOC(  1,  0,  1,  K), K= 0, 3) /  1,  0,  0,  1/
C
      DATA  NRELOC(  1,  0) /1/
C
C=======================================================================
C          LAMBDA=1  MIU=1  YLM=C * ( X + I*Y )
C          C=-SQRT(3/2)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  1,  1,      K), K= 1, 3) / -1,  3,  2/
C
C=======================================================================
C          Y11 REAL PART
C=======================================================================
C
      DATA (KRELOC(  1,  1,  1,  K), K= 0, 3) /  1,  1,  0,  0/
C
      DATA  NRELOC(  1,  1) /1/
C
C=======================================================================
C          Y11 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  1,  1,  1,  K), K= 0, 3) /  1,  0,  1,  0/
C
      DATA  NIMLOC(  1,  1) /1/
C
C=======================================================================
C          LAMBDA=2  MIU=0  YLM=C * (2Z**2 - X**2 - Y**2)
C          C=SQRT(5/4)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  2,  0,      K), K= 1, 3) /  1,  5,  4/
C
C=======================================================================
C          Y20 REAL PART ONLY
C=======================================================================
C
      DATA (KRELOC(  2,  0,  1,  K), K= 0, 3) /  2,  0,  0,  2/
      DATA (KRELOC(  2,  0,  2,  K), K= 0, 3) / -1,  2,  0,  0/
      DATA (KRELOC(  2,  0,  3,  K), K= 0, 3) / -1,  0,  2,  0/
C
      DATA  NRELOC(  2,  0) /3/
C
C=======================================================================
C          LAMBDA=2  MIU=1  YLM=C * ( X + I*Y ) * Z
C          C=-SQRT(15/2)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  2,  1,      K), K= 1, 3) / -1, 15,  2/
C
C=======================================================================
C          Y21 REAL PART
C=======================================================================
C
      DATA (KRELOC(  2,  1,  1,  K), K= 0, 3) /  1,  1,  0,  1/
C
      DATA  NRELOC(  2,  1) /1/
C
C=======================================================================
C          Y21 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  2,  1,  1,  K), K= 0, 3) /  1,  0,  1,  1/
C
      DATA  NIMLOC(  2,  1) /1/
C
C=======================================================================
C          LAMBDA=2  MIU=2  YLM=C * ( X**2 + 2I X*Y - Y**2 )
C          C=SQRT(15/8)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  2,  2,      K), K= 1, 3) /  1, 15,  8/
C
C=======================================================================
C          Y22 REAL PART
C=======================================================================
C
      DATA (KRELOC(  2,  2,  1,  K), K= 0, 3) /  1,  2,  0,  0/
      DATA (KRELOC(  2,  2,  2,  K), K= 0, 3) / -1,  0,  2,  0/
C
      DATA  NRELOC(  2,  2) /2/
C
C=======================================================================
C          Y22 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  2,  2,  1,  K), K= 0, 3) /  2,  1,  1,  0/
C
      DATA  NIMLOC(  2,  2) /1/
C
C=======================================================================
C          LAMBDA=3  MIU=0  YLM=C * (2Z**2 - 3X**2 - 3Y**2) * Z
C          C=SQRT(7/4)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  3,  0,      K), K= 1, 3) /  1,  7,  4/
C
C=======================================================================
C          Y30 REAL PART ONLY
C=======================================================================
C
      DATA (KRELOC(  3,  0,  1,  K), K= 0, 3) /  2,  0,  0,  3/
      DATA (KRELOC(  3,  0,  2,  K), K= 0, 3) / -3,  2,  0,  1/
      DATA (KRELOC(  3,  0,  3,  K), K= 0, 3) / -3,  0,  2,  1/
C
      DATA  NRELOC(  3,  0) /3/
C
C=======================================================================
C          LAMBDA=3  MIU=1  YLM= C * ( X + I*Y )*( 4Z**2 - X**2 - Y**2 )
C          C=-SQRT(21/16)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  3,  1,      K), K= 1, 3) / -1, 21, 16/
C
C=======================================================================
C          Y31 REAL PART
C=======================================================================
C
      DATA (KRELOC(  3,  1,  1,  K), K= 0, 3) /  4,  1,  0,  2/
      DATA (KRELOC(  3,  1,  2,  K), K= 0, 3) / -1,  3,  0,  0/
      DATA (KRELOC(  3,  1,  3,  K), K= 0, 3) / -1,  1,  2,  0/
C
      DATA  NRELOC(  3,  1) /3/
C
C=======================================================================
C          Y31 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  3,  1,  1,  K), K= 0, 3) /  4,  0,  1,  2/
      DATA (KIMLOC(  3,  1,  2,  K), K= 0, 3) / -1,  2,  1,  0/
      DATA (KIMLOC(  3,  1,  3,  K), K= 0, 3) / -1,  0,  3,  0/
C
      DATA  NIMLOC(  3,  1) /3/
C
C=======================================================================
C          LAMBDA=3  MIU=2  YLM=C * ( X**2 + 2*I*X*Y - Y**2 ) * Z
C          C=SQRT(105/8)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  3,  2,      K), K= 1, 3) /  1,105,  8/
C
C=======================================================================
C          Y32 REAL PART
C=======================================================================
C
      DATA (KRELOC(  3,  2,  1,  K), K= 0, 3) /  1,  2,  0,  1/
      DATA (KRELOC(  3,  2,  2,  K), K= 0, 3) / -1,  0,  2,  1/
C
      DATA  NRELOC(  3,  2) /2/
C
C=======================================================================
C          Y32 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  3,  2,  1,  K), K= 0, 3) /  2,  1,  1,  1/
C
      DATA  NIMLOC(  3,  2) /1/
C
C=======================================================================
C          LAMBDA=3  MIU=3  YLM=C*(X**3 + 3I X**2*Y - 3 X*Y**2 - I Y**3)
C          C=-SQRT(35/16)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  3,  3,      K), K= 1, 3) / -1, 35, 16/
C
C=======================================================================
C          Y33 REAL PART
C=======================================================================
C
      DATA (KRELOC(  3,  3,  1,  K), K= 0, 3) /  1,  3,  0,  0/
      DATA (KRELOC(  3,  3,  2,  K), K= 0, 3) / -3,  1,  2,  0/
C
      DATA  NRELOC(  3,  3) /2/
C
C=======================================================================
C          Y33 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  3,  3,  1,  K), K= 0, 3) /  3,  2,  1,  0/
      DATA (KIMLOC(  3,  3,  2,  K), K= 0, 3) / -1,  0,  3,  0/
C
      DATA  NIMLOC(  3,  3) /2/
C
C=======================================================================
C          LAMBDA=4  MIU=0  YLM=C * ( 8Z**4 + 3X**4 + 3Y**4
C                            -24 X**2 Z**2 -24 Y**2 Z**2 + 6 X**2 Y**2 )
C          C=3/8/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  4,  0,      K), K= 1, 3) /  1,  9, 64/
C
C=======================================================================
C          Y40 REAL PART ONLY
C=======================================================================
C
      DATA (KRELOC(  4,  0,  1,  K), K= 0, 3) /  8,  0,  0,  4/
      DATA (KRELOC(  4,  0,  2,  K), K= 0, 3) /  3,  4,  0,  0/
      DATA (KRELOC(  4,  0,  3,  K), K= 0, 3) /  3,  0,  4,  0/
      DATA (KRELOC(  4,  0,  4,  K), K= 0, 3) /-24,  2,  0,  2/
      DATA (KRELOC(  4,  0,  5,  K), K= 0, 3) /-24,  0,  2,  2/
      DATA (KRELOC(  4,  0,  6,  K), K= 0, 3) /  6,  2,  2,  0/
C
      DATA  NRELOC(  4,  0) /6/
C
C=======================================================================
C          LAMBDA=4  MIU=1  YLM=C *(X + I*Y)*(4Z**2 - 3X**2 - 3Y**2) * Z
C          C=-SQRT(45/16)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  4,  1,      K), K= 1, 3) / -1, 45, 16/
C
C=======================================================================
C          Y41 REAL PART
C=======================================================================
C
      DATA (KRELOC(  4,  1,  1,  K), K= 0, 3) /  4,  1,  0,  3/
      DATA (KRELOC(  4,  1,  2,  K), K= 0, 3) / -3,  3,  0,  1/
      DATA (KRELOC(  4,  1,  3,  K), K= 0, 3) / -3,  1,  2,  1/
C
      DATA  NRELOC(  4,  1) /3/
C
C=======================================================================
C          Y41 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  4,  1,  1,  K), K= 0, 3) /  4,  0,  1,  3/
      DATA (KIMLOC(  4,  1,  2,  K), K= 0, 3) / -3,  2,  1,  1/
      DATA (KIMLOC(  4,  1,  3,  K), K= 0, 3) / -3,  0,  3,  1/
C
      DATA  NIMLOC(  4,  1) /3/
C
C=======================================================================
C          LAMBDA=4  MIU=2  YLM=C * (   X**2 + 2*I*X*Y - Y**2 ) *
C                                  ( 6*Z**2 -   X**2  - Y**2 )
C          C=SQRT(90/64)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  4,  2,      K), K= 1, 3) /  1, 90, 64/
C
C=======================================================================
C          Y42 REAL PART
C=======================================================================
C
      DATA (KRELOC(  4,  2,  1,  K), K= 0, 3) /  6,  2,  0,  2/
      DATA (KRELOC(  4,  2,  2,  K), K= 0, 3) / -6,  0,  2,  2/
      DATA (KRELOC(  4,  2,  3,  K), K= 0, 3) / -1,  4,  0,  0/
      DATA (KRELOC(  4,  2,  4,  K), K= 0, 3) /  1,  0,  4,  0/
C
      DATA  NRELOC(  4,  2) /4/
C
C=======================================================================
C          Y42 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  4,  2,  1,  K), K= 0, 3) / 12,  1,  1,  2/
      DATA (KIMLOC(  4,  2,  2,  K), K= 0, 3) / -2,  3,  1,  0/
      DATA (KIMLOC(  4,  2,  3,  K), K= 0, 3) / -2,  1,  3,  0/
C
      DATA  NIMLOC(  4,  2) /3/
C
C=======================================================================
C          LAMBDA=4  MIU=3  YLM=C*(X**3 + 3I X**2*Y - 3X*Y**2 - IY**3)*Z
C          C=-SQRT(315/16)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  4,  3,      K), K= 1, 3) / -1,315, 16/
C
C=======================================================================
C          Y43 REAL PART
C=======================================================================
C
      DATA (KRELOC(  4,  3,  1,  K), K= 0, 3) /  1,  3,  0,  1/
      DATA (KRELOC(  4,  3,  2,  K), K= 0, 3) / -3,  1,  2,  1/
C
      DATA  NRELOC(  4,  3) /2/
C
C=======================================================================
C          Y43 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  4,  3,  1,  K), K= 0, 3) /  3,  2,  1,  1/
      DATA (KIMLOC(  4,  3,  2,  K), K= 0, 3) / -1,  0,  3,  1/
C
      DATA  NIMLOC(  4,  3) /2/
C
C=======================================================================
C          LAMBDA=4  MIU=4  YLM=C * ( X**4 + 4I X**3*Y - 6 X**2*Y**2
C                                          - 4I X*Y**3 + Y**4 )
C          C=SQRT(630/256)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  4,  4,      K), K= 1, 3) /  1,630,256/
C
C=======================================================================
C          Y44 REAL PART
C=======================================================================
C
      DATA (KRELOC(  4,  4,  1,  K), K= 0, 3) /  1,  4,  0,  0/
      DATA (KRELOC(  4,  4,  2,  K), K= 0, 3) / -6,  2,  2,  0/
      DATA (KRELOC(  4,  4,  3,  K), K= 0, 3) /  1,  0,  4,  0/
C
      DATA  NRELOC(  4,  4) /3/
C
C=======================================================================
C          Y44 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  4,  4,  1,  K), K= 0, 3) /  4,  3,  1,  0/
      DATA (KIMLOC(  4,  4,  2,  K), K= 0, 3) / -4,  1,  3,  0/
C
      DATA  NIMLOC(  4,  4) /2/
C
C=======================================================================
C          LAMBDA=5  MIU=0  YLM=C * (8Z**5 - 40X**2*Z**3 - 40Y**2*Z**3
C                                +15X**4*Z + 15Y**4*Z + 30X**2*Y**2*Z)
C          C=SQRT(11/64)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  5,  0,      K), K= 1, 3) /  1,  11, 64/
C
C=======================================================================
C          Y50 REAL PART ONLY
C=======================================================================
C
      DATA (KRELOC(  5,  0,  1,  K), K= 0, 3) /  8,  0,  0,  5/
      DATA (KRELOC(  5,  0,  2,  K), K= 0, 3) /-40,  2,  0,  3/
      DATA (KRELOC(  5,  0,  3,  K), K= 0, 3) /-40,  0,  2,  3/
      DATA (KRELOC(  5,  0,  4,  K), K= 0, 3) / 15,  4,  0,  1/
      DATA (KRELOC(  5,  0,  5,  K), K= 0, 3) / 15,  0,  4,  1/
      DATA (KRELOC(  5,  0,  6,  K), K= 0, 3) / 30,  2,  2,  1/
C
      DATA  NRELOC(  5,  0) /6/
C
C=======================================================================
C          LAMBDA=5  MIU=1  YLM=C * (X + IY)( X**4 + Y**4 + 8Z**4
C                             + 2X**2*Y**2 - 12*X**2*Z**2 - 12Y**2*Z**2)
C          C=-SQRT(165/128)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  5,  1,      K), K= 1, 3) / -1,165,128/
C
C=======================================================================
C          Y51 REAL PART
C=======================================================================
C
      DATA (KRELOC(  5,  1,  1,  K), K= 0, 3) /  8,  1,  0,  4/
      DATA (KRELOC(  5,  1,  2,  K), K= 0, 3) /  1,  5,  0,  0/
      DATA (KRELOC(  5,  1,  3,  K), K= 0, 3) /  1,  1,  4,  0/
      DATA (KRELOC(  5,  1,  4,  K), K= 0, 3) /  2,  3,  2,  0/
      DATA (KRELOC(  5,  1,  5,  K), K= 0, 3) /-12,  3,  0,  2/
      DATA (KRELOC(  5,  1,  6,  K), K= 0, 3) /-12,  1,  2,  2/
C
      DATA  NRELOC(  5,  1) /6/
C
C=======================================================================
C          Y51 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  5,  1,  1,  K), K= 0, 3) /  8,  0,  1,  4/
      DATA (KIMLOC(  5,  1,  2,  K), K= 0, 3) /  1,  4,  1,  0/
      DATA (KIMLOC(  5,  1,  3,  K), K= 0, 3) /  1,  0,  5,  0/
      DATA (KIMLOC(  5,  1,  4,  K), K= 0, 3) /  2,  2,  3,  0/
      DATA (KIMLOC(  5,  1,  5,  K), K= 0, 3) /-12,  2,  1,  2/
      DATA (KIMLOC(  5,  1,  6,  K), K= 0, 3) /-12,  0,  3,  2/
C
      DATA  NIMLOC(  5,  1) /6/
C
C=======================================================================
C          LAMBDA=5  MIU=2  YLM=C * (X **2 - Y**2 + I 2*X*Y)
C                                   ( 2Z**2 - X**2 - Y**2)*Z
C          C=SQRT(1155/32)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  5,  2,      K), K= 1, 3) / 1,1155,32/
C
C=======================================================================
C          Y52 REAL PART
C=======================================================================
C
      DATA (KRELOC(  5,  2,  1,  K), K= 0, 3) /  2,  2,  0,  3/
      DATA (KRELOC(  5,  2,  2,  K), K= 0, 3) / -1,  4,  0,  1/
      DATA (KRELOC(  5,  2,  3,  K), K= 0, 3) / -2,  0,  2,  3/
      DATA (KRELOC(  5,  2,  4,  K), K= 0, 3) /  1,  0,  4,  1/
C
      DATA  NRELOC(  5,  2) /4/
C
C=======================================================================
C          Y52 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  5,  2,  1,  K), K= 0, 3) /  4,  1,  1,  3/
      DATA (KIMLOC(  5,  2,  2,  K), K= 0, 3) / -2,  3,  1,  1/
      DATA (KIMLOC(  5,  2,  3,  K), K= 0, 3) / -2,  1,  3,  1/
C
      DATA  NIMLOC(  5,  2) /3/
C
C=======================================================================
C          LAMBDA=5  MIU=3  YLM=C *
C          C=-SQRT(385/256)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  5,  3,      K), K= 1, 3) / -1,385,256/
C
C=======================================================================
C          Y53 REAL PART
C=======================================================================
C
      DATA (KRELOC(  5,  3,  1,  K), K= 0, 3) / -1,  5,  0,  0/
      DATA (KRELOC(  5,  3,  2,  K), K= 0, 3) /  8,  3,  0,  2/
      DATA (KRELOC(  5,  3,  3,  K), K= 0, 3) /  2,  3,  2,  0/
      DATA (KRELOC(  5,  3,  4,  K), K= 0, 3) /-24,  1,  2,  2/
      DATA (KRELOC(  5,  3,  5,  K), K= 0, 3) /  3,  1,  4,  0/
C
      DATA  NRELOC(  5,  3) /5/
C
C=======================================================================
C          Y53 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  5,  3,  1,  K), K= 0, 3) / -3,  4,  1,  0/
      DATA (KIMLOC(  5,  3,  2,  K), K= 0, 3) / 24,  2,  1,  2/
      DATA (KIMLOC(  5,  3,  3,  K), K= 0, 3) / -8,  0,  3,  2/
      DATA (KIMLOC(  5,  3,  4,  K), K= 0, 3) /  1,  0,  5,  0/
      DATA (KIMLOC(  5,  3,  5,  K), K= 0, 3) / -2,  2,  3,  0/
C
      DATA  NIMLOC(  5,  3) /5/
C
C=======================================================================
C          LAMBDA=5  MIU=4  YLM=C *
C          C=3*SQRT(385/128)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  5,  4,      K), K= 1, 3) /  3,385,128/
C
C=======================================================================
C          Y54 REAL PART
C=======================================================================
C
      DATA (KRELOC(  5,  4,  1,  K), K= 0, 3) /  1,  4,  0,  1/
      DATA (KRELOC(  5,  4,  2,  K), K= 0, 3) /  1,  0,  4,  1/
      DATA (KRELOC(  5,  4,  3,  K), K= 0, 3) / -6,  2,  2,  1/
C
      DATA  NRELOC(  5,  4) /3/
C
C=======================================================================
C          Y54 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  5,  4,  1,  K), K= 0, 3) /  4,  3,  1,  1/
      DATA (KIMLOC(  5,  4,  2,  K), K= 0, 3) / -4,  1,  3,  1/
C
      DATA  NIMLOC(  5,  4) /2/
C
C=======================================================================
C          LAMBDA=5  MIU=5  YLM=C *
C          C=-3*SQRT(77/256)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  5,  5,      K), K= 1, 3) /  -3, 77,256/
C
C=======================================================================
C          Y55 REAL PART
C=======================================================================
C
      DATA (KRELOC(  5,  5,  1,  K), K= 0, 3) /  1,  5,  0,  0/
      DATA (KRELOC(  5,  5,  2,  K), K= 0, 3) /  5,  1,  4,  0/
      DATA (KRELOC(  5,  5,  3,  K), K= 0, 3) /-10,  3,  2,  0/
C
      DATA  NRELOC(  5,  5) /3/
C
C=======================================================================
C          Y55 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  5,  5,  1,  K), K= 0, 3) /  1,  0,  5,  0/
      DATA (KIMLOC(  5,  5,  2,  K), K= 0, 3) /  5,  4,  1,  0/
      DATA (KIMLOC(  5,  5,  3,  K), K= 0, 3) /-10,  2,  3,  0/
C
      DATA  NIMLOC(  5,  5) /3/
C
C=======================================================================
C          LAMBDA=6  MIU=0  YLM=C *
C          C=SQRT(11/64)/SQR4PI
C=======================================================================
C
      DATA (KOELOC(  6,  0,      K), K= 1, 3) / -1,  13, 256/
C
C=======================================================================
C          Y60 REAL PART ONLY
C=======================================================================
C
      DATA (KRELOC(  6,  0,  1,  K), K= 0, 3) /   5,  6,  0,  0/
      DATA (KRELOC(  6,  0,  2,  K), K= 0, 3) /  15,  4,  2,  0/
      DATA (KRELOC(  6,  0,  3,  K), K= 0, 3) / -90,  4,  0,  2/
      DATA (KRELOC(  6,  0,  4,  K), K= 0, 3) /  15,  2,  4,  0/
      DATA (KRELOC(  6,  0,  5,  K), K= 0, 3) /-180,  2,  2,  2/
      DATA (KRELOC(  6,  0,  6,  K), K= 0, 3) / 120,  2,  0,  4/
      DATA (KRELOC(  6,  0,  7,  K), K= 0, 3) /   5,  0,  6,  0/
      DATA (KRELOC(  6,  0,  8,  K), K= 0, 3) / -90,  0,  4,  2/
      DATA (KRELOC(  6,  0,  9,  K), K= 0, 3) / 120,  0,  2,  4/
      DATA (KRELOC(  6,  0, 10,  K), K= 0, 3) / -16,  0,  0,  6/
C
      DATA  NRELOC(  6,  0) /10/
C
C=======================================================================
C          LAMBDA=6  MIU=1  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  6,  1,      K), K= 1, 3) / -1,273,128/
C
C=======================================================================
C          Y61 REAL PART
C=======================================================================
C
      DATA (KRELOC(  6,  1,  1,  K), K= 0, 3) /  5,  5,  0,  1/
      DATA (KRELOC(  6,  1,  2,  K), K= 0, 3) / 10,  3,  2,  1/
      DATA (KRELOC(  6,  1,  3,  K), K= 0, 3) /-20,  3,  0,  3/
      DATA (KRELOC(  6,  1,  4,  K), K= 0, 3) /  5,  1,  4,  1/
      DATA (KRELOC(  6,  1,  5,  K), K= 0, 3) /-20,  1,  2,  3/
      DATA (KRELOC(  6,  1,  6,  K), K= 0, 3) /  8,  1,  0,  5/
C
      DATA  NRELOC(  6,  1) /6/
C
C=======================================================================
C          Y61 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  6,  1,  1,  K), K= 0, 3) /  5,  4,  1,  1/
      DATA (KIMLOC(  6,  1,  2,  K), K= 0, 3) / 10,  2,  3,  1/
      DATA (KIMLOC(  6,  1,  3,  K), K= 0, 3) /-20,  2,  1,  3/
      DATA (KIMLOC(  6,  1,  4,  K), K= 0, 3) /  5,  0,  5,  1/
      DATA (KIMLOC(  6,  1,  5,  K), K= 0, 3) /-20,  0,  3,  3/
      DATA (KIMLOC(  6,  1,  6,  K), K= 0, 3) /  8,  0,  1,  5/
C
      DATA  NIMLOC(  6,  1) /6/
C
C=======================================================================
C          LAMBDA=6  MIU=2  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  6,  2,      K), K= 1, 3) / 1,1365,1024/
C
C=======================================================================
C          Y62 REAL PART
C=======================================================================
C
      DATA (KRELOC(  6,  2,  1,  K), K= 0, 3) /  1,  6,  0,  0/
      DATA (KRELOC(  6,  2,  2,  K), K= 0, 3) /  1,  4,  2,  0/
      DATA (KRELOC(  6,  2,  3,  K), K= 0, 3) /-16,  4,  0,  2/
      DATA (KRELOC(  6,  2,  4,  K), K= 0, 3) / -1,  2,  4,  0/
      DATA (KRELOC(  6,  2,  5,  K), K= 0, 3) / 16,  2,  0,  4/
      DATA (KRELOC(  6,  2,  6,  K), K= 0, 3) / -1,  0,  6,  0/
      DATA (KRELOC(  6,  2,  7,  K), K= 0, 3) / 16,  0,  4,  2/
      DATA (KRELOC(  6,  2,  8,  K), K= 0, 3) /-16,  0,  2,  4/
C
      DATA  NRELOC(  6,  2) /8/
C
C=======================================================================
C          Y62 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  6,  2,  1,  K), K= 0, 3) /  2,  5,  1,  0/
      DATA (KIMLOC(  6,  2,  2,  K), K= 0, 3) /  4,  3,  3,  0/
      DATA (KIMLOC(  6,  2,  3,  K), K= 0, 3) /-32,  3,  1,  2/
      DATA (KIMLOC(  6,  2,  4,  K), K= 0, 3) /  2,  1,  5,  0/
      DATA (KIMLOC(  6,  2,  5,  K), K= 0, 3) /-32,  1,  3,  2/
      DATA (KIMLOC(  6,  2,  6,  K), K= 0, 3) / 32,  1,  1,  4/
C
      DATA  NIMLOC(  6,  2) /6/
C
C=======================================================================
C          LAMBDA=6  MIU=3  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  6,  3,      K), K= 1, 3) /  1,1365,256/
C
C=======================================================================
C          Y63 REAL PART
C=======================================================================
C
      DATA (KRELOC(  6,  3,  1,  K), K= 0, 3) /  3,  5,  0,  1/
      DATA (KRELOC(  6,  3,  2,  K), K= 0, 3) / -6,  3,  2,  1/
      DATA (KRELOC(  6,  3,  3,  K), K= 0, 3) / -8,  3,  0,  3/
      DATA (KRELOC(  6,  3,  4,  K), K= 0, 3) / -9,  1,  4,  1/
      DATA (KRELOC(  6,  3,  5,  K), K= 0, 3) / 24,  1,  2,  3/
C
      DATA  NRELOC(  6,  3) /5/
C
C=======================================================================
C          Y63 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  6,  3,  1,  K), K= 0, 3) /  9,  4,  1,  1/
      DATA (KIMLOC(  6,  3,  2,  K), K= 0, 3) /  6,  2,  3,  1/
      DATA (KIMLOC(  6,  3,  3,  K), K= 0, 3) /-24,  2,  1,  3/
      DATA (KIMLOC(  6,  3,  4,  K), K= 0, 3) / -3,  0,  5,  1/
      DATA (KIMLOC(  6,  3,  5,  K), K= 0, 3) /  8,  0,  3,  3/
C
      DATA  NIMLOC(  6,  3) /5/
C
C=======================================================================
C          LAMBDA=6  MIU=4  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  6,  4,      K), K= 1, 3) /  -1,819,512/
C
C=======================================================================
C          Y64 REAL PART
C=======================================================================
C
      DATA (KRELOC(  6,  4,  1,  K), K= 0, 3) /  1,  6,  0,  0/
      DATA (KRELOC(  6,  4,  2,  K), K= 0, 3) / -5,  4,  2,  0/
      DATA (KRELOC(  6,  4,  3,  K), K= 0, 3) /-10,  4,  0,  2/
      DATA (KRELOC(  6,  4,  4,  K), K= 0, 3) / -5,  2,  4,  0/
      DATA (KRELOC(  6,  4,  5,  K), K= 0, 3) / 60,  2,  2,  2/
      DATA (KRELOC(  6,  4,  6,  K), K= 0, 3) /  1,  0,  6,  0/
      DATA (KRELOC(  6,  4,  7,  K), K= 0, 3) /-10,  0,  4,  2/
C
      DATA  NRELOC(  6,  4) /7/
C
C=======================================================================
C          Y64 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  6,  4,  1,  K), K= 0, 3) /  4,  5,  1,  0/
      DATA (KIMLOC(  6,  4,  2,  K), K= 0, 3) /-40,  3,  1,  2/
      DATA (KIMLOC(  6,  4,  3,  K), K= 0, 3) / -4,  1,  5,  0/
      DATA (KIMLOC(  6,  4,  4,  K), K= 0, 3) / 40,  1,  3,  2/
C
      DATA  NIMLOC(  6,  4) /4/
C
C=======================================================================
C          LAMBDA=6  MIU=5  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  6,  5,      K), K= 1, 3) /  -1, 9009,256/
C
C=======================================================================
C          Y65 REAL PART
C=======================================================================
C
      DATA (KRELOC(  6,  5,  1,  K), K= 0, 3) /  1,  5,  0,  1/
      DATA (KRELOC(  6,  5,  2,  K), K= 0, 3) /-10,  3,  2,  1/
      DATA (KRELOC(  6,  5,  3,  K), K= 0, 3) /  5,  1,  4,  1/
C
      DATA  NRELOC(  6,  5) /3/
C
C=======================================================================
C          Y65 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  6,  5,  1,  K), K= 0, 3) /  5,  4,  1,  1/
      DATA (KIMLOC(  6,  5,  2,  K), K= 0, 3) /-10,  2,  3,  1/
      DATA (KIMLOC(  6,  5,  3,  K), K= 0, 3) /  1,  0,  5,  1/
C
      DATA  NIMLOC(  6,  5) /3/
C
C=======================================================================
C          LAMBDA=6  MIU=6  YLM=C *
C
C          C=
C=======================================================================
C
      DATA (KOELOC(  6,  6,      K), K= 1, 3) /   1, 3003,1024/
C
C=======================================================================
C          Y66 REAL PART
C=======================================================================
C
      DATA (KRELOC(  6,  6,  1,  K), K= 0, 3) /  1,  6,  0,  0/
      DATA (KRELOC(  6,  6,  2,  K), K= 0, 3) /-15,  4,  2,  0/
      DATA (KRELOC(  6,  6,  3,  K), K= 0, 3) / 15,  2,  4,  0/
      DATA (KRELOC(  6,  6,  4,  K), K= 0, 3) / -1,  0,  6,  0/
C
      DATA  NRELOC(  6,  6) /4/
C
C=======================================================================
C          Y65 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  6,  6,  1,  K), K= 0, 3) /  6,  5,  1,  0/
      DATA (KIMLOC(  6,  6,  2,  K), K= 0, 3) /-20,  3,  3,  0/
      DATA (KIMLOC(  6,  6,  3,  K), K= 0, 3) /  6,  1,  5,  0/
C
      DATA  NIMLOC(  6,  6) /3/
C
C=======================================================================
C          LAMBDA=7  MIU=0  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  7,  0,      K), K= 1, 3) / -1,  15, 256/
C
C=======================================================================
C          Y70 REAL PART ONLY
C=======================================================================
C
      DATA (KRELOC(  7,  0,  1,  K), K= 0, 3) /  35,  6,  0,  1/
      DATA (KRELOC(  7,  0,  2,  K), K= 0, 3) / 105,  4,  2,  1/
      DATA (KRELOC(  7,  0,  3,  K), K= 0, 3) /-210,  4,  0,  3/
      DATA (KRELOC(  7,  0,  4,  K), K= 0, 3) / 105,  2,  4,  1/
      DATA (KRELOC(  7,  0,  5,  K), K= 0, 3) /-420,  2,  2,  3/
      DATA (KRELOC(  7,  0,  6,  K), K= 0, 3) / 168,  2,  0,  5/
      DATA (KRELOC(  7,  0,  7,  K), K= 0, 3) /  35,  0,  6,  1/
      DATA (KRELOC(  7,  0,  8,  K), K= 0, 3) /-210,  0,  4,  3/
      DATA (KRELOC(  7,  0,  9,  K), K= 0, 3) / 168,  0,  2,  5/
      DATA (KRELOC(  7,  0, 10,  K), K= 0, 3) / -16,  0,  0,  7/
C
      DATA  NRELOC(  7,  0) /10/
C
C=======================================================================
C          LAMBDA=7  MIU=1  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  7,  1,      K), K= 1, 3) /  1,105,2048/
C
C=======================================================================
C          Y71 REAL PART
C=======================================================================
C
      DATA (KRELOC(  7,  1,  1,  K), K= 0, 3) /   5,  7,  0,  0/
      DATA (KRELOC(  7,  1,  2,  K), K= 0, 3) /  15,  5,  2,  0/
      DATA (KRELOC(  7,  1,  3,  K), K= 0, 3) /-120,  5,  0,  2/
      DATA (KRELOC(  7,  1,  4,  K), K= 0, 3) /  15,  3,  4,  0/
      DATA (KRELOC(  7,  1,  5,  K), K= 0, 3) /-240,  3,  2,  2/
      DATA (KRELOC(  7,  1,  6,  K), K= 0, 3) / 240,  3,  0,  4/
      DATA (KRELOC(  7,  1,  7,  K), K= 0, 3) /   5,  1,  6,  0/
      DATA (KRELOC(  7,  1,  8,  K), K= 0, 3) /-120,  1,  4,  2/
      DATA (KRELOC(  7,  1,  9,  K), K= 0, 3) / 240,  1,  2,  4/
      DATA (KRELOC(  7,  1, 10,  K), K= 0, 3) / -64,  1,  0,  6/
C
      DATA  NRELOC(  7,  1) /10/
C
C=======================================================================
C          Y71 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  7,  1,  1,  K), K= 0, 3) /   5,  6,  1,  0/
      DATA (KIMLOC(  7,  1,  2,  K), K= 0, 3) /  15,  4,  3,  0/
      DATA (KIMLOC(  7,  1,  3,  K), K= 0, 3) /-120,  4,  1,  2/
      DATA (KIMLOC(  7,  1,  4,  K), K= 0, 3) /  15,  2,  5,  0/
      DATA (KIMLOC(  7,  1,  5,  K), K= 0, 3) /-240,  2,  3,  2/
      DATA (KIMLOC(  7,  1,  6,  K), K= 0, 3) / 240,  2,  1,  4/
      DATA (KIMLOC(  7,  1,  7,  K), K= 0, 3) /   5,  0,  7,  0/
      DATA (KIMLOC(  7,  1,  8,  K), K= 0, 3) /-120,  0,  5,  2/
      DATA (KIMLOC(  7,  1,  9,  K), K= 0, 3) / 240,  0,  3,  4/
      DATA (KIMLOC(  7,  1, 10,  K), K= 0, 3) / -64,  0,  1,  6/
C
      DATA  NIMLOC(  7,  1) /10/
C
C=======================================================================
C          LAMBDA=7  MIU=2  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  7,  2,      K), K= 1, 3) / 1, 315, 1024/
C
C=======================================================================
C          Y72 REAL PART
C=======================================================================
C
      DATA (KRELOC(  7,  2,  1,  K), K= 0, 3) / 15,  6,  0,  1/
      DATA (KRELOC(  7,  2,  2,  K), K= 0, 3) / 15,  4,  2,  1/
      DATA (KRELOC(  7,  2,  3,  K), K= 0, 3) /-80,  4,  0,  3/
      DATA (KRELOC(  7,  2,  4,  K), K= 0, 3) /-15,  2,  4,  1/
      DATA (KRELOC(  7,  2,  5,  K), K= 0, 3) / 48,  2,  0,  5/
      DATA (KRELOC(  7,  2,  6,  K), K= 0, 3) /-15,  0,  6,  1/
      DATA (KRELOC(  7,  2,  7,  K), K= 0, 3) / 80,  0,  4,  3/
      DATA (KRELOC(  7,  2,  8,  K), K= 0, 3) /-48,  0,  2,  5/
C
      DATA  NRELOC(  7,  2) /8/
C
C=======================================================================
C          Y72 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  7,  2,  1,  K), K= 0, 3) /  30,  5,  1,  1/
      DATA (KIMLOC(  7,  2,  2,  K), K= 0, 3) /  60,  3,  3,  1/
      DATA (KIMLOC(  7,  2,  3,  K), K= 0, 3) /-160,  3,  1,  3/
      DATA (KIMLOC(  7,  2,  4,  K), K= 0, 3) /  30,  1,  5,  1/
      DATA (KIMLOC(  7,  2,  5,  K), K= 0, 3) /-160,  1,  3,  3/
      DATA (KIMLOC(  7,  2,  6,  K), K= 0, 3) /  96,  1,  1,  5/
C
      DATA  NIMLOC(  7,  2) /6/
C
C=======================================================================
C          LAMBDA=7  MIU=3  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  7,  3,      K), K= 1, 3) /  -1, 315, 2048/
C
C=======================================================================
C          Y73 REAL PART
C=======================================================================
C
      DATA (KRELOC(  7,  3,  1,  K), K= 0, 3) /   3,  7,  0,  0/
      DATA (KRELOC(  7,  3,  2,  K), K= 0, 3) /  -3,  5,  2,  0/
      DATA (KRELOC(  7,  3,  3,  K), K= 0, 3) / -60,  5,  0,  2/
      DATA (KRELOC(  7,  3,  4,  K), K= 0, 3) / -15,  3,  4,  0/
      DATA (KRELOC(  7,  3,  5,  K), K= 0, 3) / 120,  3,  2,  2/
      DATA (KRELOC(  7,  3,  6,  K), K= 0, 3) /  80,  3,  0,  4/
      DATA (KRELOC(  7,  3,  7,  K), K= 0, 3) /  -9,  1,  6,  0/
      DATA (KRELOC(  7,  3,  8,  K), K= 0, 3) / 180,  1,  4,  2/
      DATA (KRELOC(  7,  3,  9,  K), K= 0, 3) /-240,  1,  2,  4/
C
      DATA  NRELOC(  7,  3) /9/
C
C=======================================================================
C          Y73 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  7,  3,  1,  K), K= 0, 3) /   9,  6,  1,  0/
      DATA (KIMLOC(  7,  3,  2,  K), K= 0, 3) /  15,  4,  3,  0/
      DATA (KIMLOC(  7,  3,  3,  K), K= 0, 3) /-180,  4,  1,  2/
      DATA (KIMLOC(  7,  3,  4,  K), K= 0, 3) /   3,  2,  5,  0/
      DATA (KIMLOC(  7,  3,  5,  K), K= 0, 3) /-120,  2,  3,  2/
      DATA (KIMLOC(  7,  3,  6,  K), K= 0, 3) / 240,  2,  1,  4/
      DATA (KIMLOC(  7,  3,  7,  K), K= 0, 3) /  -3,  0,  7,  0/
      DATA (KIMLOC(  7,  3,  8,  K), K= 0, 3) /  60,  0,  5,  2/
      DATA (KIMLOC(  7,  3,  9,  K), K= 0, 3) / -80,  0,  3,  4/
C
      DATA  NIMLOC(  7,  3) /9/
C
C=======================================================================
C          LAMBDA=7  MIU=4  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  7,  4,      K), K= 1, 3) /  -1,3465,512/
C
C=======================================================================
C          Y74 REAL PART
C=======================================================================
C
      DATA (KRELOC(  7,  4,  1,  K), K= 0, 3) /  3,  6,  0,  1/
      DATA (KRELOC(  7,  4,  2,  K), K= 0, 3) /-15,  4,  2,  1/
      DATA (KRELOC(  7,  4,  3,  K), K= 0, 3) /-10,  4,  0,  3/
      DATA (KRELOC(  7,  4,  4,  K), K= 0, 3) /-15,  2,  4,  1/
      DATA (KRELOC(  7,  4,  5,  K), K= 0, 3) / 60,  2,  2,  3/
      DATA (KRELOC(  7,  4,  6,  K), K= 0, 3) /  3,  0,  6,  1/
      DATA (KRELOC(  7,  4,  7,  K), K= 0, 3) /-10,  0,  4,  3/
C
      DATA  NRELOC(  7,  4) /7/
C
C=======================================================================
C          Y74 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  7,  4,  1,  K), K= 0, 3) / 12,  5,  1,  1/
      DATA (KIMLOC(  7,  4,  2,  K), K= 0, 3) /-40,  3,  1,  3/
      DATA (KIMLOC(  7,  4,  3,  K), K= 0, 3) /-12,  1,  5,  1/
      DATA (KIMLOC(  7,  4,  4,  K), K= 0, 3) / 40,  1,  3,  3/
C
      DATA  NIMLOC(  7,  4) /4/
C
C=======================================================================
C          LAMBDA=7  MIU=5  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  7,  5,      K), K= 1, 3) /  1, 3465, 2048/
C
C=======================================================================
C          Y75 REAL PART
C=======================================================================
C
      DATA (KRELOC(  7,  5,  1,  K), K= 0, 3) /  1,  7,  0,  0/
      DATA (KRELOC(  7,  5,  2,  K), K= 0, 3) / -9,  5,  2,  0/
      DATA (KRELOC(  7,  5,  3,  K), K= 0, 3) /-12,  5,  0,  2/
      DATA (KRELOC(  7,  5,  4,  K), K= 0, 3) / -5,  3,  4,  0/
      DATA (KRELOC(  7,  5,  5,  K), K= 0, 3) /120,  3,  2,  2/
      DATA (KRELOC(  7,  5,  6,  K), K= 0, 3) /  5,  1,  6,  0/
      DATA (KRELOC(  7,  5,  7,  K), K= 0, 3) /-60,  1,  4,  2/
C
      DATA  NRELOC(  7,  5) /7/
C
C=======================================================================
C          Y75 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  7,  5,  1,  K), K= 0, 3) /  5,  6,  1,  0/
      DATA (KIMLOC(  7,  5,  2,  K), K= 0, 3) / -5,  4,  3,  0/
      DATA (KIMLOC(  7,  5,  3,  K), K= 0, 3) /-60,  4,  1,  2/
      DATA (KIMLOC(  7,  5,  4,  K), K= 0, 3) / -9,  2,  5,  0/
      DATA (KIMLOC(  7,  5,  5,  K), K= 0, 3) /120,  2,  3,  2/
      DATA (KIMLOC(  7,  5,  6,  K), K= 0, 3) /  1,  0,  7,  0/
      DATA (KIMLOC(  7,  5,  7,  K), K= 0, 3) /-12,  0,  5,  2/
C
      DATA  NIMLOC(  7,  5) /7/
C
C=======================================================================
C          LAMBDA=7  MIU=6  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  7,  6,      K), K= 1, 3) /   1, 45045, 1024/
C
C=======================================================================
C          Y76 REAL PART
C=======================================================================
C
      DATA (KRELOC(  7,  6,  1,  K), K= 0, 3) /  1,  6,  0,  1/
      DATA (KRELOC(  7,  6,  2,  K), K= 0, 3) /-15,  4,  2,  1/
      DATA (KRELOC(  7,  6,  3,  K), K= 0, 3) / 15,  2,  4,  1/
      DATA (KRELOC(  7,  6,  4,  K), K= 0, 3) / -1,  0,  6,  1/
C
      DATA  NRELOC(  7,  6) /4/
C
C=======================================================================
C          Y76 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  7,  6,  1,  K), K= 0, 3) /  6,  5,  1,  1/
      DATA (KIMLOC(  7,  6,  2,  K), K= 0, 3) /-20,  3,  3,  1/
      DATA (KIMLOC(  7,  6,  3,  K), K= 0, 3) /  6,  1,  5,  1/
C
      DATA  NIMLOC(  7,  6) /3/
C
C=======================================================================
C          LAMBDA=7  MIU=7  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  7,  7,      K), K= 1, 3) /  -1, 6435, 2048/
C
C=======================================================================
C          Y77 REAL PART
C=======================================================================
C
      DATA (KRELOC(  7,  7,  1,  K), K= 0, 3) /  1,  7,  0,  0/
      DATA (KRELOC(  7,  7,  2,  K), K= 0, 3) /-21,  5,  2,  0/
      DATA (KRELOC(  7,  7,  3,  K), K= 0, 3) / 35,  3,  4,  0/
      DATA (KRELOC(  7,  7,  4,  K), K= 0, 3) / -7,  1,  6,  0/
C
      DATA  NRELOC(  7,  7) /4/
C
C=======================================================================
C          Y77 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  7,  7,  1,  K), K= 0, 3) /  7,  6,  1,  0/
      DATA (KIMLOC(  7,  7,  2,  K), K= 0, 3) /-35,  4,  3,  0/
      DATA (KIMLOC(  7,  7,  3,  K), K= 0, 3) / 21,  2,  5,  0/
      DATA (KIMLOC(  7,  7,  4,  K), K= 0, 3) / -1,  0,  7,  0/
C
      DATA  NIMLOC(  7,  7) /4/
C
C
C=======================================================================
C          LAMBDA=8  MIU=0  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  8,  0,      K), K= 1, 3) /  1,  17, 16384/
C
C=======================================================================
C          Y80 REAL PART ONLY
C=======================================================================
C
      DATA (KRELOC(  8,  0,  1,  K), K= 0, 3) /   35,  8,  0,  0/
      DATA (KRELOC(  8,  0,  2,  K), K= 0, 3) /  140,  6,  2,  0/
      DATA (KRELOC(  8,  0,  3,  K), K= 0, 3) /-1120,  6,  0,  2/
      DATA (KRELOC(  8,  0,  4,  K), K= 0, 3) /  210,  4,  4,  0/
      DATA (KRELOC(  8,  0,  5,  K), K= 0, 3) /-3360,  4,  2,  2/
      DATA (KRELOC(  8,  0,  6,  K), K= 0, 3) / 3360,  4,  0,  4/
      DATA (KRELOC(  8,  0,  7,  K), K= 0, 3) /  140,  2,  6,  0/
      DATA (KRELOC(  8,  0,  8,  K), K= 0, 3) /-3360,  2,  4,  2/
      DATA (KRELOC(  8,  0,  9,  K), K= 0, 3) / 6720,  2,  2,  4/
      DATA (KRELOC(  8,  0, 10,  K), K= 0, 3) /-1792,  2,  0,  6/
      DATA (KRELOC(  8,  0, 11,  K), K= 0, 3) /   35,  0,  8,  0/
      DATA (KRELOC(  8,  0, 12,  K), K= 0, 3) /-1120,  0,  6,  2/
      DATA (KRELOC(  8,  0, 13,  K), K= 0, 3) / 3360,  0,  4,  4/
      DATA (KRELOC(  8,  0, 14,  K), K= 0, 3) /-1792,  0,  2,  6/
      DATA (KRELOC(  8,  0, 15,  K), K= 0, 3) /  128,  0,  0,  8/
C
      DATA  NRELOC(  8,  0) /15/
C
C=======================================================================
C          LAMBDA=8  MIU=1  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  8,  1,      K), K= 1, 3) /  1,153,2048/
C
C=======================================================================
C          Y81 REAL PART
C=======================================================================
C
      DATA (KRELOC(  8,  1,  1,  K), K= 0, 3) /  35,  7,  0,  1/
      DATA (KRELOC(  8,  1,  2,  K), K= 0, 3) / 105,  5,  2,  1/
      DATA (KRELOC(  8,  1,  3,  K), K= 0, 3) /-280,  5,  0,  3/
      DATA (KRELOC(  8,  1,  4,  K), K= 0, 3) / 105,  3,  4,  1/
      DATA (KRELOC(  8,  1,  5,  K), K= 0, 3) /-560,  3,  2,  3/
      DATA (KRELOC(  8,  1,  6,  K), K= 0, 3) / 336,  3,  0,  5/
      DATA (KRELOC(  8,  1,  7,  K), K= 0, 3) /  35,  1,  6,  1/
      DATA (KRELOC(  8,  1,  8,  K), K= 0, 3) /-280,  1,  4,  3/
      DATA (KRELOC(  8,  1,  9,  K), K= 0, 3) / 336,  1,  2,  5/
      DATA (KRELOC(  8,  1, 10,  K), K= 0, 3) / -64,  1,  0,  7/
C
      DATA  NRELOC(  8,  1) /10/
C
C=======================================================================
C          Y81 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  8,  1,  1,  K), K= 0, 3) /  35,  6,  1,  1/
      DATA (KIMLOC(  8,  1,  2,  K), K= 0, 3) / 105,  4,  3,  1/
      DATA (KIMLOC(  8,  1,  3,  K), K= 0, 3) /-280,  4,  1,  3/
      DATA (KIMLOC(  8,  1,  4,  K), K= 0, 3) / 105,  2,  5,  1/
      DATA (KIMLOC(  8,  1,  5,  K), K= 0, 3) /-560,  2,  3,  3/
      DATA (KIMLOC(  8,  1,  6,  K), K= 0, 3) / 336,  2,  1,  5/
      DATA (KIMLOC(  8,  1,  7,  K), K= 0, 3) /  35,  0,  7,  1/
      DATA (KIMLOC(  8,  1,  8,  K), K= 0, 3) /-280,  0,  5,  3/
      DATA (KIMLOC(  8,  1,  9,  K), K= 0, 3) / 336,  0,  3,  5/
      DATA (KIMLOC(  8,  1, 10,  K), K= 0, 3) / -64,  0,  1,  7/
C
      DATA  NIMLOC(  8,  1) /10/
C
C=======================================================================
C          LAMBDA=8  MIU=2  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  8,  2,      K), K= 1, 3) /-1, 5355, 4096/
C
C=======================================================================
C          Y82 REAL PART
C=======================================================================
C
      DATA (KRELOC(  8,  2,  1,  K), K= 0, 3) /  1,  8,  0,  0/
      DATA (KRELOC(  8,  2,  2,  K), K= 0, 3) /  2,  6,  2,  0/
      DATA (KRELOC(  8,  2,  3,  K), K= 0, 3) /-30,  6,  0,  2/
      DATA (KRELOC(  8,  2,  4,  K), K= 0, 3) /-30,  4,  2,  2/
      DATA (KRELOC(  8,  2,  5,  K), K= 0, 3) / 80,  4,  0,  4/
      DATA (KRELOC(  8,  2,  6,  K), K= 0, 3) / -2,  2,  6,  0/
      DATA (KRELOC(  8,  2,  7,  K), K= 0, 3) / 30,  2,  4,  2/
      DATA (KRELOC(  8,  2,  8,  K), K= 0, 3) /-32,  2,  0,  6/
      DATA (KRELOC(  8,  2,  9,  K), K= 0, 3) / -1,  0,  8,  0/
      DATA (KRELOC(  8,  2, 10,  K), K= 0, 3) / 30,  0,  6,  2/
      DATA (KRELOC(  8,  2, 11,  K), K= 0, 3) /-80,  0,  4,  4/
      DATA (KRELOC(  8,  2, 12,  K), K= 0, 3) / 32,  0,  2,  6/
C
      DATA  NRELOC(  8,  2) /12/
C
C=======================================================================
C          Y82 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  8,  2,  1,  K), K= 0, 3) /   2,  7,  1,  0/
      DATA (KIMLOC(  8,  2,  2,  K), K= 0, 3) /   6,  5,  3,  0/
      DATA (KIMLOC(  8,  2,  3,  K), K= 0, 3) / -60,  5,  1,  2/
      DATA (KIMLOC(  8,  2,  4,  K), K= 0, 3) /   6,  3,  5,  0/
      DATA (KIMLOC(  8,  2,  5,  K), K= 0, 3) /-120,  3,  3,  2/
      DATA (KIMLOC(  8,  2,  6,  K), K= 0, 3) / 160,  3,  1,  4/
      DATA (KIMLOC(  8,  2,  7,  K), K= 0, 3) /   2,  1,  7,  0/
      DATA (KIMLOC(  8,  2,  8,  K), K= 0, 3) / -60,  1,  5,  2/
      DATA (KIMLOC(  8,  2,  9,  K), K= 0, 3) / 160,  1,  3,  4/
      DATA (KIMLOC(  8,  2, 10,  K), K= 0, 3) / -64,  1,  1,  6/
C
      DATA  NIMLOC(  8,  2) /10/
C
C=======================================================================
C          LAMBDA=8  MIU=3  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  8,  3,      K), K= 1, 3) /  -1, 19635, 2048/
C
C=======================================================================
C          Y83 REAL PART
C=======================================================================
C
      DATA (KRELOC(  8,  3,  1,  K), K= 0, 3) /   3,  7,  0,  1/
      DATA (KRELOC(  8,  3,  2,  K), K= 0, 3) /  -3,  5,  2,  1/
      DATA (KRELOC(  8,  3,  3,  K), K= 0, 3) / -20,  5,  0,  3/
      DATA (KRELOC(  8,  3,  4,  K), K= 0, 3) / -15,  3,  4,  1/
      DATA (KRELOC(  8,  3,  5,  K), K= 0, 3) /  40,  3,  2,  3/
      DATA (KRELOC(  8,  3,  6,  K), K= 0, 3) /  16,  3,  0,  5/
      DATA (KRELOC(  8,  3,  7,  K), K= 0, 3) /  -9,  1,  6,  1/
      DATA (KRELOC(  8,  3,  8,  K), K= 0, 3) /  60,  1,  4,  3/
      DATA (KRELOC(  8,  3,  9,  K), K= 0, 3) / -48,  1,  2,  5/
C
      DATA  NRELOC(  8,  3) /9/
C
C=======================================================================
C          Y83 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  8,  3,  1,  K), K= 0, 3) /   9,  6,  1,  1/
      DATA (KIMLOC(  8,  3,  2,  K), K= 0, 3) /  15,  4,  3,  1/
      DATA (KIMLOC(  8,  3,  3,  K), K= 0, 3) / -60,  4,  1,  3/
      DATA (KIMLOC(  8,  3,  4,  K), K= 0, 3) /   3,  2,  5,  1/
      DATA (KIMLOC(  8,  3,  5,  K), K= 0, 3) / -40,  2,  3,  3/
      DATA (KIMLOC(  8,  3,  6,  K), K= 0, 3) /  48,  2,  1,  5/
      DATA (KIMLOC(  8,  3,  7,  K), K= 0, 3) /  -3,  0,  7,  1/
      DATA (KIMLOC(  8,  3,  8,  K), K= 0, 3) /  20,  0,  5,  3/
      DATA (KIMLOC(  8,  3,  9,  K), K= 0, 3) / -16,  0,  3,  5/
C
      DATA  NIMLOC(  8,  3) /9/
C
C=======================================================================
C          LAMBDA=8  MIU=4  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  8,  4,      K), K= 1, 3) /   1,11781,8192/
C
C=======================================================================
C          Y84 REAL PART
C=======================================================================
C
      DATA (KRELOC(  8,  4,  1,  K), K= 0, 3) /   1,  8,  0,  0/
      DATA (KRELOC(  8,  4,  2,  K), K= 0, 3) /  -4,  6,  2,  0/
      DATA (KRELOC(  8,  4,  3,  K), K= 0, 3) / -24,  6,  0,  2/
      DATA (KRELOC(  8,  4,  4,  K), K= 0, 3) / -10,  4,  4,  0/
      DATA (KRELOC(  8,  4,  5,  K), K= 0, 3) / 120,  4,  2,  2/
      DATA (KRELOC(  8,  4,  6,  K), K= 0, 3) /  40,  4,  0,  4/
      DATA (KRELOC(  8,  4,  7,  K), K= 0, 3) /  -4,  2,  6,  0/
      DATA (KRELOC(  8,  4,  8,  K), K= 0, 3) / 120,  2,  4,  2/
      DATA (KRELOC(  8,  4,  9,  K), K= 0, 3) /-240,  2,  2,  4/
      DATA (KRELOC(  8,  4, 10,  K), K= 0, 3) /   1,  0,  8,  0/
      DATA (KRELOC(  8,  4, 11,  K), K= 0, 3) / -24,  0,  6,  2/
      DATA (KRELOC(  8,  4, 12,  K), K= 0, 3) /  40,  0,  4,  4/
C
      DATA  NRELOC(  8,  4) /12/
C
C=======================================================================
C          Y84 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  8,  4,  1,  K), K= 0, 3) /   4,  7,  1,  0/
      DATA (KIMLOC(  8,  4,  2,  K), K= 0, 3) /   4,  5,  3,  0/
      DATA (KIMLOC(  8,  4,  3,  K), K= 0, 3) / -96,  5,  1,  2/
      DATA (KIMLOC(  8,  4,  4,  K), K= 0, 3) /  -4,  3,  5,  0/
      DATA (KIMLOC(  8,  4,  5,  K), K= 0, 3) / 160,  3,  1,  4/
      DATA (KIMLOC(  8,  4,  6,  K), K= 0, 3) /  -4,  1,  7,  0/
      DATA (KIMLOC(  8,  4,  7,  K), K= 0, 3) /  96,  1,  5,  2/
      DATA (KIMLOC(  8,  4,  8,  K), K= 0, 3) /-160,  1,  3,  4/
C
      DATA  NIMLOC(  8,  4) /8/
C
C=======================================================================
C          LAMBDA=8  MIU=5  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  8,  5,      K), K= 1, 3) /  1, 153153, 2048/
C
C=======================================================================
C          Y85 REAL PART
C=======================================================================
C
      DATA (KRELOC(  8,  5,  1,  K), K= 0, 3) /  1,  7,  0,  1/
      DATA (KRELOC(  8,  5,  2,  K), K= 0, 3) / -9,  5,  2,  1/
      DATA (KRELOC(  8,  5,  3,  K), K= 0, 3) / -4,  5,  0,  3/
      DATA (KRELOC(  8,  5,  4,  K), K= 0, 3) / -5,  3,  4,  1/
      DATA (KRELOC(  8,  5,  5,  K), K= 0, 3) / 40,  3,  2,  3/
      DATA (KRELOC(  8,  5,  6,  K), K= 0, 3) /  5,  1,  6,  1/
      DATA (KRELOC(  8,  5,  7,  K), K= 0, 3) /-20,  1,  4,  3/
C
      DATA  NRELOC(  8,  5) /7/
C
C=======================================================================
C          Y85 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  8,  5,  1,  K), K= 0, 3) /  5,  6,  1,  1/
      DATA (KIMLOC(  8,  5,  2,  K), K= 0, 3) / -5,  4,  3,  1/
      DATA (KIMLOC(  8,  5,  3,  K), K= 0, 3) /-20,  4,  1,  3/
      DATA (KIMLOC(  8,  5,  4,  K), K= 0, 3) / -9,  2,  5,  1/
      DATA (KIMLOC(  8,  5,  5,  K), K= 0, 3) / 40,  2,  3,  3/
      DATA (KIMLOC(  8,  5,  6,  K), K= 0, 3) /  1,  0,  7,  1/
      DATA (KIMLOC(  8,  5,  7,  K), K= 0, 3) / -4,  0,  5,  3/
C
      DATA  NIMLOC(  8,  5) /7/
C
C=======================================================================
C          LAMBDA=8  MIU=6  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  8,  6,      K), K= 1, 3) /  -1,  7293, 4096/
C
C=======================================================================
C          Y86 REAL PART
C=======================================================================
C
      DATA (KRELOC(  8,  6,  1,  K), K= 0, 3) /   1,  8,  0,  0/
      DATA (KRELOC(  8,  6,  2,  K), K= 0, 3) / -14,  6,  2,  0/
      DATA (KRELOC(  8,  6,  3,  K), K= 0, 3) / -14,  6,  0,  2/
      DATA (KRELOC(  8,  6,  4,  K), K= 0, 3) / 210,  4,  2,  2/
      DATA (KRELOC(  8,  6,  5,  K), K= 0, 3) /  14,  2,  6,  0/
      DATA (KRELOC(  8,  6,  6,  K), K= 0, 3) /-210,  2,  4,  2/
      DATA (KRELOC(  8,  6,  7,  K), K= 0, 3) /  -1,  0,  8,  0/
      DATA (KRELOC(  8,  6,  8,  K), K= 0, 3) /  14,  0,  6,  2/
C
      DATA  NRELOC(  8,  6) /8/
C
C=======================================================================
C          Y86 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  8,  6,  1,  K), K= 0, 3) /  6,  7,  1,  0/
      DATA (KIMLOC(  8,  6,  2,  K), K= 0, 3) /-14,  5,  3,  0/
      DATA (KIMLOC(  8,  6,  3,  K), K= 0, 3) /-84,  5,  1,  2/
      DATA (KIMLOC(  8,  6,  4,  K), K= 0, 3) /-14,  3,  5,  0/
      DATA (KIMLOC(  8,  6,  5,  K), K= 0, 3) /280,  3,  3,  2/
      DATA (KIMLOC(  8,  6,  6,  K), K= 0, 3) /  6,  1,  7,  0/
      DATA (KIMLOC(  8,  6,  7,  K), K= 0, 3) /-84,  1,  5,  2/
C
      DATA  NIMLOC(  8,  6) /7/
C
C=======================================================================
C          LAMBDA=8  MIU=7  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  8,  7,      K), K= 1, 3) /  -1, 109395, 2048/
C
C=======================================================================
C          Y87 REAL PART
C=======================================================================
C
      DATA (KRELOC(  8,  7,  1,  K), K= 0, 3) /  1,  7,  0,  1/
      DATA (KRELOC(  8,  7,  2,  K), K= 0, 3) /-21,  5,  2,  1/
      DATA (KRELOC(  8,  7,  3,  K), K= 0, 3) / 35,  3,  4,  1/
      DATA (KRELOC(  8,  7,  4,  K), K= 0, 3) / -7,  1,  6,  1/
C
      DATA  NRELOC(  8,  7) /4/
C
C=======================================================================
C          Y87 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  8,  7,  1,  K), K= 0, 3) /  7,  6,  1,  1/
      DATA (KIMLOC(  8,  7,  2,  K), K= 0, 3) /-35,  4,  3,  1/
      DATA (KIMLOC(  8,  7,  3,  K), K= 0, 3) / 21,  2,  5,  1/
      DATA (KIMLOC(  8,  7,  4,  K), K= 0, 3) / -1,  0,  7,  1/
C
      DATA  NIMLOC(  8,  7) /4/
C
C=======================================================================
C          LAMBDA=8  MIU=8  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  8,  8,      K), K= 1, 3) /  1, 109395, 32768/
C
C=======================================================================
C          Y88 REAL PART
C=======================================================================
C
      DATA (KRELOC(  8,  8,  1,  K), K= 0, 3) /  1,  8,  0,  0/
      DATA (KRELOC(  8,  8,  2,  K), K= 0, 3) /-28,  6,  2,  0/
      DATA (KRELOC(  8,  8,  3,  K), K= 0, 3) / 70,  4,  4,  0/
      DATA (KRELOC(  8,  8,  4,  K), K= 0, 3) /-28,  2,  6,  0/
      DATA (KRELOC(  8,  8,  5,  K), K= 0, 3) /  1,  0,  8,  0/
C
      DATA  NRELOC(  8,  8) /5/
C
C=======================================================================
C          Y88 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  8,  8,  1,  K), K= 0, 3) /  8,  7,  1,  0/
      DATA (KIMLOC(  8,  8,  2,  K), K= 0, 3) /-56,  5,  3,  0/
      DATA (KIMLOC(  8,  8,  3,  K), K= 0, 3) / 56,  3,  5,  0/
      DATA (KIMLOC(  8,  8,  4,  K), K= 0, 3) / -8,  1,  7,  0/
C
      DATA  NIMLOC(  8,  8) /4/
C=======================================================================
C          LAMBDA=9  MIU=0  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  9,  0,      K), K= 1, 3) /  1,  19, 16384/
C
C=======================================================================
C          Y90 REAL PART ONLY
C=======================================================================
C
      DATA (KRELOC(  9,  0,  1,  K), K= 0, 3) /   315,  8,  0,  1/
      DATA (KRELOC(  9,  0,  2,  K), K= 0, 3) /  1260,  6,  2,  1/
      DATA (KRELOC(  9,  0,  3,  K), K= 0, 3) / -3360,  6,  0,  3/
      DATA (KRELOC(  9,  0,  4,  K), K= 0, 3) /  1890,  4,  4,  1/
      DATA (KRELOC(  9,  0,  5,  K), K= 0, 3) /-10080,  4,  2,  3/
      DATA (KRELOC(  9,  0,  6,  K), K= 0, 3) /  6048,  4,  0,  5/
      DATA (KRELOC(  9,  0,  7,  K), K= 0, 3) /  1260,  2,  6,  1/
      DATA (KRELOC(  9,  0,  8,  K), K= 0, 3) /-10080,  2,  4,  3/
      DATA (KRELOC(  9,  0,  9,  K), K= 0, 3) / 12096,  2,  2,  5/
      DATA (KRELOC(  9,  0, 10,  K), K= 0, 3) / -2304,  2,  0,  7/
      DATA (KRELOC(  9,  0, 11,  K), K= 0, 3) /   315,  0,  8,  1/
      DATA (KRELOC(  9,  0, 12,  K), K= 0, 3) / -3360,  0,  6,  3/
      DATA (KRELOC(  9,  0, 13,  K), K= 0, 3) /  6048,  0,  4,  5/
      DATA (KRELOC(  9,  0, 14,  K), K= 0, 3) / -2304,  0,  2,  7/
      DATA (KRELOC(  9,  0, 15,  K), K= 0, 3) /   128,  0,  0,  9/
C
      DATA  NRELOC(  9,  0) /15/
C
C=======================================================================
C          LAMBDA=9  MIU=1  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  9,  1,      K), K= 1, 3) /  -1, 855, 32768/
C
C=======================================================================
C          Y91 REAL PART
C=======================================================================
C
      DATA (KRELOC(  9,  1,  1,  K), K= 0, 3) /   7,  9,  0,  0/
      DATA (KRELOC(  9,  1,  2,  K), K= 0, 3) /  28,  7,  2,  0/
      DATA (KRELOC(  9,  1,  3,  K), K= 0, 3) /-280,  7,  0,  2/
      DATA (KRELOC(  9,  1,  4,  K), K= 0, 3) /  42,  5,  4,  0/
      DATA (KRELOC(  9,  1,  5,  K), K= 0, 3) /-840,  5,  2,  2/
      DATA (KRELOC(  9,  1,  6,  K), K= 0, 3) /1120,  5,  0,  4/
      DATA (KRELOC(  9,  1,  7,  K), K= 0, 3) /  28,  3,  6,  0/
      DATA (KRELOC(  9,  1,  8,  K), K= 0, 3) /-840,  3,  4,  2/
      DATA (KRELOC(  9,  1,  9,  K), K= 0, 3) /2240,  3,  2,  4/
      DATA (KRELOC(  9,  1, 10,  K), K= 0, 3) /-896,  3,  0,  6/
      DATA (KRELOC(  9,  1, 11,  K), K= 0, 3) /   7,  1,  8,  0/
      DATA (KRELOC(  9,  1, 12,  K), K= 0, 3) /-280,  1,  6,  2/
      DATA (KRELOC(  9,  1, 13,  K), K= 0, 3) /1120,  1,  4,  4/
      DATA (KRELOC(  9,  1, 14,  K), K= 0, 3) /-896,  1,  2,  6/
      DATA (KRELOC(  9,  1, 15,  K), K= 0, 3) / 128,  1,  0,  8/
C
      DATA  NRELOC(  9,  1) /15/
C
C=======================================================================
C          Y91 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  9,  1,  1,  K), K= 0, 3) /   7,  8,  1,  0/
      DATA (KIMLOC(  9,  1,  2,  K), K= 0, 3) /  28,  6,  3,  0/
      DATA (KIMLOC(  9,  1,  3,  K), K= 0, 3) /-280,  6,  1,  2/
      DATA (KIMLOC(  9,  1,  4,  K), K= 0, 3) /  42,  4,  5,  0/
      DATA (KIMLOC(  9,  1,  5,  K), K= 0, 3) /-840,  4,  3,  2/
      DATA (KIMLOC(  9,  1,  6,  K), K= 0, 3) /1120,  4,  1,  4/
      DATA (KIMLOC(  9,  1,  7,  K), K= 0, 3) /  28,  2,  7,  0/
      DATA (KIMLOC(  9,  1,  8,  K), K= 0, 3) /-840,  2,  5,  2/
      DATA (KIMLOC(  9,  1,  9,  K), K= 0, 3) /2240,  2,  3,  4/
      DATA (KIMLOC(  9,  1, 10,  K), K= 0, 3) /-896,  2,  1,  6/
      DATA (KIMLOC(  9,  1, 11,  K), K= 0, 3) /   7,  0,  9,  0/
      DATA (KIMLOC(  9,  1, 12,  K), K= 0, 3) /-280,  0,  7,  2/
      DATA (KIMLOC(  9,  1, 13,  K), K= 0, 3) /1120,  0,  5,  4/
      DATA (KIMLOC(  9,  1, 14,  K), K= 0, 3) /-896,  0,  3,  6/
      DATA (KIMLOC(  9,  1, 15,  K), K= 0, 3) / 128,  0,  1,  8/
C
      DATA  NIMLOC(  9,  1) /15/
C
C=======================================================================
C          LAMBDA=9  MIU=2  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  9,  2,      K), K= 1, 3) /-1, 9405, 4096/
C
C=======================================================================
C          Y92 REAL PART
C=======================================================================
C
      DATA (KRELOC(  9,  2,  1,  K), K= 0, 3) /   7,  8,  0,  1/
      DATA (KRELOC(  9,  2,  2,  K), K= 0, 3) /  14,  6,  2,  1/
      DATA (KRELOC(  9,  2,  3,  K), K= 0, 3) / -70,  6,  0,  3/
      DATA (KRELOC(  9,  2,  4,  K), K= 0, 3) / -70,  4,  2,  3/
      DATA (KRELOC(  9,  2,  5,  K), K= 0, 3) / 112,  4,  0,  5/
      DATA (KRELOC(  9,  2,  6,  K), K= 0, 3) / -14,  2,  6,  1/
      DATA (KRELOC(  9,  2,  7,  K), K= 0, 3) /  70,  2,  4,  3/
      DATA (KRELOC(  9,  2,  8,  K), K= 0, 3) / -32,  2,  0,  7/
      DATA (KRELOC(  9,  2,  9,  K), K= 0, 3) /  -7,  0,  8,  1/
      DATA (KRELOC(  9,  2, 10,  K), K= 0, 3) /  70,  0,  6,  3/
      DATA (KRELOC(  9,  2, 11,  K), K= 0, 3) /-112,  0,  4,  5/
      DATA (KRELOC(  9,  2, 12,  K), K= 0, 3) /  32,  0,  2,  7/
C
      DATA  NRELOC(  9,  2) /12/
C
C=======================================================================
C          Y92 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  9,  2,  1,  K), K= 0, 3) /  14,  7,  1,  1/
      DATA (KIMLOC(  9,  2,  2,  K), K= 0, 3) /  42,  5,  3,  1/
      DATA (KIMLOC(  9,  2,  3,  K), K= 0, 3) /-140,  5,  1,  3/
      DATA (KIMLOC(  9,  2,  4,  K), K= 0, 3) /  42,  3,  5,  1/
      DATA (KIMLOC(  9,  2,  5,  K), K= 0, 3) /-280,  3,  3,  3/
      DATA (KIMLOC(  9,  2,  6,  K), K= 0, 3) / 224,  3,  1,  5/
      DATA (KIMLOC(  9,  2,  7,  K), K= 0, 3) /  14,  1,  7,  1/
      DATA (KIMLOC(  9,  2,  8,  K), K= 0, 3) /-140,  1,  5,  3/
      DATA (KIMLOC(  9,  2,  9,  K), K= 0, 3) / 224,  1,  3,  5/
      DATA (KIMLOC(  9,  2, 10,  K), K= 0, 3) / -64,  1,  1,  7/
C
      DATA  NIMLOC(  9,  2) /10/
C
C=======================================================================
C          LAMBDA=9  MIU=3  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  9,  3,      K), K= 1, 3) /  1, 21945, 16384/
C
C=======================================================================
C          Y93 REAL PART
C=======================================================================
C
      DATA (KRELOC(  9,  3,  1,  K), K= 0, 3) /   1,  9,  0,  0/
      DATA (KRELOC(  9,  3,  2,  K), K= 0, 3) / -36,  7,  0,  2/
      DATA (KRELOC(  9,  3,  3,  K), K= 0, 3) /  -6,  5,  4,  0/
      DATA (KRELOC(  9,  3,  4,  K), K= 0, 3) /  36,  5,  2,  2/
      DATA (KRELOC(  9,  3,  5,  K), K= 0, 3) / 120,  5,  0,  4/
      DATA (KRELOC(  9,  3,  6,  K), K= 0, 3) /  -8,  3,  6,  0/
      DATA (KRELOC(  9,  3,  7,  K), K= 0, 3) / 180,  3,  4,  2/
      DATA (KRELOC(  9,  3,  8,  K), K= 0, 3) /-240,  3,  2,  4/
      DATA (KRELOC(  9,  3,  9,  K), K= 0, 3) / -64,  3,  0,  6/
      DATA (KRELOC(  9,  3, 10,  K), K= 0, 3) /  -3,  1,  8,  0/
      DATA (KRELOC(  9,  3, 11,  K), K= 0, 3) / 108,  1,  6,  2/
      DATA (KRELOC(  9,  3, 12,  K), K= 0, 3) /-360,  1,  4,  4/
      DATA (KRELOC(  9,  3, 13,  K), K= 0, 3) / 192,  1,  2,  6/
C
      DATA  NRELOC(  9,  3) /13/
C
C=======================================================================
C          Y93 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  9,  3,  1,  K), K= 0, 3) /   3,  8,  1,  0/
      DATA (KIMLOC(  9,  3,  2,  K), K= 0, 3) /   8,  6,  3,  0/
      DATA (KIMLOC(  9,  3,  3,  K), K= 0, 3) /-108,  6,  1,  2/
      DATA (KIMLOC(  9,  3,  4,  K), K= 0, 3) /   6,  4,  5,  0/
      DATA (KIMLOC(  9,  3,  5,  K), K= 0, 3) /-180,  4,  3,  2/
      DATA (KIMLOC(  9,  3,  6,  K), K= 0, 3) / 360,  4,  1,  4/
      DATA (KIMLOC(  9,  3,  7,  K), K= 0, 3) / -36,  2,  5,  2/
      DATA (KIMLOC(  9,  3,  8,  K), K= 0, 3) / 240,  2,  3,  4/
      DATA (KIMLOC(  9,  3,  9,  K), K= 0, 3) /-192,  2,  1,  6/
      DATA (KIMLOC(  9,  3, 10,  K), K= 0, 3) /  -1,  0,  9,  0/
      DATA (KIMLOC(  9,  3, 11,  K), K= 0, 3) /  36,  0,  7,  2/
      DATA (KIMLOC(  9,  3, 12,  K), K= 0, 3) /-120,  0,  5,  4/
      DATA (KIMLOC(  9,  3, 13,  K), K= 0, 3) /  64,  0,  3,  6/
C
      DATA  NIMLOC(  9,  3) /13/
C
C=======================================================================
C          LAMBDA=9  MIU=4  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  9,  4,      K), K= 1, 3) /   1,855855,8192/
C
C=======================================================================
C          Y94 REAL PART
C=======================================================================
C
      DATA (KRELOC(  9,  4,  1,  K), K= 0, 3) /  1,  8,  0,  1/
      DATA (KRELOC(  9,  4,  2,  K), K= 0, 3) / -4,  6,  2,  1/
      DATA (KRELOC(  9,  4,  3,  K), K= 0, 3) / -8,  6,  0,  3/
      DATA (KRELOC(  9,  4,  4,  K), K= 0, 3) /-10,  4,  4,  1/
      DATA (KRELOC(  9,  4,  5,  K), K= 0, 3) / 40,  4,  2,  3/
      DATA (KRELOC(  9,  4,  6,  K), K= 0, 3) /  8,  4,  0,  5/
      DATA (KRELOC(  9,  4,  7,  K), K= 0, 3) / -4,  2,  6,  1/
      DATA (KRELOC(  9,  4,  8,  K), K= 0, 3) / 40,  2,  4,  3/
      DATA (KRELOC(  9,  4,  9,  K), K= 0, 3) /-48,  2,  2,  5/
      DATA (KRELOC(  9,  4, 10,  K), K= 0, 3) /  1,  0,  8,  1/
      DATA (KRELOC(  9,  4, 11,  K), K= 0, 3) / -8,  0,  6,  3/
      DATA (KRELOC(  9,  4, 12,  K), K= 0, 3) /  8,  0,  4,  5/
C
      DATA  NRELOC(  9,  4) /12/
C
C=======================================================================
C          Y94 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  9,  4,  1,  K), K= 0, 3) /   4,  7,  1,  1/
      DATA (KIMLOC(  9,  4,  2,  K), K= 0, 3) /   4,  5,  3,  1/
      DATA (KIMLOC(  9,  4,  3,  K), K= 0, 3) / -32,  5,  1,  3/
      DATA (KIMLOC(  9,  4,  4,  K), K= 0, 3) /  -4,  3,  5,  1/
      DATA (KIMLOC(  9,  4,  5,  K), K= 0, 3) /  32,  3,  1,  5/
      DATA (KIMLOC(  9,  4,  6,  K), K= 0, 3) /  -4,  1,  7,  1/
      DATA (KIMLOC(  9,  4,  7,  K), K= 0, 3) /  32,  1,  5,  3/
      DATA (KIMLOC(  9,  4,  8,  K), K= 0, 3) / -32,  1,  3,  5/
C
      DATA  NIMLOC(  9,  4) /8/
C
C=======================================================================
C          LAMBDA=9  MIU=5  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  9,  5,      K), K= 1, 3) / -1, 24453, 16384/
C
C=======================================================================
C          Y95 REAL PART
C=======================================================================
C
      DATA (KRELOC(  9,  5,  1,  K), K= 0, 3) /   1,  9,  0,  0/
      DATA (KRELOC(  9,  5,  2,  K), K= 0, 3) /  -8,  7,  2,  0/
      DATA (KRELOC(  9,  5,  3,  K), K= 0, 3) / -28,  7,  0,  2/
      DATA (KRELOC(  9,  5,  4,  K), K= 0, 3) / -14,  5,  4,  0/
      DATA (KRELOC(  9,  5,  5,  K), K= 0, 3) / 252,  5,  2,  2/
      DATA (KRELOC(  9,  5,  6,  K), K= 0, 3) /  56,  5,  0,  4/
      DATA (KRELOC(  9,  5,  7,  K), K= 0, 3) / 140,  3,  4,  2/
      DATA (KRELOC(  9,  5,  8,  K), K= 0, 3) /-560,  3,  2,  4/
      DATA (KRELOC(  9,  5,  9,  K), K= 0, 3) /   5,  1,  8,  0/
      DATA (KRELOC(  9,  5, 10,  K), K= 0, 3) /-140,  1,  6,  2/
      DATA (KRELOC(  9,  5, 11,  K), K= 0, 3) / 280,  1,  4,  4/
C
      DATA  NRELOC(  9,  5) /11/
C
C=======================================================================
C          Y95 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  9,  5,  1,  K), K= 0, 3) /   5,  8,  1,  0/
      DATA (KIMLOC(  9,  5,  2,  K), K= 0, 3) /-140,  6,  1,  2/
      DATA (KIMLOC(  9,  5,  3,  K), K= 0, 3) / -14,  4,  5,  0/
      DATA (KIMLOC(  9,  5,  4,  K), K= 0, 3) / 140,  4,  3,  2/
      DATA (KIMLOC(  9,  5,  5,  K), K= 0, 3) / 280,  4,  1,  4/
      DATA (KIMLOC(  9,  5,  6,  K), K= 0, 3) /  -8,  2,  7,  0/
      DATA (KIMLOC(  9,  5,  7,  K), K= 0, 3) / 252,  2,  5,  2/
      DATA (KIMLOC(  9,  5,  8,  K), K= 0, 3) /-560,  2,  3,  4/
      DATA (KIMLOC(  9,  5,  9,  K), K= 0, 3) /   1,  0,  9,  0/
      DATA (KIMLOC(  9,  5, 10,  K), K= 0, 3) / -28,  0,  7,  2/
      DATA (KIMLOC(  9,  5, 11,  K), K= 0, 3) /  56,  0,  5,  4/
C
      DATA  NIMLOC(  9,  5) /11/
C
C=======================================================================
C          LAMBDA=9  MIU=6  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  9,  6,      K), K= 1, 3) /  -1,  40755, 4096/
C
C=======================================================================
C          Y96 REAL PART
C=======================================================================
C
      DATA (KRELOC(  9,  6,  1,  K), K= 0, 3) /   3,  8,  0,  1/
      DATA (KRELOC(  9,  6,  2,  K), K= 0, 3) / -42,  6,  2,  1/
      DATA (KRELOC(  9,  6,  3,  K), K= 0, 3) / -14,  6,  0,  3/
      DATA (KRELOC(  9,  6,  4,  K), K= 0, 3) / 210,  4,  2,  3/
      DATA (KRELOC(  9,  6,  5,  K), K= 0, 3) /  42,  2,  6,  1/
      DATA (KRELOC(  9,  6,  6,  K), K= 0, 3) /-210,  2,  4,  3/
      DATA (KRELOC(  9,  6,  7,  K), K= 0, 3) /  -3,  0,  8,  1/
      DATA (KRELOC(  9,  6,  8,  K), K= 0, 3) /  14,  0,  6,  3/
C
      DATA  NRELOC(  9,  6) /8/
C
C=======================================================================
C          Y96 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  9,  6,  1,  K), K= 0, 3) / 18,  7,  1,  1/
      DATA (KIMLOC(  9,  6,  2,  K), K= 0, 3) /-42,  5,  3,  1/
      DATA (KIMLOC(  9,  6,  3,  K), K= 0, 3) /-84,  5,  1,  3/
      DATA (KIMLOC(  9,  6,  4,  K), K= 0, 3) /-42,  3,  5,  1/
      DATA (KIMLOC(  9,  6,  5,  K), K= 0, 3) /280,  3,  3,  3/
      DATA (KIMLOC(  9,  6,  6,  K), K= 0, 3) / 18,  1,  7,  1/
      DATA (KIMLOC(  9,  6,  7,  K), K= 0, 3) /-84,  1,  5,  3/
C
      DATA  NIMLOC(  9,  6) /7/
C
C=======================================================================
C          LAMBDA=9  MIU=7  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  9,  7,      K), K= 1, 3) /  1, 122265, 65536/
C
C=======================================================================
C          Y97 REAL PART
C=======================================================================
C
      DATA (KRELOC(  9,  7,  1,  K), K= 0, 3) /   1,  9,  0,  0/
      DATA (KRELOC(  9,  7,  2,  K), K= 0, 3) / -20,  7,  2,  0/
      DATA (KRELOC(  9,  7,  3,  K), K= 0, 3) / -16,  7,  0,  2/
      DATA (KRELOC(  9,  7,  4,  K), K= 0, 3) /  14,  5,  4,  0/
      DATA (KRELOC(  9,  7,  5,  K), K= 0, 3) / 336,  5,  2,  2/
      DATA (KRELOC(  9,  7,  6,  K), K= 0, 3) /  28,  3,  6,  0/
      DATA (KRELOC(  9,  7,  7,  K), K= 0, 3) /-560,  3,  4,  2/
      DATA (KRELOC(  9,  7,  8,  K), K= 0, 3) /  -7,  1,  8,  0/
      DATA (KRELOC(  9,  7,  9,  K), K= 0, 3) / 112,  1,  6,  2/
C
      DATA  NRELOC(  9,  7) /9/
C
C=======================================================================
C          Y97 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  9,  7,  1,  K), K= 0, 3) /   7,  8,  1,  0/
      DATA (KIMLOC(  9,  7,  2,  K), K= 0, 3) / -28,  6,  3,  0/
      DATA (KIMLOC(  9,  7,  3,  K), K= 0, 3) /-112,  6,  1,  2/
      DATA (KIMLOC(  9,  7,  4,  K), K= 0, 3) / -14,  4,  5,  0/
      DATA (KIMLOC(  9,  7,  5,  K), K= 0, 3) / 560,  4,  3,  2/
      DATA (KIMLOC(  9,  7,  6,  K), K= 0, 3) /  20,  2,  7,  0/
      DATA (KIMLOC(  9,  7,  7,  K), K= 0, 3) /-336,  2,  5,  2/
      DATA (KIMLOC(  9,  7,  8,  K), K= 0, 3) /  -1,  0,  9,  0/
      DATA (KIMLOC(  9,  7,  9,  K), K= 0, 3) /  16,  0,  7,  2/
C
      DATA  NIMLOC(  9,  7) /9/
C
C=======================================================================
C          LAMBDA=9  MIU=8  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  9,  8,      K), K= 1, 3) /  1, 2078505, 32768/
C
C=======================================================================
C          Y98 REAL PART
C=======================================================================
C
      DATA (KRELOC(  9,  8,  1,  K), K= 0, 3) /  1,  8,  0,  1/
      DATA (KRELOC(  9,  8,  2,  K), K= 0, 3) /-28,  6,  2,  1/
      DATA (KRELOC(  9,  8,  3,  K), K= 0, 3) / 70,  4,  4,  1/
      DATA (KRELOC(  9,  8,  4,  K), K= 0, 3) /-28,  2,  6,  1/
      DATA (KRELOC(  9,  8,  5,  K), K= 0, 3) /  1,  0,  8,  1/
C
      DATA  NRELOC(  9,  8) /5/
C
C=======================================================================
C          Y98 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  9,  8,  1,  K), K= 0, 3) /  8,  7,  1,  1/
      DATA (KIMLOC(  9,  8,  2,  K), K= 0, 3) /-56,  5,  3,  1/
      DATA (KIMLOC(  9,  8,  3,  K), K= 0, 3) / 56,  3,  5,  1/
      DATA (KIMLOC(  9,  8,  4,  K), K= 0, 3) / -8,  1,  7,  1/
C
      DATA  NIMLOC(  9,  8) /4/
C
C=======================================================================
C          LAMBDA=9  MIU=9  YLM=C *
C          C=
C=======================================================================
C
      DATA (KOELOC(  9,  9,      K), K= 1, 3) / -1, 230945, 65536/
C
C=======================================================================
C          Y99 REAL PART
C=======================================================================
C
      DATA (KRELOC(  9,  9,  1,  K), K= 0, 3) /  1,  9,  0,  0/
      DATA (KRELOC(  9,  9,  2,  K), K= 0, 3) /-36,  7,  2,  0/
      DATA (KRELOC(  9,  9,  3,  K), K= 0, 3) /126,  5,  4,  0/
      DATA (KRELOC(  9,  9,  4,  K), K= 0, 3) /-84,  3,  6,  0/
      DATA (KRELOC(  9,  9,  5,  K), K= 0, 3) /  9,  1,  8,  0/
C
      DATA  NRELOC(  9,  9) /5/
C
C=======================================================================
C          Y99 IMAGINARY PART
C=======================================================================
C
      DATA (KIMLOC(  9,  9,  1,  K), K= 0, 3) /  9,  8,  1,  0/
      DATA (KIMLOC(  9,  9,  2,  K), K= 0, 3) /-84,  6,  3,  0/
      DATA (KIMLOC(  9,  9,  3,  K), K= 0, 3) /126,  4,  5,  0/
      DATA (KIMLOC(  9,  9,  4,  K), K= 0, 3) /-36,  2,  7,  0/
      DATA (KIMLOC(  9,  9,  5,  K), K= 0, 3) /  1,  0,  9,  0/
C
      DATA  NIMLOC(  9,  9) /5/
C
C=======================================================================
C          Determining the constant factors and rewriting the local
C          variables defined in the data statements into commons.
C=======================================================================
C
      IF (MULMAX.NE.NDMULT) STOP 'MULMAX.NE.NDMULT IN DEFYLM'
C
      SQR4PI=SQRT(16.0*ATAN(1.0))
C
      DO LAMBDA=0,MULMAX
         DO MIU=0,LAMBDA
C
            COEYLM(LAMBDA,MIU) =           KOELOC(LAMBDA,MIU,1)
     *                         * SQRT(DBLE(KOELOC(LAMBDA,MIU,2))/
     *                                DBLE(KOELOC(LAMBDA,MIU,3)))/SQR4PI
C
            NREYLM(LAMBDA,MIU) = NRELOC(LAMBDA,MIU)
C
            DO NUTERM=1,NREYLM(LAMBDA,MIU)
               DO KARTEZ=0,NDKART
C
                  KREYLM(LAMBDA,MIU,NUTERM,KARTEZ)=
     *            KRELOC(LAMBDA,MIU,NUTERM,KARTEZ)
C
               END DO
            END DO
C
            IF (MIU.GT.0) THEN
C
               NIMYLM(LAMBDA,MIU) = NIMLOC(LAMBDA,MIU)
C
               DO NUTERM=1,NIMYLM(LAMBDA,MIU)
                  DO KARTEZ=0,NDKART
C
                     KIMYLM(LAMBDA,MIU,NUTERM,KARTEZ)=
     *               KIMLOC(LAMBDA,MIU,NUTERM,KARTEZ)
C
                  END DO
               END DO
C
            END IF
C
         END DO
      END DO
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_MAGMOM(ICHARG,IZ_FIX,IN_FIX,IFMULT,NMOMAG,KRANKG,
     *                                          IINDIV,LAMIND,MIUIND)
C=======================================================================
      USE params
C=======================================================================
C
      COMPLEX
     *          ELESPI,RESULT,RESODD,SINGPA
      COMPLEX
     *          BIG_PP,BIG_PM,EWAVEF,WAVOCC
C
      DIMENSION
     *          G_SPIN(0:NDISOS),G_ORBI(0:NDISOS)
      DIMENSION
     *          SMOMUL(0:NDMULT,-NDMULT:NDMULT)
C=======================================================================
      ALLOCATABLE ELEORB(:,:),ELESPI(:,:)
C=======================================================================
C
      COMMON
     *       /PAIRIN_SWITCH/ IFPAIR,IFPROJ,IF_HFB,IF_LIP
      COMMON
     *       /SPECTR_SPINOR/ EWAVEF(1:NDBASE,1:NDBASE)
      COMMON
     *       /WSEXEC_V2OCCU/ V2OCCU(1:NDBASE)
      COMMON
     *       /CRAN3D_MAT_PP/ BIG_PP(1:NDBASE,1:NDBASE,0:NDREVE)
     *       /CRAN3D_MAT_PM/ BIG_PM(1:NDBASE,1:NDBASE,0:NDREVE)
      COMMON
     *       /CRAN3D_ALLWAV/ WAVOCC(1:NDBASE,1:2*NDBASE,0:NDREVE)
      COMMON
     *       /SETBAS_DIMENS/ LDBASE
      COMMON
     *       /CONSER_SIMPLY/ ISIMPY
      COMMON
     *       /MAGMOM_AMOMUL/ AMOMUL(0:NDMULT,-NDMULT:NDMULT,0:NDISOS)
C
      DATA
     *      G_SPIN /-3.826,5.586/,
     *      G_ORBI / 0.000,1.000/
C
C=======================================================================
C
C       This subroutine calculates and returns in "AMOMUL" the
C       average values of the magnetic multipole moments
C
C          grad[r**lambda*conjg(y(lambda,miu))]*
C          *(g_spin*s+(2/(lambda+1))*g_orbi*l)
C
C       This formula differs from eq.(b.24) in Ring & Schuck
C       by the complex conjugate of the spherical harmonics.
C       The result is expressed in nuclear magnetons eh/2M
C
C       At the moment, the following cases are supported:
C         - static nucleus with odd or even number of particles
C         - 1D cranking with odd or even number of particles
C         - 3D cranking with odd or even number of particles
C
C=======================================================================
C
      IF (IFMULT.EQ.0) RETURN
C
C=======================================================================
C
      CALL WS_CPUTIM('MAGMOM',1)
C
C=======================================================================
C
C      Initialization of several quantities: Fermi level and whether
C      the number of particles is even or odd
C
      IF (ICHARG.EQ.0) THEN
          NFERMI=IN_FIX/2
	  IMODUL=MOD(IN_FIX,2)
      ELSE
          NFERMI=IZ_FIX/2
	  IMODUL=MOD(IZ_FIX,2)
      END IF
C
      IF (KRANKG.LT.0) THEN
C
          IMODUL=0
C
                           NFERMI=IZ_FIX
          IF (ICHARG.EQ.0) NFERMI=IN_FIX
C
      END IF
C
C      In case of pairing, levels beyond the Fermi level should also
C      be included
C
                       LDSUMM=NFERMI
      IF (IFPAIR.EQ.1) LDSUMM=2*NFERMI
C
      IF (IFPAIR.EQ.0.AND.IINDIV.EQ.0) LDSUMM=NFERMI+1
C
C      In case of no-cranking, one must put the static wave-functions
C      into the proper matrices
C
      IF (KRANKG.EQ.0) THEN
C
          DO J=1,LDBASE
	     DO I=1,LDBASE
		BIG_PP(I,J,0)=      EWAVEF(I,J)
		BIG_PP(I,J,1)=CONJG(EWAVEF(I,J))
	     END DO
	  END DO
C
      END IF
C
C=======================================================================
C
C                       SPIN MOMENTS
C
C=======================================================================
C
C      Zeroing the spin moments
C
      DO LAMBDA=0,NDMULT
         DO MIU=-LAMBDA,LAMBDA
            SMOMUL(LAMBDA,MIU)=0.0
         END DO
      END DO
C
C=======================================================================
C      Calculating the real parts of the spin moments.
C
C      This happens either when the time-reversal and simplex symmetry
C      are broken (3D cranking), or in the case of an odd-even nucleus
C=======================================================================
C
      ALLOCATE(ELESPI(1:NDBASE,1:NDBASE))
C
      IF (KRANKG.LT.0) THEN
C
          IMAREA=1
C
          DO LAMBDA=0,NMOMAG
             DO MIU=0,LAMBDA
C
                CALL WS_INTSPI(ELESPI,LAMBDA,MIU,IMAREA)
C
                RESULT=CMPLX(0.0,0.0)
C
                DO ISTATE=1,LDSUMM
C
		   SINGPA=CMPLX(0.0,0.0)
C
                   DO J=1,LDBASE
                      DO I=1,LDBASE
C
                         SINGPA=SINGPA+CONJG(WAVOCC(I,ISTATE,0))
     *                                      *WAVOCC(J,ISTATE,1)
     *                                      *ELESPI(I,J)
     *                                      *V2OCCU(ISTATE)
C
                      END DO
                   END DO
C
		   RESULT=RESULT+SINGPA
C
      IF (IINDIV.EQ.1.AND.(LAMBDA.EQ.1.OR.LAMBDA.EQ.3)
     *               .AND.MIU.EQ.0) THEN
      WRITE(6,'(''ISTATE = '',I3,
     *          '' s('',I1,'','',I2.2,'') = '',F20.12)')
     *            ISTATE,LAMBDA,MIU,REAL(SINGPA)
      END IF
C
C
		END DO
C
                SMOMUL(LAMBDA,MIU)=2.0*REAL(RESULT)*G_SPIN(ICHARG)
C
             END DO
          END DO
C
      END IF
C
C      Case of an odd nucleus in 1D cranking: only the single nucleon
C      contributes to the spin moment (all other terms cancel out due
C      to the conserved time-reversal symmetry)
C
      IF (KRANKG.GE.0.AND.IMODUL.EQ.1) THEN
C
          IMAREA=1
C
          DO LAMBDA=0,NMOMAG
             DO MIU=0,LAMBDA
C
                CALL WS_INTSPI(ELESPI,LAMBDA,MIU,IMAREA)
C
                RESULT=CMPLX(0.0,0.0)
C
                DO ISTATE=1,LDSUMM+10
C
                   SINGPA=CMPLX(0.0,0.0)
C
                   DO J=1,LDBASE
                      DO I=1,LDBASE
C
                         SINGPA=SINGPA+CONJG(BIG_PP(I,ISTATE,0))
     *                                      *BIG_PP(J,ISTATE,1)
     *                                      *ELESPI(I,J)
     *                                      *V2OCCU(ISTATE)
C
                      END DO
                   END DO
C
		   RESULT=RESULT+SINGPA
C
                   SMOMUL(LAMBDA,MIU)=REAL(RESULT)*G_SPIN(ICHARG)
C
      IF (IINDIV.EQ.1.AND.(LAMBDA.EQ.1.OR.LAMBDA.EQ.3)
     *               .AND.MIU.EQ.0) THEN
      WRITE(6,'(''ISTATE = '',I4,''   - ICHARG = '',I2,
     *          '' S('',I1,'','',I2,'') = '',F12.7)')
     *       ISTATE,ICHARG,LAMBDA,MIU,REAL(SINGPA)*G_SPIN(ICHARG)
      END IF
C
                END DO
	     END DO
C
          END DO
C
      END IF
C
C=======================================================================
C      Calculating the imaginary parts of the spin moments
C=======================================================================
C
      IF (NMOMAG.GE.1) THEN
C
          IMAREA=-1
C
          DO LAMBDA=1,NMOMAG
             DO MIU=-LAMBDA,-1
C
                CALL WS_INTSPI(ELESPI,LAMBDA,MIU,IMAREA)
C
                RESULT=CMPLX(0.0,0.0)
C
                ODDPAR=0.0
C
                IF (KRANKG.GE.0) THEN
C
                    DO ISTATE=1,LDSUMM
C
                       SINGPA=CMPLX(0.0,0.0)
C
                       DO I=1,LDBASE
                          DO J=1,LDBASE
C
                             SINGPA=SINGPA+CONJG(BIG_PP(I,ISTATE,0))
     *                                          *BIG_PP(J,ISTATE,0)
     *                                          *ELESPI(I,J)
     *                                          *V2OCCU(ISTATE)
     *
     *                                    +CONJG(BIG_PP(I,ISTATE,1))
     *                                          *BIG_PP(J,ISTATE,1)
     *                                    *CONJG(ELESPI(I,J))
     *                                          *V2OCCU(ISTATE)
C
                          END DO
                       END DO
c
		       RESULT=RESULT+SINGPA
C
		    END DO
C
                    SMOMUL(LAMBDA,MIU)=AIMAG(RESULT)*G_SPIN(ICHARG)
C
                END IF
C
		IF (KRANKG.LT.0) THEN
C
                    DO ISTATE=1,LDSUMM
C
                       DO I=1,LDBASE
                          DO J=1,LDBASE
C
                             RESULT=RESULT+CONJG(WAVOCC(I,ISTATE,0))
     *                                          *WAVOCC(J,ISTATE,1)
     *                                          *ELESPI(I,J)
     *                                          *V2OCCU(ISTATE)
C
C                             RESULT=RESULT+CONJG(WAVOCC(I,ISTATE,0))
C     *                                          *WAVOCC(J,ISTATE,0)
C     *                                          *ELESPI(I,J)
C     *
C     *                                    +CONJG(WAVOCC(I,ISTATE,1))
C     *                                          *WAVOCC(J,ISTATE,1)
C     *                                    *CONJG(ELESPI(I,J))
C
                          END DO
                       END DO
C
		    END DO
C
                    SMOMUL(LAMBDA,MIU)=2.0*REAL(RESULT)*G_SPIN(ICHARG)
C
		END IF
C
             END DO
          END DO
C
      END IF
C
      DEALLOCATE(ELESPI)
C
C=======================================================================
C
C                       ORBITAL MOMENTS
C
C=======================================================================
C
C      Zeroing the orbital moments
C
      DO LAMBDA=0,NDMULT
         DO MIU=-LAMBDA,LAMBDA
            AMOMUL(LAMBDA,MIU,ICHARG)=0.0
         END DO
      END DO
C
C=======================================================================
C      Calculating the real parts of the orbital moments
C=======================================================================
C
      ALLOCATE(ELEORB(1:NDBASE,1:NDBASE))
C
      IF (KRANKG.LT.0.AND.ICHARG.EQ.1) THEN
C
          IMAREA=1
C
          DO LAMBDA=0,NMOMAG
             DO MIU=0,LAMBDA
C
                CALL WS_INTORB(ELEORB,LAMBDA,MIU,IMAREA)
C
                RESULT=CMPLX(0.0,0.0)
C
                DO ISTATE=1,LDSUMM
C
                   DO I=1,LDBASE
                      DO J=1,LDBASE
C
                         RESULT=RESULT+CONJG(WAVOCC(I,ISTATE,0))
     *                                      *WAVOCC(J,ISTATE,1)
     *                                      *ELEORB(I,J)
     *                                      *V2OCCU(ISTATE)
C
                      END DO
                   END DO
C
		END DO
C
                AMOMUL(LAMBDA,MIU,ICHARG)=-2*AIMAG(RESULT)
     *                                     *G_ORBI(ICHARG)
             END DO
          END DO
C
      END IF
C
C      Case of an odd nucleus: only the single nucleon contributes to
C      the spin moment (all other terms cancel out due to the conserved
C      time-reversal symmetry)
C
      IF (KRANKG.GE.0.AND.IMODUL.EQ.1) THEN
C
          IMAREA=1
C
          DO LAMBDA=0,NMOMAG
             DO MIU=0,LAMBDA
C
                CALL WS_INTORB(ELEORB,LAMBDA,MIU,IMAREA)
C
                RESULT=CMPLX(0.0,0.0)
C
                DO ISTATE=1,LDSUMM+10
C
                   SINGPA=CMPLX(0.0,0.0)
C
                   DO J=1,LDBASE
                      DO I=1,LDBASE
C
                         SINGPA=SINGPA+CONJG(BIG_PP(I,ISTATE,0))
     *                                      *BIG_PP(J,ISTATE,1)
     *                                      *ELEORB(I,J)
     *                                      *V2OCCU(ISTATE)
C
                      END DO
                   END DO
C
		   RESULT=RESULT+SINGPA
C
      IF (IINDIV.EQ.1.AND.(LAMBDA.EQ.1.OR.LAMBDA.EQ.3)
     *               .AND.MIU.EQ.0) THEN
      WRITE(6,'(''ISTATE = '',I4,''   - ICHARG = '',I2,
     *          '' a('',I1,'','',I2,'') = '',F12.7)')
     *      ISTATE,ICHARG,LAMBDA,MIU,-AIMAG(SINGPA)*G_ORBI(ICHARG)
      END IF
C
		END DO
C
                AMOMUL(LAMBDA,MIU,ICHARG)=-AIMAG(RESULT)*G_ORBI(ICHARG)
C
             END DO
          END DO
C
      END IF
C
C=======================================================================
C      Calculating the imaginary parts of the orbital moments
C=======================================================================
C
      IF (NMOMAG.GE.1.AND.ICHARG.EQ.1) THEN
C
          IMAREA=-1
C
          DO LAMBDA=1,NMOMAG
             DO MIU=-LAMBDA,-1
C
                CALL WS_INTORB(ELEORB,LAMBDA,MIU,IMAREA)
C
                RESULT=CMPLX(0.0,0.0)
C
		ODDPAR=0.0
C
                IF (KRANKG.GE.0) THEN
C
                    DO ISTATE=1,LDSUMM
C
		       SINGPA=CMPLX(0.0,0.0)
C
                       DO I=1,LDBASE
                          DO J=1,LDBASE
C
                             SINGPA=SINGPA+CONJG(BIG_PP(I,ISTATE,0))
     *                                          *BIG_PP(J,ISTATE,0)
     *                                          *ELEORB(I,J)
     *                                          *V2OCCU(ISTATE)
     *
     *                                    +CONJG(BIG_PP(I,ISTATE,1))
     *                                          *BIG_PP(J,ISTATE,1)
     *                                          *ELEORB(I,J)
     *                                          *V2OCCU(ISTATE)
C
                          END DO
                       END DO
C
		       RESULT=RESULT+SINGPA
C
      IF (IINDIV.EQ.1.AND.(LAMBDA.EQ.1.OR.LAMBDA.EQ.3)
     *               .AND.MIU.EQ.0) THEN
      WRITE(6,'(''ISTATE = '',I3,
     *          '' o('',I1,'','',I2.2,'') = '',F20.12)')
     *            ISTATE,LAMBDA,MIU,AIMAG(SINGPA)*G_ORBI(ICHARG)
      END IF
C
		    END DO
C
		END IF
C
		IF (KRANKG.LT.0) THEN
C
                    DO ISTATE=1,LDSUMM
C
                       DO I=1,LDBASE
                          DO J=1,LDBASE
C
                             RESULT=RESULT+CONJG(WAVOCC(I,ISTATE,0))
     *                                          *WAVOCC(J,ISTATE,0)
     *                                          *ELEORB(I,J)
     *                                          *V2OCCU(ISTATE)
     *
     *                                    +CONJG(WAVOCC(I,ISTATE,1))
     *                                          *WAVOCC(J,ISTATE,1)
     *                                          *ELEORB(I,J)
     *                                          *V2OCCU(ISTATE)
C
                          END DO
                       END DO
C
		    END DO
C
		END IF
C
                AMOMUL(LAMBDA,MIU,ICHARG)=AIMAG(RESULT)*G_ORBI(ICHARG)
     *                                                 +ODDPAR
C
             END DO
          END DO
C
      END IF
C
      DEALLOCATE(ELEORB)
C
C=======================================================================
C
C      Adding orbital and spin parts of magnetic moments
C
C=======================================================================
C
      DO LAMBDA=0,NMOMAG
         DO MIU=-LAMBDA,LAMBDA
            AMOMUL(LAMBDA,MIU,ICHARG)=AMOMUL(LAMBDA,MIU,ICHARG)
     *                               +SMOMUL(LAMBDA,MIU)
         END DO
      END DO
C
C=======================================================================
C
      CONTINUE
C
C=======================================================================
C
      CALL WS_CPUTIM('MAGMOM',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_INTSPI(ELESPI,LAMBDA,MIU,IMAREA)
C=======================================================================
      USE params
C=======================================================================
C
      COMPLEX
     *          ELESPI
C
      DIMENSION
     *          ELESPI(1:NDBASE,1:NDBASE)
C
      COMMON
     *       /SETBAS_NXNYNZ/ NXVECT(1:NDBASE),
     *                       NYVECT(1:NDBASE),
     *                       NZVECT(1:NDBASE)
      COMMON
     *       /SETBAS_DIMENS/ LDBASE
      COMMON
     *       /INTMUL_COEYLM/ COEYLM(0:NDMULT,0:NDMULT)
      COMMON
     *       /INTMUL_IMAREA/ NREYLM(0:NDMULT,0:NDMULT),
     *                       KREYLM(0:NDMULT,0:NDMULT,NDTERM,0:NDKART),
     *
     *                       NIMYLM(0:NDMULT,1:NDMULT),
     *                       KIMYLM(0:NDMULT,1:NDMULT,NDTERM,0:NDKART)
      COMMON
     *       /DEVMUL_COEMUL/ COMULT(0:NDMULT,0:NDOSGS,0:NDOSGS,1:NDKART)
      COMMON
     *       /TPHASE_PHASES/ TPHAPP(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAPM(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAMP(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAMM(0:NDYMAX,0:NDYMAX,0:NDKART)
      COMMON
     *       /DEFUNI_MAGUNI/ AUNITS(0:NDMULT,0:NDMULT)
C
C=======================================================================
C
C
C      This subroutine calculates matrix elements of the spin part of
c      the magnetic moment operator. This operator is time-odd. The matrix
C      elements on the harmonic oscillator basis:
C
C                          O(n,n') = < n | Spin | n' >
C
C      are calculated first. These matrix elements are complex number,
C      and the full matrix elements in the t-basis have the following
C      properties:
C       - the diagonal terms are made of the imaginary-part of the O(n,n')
C         times some phase factor which corresponds to the OFF-diagonal
C         block (+,-)
C       - the off-diagonal terms are made of the real parts of the O(n,n')
C         times some phase factor wich corresponds to the DIAGONAL block
C         (+,+)
C
C      See:
C       - Phys. Rev. C62 (2000) 014310, Eq. (46b) for the reference of what
C         exactly is calculated
C       - notes to see how it is calculated.
C
C      IMAREA = 1 (-1) requests calculation of the real (imaginary) part
C
C
C=======================================================================
C
      CALL WS_CPUTIM('INTSPI',1)
C
C=======================================================================
C
      DO IBRA=1,LDBASE
         DO IKET=1,LDBASE
            ELESPI(IBRA,IKET)=CMPLX(0.0,0.0)
         END DO
      END DO
C
      NIU=IABS(MIU)
C
      IF (MIU.GE.0) THEN
          JPHASE=IMAREA
      ELSE
          JPHASE=(-1)**MIU
      END IF
C
C       Matrix elements of the real spin part: Block (+,-) in the t-basis
C
      IF (IMAREA.EQ.1) THEN
C
          DO NUTERM=1,NREYLM(LAMBDA,NIU)
C
             DO IBRA=1,LDBASE
                DO IKET=1,LDBASE
C
C                   Term proportional to s_x
C
                   IF (KREYLM(LAMBDA,NIU,NUTERM,1).NE.0)
     *                 ELESPI(IBRA,IKET) = ELESPI(IBRA,IKET)
     *               + KREYLM(LAMBDA,NIU,NUTERM,0)
     *               * KREYLM(LAMBDA,NIU,NUTERM,1)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,1)-1
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,2)
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,3)
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
     *               * TPHAPM(NYVECT(IBRA),NYVECT(IKET),1)
     *               * CMPLX(1.0,0.0)
C
C                   Term proportional to s_y
C
                   IF (KREYLM(LAMBDA,NIU,NUTERM,2).NE.0)
     *                 ELESPI(IBRA,IKET) = ELESPI(IBRA,IKET)
     *               + KREYLM(LAMBDA,NIU,NUTERM,0)
     *               * KREYLM(LAMBDA,NIU,NUTERM,2)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,1)
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,2)-1
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,3)
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
     *               * TPHAPM(NYVECT(IBRA),NYVECT(IKET),2)
     *               * CMPLX(1.0,0.0)
C
C                   Term proportional to s_z
C
                   IF (KREYLM(LAMBDA,NIU,NUTERM,3).NE.0)
     *                 ELESPI(IBRA,IKET) = ELESPI(IBRA,IKET)
     *               + KREYLM(LAMBDA,NIU,NUTERM,0)
     *               * KREYLM(LAMBDA,NIU,NUTERM,3)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,1)
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,2)
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,3)-1
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
     *               * TPHAPM(NYVECT(IBRA),NYVECT(IKET),3)
     *               * CMPLX(0.0,1.0)
C
                END DO
             END DO
C
          END DO
C
      END IF
C
C      Matrix elements of the imaginary spin part: Block (+,+) in the t-basis
C
      IF (IMAREA.EQ.-1) THEN
C
          DO NUTERM=1,NIMYLM(LAMBDA,NIU)
C
             DO IBRA=1,LDBASE
                DO IKET=1,LDBASE
C
C                   Term proportional to s_x
C
                   IF (KIMYLM(LAMBDA,NIU,NUTERM,1).NE.0)
     *                 ELESPI(IBRA,IKET) = ELESPI(IBRA,IKET)
     *               + KIMYLM(LAMBDA,NIU,NUTERM,0)
     *               * KIMYLM(LAMBDA,NIU,NUTERM,1)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,1)-1
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,2)
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,3)
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
     *               * TPHAPP(NYVECT(IBRA),NYVECT(IKET),1)
     *               * CMPLX(0.0,1.0)
C
C                   Term proportional to s_y
C
                   IF (KIMYLM(LAMBDA,NIU,NUTERM,2).NE.0)
     *                 ELESPI(IBRA,IKET) = ELESPI(IBRA,IKET)
     *               + KIMYLM(LAMBDA,NIU,NUTERM,0)
     *               * KIMYLM(LAMBDA,NIU,NUTERM,2)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,1)
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,2)-1
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,3)
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
     *               * TPHAPP(NYVECT(IBRA),NYVECT(IKET),2)
     *               * CMPLX(0.0,1.0)
C
C                   Term proportional to s_z
C
                   IF (KIMYLM(LAMBDA,NIU,NUTERM,3).NE.0)
     *                 ELESPI(IBRA,IKET) = ELESPI(IBRA,IKET)
     *               + KIMYLM(LAMBDA,NIU,NUTERM,0)
     *               * KIMYLM(LAMBDA,NIU,NUTERM,3)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,1)
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,2)
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,3)-1
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
     *               * TPHAPP(NYVECT(IBRA),NYVECT(IKET),3)
     *               * CMPLX(-1.0,0.0)
C
                END DO
             END DO
C
          END DO
C
      END IF
C
C      Normalization factor and phase
C
      DO IBRA=1,LDBASE
         DO IKET=1,LDBASE
C
            ELESPI(IBRA,IKET)=ELESPI(IBRA,IKET)
     *                       *COEYLM(LAMBDA,NIU)*JPHASE*0.5
C
         END DO
      END DO
C
C        Follows multiplication by the units defined once for all in the
C        subroutine "DEFUNI". The  same  units are implied for positive
C        and negative values of "miu".
C
      DO IBRA=1,LDBASE
         DO IKET=1,LDBASE
            ELESPI(IBRA,IKET)=ELESPI(IBRA,IKET)*AUNITS(LAMBDA,NIU)
         END DO
      END DO
C
C=======================================================================
C
      CALL WS_CPUTIM('INTSPI',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_INTORB(ELEORB,LAMBDA,MIU,IMAREA)
C=======================================================================
      USE params
C=======================================================================
C
      DIMENSION
     *          ELEORB(1:NDBASE,1:NDBASE)
C
      COMMON
     *       /SETBAS_NXNYNZ/ NXVECT(1:NDBASE),
     *                       NYVECT(1:NDBASE),
     *                       NZVECT(1:NDBASE)
      COMMON
     *       /SETBAS_DIMENS/ LDBASE
      COMMON
     *       /INTMUL_COEYLM/ COEYLM(0:NDMULT,0:NDMULT)
      COMMON
     *       /INTMUL_IMAREA/ NREYLM(0:NDMULT,0:NDMULT),
     *                       KREYLM(0:NDMULT,0:NDMULT,NDTERM,0:NDKART),
     *
     *                       NIMYLM(0:NDMULT,1:NDMULT),
     *                       KIMYLM(0:NDMULT,1:NDMULT,NDTERM,0:NDKART)
      COMMON
     *       /DEVMUL_COEMUL/ COMULT(0:NDMULT,0:NDOSGS,0:NDOSGS,1:NDKART)
      COMMON
     *       /TPHASE_PHASES/ TPHAPP(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAPM(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAMP(0:NDYMAX,0:NDYMAX,0:NDKART),
     *                       TPHAMM(0:NDYMAX,0:NDYMAX,0:NDKART)
      COMMON
     *       /DEVMUL_COEMAG/ COGRAD(0:NDMULT,0:NDOSGS,0:NDOSGS,1:NDKART)
      COMMON
     *       /DEFUNI_MAGUNI/ AUNITS(0:NDMULT,0:NDMULT)
C
C=======================================================================
C
C      This subroutine calculates matrix elements of the orbital part of
c      the magnetic moment operator. This operator is time-odd. The matrix
C      elements on the harmonic oscillator basis:
C
C                          O(n,n') = < n | Orbital | n' >
C
C      are calculated first. These matrix elements are complex number,
C      and the full matrix elements in the t-basis have the following
C      properties:
C       - the diagonal terms are made of the imaginary-part of the O(n,n')
C         times some phase factor which corresponds to the OFF-diagonal
C         block (+,-)
C       - the off-diagonal terms are made of the real parts of the O(n,n')
C         times some phase factor wich corresponds to the DIAGONAL block
C         (+,+)
C
C      See:
C       - Phys. Rev. C62 (2000) 014310, Eq. (46b) for the reference of what
C         exactly is calculated
C       - notes to see how it is calculated.
C
C      IMAREA = 1 (-1) requests calculation of the real (imaginary) part
C
C=======================================================================
C
      CALL WS_CPUTIM('INTORB',1)
C
C=======================================================================
C
      DO IBRA=1,LDBASE
         DO IKET=1,LDBASE
            ELEORB(IBRA,IKET)=0.0
         END DO
      END DO
C
      NIU=IABS(MIU)
C
      IF (MIU.GE.0) THEN
          JPHASE=IMAREA
      ELSE
          JPHASE=(-1)**MIU
      END IF
C
C      Matrix elements of the real orbital part < n | Orbital | n' >
C
      IF (IMAREA.EQ.1) THEN
C
          DO NUTERM=1,NREYLM(LAMBDA,NIU)
C
             DO IBRA=1,LDBASE
                DO IKET=1,LDBASE
C
C                   Term proportional to l_x * partial_x
C
                   IF (KREYLM(LAMBDA,NIU,NUTERM,1).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               + KREYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KREYLM(LAMBDA,NIU,NUTERM,1)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,1)-1
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,2)+1
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COGRAD(KREYLM(LAMBDA,NIU,NUTERM,3)
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
                   IF (KREYLM(LAMBDA,NIU,NUTERM,1).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               - KREYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KREYLM(LAMBDA,NIU,NUTERM,1)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,1)-1
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COGRAD(KREYLM(LAMBDA,NIU,NUTERM,2)
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,3)+1
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
C                   Term proportional to l_y * partial_y
C
                   IF (KREYLM(LAMBDA,NIU,NUTERM,2).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               + KREYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KREYLM(LAMBDA,NIU,NUTERM,2)
     *               * COGRAD(KREYLM(LAMBDA,NIU,NUTERM,1)
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,2)-1
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,3)+1
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
                   IF (KREYLM(LAMBDA,NIU,NUTERM,2).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               - KREYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KREYLM(LAMBDA,NIU,NUTERM,2)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,1)+1
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,2)-1
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COGRAD(KREYLM(LAMBDA,NIU,NUTERM,3)
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
C                   Term proportional to l_z * partial_z
C
                   IF (KREYLM(LAMBDA,NIU,NUTERM,3).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               + KREYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KREYLM(LAMBDA,NIU,NUTERM,3)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,1)+1
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COGRAD(KREYLM(LAMBDA,NIU,NUTERM,2)
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,3)-1
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
                   IF (KREYLM(LAMBDA,NIU,NUTERM,3).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               - KREYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KREYLM(LAMBDA,NIU,NUTERM,3)
     *               * COGRAD(KREYLM(LAMBDA,NIU,NUTERM,1)
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,2)+1
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KREYLM(LAMBDA,NIU,NUTERM,3)-1
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
                END DO
             END DO
          END DO
C
C          Normalization and phase for the real orbital part
C
          DO IBRA=1,LDBASE
             DO IKET=1,LDBASE
C
                ELEORB(IBRA,IKET)=ELEORB(IBRA,IKET)*(-1.0)
     *                           *COEYLM(LAMBDA,NIU)*JPHASE
     *                           *TPHAPM(NYVECT(IBRA),NYVECT(IKET),0)
     *                           *2.0/(LAMBDA+1)
C
             END DO
          END DO
C
      END IF
C
C      Matrix elements of the imaginary orbital part
C
      IF (IMAREA.EQ.-1) THEN
C
          DO NUTERM=1,NIMYLM(LAMBDA,NIU)
C
             DO IBRA=1,LDBASE
                DO IKET=1,LDBASE
C
                   IF (KIMYLM(LAMBDA,NIU,NUTERM,1).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               + KIMYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KIMYLM(LAMBDA,NIU,NUTERM,1)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,1)-1
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,2)+1
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COGRAD(KIMYLM(LAMBDA,NIU,NUTERM,3)
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
                   IF (KIMYLM(LAMBDA,NIU,NUTERM,1).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               - KIMYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KIMYLM(LAMBDA,NIU,NUTERM,1)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,1)-1
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COGRAD(KIMYLM(LAMBDA,NIU,NUTERM,2)
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,3)+1
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
                   IF (KIMYLM(LAMBDA,NIU,NUTERM,2).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               + KIMYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KIMYLM(LAMBDA,NIU,NUTERM,2)
     *               * COGRAD(KIMYLM(LAMBDA,NIU,NUTERM,1)
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,2)-1
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,3)+1
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
                   IF (KIMYLM(LAMBDA,NIU,NUTERM,2).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               - KIMYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KIMYLM(LAMBDA,NIU,NUTERM,2)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,1)+1
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,2)-1
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COGRAD(KIMYLM(LAMBDA,NIU,NUTERM,3)
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
                   IF (KIMYLM(LAMBDA,NIU,NUTERM,3).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               + KIMYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KIMYLM(LAMBDA,NIU,NUTERM,3)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,1)+1
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COGRAD(KIMYLM(LAMBDA,NIU,NUTERM,2)
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,3)-1
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
                   IF (KIMYLM(LAMBDA,NIU,NUTERM,3).NE.0)
     *
     *                 ELEORB(IBRA,IKET) = ELEORB(IBRA,IKET)
     *
     *               - KIMYLM(LAMBDA,NIU,NUTERM,0)
     *
     *               * KIMYLM(LAMBDA,NIU,NUTERM,3)
     *               * COGRAD(KIMYLM(LAMBDA,NIU,NUTERM,1)
     *                                     ,NXVECT(IBRA),NXVECT(IKET),1)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,2)+1
     *                                     ,NYVECT(IBRA),NYVECT(IKET),2)
     *               * COMULT(KIMYLM(LAMBDA,NIU,NUTERM,3)-1
     *                                     ,NZVECT(IBRA),NZVECT(IKET),3)
C
                END DO
             END DO
          END DO
C
C          Normalization and phase for the imaginary orbital part
C
          DO IBRA=1,LDBASE
             DO IKET=1,LDBASE
C
                ELEORB(IBRA,IKET)=ELEORB(IBRA,IKET)
     *                           *COEYLM(LAMBDA,NIU)*JPHASE
     *                           *TPHAPP(NYVECT(IBRA),NYVECT(IKET),0)
     *                           *2.0/(LAMBDA+1)
C
             END DO
          END DO
C
      END IF
C
C      Follows multiplication by the units defined once for all in the
C      subroutine "DEFUNI". The  same  units are implied for positive
C      and negative values of "miu".
C
      DO IBRA=1,LDBASE
         DO IKET=1,LDBASE
            ELEORB(IBRA,IKET)=ELEORB(IBRA,IKET)*AUNITS(LAMBDA,NIU)
         END DO
      END DO
C
C=======================================================================
C
      CALL WS_CPUTIM('INTORB',0)
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE WS_EULROT(KSHIMO,KROTMO)
C=======================================================================
      USE params
C=======================================================================
C
      DIMENSION
     *          QUAAUX(-2:2)
      DIMENSION
     *          QUAMOM(1:NDKART,1:NDKART),
     *          ROTMAT(1:NDKART,1:NDKART),
     *          VECTOR(1:NDKART,1:NDKART)
      DIMENSION
     *          SPEQUA(1:NDKART)
      DIMENSION
     *          CELMTS(1:((NDKART+1)*NDKART)/2)
C
      COMPLEX
     *          CELMTS,VECTOR
C
      COMMON
     *       /MOMETS_MULMOM/ QMOMUL(0:NDMULT,-NDMULT:NDMULT,0:NDISOS)
      COMMON
     *       /DEFUNI_OURUNI/ QUNITS(0:NDMULT,0:NDMULT)
      COMMON
     *       /EULROT_EANGLE/ ALPEUL,BETEUL,GAMEUL
C
C=======================================================================
C
C      This routine finds the Euler angles of the principal-axes-frame
C      by using the diagonalisation of the cartesian quadrupole moment
C
C=======================================================================
C
      PI=4.0*ATAN(1.0)
C
      SQRT_6=SQRT(6.0)
C
      ALPEUL=0.0
      BETEUL=0.0
      GAMEUL=0.0
C
      DO J_KART=1,NDKART
         DO I_KART=1,NDKART
            VECTOR(I_KART,J_KART)=0.0
         END DO
      END DO
C
C=======================================================================
C
C      IF (KROTMO.EQ.0) GO TO 2
C      IF (KSHIMO.LT.2) STOP 'KSHIMO<2 IN EULROT'
C
      DO MIU=-2,2
         QUAAUX(MIU)=(QMOMUL(2,MIU,0)+QMOMUL(2,MIU,1))
     *                           /QUNITS(2,IABS(MIU))
      END DO
C
C      Constructing the cartesian matrix of the quadrupole moment
C      see varshalovich eq. 2.6(8)
C
      QUAMOM(1,1)= QUAAUX( 2)-QUAAUX(0)/SQRT_6
      QUAMOM(1,2)=-QUAAUX(-2)
      QUAMOM(1,3)=-QUAAUX( 1)
C
      QUAMOM(2,2)=-QUAAUX( 2)-QUAAUX(0)/SQRT_6
      QUAMOM(2,3)=-QUAAUX(-1)
C
      QUAMOM(3,3)= QUAAUX( 0)*2/SQRT_6
C
C      Symmetrizing the cartesian matrix of the quadrupole moment
C
      QUAMOM(2,1)=QUAMOM(1,2)
C
      QUAMOM(3,1)=QUAMOM(1,3)
      QUAMOM(3,2)=QUAMOM(2,3)
C
C=======================================================================
C        Calling the diagonalization routine. Matrices "CELMTS"  and
C        "VECTOR" must be declared complex, even though they contain
C        real numbers.
C=======================================================================
C
      NCOUNT=0
C
      DO KARKET=1,NDKART
         DO KARBRA=KARKET,NDKART
C
            NCOUNT=NCOUNT+1
C
            CELMTS(NCOUNT)=QUAMOM(KARBRA,KARKET)
C
         END DO
      END DO
C
C      CALL WS_DIAMAT(CELMTS,SPEQUA,VECTOR,NDKART,NDKART,NDKART)
C
C=======================================================================
C      Calculating the rotation matrix that is equal to the matrix  of
C      the eigenvectors, see notes 7.54b. Note  that  the eigenvectors
C      nos. 1 and 2 (x and y) have to be exchanged  in order to obtain
C      the Euler angles for the intrinsic system in the  first  sector
C      of the Bohr_gamma angles, 0<=bohr_gamma<=60, i.e.,
C      <q_yy>.le.<q_xx>.le.<q_zz>    or     <y**2>.le.<x**2>.le.<z**2>
C=======================================================================
C
      DO KARBRA=1,NDKART
         ROTMAT(KARBRA,2)=VECTOR(KARBRA,1)
         ROTMAT(KARBRA,1)=VECTOR(KARBRA,2)
         ROTMAT(KARBRA,3)=VECTOR(KARBRA,3)
      END DO
C
C      Changing the phases of cols no. 2 and 3 of the rotation matrix
C      (i.e., the phases of eigenvectors of quadrupole moment) in such
C      a way that the elements in the last row are non-negative. This
C      corresponds to selecting the Euler angles:
C                     0<=BETA<=PI/2 and 0<=GAMMA<=PI
C
      DO KARKET=2,NDKART
C
         IF (ROTMAT(3,KARKET).LT.0) THEN
C
             FACTOR=-1
         ELSE
             FACTOR= 1
C
         END IF
C
         DO KARBRA=1,NDKART
C
            ROTMAT(KARBRA,KARKET)=FACTOR*ROTMAT(KARBRA,KARKET)
C
         END DO
C
      END DO
C
C=======================================================================
C        Changing the phase of column no. 1 of the rotation matrix in
C        such a way that the matrix has a positive determinant
C=======================================================================
C
      DETMAT=ROTMAT(1,1)*ROTMAT(2,2)*ROTMAT(3,3)
     *      +ROTMAT(1,2)*ROTMAT(2,3)*ROTMAT(3,1)
     *      +ROTMAT(1,3)*ROTMAT(2,1)*ROTMAT(3,2)
     *      -ROTMAT(1,1)*ROTMAT(2,3)*ROTMAT(3,2)
     *      -ROTMAT(1,2)*ROTMAT(2,1)*ROTMAT(3,3)
     *      -ROTMAT(1,3)*ROTMAT(2,2)*ROTMAT(3,1)
C
      IF (DETMAT.LT.0) THEN
C
          FACTOR=-1
      ELSE
          FACTOR= 1
C
      END IF
C
      KARKET=1
C
      DO KARBRA=1,NDKART
C
         ROTMAT(KARBRA,KARKET)=FACTOR*ROTMAT(KARBRA,KARKET)
C
      END DO
C
C=======================================================================
C
C      Calculating the Euler angles from the elements of the rotation
C      matrix, see Varshalovich eq. 1.6(54) and notes 7.54b.
C
C      In order to increase precision, below always the smaller matrix
C      elements are used to determine angles. This is so because when
C      the absolute value of cos(angl) is smaller then sin(angl) than
C      the acos gives a more precise angl than asin, and vice versa.
C
C=======================================================================
C
C      Determining BETEUL
C
      ROTHEL=SQRT(ROTMAT(1,3)**2+ROTMAT(2,3)**2)
C
      IF (ABS(ROTMAT(3,3)).LT.ROTHEL) THEN
C
          COSBET=ROTMAT(3,3)
C
          IF (COSBET.GT. 1.0D0) STOP ' TOO LARGE COSBET IN EULROT'
          IF (COSBET.LT. 0.0D0) STOP ' TOO SMALL COSBET IN EULROT'
C
          BETEUL=ACOS(COSBET)
C
          SINBET=SIN(BETEUL)
C
      ELSE
C
          SINBET=ROTHEL
C
          IF (SINBET.GT. 1.0D0) STOP ' TOO LARGE SINBET IN EULROT'
          IF (SINBET.LT. 0.0D0) STOP ' TOO SMALL SINBET IN EULROT'
C
          BETEUL=ASIN(SINBET)
C
      END IF
C
      SINLIM=1.0D-8
C
C      Here we deal with the situation of vanishing angle BETA. Then
C      only the sum ALPHA+GAMMA can be determined, and we can choose
C      GAMMA=0. Values of SINBET smaller than SINLIM are considered to
C      be equal to zero, because what we need then is an approximation
C      of COS(EULBET) by 1, which is very good up to SINLIM**2.
C
      IF (SINBET.LT.SINLIM) THEN
C
          GAMEUL=0.0
C
C          Determining ALPEUL
C
          IF (ABS(ROTMAT(1,1)).LT.ABS(ROTMAT(2,1))) THEN
C
              COSALP= ROTMAT(1,1)
C
              IF (COSALP.GT. 1.0) STOP ' TOO LARGE COSALP IN EULROT'
              IF (COSALP.LT.-1.0) STOP ' TOO SMALL COSALP IN EULROT'
C
              ALPEUL=ACOS(COSALP)
C
              IF (ROTMAT(2,1).LT.0.0) ALPEUL=2*PI-ALPEUL
C
          ELSE
C
              SINALP= ROTMAT(2,1)
C
              IF (SINALP.GT. 1.0) STOP ' TOO LARGE SINALP IN EULROT'
              IF (SINALP.LT.-1.0) STOP ' TOO SMALL SINALP IN EULROT'
C
              ALPEUL=ASIN(SINALP)
C
              IF (ROTMAT(1,1).LT.0.0) ALPEUL=PI-ALPEUL
C
          END IF
C
      ELSE
C
C        Here we deal with the situation of non-zero angle BETA.
C
C          Determining ALPEUL
C
          IF (ABS(ROTMAT(1,3)).LT.ABS(ROTMAT(2,3))) THEN
C
              COSALP= ROTMAT(1,3)/SINBET
C
              IF (COSALP.GT. 1.0) STOP ' TOO LARGE COSALP IN EULROT'
              IF (COSALP.LT.-1.0) STOP ' TOO SMALL COSALP IN EULROT'
C
              ALPEUL=ACOS(COSALP)
C
              IF (ROTMAT(2,3).LT.0.0) ALPEUL=2*PI-ALPEUL
C
          ELSE
C
              SINALP= ROTMAT(2,3)/SINBET
C
              IF (SINALP.GT. 1.0) STOP ' TOO LARGE SINALP IN EULROT'
              IF (SINALP.LT.-1.0) STOP ' TOO SMALL SINALP IN EULROT'
C
              ALPEUL=ASIN(SINALP)
C
              IF (ROTMAT(1,3).LT.0.0) ALPEUL=PI-ALPEUL
C
          END IF
C
C          Determining GAMEUL
C
          IF (ABS(ROTMAT(3,1)).LT.ABS(ROTMAT(3,2))) THEN
C
              COSGAM=-ROTMAT(3,1)/SINBET
C
              IF (COSGAM.GT. 1.0) STOP ' TOO LARGE COSGAM IN EULROT'
              IF (COSGAM.LT.-1.0) STOP ' TOO SMALL COSGAM IN EULROT'
C
              GAMEUL=ACOS(COSGAM)
C
          ELSE
C
              SINGAM= ROTMAT(3,2)/SINBET
C
              IF (SINGAM.GT. 1.0) STOP ' TOO LARGE SINGAM IN EULROT'
              IF (SINGAM.LT. 0.0) STOP ' TOO SMALL SINGAM IN EULROT'
C
              GAMEUL=ASIN(SINGAM)
C
              IF (ROTMAT(3,1).GT.0.0) GAMEUL=PI-GAMEUL
C
          END IF
C
      END IF
C
C=======================================================================
C
  2   CONTINUE
C
C=======================================================================
C
      RETURN
      END
C
C=======================================================================
